<?php
/**
 * Calendar
 *
 * @author    Jebbol
 * @version   $Id: cal.class.php,v 1.11 2006/03/08 16:58:04 jebbol Exp $
 * @package   spl
 * @category  core
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

//=========================================================================
// Class definition
//=========================================================================

/**
 * Calendar
 */
class cal extends abstractData {
  
  function cal() {

    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['cal_id'] = (int) 0;
    $this->_data['adm_id'] = $_SESSION['adm_id'];
    $this->_data['usr_id'] = $_SESSION['usr_id'];
    $this->_data['cal_event_start'] = (int) 0;
    $this->_data['cal_event_end'] = (int) 0;
    $this->_data['cal_event_title'] = '';
    $this->_data['is_active'] = (bool) true;
    $this->_data['create_date'] = time();
    $this->_data['create_usr_id'] = $_SESSION['usr_id'];
    $this->_data['create_src_id'] = $_SESSION['native_id'];
    $this->_data['update_date'] = time();
    $this->_data['update_usr_id'] = $_SESSION['usr_id'];
    $this->_data['update_src_id'] = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary = 'cal_id';
    $this->_table   = 'cal';
    $this->_name    = 'cal_event_title';
    
    // Register properties that must have a value
    $this->_required[] = 'usr_id';
    $this->_required[] = 'cal_event_start';

  }
  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Retrieves all events for a given date
   * 
   * @param  $date date     The date to check (default=today)
   * @param  $usr  integer  Only this users events (default=all users)
   * @param  $active bool   When true, only active events ar returned
   * @return array of Events
   * @access Public
   */
  function getEventsForDate($date=NULL,$usr=NULL,$active=true){
    
    global $SPL_CFG;
    
    // Make sure we have a timestamp
    $date = (empty($date)?time():$this->_parseDate($date));
    $start = strtotime(date("Y-m-d",$date)." 00:00:00");
    $end = strtotime(date("Y-m-d",$date)." 23:59:59");

    $query  = 'SELECT cal_id,cal_event_start,cal_event_end,cal_event_title FROM '.$SPL_CFG['prefix'].'cal cal';
    $query .= ' WHERE cal.adm_id = "'.$_SESSION['adm_id'].'"';
    // Restrict to a specific user
    if (is_numeric($usr)){
      $query .= ' AND cal.usr_id = "'.addslashes($usr).'"';
    }
    // Only active events
    if ($active){
      $query .= ' AND cal.is_active';
    }
    $query .= ' AND (cal.cal_event_start BETWEEN '.$start.' AND '.$end.') OR (cal.cal_event_start <= '.$start.' AND cal.cal_event_end >= "'.$start.'")';
    
    // Ordering
    $query .= ' ORDER BY cal.cal_event_start ASC';

    $result = db_query($query);
    return db_fetch_array($result);
  }
  
  /** 
   * Retrieves all events for a given month
   * 
   * @param  $date date     A date in the month to check (default=today)
   * @param  $usr  integer  Only this users events (default=all users)
   * @param  $active bool   When true, only active events ar returned
   * @return array of Events, ordered by daynum, event_start
   * @access Public
   */
  function getEventsForMonth($date=NULL,$usr=NULL,$active=true){
    
    global $SPL_CFG;
    
    // Make sure we have a timestamp
    $date = (empty($date)?time():$this->_parseDate($date));
    $start = strtotime(date("Y",$date)."-".date("m",$date)."-01 00:00:00");
    $end = strtotime(date("Y",$date)."-".date("m",$date)."-".date("t",$date)." 23:59:59");

    $query  = 'SELECT cal_id,cal_event_start,cal_event_end,cal_event_title FROM '.$SPL_CFG['prefix'].'cal cal';
    $query .= ' WHERE cal.adm_id = "'.$_SESSION['adm_id'].'"';
    // Restrict to a specific user
    if (is_numeric($usr)){
      $query .= ' AND cal.usr_id = "'.addslashes($usr).'"';
    }
    // Only active events
    if ($active){
      $query .= ' AND cal.is_active';
    }
    $query .= ' AND cal.cal_event_start >= '.$start.' AND cal.cal_event_start <= '.$end;
    
    // Ordering
    $query .= ' ORDER BY cal.cal_event_start ASC';

    $events = array();
    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      $events[gmdate("j",$row['cal_event_start'])][$row['cal_event_start']] = $row;
    }
    return $events;
  }

  /** 
   * Retrieves all events for a given week
   * 
   * Please note that Monday=1 and Sunday=7
   * 
   * @param  $date date     A date in the week to check (default=today)
   * @param  $usr  integer  Only this users events (default=all users)
   * @param  $active bool   When true, only active events ar returned
   * @return array of Events, where 'daynum' is the daynumber in the week
   * @access public
   */
  function getEventsForWeek($date=NULL,$usr=NULL,$active=true){
    global $SPL_CFG;
    
    // Make sure we have a timestamp
    $date = (empty($date)?time():$this->_parseDate($date));
    $start = mktime(0,0,0,date("n"),(date("j")-date("w")+1),date("Y"));
    $end = mktime(23,59,59,date("n"),(date("j")+(6-date("w")+1)),date("Y"));

    $query  = 'SELECT cal_id,cal_event_start,cal_event_end,cal_event_title FROM '.$SPL_CFG['prefix'].'cal cal';
    $query .= ' WHERE cal.adm_id = "'.$_SESSION['adm_id'].'"';
    // Restrict to a specific user
    if (is_numeric($usr)){
      $query .= ' AND cal.usr_id = "'.addslashes($usr).'"';
    }
    // Only active events
    if ($active){
      $query .= ' AND cal.is_active';
    }
    $query .= ' AND cal.cal_event_start >= '.$start.' AND cal.cal_event_start <= '.$end;
    
    // Ordering
    $query .= ' ORDER BY cal.cal_event_start ASC';

    $events = array();
    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      $day = gmdate("w",$row['cal_event_start']);
      if ($day==0) $day=7;
      $events[] = array('daynum'=>$day) + $row;
    }
    return $events;
  }


  /** 
   * Retrieves daynumber relative to the given period
   * 
   * @param  $date        date    A date in the period to check (default=today)
   * @param  $periodStart date    The start date of the period (default=Jan1)
   * @return              integer
   * @access Public
   */
  function getPeriodDayNumber($date=NULL,$periodStart=NULL){

    $date = (empty($date)?time():$this->_parseDate($date));
    $periodStart = (empty($periodStart)?mktime(0,0,0,1,1,date("Y")):$this->_parseDate($periodStart));

    $first = date("Y-m-d", $periodStart);
    $last = date("Y-m-d", $date);
  
    //Initialize a today and day counter variables
    $today = $first;
    $dayCount = 0;
    while ($today<=$last){
       $today = date("Y-m-d", strtotime("+1 days", strtotime($today)));
       $day = date("w", strtotime("+1 days", strtotime($today)));
       if($day != 0 && $day != 6)
           $dayCount++;
    }
    return $dayCount;
  }
  
  /** 
   * Retrieves monthnumber relative to the given period
   * 
   * @param  $date        date    A date in the month to check (default=today)
   * @param  $periodStart date    The start date of the period (default=Jan1)
   * @return              integer
   * @access Public
   */
  function getPeriodMonthNumber($date=NULL,$periodStart=NULL){

    $date = (empty($date)?time():$this->_parseDate($date));
    $periodStart = (empty($periodStart)?mktime(0,0,0,1,1,date("Y")):$this->_parseDate($periodStart));
    
    $periodMonth = date("n",$periodStart);
    $periodYear = date("Y",$periodStart);
    $thisMonth = date("n",$date);
    $thisYear = date("Y",$date);
    
    return (($thisYear<>$periodYear?12:0)+$thisMonth-$periodMonth+1);

  }

  /** 
   * Retrieves weeknumber relative to the given period
   * 
   * @param  $date        date    A date in the week to check (default=today)
   * @param  $periodStart date    The start date of the period (default=Jan1)
   * @return              integer
   * @access Public
   */
  function getPeriodWeekNumber($date=NULL,$periodStart=NULL){

    $date = (empty($date)?time():$this->_parseDate($date));
    $periodStart = (empty($periodStart)?mktime(0,0,0,1,1,date("Y")):$this->_parseDate($periodStart));
    
    $periodWeek = date("W",$periodStart);
    $periodYear = date("Y",$periodStart);
    $thisWeek = date("W",$date);
    $thisYear = date("Y",$date);
    
    $lastCalendarWeek = date("W",mktime(23,59,59,12,31,date("Y",$periodStart)));
    
    return (($thisYear<>$periodYear?$lastCalendarWeek:0)+$thisWeek-$periodWeek+1);

  }
  
  /** 
   * Retrieves the last day of a given quarter
   * 
   * @param   $str string     A quarter (e.g. "Q2 2005")
   * @return  timestamp
   * @access  Public
   * @credits Published 2005-02-28 on php.net by Guilherme Blanco, named str2time
   */
  function getQuarterLastDay($str)
  {
    $matches = array();
    $str = preg_replace("/Q(.+){1}\s+([[:digit:]]{4})(.*)/i", "\\2-X\\1X-27\\3", trim($str));
    preg_match_all("/X(.+)X/i", $str, $matches);
    $quarter = $matches[1][0];
    $month = $quarter * 3;
    if (strlen($month) == 1) $month = "0".$month;
    return strtotime(str_replace("X{$quarter}X", $month, $str));
  }
  
  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Converts a date into a timestamp
   * 
   * @param  $dateTime date     A date or datetime
   * @return           integer  The timestamp
   * @private
   * 
   * @author    Steve Powell info@scrimpnet.com
   * @copyright Copyright (c) 2004 scrimpnet.com
   */
  function _parseDate($dateTime="")
  {
    $ts = 0; // timestamp of parsed date
    switch(gettype($dateTime)) {
      case "string": 
        $ts=strtotime($dateTime); 
        break;
      case "integer":
        $ts=$dateTime;
        break;
    }
    return $ts;
  }
}
?>