<?php
/**
 * Dashboard class.
 *
 * @author    Jebbol
 * @version   1.0
 * @revision  $Id: dash.class.php,v 1.7 2006/03/08 16:58:04 jebbol Exp $
 * @package   spl
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

// Calendar class
include_once($SPL_CFG['dirroot'].'/class/cal.class.php');

// Teacher data
include_once($SPL_CFG['dirroot'].'/methodology/'.$_SESSION['methodology'].'/class/tea.class.php');
include_once($SPL_CFG['dirroot'].'/class/usr.class.php');

//=========================================================================
// Class definition
//=========================================================================

/**
 * Dashboard
 */
class dash extends abstractData {
  
  function dash() {
    
    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['spt_id']           = (int) 0;
    $this->_data['adm_id']           = $_SESSION['adm_id'];
    $this->_data['tea_id']           = (int) 0;
    $this->_data['stu_id']           = (int) 0;
    $this->_data['ins_id']           = (int) 0;
    $this->_data['ina_id']           = (int) 0;
    $this->_data['tra_id']           = (int) 0;
    $this->_data['ali_id']           = (int) 0;
    $this->_data['sub_id']           = (int) 0;
    $this->_data['spt_courseyear']   = (int) 0;
    $this->_data['prd_id']           = (int) 0;
    $this->_data['spt_number_total'] = (int) 0;
    $this->_data['spt_number_done']  = (int) 0;
    $this->_data['spt_load_total']   = (int) 0;
    $this->_data['spt_load_done']    = (int) 0;
    $this->_data['spt_reward_total'] = (int) 0;
    $this->_data['spt_reward_done']  = (int) 0;
    $this->_data['spt_sync_date']    = time();
    $this->_data['is_active']        = (bool) true;
    
    // Register the required class properties
    $this->_primary = 'spt_id';
    $this->_table   = 'spt';

    // Set user permissions for this type of data
    if (!isset($_SESSION[$this->_table.'_permission'])) {
      $_SESSION[$this->_table.'_permission'] = $this->_getPermissions();
    }
        
  }

  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Actualizes the dashboard by rebuilding the list of students
   *
   * @param    integer $tmp_usr_id    The user id 
   * @return   boolean
   * @access   public
   */
  function actualize($tmp_usr_id) {
    
    global $SPL_CFG;
    
    // Find teacher defaults
    $usr = new usr();
    if (!$usr->init($tmp_usr_id)) {
      // User unknown
      return (bool) false;
    }
    $teacher = $usr->getProperty('tea_id');
    if ($teacher){
      $tea = new tea();
      $tea->init($teacher);
    }
    else {
      // User is not a teacher
      return (bool) false;
    }
    
    // Do we have to rebuild the student list?
    $mustRebuild = $tea->getProperty('tea_rebuild_spt');
    if (!$mustRebuild) {
      return (bool)true;
    }
    else {
      // Delete old stuff
      $query  = 'DELETE FROM '.$SPL_CFG['prefix'].'spt WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
      $query .= ' tea_id = "'.$teacher.'"';
      $result = db_query($query);
    }


    // Retrieve the teachers institutions
    $institutions = '';
    $s = $tea->getInstitutions();
    foreach ($s as $record) {
      foreach ($record as $key => $item) {
        $institutions .= (strlen($institutions)?',':'');
        $institutions .= $key;
      }
    }

    // Retrieve the teachers institution addresses
    $addresses = '';
    $s = $tea->getInstitutionAddresses();
    foreach ($s as $record) {
      foreach ($record as $key => $item) {
        $addresses .= (strlen($addresses)?',':'');
        $addresses .= $key;
      }
    }

    // Retrieve the teachers trainings
    $trainings = '';

    // Retrieve the teachers alignments
    $alignments = '';

    // Retrieve the teachers courseyears
    $courseyears = '';

    // Retrieve the teachers subjects
    $subjects = '';
    $s = $tea->getSubjects();
    foreach ($s as $record) {
      foreach ($record as $key => $item) {
        $subjects .= (strlen($subjects)?',':'');
        $subjects .= $key;
      }
    }
    
    // Find students with one or more of these subjects
    $query  = 'SELECT parent_id,child_id,ins_id,ina_id,tra_id,ali_id,stu_courseyear,usr_id';
    $query .= ' FROM '.$SPL_CFG['prefix'].'apl apl LEFT JOIN '.$SPL_CFG['prefix'].'stu stu ON stu.stu_id = apl.parent_id LEFT JOIN '.$SPL_CFG['prefix'].'usr usr ON usr.stu_id = stu.stu_id';
    $query .= ' WHERE apl.adm_id = "'.$_SESSION['adm_id'].'" AND';
    $query .= ' apl.is_active AND';
    $query .= ' apl_parent = "stu" AND'; 
    $query .= ' apl_child = "sub" AND';
    if (strlen($subjects)) {
      if (substr_count($subjects, ',')==0){
        $query .= ' apl.child_id = "'.$subjects.'" AND';
      }
      else {
        $query .= ' apl.child_id IN ('.$subjects.') AND';
      }
    }
    if (strlen($institutions)) {
      if (substr_count($institutions, ',')==0){
        $query .= ' stu.ins_id = "'.$institutions.'" AND';
      }
      else {
        $query .= ' stu.ins_id IN ('.$institutions.') AND';
      }
    }
    if (strlen($addresses)) {
      if (substr_count($addresses, ',')==0){
        $query .= ' stu.ina_id = "'.$addresses.'" AND';
      }
      else {
        $query .= ' stu.ina_id IN ('.$addresses.') AND';
      }
    }
    if (strlen($trainings)) {
      if (substr_count($trainings, ',')==0){
        $query .= ' stu.tra_id = "'.$trainings.'" AND';
      }
      else {
        $query .= ' stu.tra_id IN ('.$trainings.') AND';
      }
    }
    if (strlen($alignments)) {
      if (substr_count($alignments, ',')==0){
        $query .= ' stu.ali_id = "'.$alignments.'" AND';
      }
      else {
        $query .= ' stu.ali_id IN ('.$alignments.') AND';
      }
    }
    if (strlen($courseyears)) {
      if (substr_count($courseyears, ',')==0){
        $query .= ' stu.stu_courseyear = "'.$courseyears.'" AND';
      }
      else {
        $query .= ' stu.stu_courseyear IN ('.$courseyears.') AND';
      }
    }
    $query .= ' apl.apl_valid_from <= "'.time().'" AND'; 
    $query .= ' (apl.apl_valid_until = "0" OR apl.apl_valid_until >= "'.time().'")';
    $result = db_query($query);
      
    while ($row = db_fetch_array($result)) {

      // Build students-per-teacher row for each period
      $query = 'SELECT prd_id_start AS prd_id,COUNT(sgd_id),SUM(is_done),SUM(sug_load),SUM(sgd_load_done),SUM(sug_reward),SUM(sgd_reward_done)';
      $query .= ' FROM '.$SPL_CFG['prefix'].'sgd sgd  LEFT JOIN '.$SPL_CFG['prefix'].'sug sug ON sug.sug_id = sgd.sug_id WHERE';
      $query .= ' sgd.usr_id = "'.$row['usr_id'].'" AND';
      $query .= ' sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active AND';
      $query .= ' sgd.sub_id = "'.$row['child_id'].'" GROUP BY prd_id_start';
      $sgdresult = db_query($query);
      while ($sgdrow = db_fetch_array($sgdresult)) {

        $this->_data['tea_id']           = $teacher;
        $this->_data['stu_id']           = $row['parent_id'];
        $this->_data['ins_id']           = $row['ins_id'];
        $this->_data['ina_id']           = $row['ina_id'];
        $this->_data['tra_id']           = $row['tra_id'];
        $this->_data['ali_id']           = $row['ali_id'];
        $this->_data['sub_id']           = $row['child_id'];
        $this->_data['spt_courseyear']   = $row['stu_courseyear'];
        $this->_data['prd_id']           = $sgdrow['prd_id'];
        $this->_data['spt_number_total'] = $sgdrow['COUNT(sgd_id)'];
        $this->_data['spt_number_done']  = $sgdrow['SUM(is_done)'];
        $this->_data['spt_load_total']   = $sgdrow['SUM(sug_load)'];
        $this->_data['spt_load_done']    = $sgdrow['SUM(sgd_load_done)'];
        $this->_data['spt_reward_total'] = $sgdrow['SUM(sug_reward)'];
        $this->_data['spt_reward_done']  = $sgdrow['SUM(sgd_reward_done)'];
        $this->_data['spt_sync_date']    = time();
        $this->_data['is_active']        = (bool)true;
        $this->_status['is_dirty']       = (bool)true;
        $this->_status['is_new']         = (bool)true;
      }

      // Save this data
      $this->save();
    }
   
    /*
    if ($mustRebuild) {
      // Reset the force_rebuild flag
      $tea->setProperty('tea_rebuild_spt',false);
      $tea->save();
    }
    */
  }
  
  /**
   * Retrieves an array with information about the current item
   *
   * @param integer $tmp_usr_id The user
   * @param integer $sub_id The subject
   * @return array The information
   */
  function getInfo($tmp_usr_id, $sub_id) {
    
    global $SPL_CFG;
    
    $info = array();
    $info['num_activities'] = 0;
    $info['num_activities_done'] = 0;
    $info['perc_activities_done'] = 0;
    $info['load'] = 0;
    $info['load_done'] = 0;
    $info['perc_load_done'] = 0;
    $info['reward'] = 0;
    $info['reward_done'] = 0;
    $info['perc_reward_done'] = 0;

    // Determine progress
    if ($sub_id) {
      // Query the studyguide
      $query = 'SELECT is_done FROM '.$SPL_CFG['prefix'].'sgd sgd WHERE';
      $query .= ' sgd.usr_id = "'.$tmp_usr_id.'" AND';
      $query .= ' sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active AND';
      $query .= ' sub_id = "'.$sub_id.'"';
      $result = db_query($query);
      while ($row = db_fetch_array($result)) {
        // See if the student finished this activity
        ++$info['num_activities'];
        if (isset($row['is_done'])&&$row['is_done']){
          ++$info['num_activities_done'];
        }
      }
      if ($info['num_activities_done']>0) {
        $info['perc_activities_done'] = $info['num_activities_done']/$info['num_activities']*100; 
      }
    }
    return $info;
  }
  
  //=========================================================================
  // Private methods
  //=========================================================================
}
?>