<?php
/**
 * Abstract data
 *
 * $Author: jebbol $
 * $Revision: 1.27 $
 * $Date: 2006/08/08 06:05:52 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 * 
 * $Log: data.abstract.php,v $
 * Revision 1.27  2006/08/08 06:05:52  jebbol
 * *** empty log message ***
 *
 * Revision 1.26  2006/04/14 10:56:54  jebbol
 * Added getReportData()
 *
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Configuration
include_once($SPL_CFG['dirroot'].'/inc/config.php');

// Database functions
if (!isset($SPL_CFG['dbtype'])) {$SPL_CFG['dbtype'] = 'mysql';}
include_once($SPL_CFG['dirroot'].'/db/'.$SPL_CFG['dbtype'].'/db.inc.php');

// Administration settings
include_once($SPL_CFG['dirroot'].'/inc/settings.adm.php');

// System settings
include_once($SPL_CFG['dirroot'].'/inc/settings.sys.php');

// User settings
include_once($SPL_CFG['dirroot'].'/inc/settings.usr.php');

// The ability to split text and html formatting
include_once($SPL_CFG['dirroot'].'/inc/text.formatting.php');

// Permissions
include_once($SPL_CFG['dirroot'].'/class/permissions.class.php');

// Human language translations
if (!isset($_SESSION['usp_lang'])) {$_SESSION['usp_lang'] = 'en';}
include_once($SPL_CFG['dirroot'].'/lang/'.$_SESSION['usp_lang'].'/data.lang.php');

//=========================================================================
// Class definition
//=========================================================================

/**
 * Abstract class for data objects
 *
 * This is an abstract class from which all data-handling classes are derived. 
 * When extending this class, make sure you set the following properties:
 * - {@link $_table}: the name of the database table the class will handle
 * - {@link $_primary}: the name of the primary key column
 * - {@link $_required}: An array containing properties that MUST have a value
 * - {@link $_unique}: An array containing properties that must have a unique value
 *
 * @abstract
 */
class abstractData {
  
  /**
   * Instance data
   * 
   * The objects persistent data
   * Use {@link getProperty()} to retrieve the data and {@link setProperty()} to set it.
   * 
   * @var array
   * @protected
   */
  var $_data = array();

  /**
   * Debug information
   * 
   * This is essentially a stack of debug messages. Only warnings and error are being pushed on the stack. 
   * Use {@link getDebugInfo()} to retrieve the debug information.
   * 
   * @var array
   * @protected
   */
  var $_debug = array();

  /**
   * Message
   * 
   * End-user messages. This is an erray of message ids.  
   * Use {@link getMessages()} to retrieve the messages.
   * 
   * @var array
   * @protected
   */
  var $_msg = array();

  /**
   * Required properties
   * 
   * A list of required data (data that must have a value) 
   * Use {@link getRequired()} to get a list of required data.
   * 
   * @var array
   * @protected
   */
  var $_required = array();

  /**
   * Object status
   * 
   * A set of booleans describing the objects status:
   * - is_dirty: indicating if the object's data is out of sync with the database
   * - is_loaded: indicating if the object has been initalized properly
   * - is_new: indicating the creation of an object not yet represented in the database 
   *
   * @var array 
   * @protected
   */
  var $_status = array();

  /**
   * Property type
   * 
   * Metadata regarding the type of data.
   * Each entry in this array represents a property. The property's type information is stored in an attached array.
   * The information in this array will be used to check the supplied data in {@link setProperty()}. 
   * The array may contain any or none of the following information:
   * - type: the property's datatype (email, integer, string or tiemstamp)
   * - min: the minimum value (applicable with integer values only)
   * - max: the maximum value (applicable with integer values only)
   * 
   * @var array
   * @optional When empty, no checks are made 
   * @protected
   */
  var $_type = array();

  /**
   * Unique properties
   * 
   * A list of properties that must have a system-wide unique value.
   * 
   * @var array
   * @protected
   */
  var $_unique = array();

  /**
   * Format suffix
   * 
   * Properties that contain formatting data (such as marked-up html) can have the 
   * data separated from the formatting markup. If you deisre this functionality, create
   * two properties or this. The first property will contain the actual data, the second property
   * will contain only the formatting information. The system will recognize the need to separate the 
   * markup from the data by noticing the presence of a property with the same name as the data-property,
   * but with a suffix added to its name. The suffix can be set here.
   * 
   * @var string
   * @protected
   */
  var $_formatSuffix;

  /**
   * Instance name
   * 
   * The name of the instance property to return when requesting 'the name of the instance'.
   * This is the property whose value will be returned when calling {@link getName()}'
   * 
   * @var string
   * @protected
   */
  var $_name;

  /**
   * Primary key name
   * 
   * The name of the property used as primary key in a database table
   * 
   * @var string
   * @protected
   */
  var $_primary;

  /**
   * Table name
   * 
   * The name of the database table to store the instance data in
   * 
   * @var string
   * @protected
   */
  var $_table;

  /**
   * Active data only
   * 
   * Check for active data.
   * Data can be either active or inactive. Active data will be presented to the user,
   * inactive data will be hidden (so it appears to be deleted). This property will set
   * the default check of the is_active data property. When set to 'true', data will only be
   * loaded when active. When set to 'false', data will only be loaded when set to inactive'. 
   * Alternalively set this property to 'skip' in order to skip checking this completely.
   * 
   * @var string
   * @protected
   */
  var $_active;
    
  function abstractData() {
    
    // Set the default format suffix
    $this->_formatSuffix = '_fmt';
    
    // Set the default active flag
    if (isset($_SESSION['is_admin'])){
      $this->_active = 'skip';
    }
    else {
      $this->_active = 'true';
    }
    
    // Register the default status information 
    $this->_status['is_dirty']  = (bool) false;
    $this->_status['is_loaded'] = (bool) false;
    $this->_status['is_new'] = (bool) true;

  }

  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Deletes the current instance
   *  
   * By  default a soft delete will be performed, which means the data is not
   * actually  deleted from the database, but merely flagged as 'inactive'.
   *
   * @param   (boolean) $soft Soft delete (default) or record delete (false) 
   * @return  (boolean)
   * @public
   */
  function delete($soft=true) {
    
    global $SPL_TXT;

    // This will only work on a correctly instantiated object
    if ($this->_status['is_loaded']) {
      
      // Perform a soft deleted on active rows only
      if ($soft && $this->_data['is_active']) {
        $this->setProperty('is_active',false);
        $this->_status['is_dirty'] = true;
        return $this->save();
      }
      // Perform a database delete
      else {
        return $this->_delete();
      }
        
    } 
    else {
      $_dbg['query'] = 'examining the value of is_loaded for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_loaded'];
      $_dbg['method'] = 'delete';
      $_dbg['error'] = $SPL_TXT['db_instance_not_loaded'];
      array_push($this->_debug, $_dbg);
      return (bool) false;
    }
  }

  /** 
   * Returns all properties of the current instance
   * 
   * @return (array) The objects properties
   * @public
   */
  function getData() {
    
    global $SPL_TXT;
    
    // This will only work on a correctly instantiated object
    if ($this->_status['is_loaded']) {
      // User must have permission to read data
      if (!isset($_SESSION[$this->_table.'_permission'])||!$_SESSION[$this->_table.'_permission']['read']) {
        return (bool) false;
      }
      
      return $this->_data;
    }
    else {
      $_dbg['query'] = 'examining the value of is_loaded for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_loaded'];
      $_dbg['method'] = 'getData';
      $_dbg['error'] = $SPL_TXT['db_instance_not_loaded'];
      array_push($this->_debug, $_dbg);
      return (bool) false;
    }
  }


  /** 
   * Returns an array containing debug information
   * 
   * @return (array) The debug info
   * @public
   */
  function getDebugInfo() {
    return $this->_debug;
  }

  /** 
   * Returns a set of keys and values for all occurrences in the table
   * 
   * Please note that when a restriction has been passed without a parent
   * identifier ($parent=false), the restriction is assumed to be applicable to
   * the table itself. In that case, specify the column name and the value. For
   * example: $restrict = 'ina_id = "1"'.
   * 
   * When a parent table has been passed, a 'applicability' relationship will be
   * assumed. The applicable rows will be looked up in the 'apl' table. In this
   * case, do not specify column names in the restrictions, just the key value
   * (for example: $parent='ina', $restrict='1')
   * 
   * When a child table has been passed, a 'applicability' relationship will be
   * assumed. The applicable rows will be looked up in the 'apl' table. In this
   * case, do not specify column names in the restrictions, just the key value
   * (for example: $child='ina', $restrict='1')
   * 
   * @param  (string) $parent   (optional) The name of the parent table (ex:'ina')
   * @param  (string) $restrict (optional) SQL restriction (ex:'ina_id = "1"')
   * @param  (string) $orderBy  (optional) Alternative ordering (ex:ina_sequence)
   * @param  (string) $orderDir (optional) 'ASC' or 'DESC'
   * @param  (string) $child    (optional) The name of the child table (ex:'ina')
   * @param  (string) $column   (optional) The name of the column to return
   * @return (array) 
   * @public
   */
  function getKeyValues($parent=false,$restrict=false,$orderBy=false,$orderDir='ASC',$child=false,$column=false){
    
    global $SPL_CFG;
    
    // The column name to return
    if (!$column) {
      if (!isset($this->_name)){
        return (bool)false;
      }
      else {
        $column = $this->_name;
      }
    }
    
    $values = array();
    // Table(s)
    $query = 'SELECT '.$this->_table.'.'.$this->_primary.' as "key",'.$this->_table.'.'.(!$parent&&$orderBy?$orderBy:$column).' as "value"';
    if ($restrict && ($parent||$child)) {
      $query .= ' FROM '.$SPL_CFG['prefix'].'apl apl LEFT JOIN '.$SPL_CFG['prefix'].$this->_table.' '.$this->_table.' ON '.$this->_table.'.'.$this->_primary.' = '.($parent?'apl.child_id':'apl.parent_id');
    }
    else {
      $query .= ' FROM '.$SPL_CFG['prefix'].$this->_table.' '.$this->_table;
    }
    // Columns
    $query .= ' WHERE '.$this->_table.'.adm_id = "'.$_SESSION['adm_id'].'" AND '.$this->_table.'.is_active';
    if ($restrict) {
      if ($parent){
        $query .= ' AND apl.adm_id = "'.$_SESSION['adm_id'].'"';
        $query .= ' AND apl.is_active';
        $query .= ' AND apl.apl_parent = "'.$parent.'"';
        $query .= ' AND apl.parent_id = "'.$restrict.'"';
        $query .= ' AND apl.apl_child = "'.$this->_table.'"';
        $query .= ' AND apl.apl_valid_from <= "'.time().'"';
        $query .= ' AND (apl.apl_valid_until = "0" OR apl.apl_valid_until >= "'.time().'")';
      }
      else if ($child){
        $query .= ' AND apl.adm_id = "'.$_SESSION['adm_id'].'"';
        $query .= ' AND apl.is_active';
        $query .= ' AND apl.apl_child = "'.$child.'"';
        $query .= ' AND apl.child_id = "'.$restrict.'"';
        $query .= ' AND apl.apl_parent = "'.$this->_table.'"';
        $query .= ' AND apl.apl_valid_from <= "'.time().'"';
        $query .= ' AND (apl.apl_valid_until = "0" OR apl.apl_valid_until >= "'.time().'")';
      }
      else {
        $query .= ' AND '.$restrict;
      }
    }
    // Ordering
    if ($restrict){
      if ($parent||$child){
        $query .= ' ORDER BY apl.apl_sequence '.$orderDir.', '.($orderBy?$orderBy:$column).' '.$orderDir;
      }
      else if ($orderBy){
        $query .= ' ORDER BY '.($orderBy?$orderBy:$column).' '.$orderDir;
      }
    }
    else if ($orderBy){
      $query .= ' ORDER BY '.($orderBy?$orderBy:$column).' '.$orderDir;
    }

    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      $values[] = array($row['key']=>$row['value']);
    }
 
    return $values;
  }

  /** 
   * Returns an array containing message ids
   * 
   * @return (array) The messages
   * @public
   */
  function getMessages() {
    return $this->_msg;
  }

  /** 
   * Returns the name of the current instance
   *
   * @return (string) The name of the instance
   * @public
   */
  function getName() {
    return $this->getProperty($this->_name);
  }

  /** 
   * Returns a property of the current instance
   *
   * @param  (string) $property The name of the propert to be returned
   * @return (string)           The value of the property
   * @public
   */
  function getProperty($property) {
    
    // User must have permission to read data
    if (!isset($_SESSION['usr_permission'])||!$_SESSION['usr_permission']['read']) {
      return (bool) false;
    }
    
    // Only known properties can be returned
    if (array_key_exists($property, $this->_data)) {
      //return $this->_data[$property];
      return (get_magic_quotes_gpc() ? htmlspecialchars(stripslashes($this->_data[$property])) : $this->_data[$property]);
    } 
    // If it's not a known property, return an empty string
    else {
      return '';
    }
  }

  /** 
   * Returns the primary key value of the current instance
   *
   * @return (integer)  The primary key
   * @public
   */
  function getPrimary() {

    return $this->_data[$this->_primary];
    
  }


  /**
   * Returns data for printing a report
   * 
   * @return (array) The data to print
   * @public
   */
  function getReportData() {
    
    global $SPL_CFG;
    
    $query  = 'SELECT ';
    $join   = '';
    $iFields = 0;
    foreach ($this->_data as $key => $value) {
      // Skip 
      if ($key==$this->_primary||$key=='adm_id'||$key=='cal_id'||$key=='adr_id'||$key=='is_active'||$key=='sts_id_next'||substr($key,0,7)=='create_'||substr($key,0,7)=='update_') {
        continue;
      }
      $iFields++;
      if ($iFields>1) {
        $query .= ', ';
      }
      // Guess date fields
      if (isset($this->_type[$key])&&isset($this->_type[$key]['type'])&&$this->_type[$key]['type']=='timestamp') {
        $query .= 'IF('.$key.'>0,FROM_UNIXTIME('.$key.'),"") AS '.$key;
      }
      else if (strstr($key,'valid_from')||strstr($key,'valid_until')) {
        $query .= 'IF('.$key.'>0,FROM_UNIXTIME('.$key.'),"") AS '.$key;
      }
      // Get description from related table
      else if (substr($key,3,3)=='_id'){
        $query .= substr($key,0,3).'_short AS '.$key;
        $join .= ' LEFT JOIN '.$SPL_CFG['prefix'].substr($key,0,3).' ON '.$SPL_CFG['prefix'].substr($key,0,3).'.'.$key.' = '.$SPL_CFG['prefix'].$this->_table.'.'.$key;
      }
      else {
        $query .= $key;
      }
    }
    $query .= ' FROM '.$SPL_CFG['prefix'].$this->_table.$join;
    $query .= ' WHERE '.$SPL_CFG['prefix'].$this->_table.'.adm_id = "'.$_SESSION['adm_id'].'" AND '.$SPL_CFG['prefix'].$this->_table.'.is_active';
    if (isset($this->_name)) {
      $query .= ' ORDER BY '.$this->_name;
    }

    $data = array();
    $result = db_query($query);
    while($data[] = db_retrieve($result)) {}
    return $data;

  }

  /** 
   * Returns the names of the properties that must have a value in order for 
   * the  object to represent a meaningfull entity
   *
   * @return (array)  The required property
   * @public
   */
  function getRequired() {

    return $this->_required;
  }

  /** 
   * Returns a formatted property of an instance
   *
   * @param  (string) $property The name of the property to be returned
   * @return (string)           The formatted data
   * @public
   */
  function getPropertyFormatted($property) {
    
    // Only known properties can be returned
    if (array_key_exists($property, $this->_data)) {
      $value = stripslashes($this->_data[$property]);
      
      // If there is formatting information associated with this property, apply it now
      if (array_key_exists($property.$this->_formatSuffix, $this->_data)) {
        $format = explode_with_keys(";",$this->_data[$property.$this->_formatSuffix]);
        return ApplyFormatting($value, $format);
      }
      // otherwise return the unformatted information
      else {
        return $value;
      }
    } 
    // If it's not a known property, return an empty string
    else {
      return '';
    }
  }

  /** 
   * Instantiates the object
   *
   * @param  (string)  $column  The name of the db column corresponding with the id, or the id itself (in which case $id can be omitted)
   * @param  (integer) $id      The id of the instance data (optional)  
   * @return (boolean)          Succes or Failure
   * @public
   */
  function init($column, $id=0) {
    
    global $SPL_TXT;
    
    if ( func_num_args() == 1 ) {
    	$id     = $column;
    	$column = $this->_primary;
    }
    // Get data from the database
    $data = $this->_select($column,$id,'1',$this->_active);
    if (!$data){
      $_dbg['query'] = 'calling _select with '.$column.' = '.$id.' and limit = 1';
      $_dbg['result'] = $data;
      $_dbg['method'] = 'init';
      $_dbg['error'] = $SPL_TXT['cannot_instantiate'];
      array_push($this->_debug, $_dbg);
      return (bool) false;
    }
    
    // Populate the objects properties
    $this->_populate($data);

    // Set user permissions for this type of data
    if (!isset($_SESSION[$this->_table.'_permission'])) {
      $_SESSION[$this->_table.'_permission'] = $this->_getPermissions();
    }
    
    return (bool) true;
  }

  /** 
   * Checks if the object is properly instantiated or not
   * 
   * @return (boolean) Instantiated or Not
   * @public
   */
  function isLoaded() {
    return (bool)$this->_status['is_loaded'];
  }

  /** 
   * Checks if the object is new or not
   * 
   * @return (boolean) New or Not
   * @public
   */
  function isNew() {
    return (bool)$this->_status['is_new'];
  }

  /**
   * Checks if there is only one active row in the table and returns the id if that is the case
   * 
   * @return (integer) The row id, or false if there is either zer or more than one row
   * @public
   */
  function isSingle() {
    
    global $SPL_CFG;
    
    $query = 'SELECT '.$this->_primary.' FROM '.$SPL_CFG['prefix'].$this->_table.' WHERE adm_id = "'.$_SESSION['adm_id'].'" AND is_active';
    $result = db_query($query);
    if (db_num_rows($result)==1){
      $result = db_retrieve($result);
      return $result[$this->_primary];
    }
    return (bool)false;
  }

  /**
   * Checks if the given column/value combination is referenced from any objects
   * of this class
   * 
   * @param  (string)  $column The name of the column
   * @param  (string)  $value  The value to check
   * @return (boolean) Used or Unused
   * @public
   */
  function isUsed($column,$value){
    
    global $SPL_CFG;
    
    $query = 'SELECT '.$this->_primary.' FROM '.$SPL_CFG['prefix'].$this->_table.' WHERE '.$column.' = "'.$value.'" AND adm_id = "'.$_SESSION['adm_id'].'" LIMIT 1'; 
    $result = db_retrieve(db_query($query));
    return (isset($result[$this->_primary]));
  }
  

  /** 
   * Saves the instance data to the database
   *
   * @return (boolean) Succes or Failure
   * @public
   */
  function save() {
    
    global $SPL_TXT;

    // If the instance data is not dirty, no action is needed
    if (!$this->_status['is_dirty']) {
      $_dbg['query'] = 'examining the value of is_dirty for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_dirty'];
      $_dbg['method'] = 'save';
      $_dbg['error'] = $SPL_TXT['db_instance_not_dirty'];
      array_push($this->_debug, $_dbg);
      return (bool) true;         
    }
    
    // Check if required columns do have a value
    if (count($this->_required)>0) {
      foreach ($this->_required as $attribute){
        if (strlen($this->_data[$attribute])==0) {
          $_dbg['query'] = 'checking required attribute '.$attribute;
          $_dbg['result'] = 'empty';
          $_dbg['method'] = 'save';
          $_dbg['error'] = $SPL_TXT['db_column_empty'];
          array_push($this->_debug, $_dbg);
          return (bool) false;
        }
      }
    }

    // Check unique columns 
    if (count($this->_unique)>0 && $this->_status['is_new']) {
      foreach ($this->_unique as $attribute){
        if (!$this->_isUnique($attribute,$this->_data[$attribute])) {
          $_dbg['query'] = 'checking unique attribute '.$attribute;
          $_dbg['result'] = 'empty';
          $_dbg['method'] = 'save';
          $_dbg['error'] = $SPL_TXT['db_column_not_unique'];
          return (bool) false;
        }
      } 
    }

    // If this is a new instance, insert a record
    if ($this->_status['is_new']) {
      return $this->_insert();
    }
    // otherwise update the existing data
    else {
      return $this->_update();
    }
  }

  /** 
   * Sets the instance property with the given value
   *
   * @param  (string)  $property The name of the property to be set
   * @param  (string)  $value    The data to bet set
   * @return (boolean) Succes or Failure
   * @public
   */
  function setProperty($property, $value) {
    
    global $SPL_TXT;

    // Only known properties can be set
    if (array_key_exists($property, $this->_data)) {
      
      // Split any formatted text field into plain text and formatting data
      if (array_key_exists($property.$this->_formatSuffix, $this->_data)) {
        list($value,$format) = ExtractFormatting(rtrim($value));
        $this->_data[$property.$this->_formatSuffix] = implode_with_keys(";",$format);
      }

      // If the fieldtype is known, validate the value
      if (array_key_exists($property, $this->_type)){
        if (isset($this->_type[$property]['type'])){
          switch ($this->_type[$property]['type']) {
            case 'boolean':
              break;
            case 'email':
              $expression="^([-!#\$%&'*+./0-9=?A-Z^_`a-z{|}~])+@([-!#\$%&'*+/0-9=?A-Z^_`a-z{|}~]+\\.)+[a-zA-Z]{2,4}\$";
              if(!eregi($expression,$value)){
                $_dbg['query'] = 'checking email format of '.$property;
                $_dbg['result'] = $value;
                $_dbg['method'] = 'setProperty';
                $_dbg['error'] = $SPL_TXT['db_column_format'];
                array_push($this->_debug, $_dbg);
                return (bool) false;
              }
              break;
            case 'integer':
              // Check type
              $value = ltrim(html_entity_decode(strip_tags($value)));
              if (empty($value)){
                $value = '0';
              }
              $integer_value=intval($value);
              if (strcmp($value,strval($integer_value))){
                $_dbg['query'] = 'checking integer value of '.$property;
                $_dbg['result'] = $value.' <> '.strval($integer_value);
                $_dbg['method'] = 'setProperty';
                $_dbg['error'] = $SPL_TXT['db_column_format'];
                array_push($this->_debug, $_dbg);
                return (bool) false;
              }
              // Check minimum value
              if (isset($this->_type[$property]['min'])){
                if (intval($value)<intval($this->_type[$property]['min'])){
                  $_dbg['query'] = 'checking minimum value of '.$property;
                  $_dbg['result'] = $value;
                  $_dbg['method'] = 'setProperty';
                  $_dbg['error'] = $SPL_TXT['db_column_value'];
                  array_push($this->_debug, $_dbg);
                }
              }
              // Check maximum value
              if (isset($this->_type[$property]['max'])){
                if (intval($value)>intval($this->_type[$property]['max'])){
                  $_dbg['query'] = 'checking maximum value of '.$property;
                  $_dbg['result'] = $value;
                  $_dbg['method'] = 'setProperty';
                  $_dbg['error'] = $SPL_TXT['db_column_value'];
                  array_push($this->_debug, $_dbg);
                }
              }
              $value = $integer_value;
              break;
            case 'string':
              // Check minimum length
              if (isset($this->_type[$property]['min'])){
                if (strlen($value)<intval($this->_type[$property]['min'])){
                  $_dbg['query'] = 'checking minimum length of '.$property;
                  $_dbg['result'] = $value;
                  $_dbg['method'] = 'setProperty';
                  $_dbg['error'] = $SPL_TXT['db_column_value'];
                  array_push($this->_debug, $_dbg);
                }
              }
              // Check maximum length
              if (isset($this->_type[$property]['max'])){
                if (strlen($value)>intval($this->_type[$property]['max'])){
                  $_dbg['query'] = 'checking maximum length of '.$property;
                  $_dbg['result'] = $value;
                  $_dbg['method'] = 'setProperty';
                  $_dbg['error'] = $SPL_TXT['db_column_value'];
                  array_push($this->_debug, $_dbg);
                }
              }
              break;
            case 'timestamp':
              if ($value) {
                $value = $this->_parseDate($value);
              }
              break;
            default:
              break;
          }
        }
      }
      
      // Set the property
      if (!get_magic_quotes_gpc()) {
        $this->_data[$property] = addslashes($value);
      } 
      else {
        $this->_data[$property] = $value;
      }
      $this->_status['is_dirty'] = (bool) true;
      return (bool) true;
    } 
    else {
      return (bool) false;
    }
  }

  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Deletes a row from the database.
   * 
   * @return (boolean) Success or Failure
   * @protected
   */
  function _delete() {
    
    global $SPL_CFG,$SPL_TXT;

    if (!$this->_status['is_loaded']){
      $_dbg['query'] = 'examining the value of is_loaded for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_loaded'];
      $_dbg['method'] = '_delete';
      $_dbg['error'] = $SPL_TXT['db_instance_not_loaded'];
      array_push($this->_debug, $_dbg);
      return (bool) false;
    }
    
    // Delete this objects references in the 'applicability' table
    $query = 'DELETE FROM '.$SPL_CFG['prefix'].'apl WHERE apl_child = "'.$this->_table.'" AND child_id = "'.$this->_data[$this->_primary].'"';
    db_query($query);
    $query = 'DELETE FROM '.$SPL_CFG['prefix'].'apl WHERE apl_parent = "'.$this->_table.'" AND parent_id = "'.$this->_data[$this->_primary].'"';
    db_query($query);
    
    // Delete the record associated with this object
    $query = 'DELETE FROM '.$SPL_CFG['prefix'].$this->_table.' WHERE '.$this->_primary.' = "'.$this->_data[$this->_primary].'" LIMIT 1'; 
    $result = db_query($query);
    if ($result) {
      $this->_status['is_loaded'] = (bool) false;
      return (bool) true;
    }
    else {
      $_dbg['query'] = $query;
      $_dbg['result'] = $result;
      $_dbg['method'] = '_delete';
      $_dbg['error'] = $SPL_TXT['db_cannot_delete'];
      array_push($this->_debug, $_dbg);         
      return (bool) false;
    }

  }

  /** 
   * Transforms a permission bitmask into an array of permissions
   *
   * @param  (integer)  $bitmask  The permission bitmask
   * @return (array) The permissions
   * @private
   */
  function _explodePermissions($bitmask) {
    $perms = new permissions();
    return ($perms->getPermissions($bitmask));
  } 

  /** 
   * Retrieves user permissions
   * 
   * The array contains the following permissions (each set to true or false):
   * - read: user is allowed to read data
   * - write: user is allowed to add and change data
   * - delete: user is allowed to delete data
   * - change: user is allowed to change data (not to add it)
   * - admin: user is allowed to perform admin tasks on this data
   * All permissions default to false.
   *
   * @return (array) The permissions
   * @protected
   */
  function _getPermissions($class=NULL) {

    global $SPL_CFG,$SPL_TXT;
    
    // First check user specific permissions
    $query  =  "SELECT usa_permission FROM ".$SPL_CFG['prefix']."usa WHERE";
    $query .= " usr_id = '".$_SESSION['usr_id']."'";
    $query .= " AND adm_id = '".$_SESSION['adm_id']."'";
    $query .= " AND usa_key = '".($class?$class:$this->_table)."'";
    $query .= " LIMIT 1";
    $result = db_retrieve(db_query($query));
    if (isset($result['usa_permission'])) {
      return ($this->_explodePermissions($result['usa_permission']));
    }
    
    // Otherwise, return the default permissions
    return array("read"=>false,"write"=>false,"delete"=>false,"change"=>false,"admin"=>false);
  } 

  /** 
   * Inserts a new row in the database.
   * 
   * This function will be called by {@link save()} when adding a new instance to the database.
   * 
   * @private
   * @return (boolean) Success or Failure
   */
  function _insert() {
    
    global $SPL_CFG,$SPL_TXT;
    
    // Instance must be new. If not, update should be used
    if (!$this->_status['is_new']) {
      $_dbg['query'] = 'examining the value of is_new for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_new'];
      $_dbg['method'] = '_insert';
      $_dbg['error'] = $SPL_TXT['db_instance_not_new'];
      array_push($this->_debug, $_dbg);         
    }
    
    // Instance must dirty, otherwise it is an empty instance
    if (!$this->_status['is_dirty']) {
      $_dbg['query'] = 'examining the value of is_dirty for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_dirty'];
      $_dbg['method'] = '_insert';
      $_dbg['error'] = $SPL_TXT['db_instance_not_dirty'];
      array_push($this->_debug, $_dbg);         
    }

    // To keep track of the current position in the array
    $currentPos = 0;
    // The number of items in the array
    $maxPos = count($this->_data);

    $query = "INSERT INTO ".$SPL_CFG['prefix'].$this->_table." (";
    foreach ($this->_data as $property => $value) {
      $currentPos++;
      $query .= $property.($currentPos<$maxPos?",":"");
    }
    $query .= ") VALUES (NULL,'";
    $currentPos = 0;
    foreach ($this->_data as $property => $value) {
      $currentPos++;
      if ($property != $this->_primary){
        
        // Activate the data
        if ($property == 'is_active') {
          $value = '1';
        }
        
        // Create date
        if ($property == 'create_date') {
          $value = time();
        }
        
        // User who created the data 
        if ($property == 'create_usr_id') {
          $value = $_SESSION['usr_id'];
        }
        
        // Source that created the data 
        if ($property == 'create_src_id') {
          $value = $_SESSION['src_id'];
        }

        // Update date
        if ($property == 'update_date') {
          $value = time();
        }
        
        // User who updated the data 
        if ($property == 'update_usr_id') {
          $value = $_SESSION['usr_id'];
        }
        
        // Source that updated the data 
        if ($property == 'update_src_id') {
          $value = $_SESSION['src_id'];
        }

        // Add the property to the sql statement
        $query .= db_escape_string($value).($currentPos<$maxPos?"','":"')");
      }
    }

    $result = db_query($query);
    if ($result) {
      $this->_data[$this->_primary] = db_get_generated_key();
      $this->_status['is_new'] = (bool) false;
      $this->_status['is_dirty'] = (bool) false;
      $this->_status['is_loaded'] = (bool) true;
      return (bool) true;
    }
    else {
      $_dbg['query'] = $query;
      $_dbg['result'] = $result;
      $_dbg['method'] = '_insert';
      $_dbg['error'] = $SPL_TXT['db_cannot_insert'];
      array_push($this->_debug, $_dbg);         
      return (bool) false;
    }

  }

  /** 
   * Checks if the given value is unique within the table.
   * 
   * @param  (string)  $attribute The column name to check
   * @param  (string)  $value     The value to check
   * @return (boolean) Unique or Not Unique
   * @private
   */
  function _isUnique($column,$value) {
    
    global $SPL_CFG,$SPL_TXT;

    $query = "SELECT ".$this->_primary." FROM ".$SPL_CFG['prefix'].$this->_table." WHERE ".$column." = '".$value."'";
    $result = db_retrieve(db_query($query));
    return (!isset($result[$this->_primary]));
  }

  /** 
   * Converts a date into a timestamp
   * 
   * @author    Steve Powell info@scrimpnet.com
   * @copyright Copyright (c) 2004 scrimpnet.com
   * @param  (date) $dateTime A date or datetime
   * @return (integer)  The timestamp
   * @protected
   */
  function _parseDate($dateTime="")
  {
    $ts = 0; // timestamp of parsed date
    switch(gettype($dateTime)) {
      case "string": 
        $ts=strtotime($dateTime); 
        break;
      case "integer":
        $ts=$dateTime;
        break;
      case "object":
        if (get_class($dateTime) == "dateclass") {
          $ts = $dateTime->TimeStamp();
        }
    }
    return $ts;
  }

  /** 
   * Populates the objects attributes.
   * 
   * @param  (array) $data An array containing the data to populate
   * @return (boolean) Success or Failure
   * @private
   */
  function _populate($data) {
    
    // Populate object properties from database columns
    foreach ($data as $property => $value) {
      $this->_data[$property] = $value;
    }
    
    // Populate housekeeping data
    $this->_status['is_dirty']  = (bool) false;
    $this->_status['is_loaded'] = (bool) true;
    $this->_status['is_new'] = (bool) false;
    
    return (bool) true;
  }

  /** 
   * Selects rows from the database.
   * 
   * @param  (string)  $column  The column to select on
   * @param  (string)  $value   The value to match
   * @param  (integer) $limit   (optional) The maximum number of records
   * @param  (string)  $active  Only active ('true'), only inactive ('false') or 'skip'
   * @return (array) The result set
   * @protected
   */
  function _select($column,$value,$limit=0,$active='true') {
    
    global $SPL_CFG,$SPL_TXT;

    $is_active = ''; // Default to 'skip'
    if ($active=='true') {
      $is_active = ' AND is_active';
    }
    else if ($active=='false') {
      $is_active = ' AND NOT is_active';
    }
    $query = "SELECT * FROM ".$SPL_CFG['prefix'].$this->_table." WHERE ".$column." = '".$value."'".$is_active.($limit>0?" LIMIT ".$limit:"");
    $result = db_retrieve(db_query($query));
    if (!$result) {
      $_dbg['query'] = $query;
      $_dbg['result'] = $result;
      $_dbg['method'] = '_select';
      $_dbg['error'] = $SPL_TXT['db_cannot_select'];
      array_push($this->_debug, $_dbg);         
      return (bool) false;
    }
    else {
      return $result;
    }      
  }

  /** 
   * Updates an existing row in the database.
   * 
   * This function will be called by {@link save()} when updating an existing instance in the database.
   * 
   * @return (boolean) Success or Failure
   * @private
   */
  function _update() {

    global $SPL_CFG,$SPL_TXT;

    // To keep track of the current position in the array
    $currentPos = 0;
    // The number of items in the array
    $maxPos = count($this->_data);
    
    // Assemble the update statement
    $query = "UPDATE ".$SPL_CFG['prefix'].$this->_table." SET ";
    foreach ($this->_data as $property => $value) {
      $currentPos++;
      if ($property != $this->_primary){

        // Update date
        if ($property == 'update_date') {
          $value = time();
        }
        
        // User who updated the data 
        if ($property == 'update_usr_id') {
          $value = $_SESSION['usr_id'];
        }
        
        // Source that updated the data 
        if ($property == 'update_src_id') {
          $value = $_SESSION['src_id'];
        }

        $query .= $property." = '".db_escape_string($value)."'".($currentPos<$maxPos?",":"");
      }
    }
    $query .= " WHERE ".$this->_primary." = '".$this->_data[$this->_primary]."'"; 
    $result = db_query($query);
    if ($result) {
      $this->_status['is_new'] = (bool) false;
      $this->_status['is_dirty'] = (bool) false;
      $this->_status['is_loaded'] = (bool) true;
      return (bool) true;
    }
    else {
      $_dbg['query'] = $query;
      $_dbg['result'] = $result;
      $_dbg['method'] = '_update';
      $_dbg['error'] = $SPL_TXT['db_cannot_update'];
      array_push($this->_debug, $_dbg);
      return (bool) false;
    }
  }

}

?>
