<?php
/**
 * Extention to fpdf 
 *
 * $Author: jebbol $
 * $Revision: 1.1 $
 * $Date: 2006/08/08 06:07:23 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

include_once($SPL_CFG['dirroot'].'/class/fpdf/fpdf.php');

//=========================================================================
// Class
//=========================================================================

class PDF extends FPDF {
  
  // Used by html parser
  var $B;
  var $I;
  var $U;
  var $HREF;
  var $fontList;
  var $issetfont;
  var $issetcolor;
  
  // Used by database table
  var $ProcessingTable=false;
  var $aCols=array();
  var $TableX;
  var $HeaderColor;
  var $RowColors;
  var $ColorIndex;  
  
  // Used by table-based multicell
  var $widths;
  var $aligns;

  function PDF($orientation='P',$unit='mm',$format='A4')  {
    //Call parent constructor
    $this->FPDF($orientation,$unit,$format);
    //Initialization
    $this->B=0;
    $this->I=0;
    $this->U=0;
    $this->HREF='';
    $this->fontlist=array("arial","times","courier","helvetica","symbol");
    $this->issetfont=false;
    $this->issetcolor=false;
  }

  // Page header
  function Header() {
    global $SPL_CFG,$name,$courseyear,$prefSub,$subCode,$prefPer,$prdCode,$prefWk,$wkCode,$prefSts,$stsCode;
    
    if ($this->PageNo()==1) {
      $this->Image($SPL_CFG['dirroot'].'/ui/skin/'.$_SESSION['usp_skin'].'/images/pageheader.png',0,0);
    }
    else {
      $this->Image($SPL_CFG['dirroot'].'/ui/skin/'.$_SESSION['usp_skin'].'/images/pageheader_small.png',0,0);
      $this->SetFont('Times','B',8);
      $this->Cell(0,0,$name,0,1,'R');
      $this->Ln(3);
      $this->Cell(0,0,$courseyear,0,1,'R');
      $this->Ln(3);
      if ($prefSub >0) {
        $this->Cell(0,0,$subCode,0,1,'R');
        $this->Ln(3);
      }
      if ($prefPer >0) {
        $this->Cell(0,0,$prdCode,0,1,'R');
        $this->Ln(3);
      }
      if ($prefWk >0) {
        $this->Cell(0,0,$wkCode,0,1,'R');
        $this->Ln(3);
      }
      if ($prefSts >0) {
        $this->Cell(0,0,$stsCode,0,1,'R');
        $this->Ln(3);
      }
    }
    $this->SetFont('Times','B',12);
    $this->Ln(20);                        // Line break

    //Print the table header if necessary
    if($this->ProcessingTable) {
      $this->TableHeader();
    }
    
  }
  
  // Page footer
  function Footer() {
    global $SPL_TXT;
    $this->SetY(-15);
    $this->SetFont('Arial','I',8);
    $this->Cell(0,10,$this->PageNo().'/{nb}',0,0,'C');
  }

  function SetWidths($w) {
    //Set the array of column widths
    $this->widths=$w;
  }

  function SetAligns($a) {
    //Set the array of column alignments
    $this->aligns=$a;
  }

  function Wrap(&$text, $maxwidth) {
    $text = trim($text);
    if ($text==='')
        return 0;
    $space = $this->GetStringWidth(' ');
    $lines = explode("\n", $text);
    $text = '';
    $count = 0;

    foreach ($lines as $line) {
      $words = preg_split('/ +/', $line);
      $width = 0;

      foreach ($words as $word) {
        $wordwidth = $this->GetStringWidth($word);
        if ($wordwidth > $maxwidth) {
          // Word is too long, we cut it
          for($i=0; $i<strlen($word); $i++) {
            $wordwidth = $this->GetStringWidth(substr($word, $i, 1));
            if($width + $wordwidth <= $maxwidth) {
              $width += $wordwidth;
              $text .= substr($word, $i, 1);
            }
            else {
              $width = $wordwidth;
              $text = rtrim($text)."\n".substr($word, $i, 1);
              $count++;
            }
          }
        }
        elseif($width + $wordwidth <= $maxwidth) {
          $width += $wordwidth + $space;
          $text .= $word.' ';
        }
        else {
          $width = $wordwidth + $space;
          $text = rtrim($text)."\n".$word.' ';
          $count++;
        }
      }
      $text = rtrim($text)."\n";
      $count++;
    }
    $text = rtrim($text);
    return $count;
  }
  
  function WriteHTML($html) {

    $html=strip_tags($html,"<b><u><i><a><img><p><br><strong><em><font><tr><blockquote>"); //remove all unsupported tags
    $html=str_replace("\n",' ',$html); //replace carriage returns by spaces
    $a=preg_split('/<(.*)>/U',$html,-1,PREG_SPLIT_DELIM_CAPTURE); //explodes the string
    foreach($a as $i=>$e)
    {
        if($i%2==0)
        {
            //Text
            if($this->HREF)
                $this->PutLink($this->HREF,$e);
            else
                $this->Write(5,stripslashes(txtentities($e)));
        }
        else
        {
            //Tag
            if($e{0}=='/')
                $this->CloseTag(strtoupper(substr($e,1)));
            else
            {
                //Extract attributes
                $a2=explode(' ',$e);
                $tag=strtoupper(array_shift($a2));
                $attr=array();
                foreach($a2 as $v)
                    if(ereg('^([^=]*)=["\']?([^"\']*)["\']?$',$v,$a3))
                        $attr[strtoupper($a3[1])]=$a3[2];
                $this->OpenTag($tag,$attr);
            }
        }
    }

  }

  function OpenTag($tag,$attr) {

    switch($tag){
        case 'STRONG':
            $this->SetStyle('B',true);
            break;
        case 'EM':
            $this->SetStyle('I',true);
            break;
        case 'B':
        case 'I':
        case 'U':
            $this->SetStyle($tag,true);
            break;
        case 'A':
            $this->HREF=$attr['HREF'];
            break;
        case 'IMG':
            if(isset($attr['SRC']) and (isset($attr['WIDTH']) or isset($attr['HEIGHT']))) {
                if(!isset($attr['WIDTH']))
                    $attr['WIDTH'] = 0;
                if(!isset($attr['HEIGHT']))
                    $attr['HEIGHT'] = 0;
                $this->Image($attr['SRC'], $this->GetX(), $this->GetY(), px2mm($attr['WIDTH']), px2mm($attr['HEIGHT']));
            }
            break;
        case 'TR':
        case 'BLOCKQUOTE':
        case 'BR':
            $this->Ln(5);
            break;
        case 'P':
            $this->Ln(10);
            break;
        case 'FONT':
            if (isset($attr['COLOR']) and $attr['COLOR']!='') {
                $coul=hex2dec($attr['COLOR']);
                $this->SetTextColor($coul['R'],$coul['G'],$coul['B']);
                $this->issetcolor=true;
            }
            if (isset($attr['FACE']) and in_array(strtolower($attr['FACE']), $this->fontlist)) {
                $this->SetFont(strtolower($attr['FACE']));
                $this->issetfont=true;
            }
            break;
    }
    
  }

  function CloseTag($tag) {

    //Closing tag
    if($tag=='STRONG')
        $tag='B';
    if($tag=='EM')
        $tag='I';
    if($tag=='B' or $tag=='I' or $tag=='U')
        $this->SetStyle($tag,false);
    if($tag=='A')
        $this->HREF='';
    if($tag=='FONT'){
        if ($this->issetcolor==true) {
            $this->SetTextColor(0);
        }
        if ($this->issetfont) {
            $this->SetFont('arial');
            $this->issetfont=false;
        }
    }

  }

  function SetStyle($tag,$enable) {
    //Wijzig stijl en selecteer lettertype
    $this->$tag+=($enable ? 1 : -1);
    $style='';
    foreach(array('B','I','U') as $s)
        if($this->$s>0)
            $style.=$s;
    $this->SetFont('',$style);
  }

  function PutLink($URL,$txt) {
    //Plaatst een  hyperlink
    $this->SetTextColor(0,0,255);
    $this->SetStyle('U',true);
    $this->Write(5,$txt,$URL);
    $this->SetStyle('U',false);
    $this->SetTextColor(0);
  }

  function TableHeader() {
    $this->SetFont('Arial','B',8);
    $this->SetX($this->TableX);
    $fill=!empty($this->HeaderColor);
    if($fill)
        $this->SetFillColor($this->HeaderColor[0],$this->HeaderColor[1],$this->HeaderColor[2]);
    foreach($this->aCols as $col)
        $this->Cell($col['w'],6,$col['c'],1,0,'C',$fill);
    $this->Ln();
  }

  function Row($data) {
    $this->SetX($this->TableX);
    $ci=$this->ColorIndex;
    $fill=!empty($this->RowColors[$ci]);
    if($fill) {
      $this->SetFillColor($this->RowColors[$ci][0],$this->RowColors[$ci][1],$this->RowColors[$ci][2]);
    }

    //Calculate the height of the row
    $nb=0;
    for($i=0;$i<count($data);$i++) {
      if (isset($this->widths[$i])) {
        $nb=max($nb,$this->NbLines($this->widths[$i],$data[$i]));
      }
    }
    $h=5*$nb;
    //Issue a page break first if needed
    $this->CheckPageBreak($h);

    /*
    // Determine the maximum height
    $nrOfLines = 1;
    foreach($this->aCols as $col) {
      $lines = $this->Wrap($data[$col['f']],$col['w']);
      if ($lines>$nrOfLines) {
        $nrOfLines = $lines;
      }
    }
    $height = $nrOfLines*5;
    */
    
    $i=0;
    foreach($this->aCols as $col) {
        $w=$this->widths[$i];
        $a=isset($this->aligns[$i]) ? $this->aligns[$i] : 'L';
        //Save the current position
        $x=$this->GetX();
        $y=$this->GetY();
        //Draw the border
        $this->Rect($x,$y,$w,$h);
        //Print the text
        $this->MultiCell($w,5,$data[$col['f']],0,$col['a'],$fill);
        //Put the position to the right of the cell
        $this->SetXY($x+$w,$y);
      $i++;
      
      
      //$this->Cell($col['w'],$height,$data[$col['f']],1,0,$col['a'],$fill);
    }
    $this->Ln($h);
    $this->ColorIndex=1-$ci;
  }

  function CalcWidths($width,$align) {
    //Compute the widths of the columns
    $TableWidth=0;
    foreach($this->aCols as $i=>$col)
    {
        $w=$col['w'];
        if($w==-1)
            $w=$width/count($this->aCols);
        elseif(substr($w,-1)=='%')
            $w=$w/100*$width;
        $this->aCols[$i]['w']=$w;
        $TableWidth+=$w;
    }
    //Compute the abscissa of the table
    if($align=='C')
        $this->TableX=max(($this->w-$TableWidth)/2,0);
    elseif($align=='R')
        $this->TableX=max($this->w-$this->rMargin-$TableWidth,0);
    else
        $this->TableX=$this->lMargin;
  }

  function AddCol($field=-1,$width=-1,$caption='',$align='L') {
    //Add a column to the table
    if($field==-1)
        $field=count($this->aCols);
    $this->aCols[]=array('f'=>$field,'c'=>$caption,'w'=>$width,'a'=>$align);
  }

  function Table($query,$prop=array()) {
    //Issue query
    $res=mysql_query($query) or die('Error: '.mysql_error()."<BR>Query: $query");
    //Add all columns if none was specified
    if(count($this->aCols)==0)
    {
        $nb=mysql_num_fields($res);
        for($i=0;$i<$nb;$i++)
            $this->AddCol();
    }
    //Retrieve column names when not specified
    foreach($this->aCols as $i=>$col)
    {
        if($col['c']=='')
        {
            if(is_string($col['f']))
                $this->aCols[$i]['c']=ucfirst($col['f']);
            else
                $this->aCols[$i]['c']=ucfirst(mysql_field_name($res,$col['f']));
        }
    }
    //Handle properties
    if(!isset($prop['width']))
        $prop['width']=0;
    if($prop['width']==0)
        $prop['width']=$this->w-$this->lMargin-$this->rMargin;
    if(!isset($prop['align']))
        $prop['align']='C';
    if(!isset($prop['padding']))
        $prop['padding']=$this->cMargin;
    $cMargin=$this->cMargin;
    $this->cMargin=$prop['padding'];
    if(!isset($prop['HeaderColor']))
        $prop['HeaderColor']=array();
    $this->HeaderColor=$prop['HeaderColor'];
    if(!isset($prop['color1']))
        $prop['color1']=array();
    if(!isset($prop['color2']))
        $prop['color2']=array();
    $this->RowColors=array($prop['color1'],$prop['color2']);
    //Compute column widths
    $this->CalcWidths($prop['width'],$prop['align']);
    //Print header
    $this->TableHeader();
    //Print rows
    $this->SetFont('Arial','',8);
    $this->ColorIndex=0;
    $this->ProcessingTable=true;
    while($row=mysql_fetch_array($res))
        $this->Row($row);
    $this->ProcessingTable=false;
    $this->cMargin=$cMargin;
    $this->aCols=array();
  }

  function CheckPageBreak($h) {
    //If the height h would cause an overflow, add a new page immediately
    if($this->GetY()+$h>$this->PageBreakTrigger)
        $this->AddPage($this->CurOrientation);
  }

  
  function NbLines($w,$txt) {
    //Computes the number of lines a MultiCell of width w will take
    $cw=&$this->CurrentFont['cw'];
    if($w==0)
        $w=$this->w-$this->rMargin-$this->x;
    $wmax=($w-2*$this->cMargin)*1000/$this->FontSize;
    $s=str_replace("\r",'',$txt);
    $nb=strlen($s);
    if($nb>0 and $s[$nb-1]=="\n")
        $nb--;
    $sep=-1;
    $i=0;
    $j=0;
    $l=0;
    $nl=1;
    while($i<$nb)
    {
        $c=$s[$i];
        if($c=="\n")
        {
            $i++;
            $sep=-1;
            $j=$i;
            $l=0;
            $nl++;
            continue;
        }
        if($c==' ')
            $sep=$i;
        $l+=$cw[$c];
        if($l>$wmax)
        {
            if($sep==-1)
            {
                if($i==$j)
                    $i++;
            }
            else
                $i=$sep+1;
            $sep=-1;
            $j=$i;
            $l=0;
            $nl++;
        }
        else
            $i++;
    }
    return $nl;
  }
  
}

//function hex2dec
//returns an associative array (keys: R,G,B) from
//a hex html code (e.g. #3FE5AA)
function hex2dec($couleur = "#000000"){
    $R = substr($couleur, 1, 2);
    $rouge = hexdec($R);
    $V = substr($couleur, 3, 2);
    $vert = hexdec($V);
    $B = substr($couleur, 5, 2);
    $bleu = hexdec($B);
    $tbl_couleur = array();
    $tbl_couleur['R']=$rouge;
    $tbl_couleur['G']=$vert;
    $tbl_couleur['B']=$bleu;
    return $tbl_couleur;
}

//conversion pixel -> millimeter in 72 dpi
function px2mm($px){
    return $px*25.4/72;
}

function txtentities($html){
    $trans = get_html_translation_table(HTML_ENTITIES);
    $trans = array_flip($trans);
    return strtr($html, $trans);
}

/**
 * $Log: fpdf.class.php,v $
 * Revision 1.1  2006/08/08 06:07:23  jebbol
 * Initial upload
 *
 */
?>