<?php
/**
 * Group class.
 *
 * @author    Jebbol
 * @version   1.0
 * @revision  $Id: grp.class.php,v 1.5 2006/08/08 06:05:52 jebbol Exp $
 * @package   spl
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');


//=========================================================================
// Class definition
//=========================================================================

class grp extends abstractData {
  
  function grp() {

    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['grp_id']              = (int) 0;
    $this->_data['adm_id']              = $_SESSION['adm_id'];
    $this->_data['grp_name']            = '';
    $this->_data['grp_short']           = '';
    $this->_data['grp_code']            = '';
    $this->_data['grp_type']            = 'class';
    $this->_data['ins_id']              = (int) 0;
    $this->_data['ina_id']              = (int) 0;
    $this->_data['tra_id']              = (int) 0;
    $this->_data['ali_id']              = (int) 0;
    $this->_data['sub_id']              = (int) 0;
    $this->_data['sug_id']              = (int) 0;
    $this->_data['prd_id']              = (int) 0;
    $this->_data['cal_id']              = (int) 0;
    $this->_data['grp_calyear']         = (int) 0;
    $this->_data['grp_courseyear']      = (int) 0;
    $this->_data['is_active']           = (bool) true;
    $this->_data['create_date']         = time();
    $this->_data['create_usr_id']       = $_SESSION['usr_id'];
    $this->_data['create_src_id']       = $_SESSION['native_id'];
    $this->_data['update_date']         = time();
    $this->_data['update_usr_id']       = $_SESSION['usr_id'];
    $this->_data['update_src_id']       = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary = 'grp_id';
    $this->_table   = 'grp';
    $this->_name    = 'grp_short';
    
    // Register properties that must have a value
    $this->_required[] = $this->_name;
    $this->_required[] = 'grp_code';
    
    // Unique values
    $this->_unique[] = 'grp_code';

    // Set user permissions for this type of data
    if (!isset($_SESSION[$this->_table.'_permission'])) {
      $_SESSION[$this->_table.'_permission'] = $this->_getPermissions();
    }

  }
  //=========================================================================
  // Public methods
  //=========================================================================

  /**
   * Retrieve a list of groups
   * 
   * @param  (string) $displayField The field to return (optional)
   * @param  (string) $sortField    The field to used for sorting (optional)
   * @param  (string) $exclude      A comma-seperated list of group id's to exclude from the result (optional)
   * @param  (int)    $insId        The institution id (optional)
   * @param  (int)    $inaId        The institution address id (optional)
   * @param  (int)    $traId        The training id (optional)
   * @param  (int)    $aliId        The alignment id (optional)
   * @param  (string) $subjects     A comma-seperated list of subject id's limiting the groups (optional)
   * @return array An array of students (key => value)
   * @public
   */
  function getGroups($displayField='grp_name',$sortField='grp_name',$exclude=null,$insId=null,$inaId=null,$traId=null,$aliId=null,$subjects=null) {
    
    global $SPL_CFG;
    
    $values = array();
    
    $query  = 'SELECT grp_id as "key",'.$displayField.' as "value" FROM '.$SPL_CFG['prefix'].'grp grp';
    $query .= ' WHERE grp.adm_id = "'.$_SESSION['adm_id'].'" AND grp.is_active';
    if (isset($insId)) {
      $query .= ' AND ins_id = "'.$insId.'"';
    }
    if (isset($inaId)) {
      $query .= ' AND ina_id = "'.$inaId.'"';
    }
    if (isset($traId)) {
      $query .= ' AND tra_id = "'.$traId.'"';
    }
    if (isset($aliId)) {
      $query .= ' AND ali_id = "'.$aliId.'"';
    }
    if (isset($subjects)) {
      $query .= ' AND sub_id IN ('.$subjects.')';;
    }
    if (isset($exclude)) {
      $query .= ' AND grp_id NOT IN ('.$exclude.')';;
    }
    $query .= ' ORDER BY grp.'.$sortField;

    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      $values[] = array($row['key']=>$row['value']);
    }
    return $values;
  }


  //=========================================================================
  // Private methods
  //=========================================================================

}

?>