<?php
/**
 * Institution address class.
 *
 * $Author: jebbol $
 * $Revision: 1.10 $
 * $Date: 2006/08/24 17:31:59 $
 * 
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

//=========================================================================
// Class definition
//=========================================================================

class ina extends abstractData {
  
  function ina() {

    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['ina_id']                   = (int) 0;
    $this->_data['adm_id']                   = $_SESSION['adm_id'];
    $this->_data['adr_id']                   = (int) 0;
    $this->_data['ina_type']                 = 'Location';
    $this->_data['ina_name']                 = '';
    $this->_data['ina_short']                = '';
    $this->_data['ina_email']                = '';
    $this->_data['ina_code']                 = '';
    $this->_data['ina_schoolyear_start']     = (int) 0;
    $this->_data['ina_schoolyear_end']       = (int) 0;
    $this->_data['ina_methodology']          = 'traditional';
    $this->_data['ina_url_schedule_group']   = '';
    $this->_data['ina_url_schedule_teacher'] = '';
    $this->_data['ina_url_schedule_student'] = '';
    $this->_data['ina_url_schedule_room']    = '';
    $this->_data['ina_alerttext']            = '';
    $this->_data['ina_alerttext_fmt']        = '';
    $this->_data['ina_valid_from']           = (int) 0;
    $this->_data['ina_valid_until']          = (int) 4294967295; // Always valid
    $this->_data['is_active']                = (bool) true;
    $this->_data['create_date']              = time();
    $this->_data['create_usr_id']            = $_SESSION['usr_id'];
    $this->_data['create_src_id']            = $_SESSION['native_id'];
    $this->_data['update_date']              = time();
    $this->_data['update_usr_id']            = $_SESSION['usr_id'];
    $this->_data['update_src_id']            = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary = 'ina_id';
    $this->_table   = 'ina';
    $this->_name    = 'ina_name';
    
    // Register properties that must have a value
    $this->_required[] = $this->_name;
    $this->_required[] = 'ina_code';
    
    // Set field types
    $this->_type['ina_schoolyear_start'] = array('type'=>'timestamp');
    $this->_type['ina_schoolyear_end'] = array('type'=>'timestamp');
    $this->_type['ina_valid_from'] = array('type'=>'timestamp');
    $this->_type['ina_valid_until'] = array('type'=>'timestamp');

    // Set user permissions for this type of data
    if (!isset($_SESSION[$this->_table.'_permission'])) {
      $_SESSION[$this->_table.'_permission'] = $this->_getPermissions();
    }

  }
  //=========================================================================
  // Public methods
  //=========================================================================


  /**
   * Checks if there is only one active row in the table and returns the id if that is the case
   * 
   * @param  integer $tmp_ins_id When>0, this will check if there is only one address for this institution
   * @return integer The row id, or false if there is either zer or more than one row
   * @access public
   */
  function isSingle($tmp_ins_id=0) {
    
    global $SPL_CFG;
    
    // When no institution is specified, check the number of addresses within the administration
    if ($tmp_ins_id==0) {
      return parent::isSingle();
    }
    
    // Else check the number of addresses for the given institution
    $query  = 'SELECT child_id FROM '.$SPL_CFG['prefix'].'apl apl';
    $query .= ' LEFT JOIN '.$SPL_CFG['prefix'].'ina ina ON ina.ina_id = apl.child_id';
    $query .= ' WHERE apl.adm_id = "'.$_SESSION['adm_id'].'" AND apl.is_active AND ina.is_active';
    $query .= ' AND apl_parent = "ins" AND parent_id = "'.$tmp_ins_id.'" AND apl_child = "ina"';
    $result = db_query($query);
    if (db_num_rows($result)==1){
      $result = db_retrieve($result);
      return $result['child_id'];
    }
    return (bool)false;
  }

  /** 
   * Sets the instance property with the given value.
   * This overrides the function in data.abstract.php, because in case of
   * an 'is_active' property all 'referees' will have their state switched as well
   *
   * @param  string  $property The name of the property to be set
   * @param  string  $value    The data to bet set
   * @return boolean Succes or Failure
   * @access public
   */
  function setProperty($property, $value) {
    
    global $SPL_CFG;
    
    $succes = parent::setProperty($property, $value); 
    
    if ($succes){

      // Switch state in the catalog
      $query  = 'UPDATE '.$SPL_CFG['prefix'].'cat';
      $query .= ' SET is_active = "'.$value.'" WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
      $query .= ' ina_id = "'.$this->_data['ina_id'].'"';
      $result = db_query($query);

    }
  }


  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Deletes a row from the database.
   * Override the default behaviour by adding the removal of child records
   * 
   * @return boolean Success or Failure
   * @access protected
   * @uses   array   $SPL_CFG Global configuration data
   */
  function _delete() {
    
    global $SPL_CFG;
    
    $succes = parent::_delete();
    
    if ($succes) {
      $query  = 'DELETE FROM '.$SPL_CFG['prefix'].'cat WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
      $query .= ' ina_id = "'.$this->_data['ina_id'].'"';
      $result = db_query($query);
    }
    
    return $succes;
  }

}

/**
 * $Log: ina.class.php,v $
 * Revision 1.10  2006/08/24 17:31:59  jebbol
 * Added support for alerts
 *
 */
?>