<?php
/**
 * Class that handles a methodology
 *
 * $Author: jebbol $
 * $Revision: 1.14 $
 * $Date: 2006/08/08 05:49:15 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 * 
 * $Log: methodology.class.php,v $
 * Revision 1.14  2006/08/08 05:49:15  jebbol
 * Added support for parent and mentor
 *
 * Revision 1.13  2006/04/14 10:58:10  jebbol
 * Added getReportScript()
 *
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Configuration
//include_once($SPL_CFG['dirroot'].'/inc/config.php');

// XML parser
include_once($SPL_CFG['dirroot'].'/inc/domit/xml_domit_lite_include.php');

class methodology {
  
  var $_id;               // The id (directoryname) of the methodology
  var $_methodology;      // The xml definition
  var $_status = array(); // The object status

  function methodology() {

    $this->_methodology =& new DOMIT_Lite_Document();
    $this->_methodology->resolveErrors(true);

    $this->_status['is_loaded'] = (bool) false;
  }


  /** 
   * Returns the path to the catalog browse include
   *
   * @return string The path to the catalog include file
   * @access public
   */
  function getCatalogBrowseScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'catalog/browse_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the metadata script
   *
   * @return string The path to the metadata script
   * @access public
   */
  function getCatalogMetadata() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'catalog/metadata', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
 
  }

  /** 
   * Returns the path to the catalog display include
   *
   * @return string The path to the catalog include file
   * @access public
   */
  function getCatalogDisplayScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'catalog/display_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the catalog edit include
   *
   * @return string The path to the catalog include file
   * @access public
   */
  function getCatalogEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'catalog/edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the catalog report include
   *
   * @return string The path to the catalog include file
   * @access public
   */
  function getCatalogReportScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'catalog/report_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the translation script
   *
   * @return string The path to the translation script
   * @access public
   */
  function getCatalogTranslation() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    if (!isset($_SESSION['usp_lang'])) {$_SESSION['usp_lang'] = 'en';}
    
    $element = &$this->_methodology->getElementsByPath( 'catalog/translation', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/ui/lang/'.$_SESSION['usp_lang'].'/'.$element->getText() : '';
 
  }

  /** 
   * Returns the path to the chronicle browse include
   *
   * @return string The path to the chronicle include file
   * @access public
   */
  function getChronicleBrowseScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'chronicle/browse_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the chronicle display include
   *
   * @return string The path to the chronicle include file
   * @access public
   */
  function getChronicleDisplayScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'chronicle/display_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the chronicle edit include
   *
   * @return string The path to the chronicle include file
   * @access public
   */
  function getChronicleEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'chronicle/edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the metadata script
   *
   * @return string The path to the metadata script
   * @access public
   */
  function getChronicleMetadata() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'chronicle/metadata', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
 
  }

  /** 
   * Returns the path to the chronicle report include
   *
   * @return string The path to the chronicle include file
   * @access public
   */
  function getChronicleReportScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'chronicle/report_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the translation script
   *
   * @return string The path to the translation script
   * @access public
   */
  function getChronicleTranslation() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    if (!isset($_SESSION['usp_lang'])) {$_SESSION['usp_lang'] = 'en';}
    
    $element = &$this->_methodology->getElementsByPath( 'chronicle/translation', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/ui/lang/'.$_SESSION['usp_lang'].'/'.$element->getText() : '';
 
  }

  /** 
   * Returns the supported classes
   *
   * @return string A comma separated list of classes
   * @access public
   */
  function getClasses() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'classes', 1 );
    return $element ? $element->getText() : '';
  }

  /** 
   * Returns the path to the browse include
   *
   * @return string The path to the include file
   * @access public
   */
  function getDashboardBrowseScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'dashboard/browse_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the display include
   *
   * @return string The path to the include file
   * @access public
   */
  function getDashboardDisplayScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'dashboard/display_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the edit include
   *
   * @return string The path to the include file
   * @access public
   */
  function getDashboardEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'dashboard/edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the metadata script
   *
   * @return string The path to the metadata script
   * @access public
   */
  function getDashboardMetadata() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'dashboard/metadata', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
 
  }

  /** 
   * Returns the path to the report include
   *
   * @return string The path to the include file
   * @access public
   */
  function getDashboardReportScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'dashboard/report_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the translation script
   *
   * @return string The path to the translation script
   * @access public
   */
  function getDashboardTranslation() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    if (!isset($_SESSION['usp_lang'])) {$_SESSION['usp_lang'] = 'en';}
    
    $element = &$this->_methodology->getElementsByPath( 'dashboard/translation', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/ui/lang/'.$_SESSION['usp_lang'].'/'.$element->getText() : '';
 
  }

  /** 
   * Returns the path to the mentor edit include
   *
   * @return string The path to the mentor include file
   * @access public
   */
  function getMentorEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'mentor/edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the mentor groups edit include
   *
   * @return string The path to the mentor include file
   * @access public
   */
  function getMentorGroupsEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'mentor/groups_edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the mentor group set include
   *
   * @return string The path to the mentor include file
   * @access public
   */
  function getMentorGroupsSetScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'mentor/groups_set_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }
  
  /** 
   * Returns the path to the mentors students edit include
   *
   * @return string The path to the mentor include file
   * @access public
   */
  function getMentorStudentsEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'mentor/students_edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the mentor subjects edit include
   *
   * @return string The path to the mentor include file
   * @access public
   */
  function getMentorSubjectsEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'mentor/subjects_edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the mentor set include
   *
   * @return string The path to the mentor include file
   * @access public
   */
  function getMentorSetScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'mentor/set_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the mentor subjects set include
   *
   * @return string The path to the mentor include file
   * @access public
   */
  function getMentorSubjectsSetScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teacher/subjects_set_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the name
   *
   * @return string The name
   * @access public
   */
  function getName() {
    if (!$this->_status['is_loaded']) return (bool)false;
    
    $element = &$this->_methodology->getElementsByPath( 'name', 1 );
    return $element ? $element->getText() : '';
  }

  /** 
   * Returns the path to the student edit include
   *
   * @return string The path to the student include file
   * @access public
   */
  function getStudentEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'student/edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the student set include
   *
   * @return string The path to the student include file
   * @access public
   */
  function getStudentSetScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'student/set_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the studyguide browse include
   *
   * @return string The path to the studyguide include file
   * @access public
   */
  function getStudyguideBrowseScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'studyguide/browse_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the studyguide display include
   *
   * @return string The path to the studyguide include file
   * @access public
   */
  function getStudyguideDisplayScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'studyguide/display_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the studyguide edit include
   *
   * @return string The path to the studyguide include file
   * @access public
   */
  function getStudyguideEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'studyguide/edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the metadata script
   *
   * @return string The path to the metadata script
   * @access public
   */
  function getStudyguideMetadata() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'studyguide/metadata', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
 
  }

  /** 
   * Returns the path to the studyguide report include
   *
   * @return string The path to the studyguide include file
   * @access public
   */
  function getStudyguideReportScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'studyguide/report_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the translation script
   *
   * @return string The path to the translation script
   * @access public
   */
  function getStudyguideTranslation() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    if (!isset($_SESSION['usp_lang'])) {$_SESSION['usp_lang'] = 'en';}
    
    $element = &$this->_methodology->getElementsByPath( 'studyguide/translation', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/ui/lang/'.$_SESSION['usp_lang'].'/'.$element->getText() : '';
 
  }

  /** 
   * Returns the path to the teacher edit include
   *
   * @return string The path to the teacher include file
   * @access public
   */
  function getTeacherEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teacher/edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  
  /** 
   * Returns the path to the teacher groups edit include
   *
   * @return string The path to the teacher include file
   * @access public
   */
  function getTeacherGroupsEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teacher/groups_edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the teacher group set include
   *
   * @return string The path to the teacher include file
   * @access public
   */
  function getTeacherGroupsSetScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teacher/groups_set_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }
  
  /** 
   * Returns the path to the teacher students edit include
   *
   * @return string The path to the teacher include file
   * @access public
   */
  function getTeacherStudentsEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teacher/students_edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the teacher subjects edit include
   *
   * @return string The path to the teacher include file
   * @access public
   */
  function getTeacherSubjectsEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teacher/subjects_edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }


  /** 
   * Returns the path to the teacher set include
   *
   * @return string The path to the teacher include file
   * @access public
   */
  function getTeacherSetScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teacher/set_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the teacher students set include
   *
   * @return string The path to the teacher include file
   * @access public
   */
  function getTeacherStudentsSetScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teacher/students_set_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }
  
  /** 
   * Returns the path to the teacher ssubjects set include
   *
   * @return string The path to the teacher include file
   * @access public
   */
  function getTeacherSubjectsSetScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teacher/subjects_set_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }


  /** 
   * Returns the path to the tasklist browse include
   *
   * @return string The path to the include file
   * @access public
   */
  function getTeacherTaskBrowseScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teachertask/browse_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the tasklist display include
   *
   * @return string The path to the include file
   * @access public
   */
  function getTeacherTaskDisplayScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teachertask/display_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the tasklist edit include
   *
   * @return string The path to the include file
   * @access public
   */
  function getTeacherTaskEditScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'tasklist/edit_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the metadata script
   *
   * @return string The path to the metadata script
   * @access public
   */
  function getTeacherTaskMetadata() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teachertask/metadata', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
 
  }

  /** 
   * Returns the path to the tasklist report include
   *
   * @return string The path to the include file
   * @access public
   */
  function getTeacherTaskReportScript() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    $element = &$this->_methodology->getElementsByPath( 'teachertask/report_script', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/'.$element->getText() : '';
  }

  /** 
   * Returns the path to the translation script
   *
   * @return string The path to the translation script
   * @access public
   */
  function getTeacherTaskTranslation() {
    
    global $SPL_CFG;
    
    if (!$this->_status['is_loaded']) return (bool)false;

    if (!isset($_SESSION['usp_lang'])) {$_SESSION['usp_lang'] = 'en';}
    
    $element = &$this->_methodology->getElementsByPath( 'teachertask/translation', 1 );
    return $element ? $SPL_CFG['dirroot'].'/methodology/'.$this->_id.'/ui/lang/'.$_SESSION['usp_lang'].'/'.$element->getText() : '';
 
  }

  /** 
   * Instantiates the object
   *
   * @param  string  The name of the methodology  
   * @return boolean
   * @access public
   */
  function init($name) {
    
    global $SPL_CFG;
  
    $xmlfile = $SPL_CFG['dirroot']. "/methodology/".$name."/methodology.xml";
    if (!$this->_methodology->loadXML($xmlfile)) {
      return (bool)false;
    }
    $this->_id = $name;
    $this->_status['is_loaded'] = (bool) true;
  
  }

}
?>