<?php
/**
 * par.class.php
 *
 * $Author: jebbol $
 * $Revision: 1.5 $
 * $Date: 2006/08/21 18:33:19 $
 * 
 * @copyright Copyright (c) 2005,2006 Studiewijzer.nl, all rights reserved 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

//=========================================================================
// Class definition
//=========================================================================

/**
 * Parent data
 * This class is an addition to the user data (usr)
 */
class par extends abstractData {

  var $associatedUser; // When adding a record, it must be attached to a valid user.
  var $childrenQueue;  // Queue children for a new parent row
  
  function par() {
    
    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['par_id']            = (int) 0;
    $this->_data['adm_id']            = $_SESSION['adm_id'];
    $this->_data['par_dispname']      = '';
    $this->_data['par_sortname']      = '';
    $this->_data['is_active']         = (bool) true;
    $this->_data['create_date']       = time();
    $this->_data['create_usr_id']     = $_SESSION['usr_id'];
    $this->_data['create_src_id']     = $_SESSION['native_id'];
    $this->_data['update_date']       = time();
    $this->_data['update_usr_id']     = $_SESSION['usr_id'];
    $this->_data['update_src_id']     = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary    = 'par_id';
    $this->_table      = 'par';
    $this->_required[] = 'par_sortname';
    
    // Initialize the children queue
    $this->childrenQueue = (bool) false;
    
      // Set user permissions for this type of data
    if (!isset($_SESSION[$this->_table.'_permission'])) {
      $_SESSION[$this->_table.'_permission'] = $this->_getPermissions();
    }
  
  }

  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Returns the name of the parent
   *
   * @return (string) The (formatted) name
   * @public
   */
  function getName() {

    global $SPL_CFG,$SPL_TXT;
    
    // Make sure session information exists
    if (!isset($_SESSION['usp_nameformat'])) {
      $_SESSION['usp_nameformat'] = $SPL_CFG['nameformat'];
    }

    // This will only work on a correctly instantiated object
    if ($this->_status['is_loaded']) {

      // The user class owns the name of the student
      include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
      $usr = new usr();
      $result = $usr->initStudent($this->_data[$this->_primary]);
      if ($result) {
        return $usr->getName();
      }
      else {
        return (bool)false;
      }
    }
    else {
      $_dbg['query']  = 'getting name for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_loaded'];
      $_dbg['method'] = 'getName';
      $_dbg['error']  = $SPL_TXT['db_instance_not_loaded'];
      array_push($this->_debug, $_dbg);
      return (bool)false;
    }
    
  }

  /** 
   * Returns the parents children
   *
   * @param  (string) $valueType Indicates the type of value to return (long,short,code)
   * @return (array)             An array of children (key => value)
   * @public
   */
  function getChildren($valueType='long') {

    global $SPL_CFG;
    
    $values = array();
    $field = 'stu_dispname';

    $query  = 'SELECT stu.stu_id as "key",stu.'.$field.' as "value"';
    $query .= ' FROM '.$SPL_CFG['prefix'].'apl apl LEFT JOIN '.$SPL_CFG['prefix'].'stu stu ON stu.stu_id = apl.child_id';
    $query .= ' WHERE apl.adm_id = "'.$_SESSION['adm_id'].'" AND apl.is_active';
    $query .= ' AND apl.apl_parent = "par"';
    $query .= ' AND apl.parent_id = "'.$this->_data['par_id'].'"';
    $query .= ' AND apl.apl_child = "stu"';
    $query .= ' AND apl.apl_valid_from <= "'.time().'"';
    $query .= ' AND (apl.apl_valid_until = "0" OR apl.apl_valid_until >= "'.time().'")';
    $query .= ' ORDER BY apl.apl_sequence ASC, stu.stu_sortname ASC';

    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      // Try to extract first name
      if (($valueType != 'long') && strstr($row['value'],' ')) {
        $short = explode(' ',$row['value']);
        $values[] = array($row['key']=>$short[0]);
      }
      // Get full name
      else {
        $values[] = array($row['key']=>$row['value']);
      }
    }
    return $values;
  }

  /** 
   * Saves the instance data to the database
   *
   * @return (boolean)
   * @public
   */
  function save() {
    
    global $SPL_CFG;
    
    // When creating a new record, attach it to the user
    if ($this->_status['is_new']) {
      if (empty($this->associatedUser)){
        $_dbg['query']  = 'examining the value of associatedUser id '.$this->_data[$this->_primary];
        $_dbg['result'] = 'empty';
        $_dbg['method'] = 'save';
        $_dbg['error']  = 'No user set. Call setUser prior to saving.';
        array_push($this->_debug, $_dbg);
        return (bool)false;
      }
      include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
      $usr = new usr();
      if (!$usr->init($this->associatedUser)){
        $_dbg['query']  = 'initializing user id '.$this->associatedUser;
        $_dbg['result'] = 'failed';
        $_dbg['method'] = 'save';
        $_dbg['error']  = 'Could not initialize user object.';
        array_push($this->_debug, $_dbg);
        return (bool)false;
      }
      parent::save();
      $usr->setProperty('par_id',$this->_data['par_id']);
      $usr->save();
      $usr->setPreference('usp_startpane','menu_dash.php');
      $usr->save();
    }
    else {
      parent::save();
    }

    // Check for children in the queue
    if ($this->childrenQueue) {
      $this->setChildren($this->childrenQueue);
      $this->childrenQueue = (bool) false;
    }

  }
  
  /** 
   * Sets the parents children
   *
   * @param  (array)   $children An array of children to set for the parent
   * @return (boolean)           Success or Failure
   * @public
   */
  function setChildren($children=NULL) {

    global $SPL_CFG;
    
    if (isset($children)){
      
      // First delete old children first
      $this->unsetChildren();
      
      // Than insert the new ones
      $seq = (int)0;
      foreach ($children as $id){
        // Find subject code
        $query  = 'SELECT stu_sortname FROM '.$SPL_CFG['prefix'].'stu stu ';
        $query .= 'WHERE stu.adm_id = "'.$_SESSION['adm_id'].'" ';
        $query .= 'AND stu.stu_id = "'.$id.'" LIMIT 1';
        $result = db_retrieve(db_query($query));
        $code = (isset($result['stu_sortname'])?$result['stu_sortname']:'');
        
        $seq = $seq + 100;
        $query  = 'INSERT INTO '.$SPL_CFG['prefix'].'apl ';
        $query .= '(apl_id,adm_id,apl_parent,parent_id,apl_child,child_id,apl_sequence,apl_code,apl_valid_from,apl_valid_until,is_active) ';
        $query .= 'VALUES (NULL,';
        $query .= '"'.$_SESSION['adm_id'].'",';
        $query .= '"par",';
        $query .= '"'.$this->_data['par_id'].'",';
        $query .= '"stu",';
        $query .= '"'.$id.'",';
        $query .= '"'.$seq.'",';
        $query .= '"'.$code.'",';
        $query .= '"0",';
        $query .= '"0",';
        $query .= '"1")';
        $result = db_query($query);
      }
    }
  }

  /**
   * Queues the selection of children until a new parent row has been created
   * 
   * @param (string) $children A list of subjects
   * @public
   */
  function queueChildren($children) {
    $this->childrenQueue = $children;
  }

  /** 
   * Sets the userid of the associated user
   *
   * @param (integer) $userid A valid user id
   * @public
   */
  function setUser($userid) {
    $this->associatedUser = $userid;
  }
  
  /** 
   * Unsets the parents children
   *
   * @return (boolean) Success or Failure
   * @public
   */
  function unsetChildren() {

    global $SPL_CFG;

    if (!$this->_status['is_new']) {
      $query  = 'DELETE FROM '.$SPL_CFG['prefix'].'apl';
      $query .= ' WHERE adm_id = "'.$_SESSION['adm_id'].'"';
      $query .= ' AND apl_parent = "par"';
      $query .= ' AND parent_id = "'.$this->_data['par_id'].'"';
      $query .= ' AND apl_child = "stu"';
      $result = db_query($query);
    }
    return (bool) true;

  }
  
  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Deletes a row from the database.
   * 
   * @return (boolean) Success or Failure
   * @protected
   */
  function _delete() {
    
    global $SPL_CFG;
    
      // Remove student id from user data
      include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
      $usr = new usr();
      $result = $usr->initParent($this->_data[$this->_primary]);
      if ($result) {
        $usr->setProperty('par_id',0);
        $usr->save();
      }
      
      // Remove parents children
      $this->unsetChildren();

      return (parent::_delete());
  }
  
  /** 
   * Inserts a new row in the database.
   * 
   * This function will be called by {@link save()} when adding a new instance to the database.
   * 
   * @return (boolean) Success or Failure
   * @protected
   */
  function _insert() {
    
    global $SPL_CFG;
    
    if (parent::_insert()) {
      // When insert is succesfull, set default authorization
      include_once($SPL_CFG['dirroot'].'/class/usa.class.php');
      
      // Calendar
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','cal');      
      $usa->setProperty('usa_permission','1');   
      $usa->save();   

      // Catalog
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','cat');      
      $usa->setProperty('usa_permission','1');      
      $usa->save();   

      // Studyguide
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','sgd');      
      $usa->setProperty('usa_permission','1');   
      $usa->save();   

      // Suggestions
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','sug');      
      $usa->setProperty('usa_permission','1');   
      $usa->save();   

      // Period
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','prd');      
      $usa->setProperty('usa_permission','1');   
      $usa->save();   

      // Statusses
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','sts');      
      $usa->setProperty('usa_permission','1');   
      $usa->save();
      
      // Set preferences
      include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
      $usr = new usr();
      if ($usr->init($this->associatedUser)) {
        $usr->setPreference('usp_calmode','calendardate');
        $usr->setPreference('usp_startpane','menu_parent.php');
        $usr->setPreference('usp_menumode','normal');
        $usr->setPreference('usp_numrows','16');
      }
    }

  }  

}

/** 
 * $Log: par.class.php,v $
 * Revision 1.5  2006/08/21 18:33:19  jebbol
 * Changed default skin when creating a new record
 *
 * Revision 1.4  2006/08/08 06:05:52  jebbol
 * *** empty log message ***
 *
 * Revision 1.1  2006/04/14 10:41:39  jebbol
 * Added support for mentors and parents
 */

?>