<?php
/**
 * Period class.
 *
 * $Author: jebbol $
 * $Revision: 1.7 $
 * $Date: 2006/05/30 12:56:38 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.nl
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

//=========================================================================
// Class definition
//=========================================================================

class prd extends abstractData {
  
  function prd() {

    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['prd_id'] = (int) 0;
    $this->_data['adm_id'] = $_SESSION['adm_id'];
    $this->_data['prd_name'] = '';
    $this->_data['prd_short'] = '';
    $this->_data['prd_code'] = '';
    $this->_data['prd_month_start'] = (int) 8;
    $this->_data['prd_month_end'] = (int) 7;
    $this->_data['prd_day_start'] = (int) 1;
    $this->_data['prd_day_end'] = (int) 31;
    $this->_data['prd_valid_from'] = (int) 0;
    $this->_data['prd_valid_until'] = (int) 4294967295; // Always valid
    $this->_data['is_active'] = (bool) true;
    $this->_data['create_date'] = time();
    $this->_data['create_usr_id'] = $_SESSION['usr_id'];
    $this->_data['create_src_id'] = $_SESSION['native_id'];
    $this->_data['update_date'] = time();
    $this->_data['update_usr_id'] = $_SESSION['usr_id'];
    $this->_data['update_src_id'] = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary = 'prd_id';
    $this->_table   = 'prd';
    $this->_name    = 'prd_name';
    
    // Register properties that must have a value
    $this->_required[] = $this->_name;
    // Bug #1493089: didn't save data when no code had been entered
    // $this->_required[] = 'prd_code';
    
    // Set field types
    $this->_type['prd_valid_from'] = array('type'=>'timestamp');
    $this->_type['prd_valid_until'] = array('type'=>'timestamp');

    // Set user permissions for this type of data
    if (!isset($_SESSION[$this->_table.'_permission'])) {
      $_SESSION[$this->_table.'_permission'] = $this->_getPermissions();
    }

  }
  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Deletes the instance. 
   * By  default a soft delete will be performed, which means the data is not
   * actually  deleted from the database, but merely flagged as 'inactive'.
   *
   * @param    boolean $soft    Soft delete (default) or record delete (false) 
   * @return   boolean
   * @access   public
   */
  function delete($soft=true) {

    global $SPL_CFG;    
    
    // When performing a record delete, check references to this object
    if (!$soft) {
      // Check the catalog
      include_once($SPL_CFG['dirroot'].'/class/cat.class.php');
      $cat = new cat();
      if ($cat->isUsed('ina_id',$this->_data['ina_id'])){
        return (bool)false;
      }
    }
    
    // Delete the record
    if (parent::delete($soft)) {
      // When deleting is succesfull, delete al related information as well
      return (bool) true;
    }
    else {
      return (bool) false;
    }
  }

  /**
   * Calculates a period given a date
   * 
   * @param date $date The date to calculate a period from (default=today)
   * @return array An arra containing the period id and the period start- and endingdate
   * @access public
   */
  function getPeriod($date=null) {
    
    global $SPL_CFG;
    
    // Make sure we have a timestamp
    if (empty($date)) {
      $date = time();
    }
    else {
      $date = $this->_parseDate($date);
    }
    
    // Get the dates month
    $month = date('n',$date);
    
    // Get the dates day
    $day = date('j',$date);
    
    // Find the period
    $query  = 'SELECT prd_id,prd_month_start,prd_day_start,prd_month_end,prd_day_end FROM '.$SPL_CFG['prefix'].'prd prd WHERE';
    $query .= ' prd.adm_id = "'.$_SESSION['adm_id'].'" AND is_active AND';
    $query .= ' prd_month_start <= "'.$month.'" AND';
    $query .= ' IF(prd_month_start = "'.$month.'",prd_day_start <= "'.$day.'",1) AND';
    $query .= ' (prd_month_end >= "'.$month.'" OR prd_month_end < prd_month_start) AND';
    $query .= ' IF(prd_month_end = "'.$month.'",prd_day_end >= "'.$day.'",1) AND';
    $query .= ' prd_valid_from <= "'.$date.'" AND';
    $query .= ' (prd_valid_until = "0" OR prd_valid_until >= "'.$date.'")';
    $query .= ' LIMIT 1';
    $period = db_retrieve(db_query($query));
    if (isset($period['prd_id'])){
      $period['prd_date_start']=date("Y-m-d",mktime(0,0,0,$period['prd_month_start'],$period['prd_day_start'],date('Y',$date)));
      $period['prd_date_end']=date("Y-m-d",mktime(0,0,0,$period['prd_month_end'],$period['prd_day_end'],($period['prd_month_end']<$period['prd_month_start']?date('Y',$date)+1:date('Y',$date))));
      return $period;
    }
  }

  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Converts a date into a timestamp
   * 
   * @param  $dateTime date     A date or datetime
   * @return           integer  The timestamp
   * @access Public
   * 
   * @author    Steve Powell info@scrimpnet.com
   * @copyright Copyright (c) 2004 scrimpnet.com
   */
  function _parseDate($dateTime="")
  {
    $ts = 0; // timestamp of parsed date
    switch(gettype($dateTime)) {
      case "string": 
        $ts=strtotime($dateTime); 
        break;
      case "integer":
        $ts=$dateTime;
        break;
    }
    return $ts;
  }

}

/**
 * $Log: prd.class.php,v $
 * Revision 1.7  2006/05/30 12:56:38  jebbol
 * Bug #1493089: Didn't save record when no code had been entered
 *
 */
 
?>