<?php
/**
 * sgd.class.php
 *
 * $Author: jebbol $
 * $Revision: 1.22 $
 * $date$
 * 
 * @copyright Copyright (c) 2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

// Calendar class
include_once($SPL_CFG['dirroot'].'/class/cal.class.php');


//=========================================================================
// Class definition
//=========================================================================

class sgd extends abstractData {
  
  function sgd() {
    
    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['sgd_id']                 = (int) 0;
    $this->_data['adm_id']                 = $_SESSION['adm_id'];
    $this->_data['usr_id']                 = $_SESSION['usr_id'];
    $this->_data['cat_id']                 = (int) 0;
    $this->_data['sub_id']                 = (int) 0;
    $this->_data['sug_id']                 = (int) 0;
    $this->_data['sgd_courseyear_start']   = (int) 0;
    $this->_data['prd_id_start']           = (int) 0;
    $this->_data['sgd_coursemonth_start']  = (int) 0;
    $this->_data['sgd_courseweek_start']   = (int) 0;
    $this->_data['sgd_courseday_start']    = (int) 0;
    $this->_data['sgd_courseyear_finish']  = (int) 0;
    $this->_data['prd_id_finish']          = (int) 0;
    $this->_data['sgd_coursemonth_finish'] = (int) 0;
    $this->_data['sgd_courseweek_finish']  = (int) 0;
    $this->_data['sgd_courseday_finish']   = (int) 0;
    $this->_data['sgd_calyear_start']      = (int) 0;
    $this->_data['sgd_calmonth_start']     = (int) 0;
    $this->_data['sgd_calweek_start']      = (int) 0;
    $this->_data['sgd_calday_start']       = (int) 0;
    $this->_data['sgd_caldate_start']      = (int) 0;
    $this->_data['sgd_calyear_finish']     = (int) 0;
    $this->_data['sgd_calmonth_finish']    = (int) 0;
    $this->_data['sgd_calweek_finish']     = (int) 0;
    $this->_data['sgd_calday_finish']      = (int) 0;
    $this->_data['sgd_caldate_finish']     = (int) 0;
    $this->_data['cal_id']                 = (int) 0;
    $this->_data['sgd_actualdate_start']   = (int) 0;
    $this->_data['sgd_actualdate_finish']  = (int) 0;
    $this->_data['sgd_sync_date']          = (int) 0;
    $this->_data['sts_id']                 = (int) 0;
    $this->_data['sgd_load_done']          = (int) 0;
    $this->_data['sgd_reward_done']        = (int) 0;
    $this->_data['sgd_assessment']         = '';
    $this->_data['sgd_compliment']         = (bool) false;
    $this->_data['is_mandatory']           = (bool) false;
    $this->_data['is_changed']             = (bool) false;
    $this->_data['is_alerted']             = (bool) false;
    $this->_data['is_done']                = (bool) false;
    $this->_data['is_active']              = (bool) true;
    $this->_data['create_date']            = time();
    $this->_data['create_usr_id']          = $_SESSION['usr_id'];
    $this->_data['create_src_id']          = $_SESSION['native_id'];
    $this->_data['update_date']            = time();
    $this->_data['update_usr_id']          = $_SESSION['usr_id'];
    $this->_data['update_src_id']          = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary                        = 'sgd_id';
    $this->_table                          = 'sgd';
    
    // Set teacher statistics notification
    $this->_status['notify_teacher'] = (bool) false;

    // Set user permissions for this type of data
    if (!isset($_SESSION[$this->_table.'_permission'])) {
      $_SESSION[$this->_table.'_permission'] = $this->_getPermissions();
    }
        
  }

  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Actualizes the studyguide by examining the catalog.
   * The catalog contains a general description of suggestions, the studyguide
   * contains a personalized version of it  
   *
   * @param    (integer) $tmp_usr_id    The user id 
   * @return   (boolean)
   * @public
   */
  function actualize($tmp_usr_id) {
    
    global $SPL_CFG;
    
    // Find student defaults
    include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
    $usr = new usr();
    if (!$usr->init($tmp_usr_id)) {
      // User unknown
      return (bool) false;
    }
    $tmp_stu_id = $usr->getProperty('stu_id');

    include_once($SPL_CFG['dirroot'].'/class/stu.class.php');
    $stu = new stu();
    $stu->init($tmp_stu_id);

    // Institution
    $tmp_ins_id = $stu->getProperty('ins_id');
    // Institution address
    $tmp_ina_id = $stu->getProperty('ina_id');
    // Training
    $tmp_tra_id = $stu->getProperty('tra_id');
    // Alignment
    $tmp_ali_id = $stu->getProperty('ali_id');
    // Current courseyear
    $courseyear = $stu->getProperty('stu_courseyear');
    if ($courseyear<1) $courseyear = 1;
    // The year the student entered the school
    $entryyear = $stu->getProperty('stu_entryyear');
    if ($entryyear<1) $entryyear = $courseyear;
    // Must we rebuild the entire studyguide?
    $mustRebuild = (bool) $stu->getProperty('stu_rebuild_sgd');
        
    // Find schoolyear calendar data
    // The students schoolyear starts with the start of the institution addresses year 
    include_once($SPL_CFG['dirroot'].'/class/ina.class.php');
    $ina = new ina();
    
    // Set defaults
    $schoolyear_start  = (int)date("Y");
    $schoolmonth_start = (int)8;
    $schoolday_start   = (int)1;
    $schoolyear_end    = (int)$schoolyear_start+1;
    $schoolmonth_end   = (int)7;
    $schoolday_end     = (int)31;
    
    // If available, use the data from the institution address
    if ($ina->init($tmp_ina_id)) {
      $schoolyear_start  = (int)date("Y",$ina->getProperty('ina_schoolyear_start'));
      $schoolmonth_start = (int)date("n",$ina->getProperty('ina_schoolyear_start'));
      $schoolday_start   = (int)date("j",$ina->getProperty('ina_schoolyear_start'));
      $schoolyear_end    = (int)date("Y",$ina->getProperty('ina_schoolyear_end'));
      $schoolmonth_end   = (int)date("n",$ina->getProperty('ina_schoolyear_end'));
      $schoolday_end     = (int)date("j",$ina->getProperty('ina_schoolyear_end'));
    }

    // Retrieve the students subjects
    $subjects = '';
    $s = $stu->getSubjects();
    foreach ($s as $record) {
      foreach ($record as $key => $item) {
        $subjects .= (strlen($subjects)?',':'');
        $subjects .= $key;
      }
    }
        

    // If there is no forced rebuild requested, find the date the catalog was last updated
    if (!$mustRebuild) {
      $query  = 'SELECT update_date FROM '.$SPL_CFG['prefix'].'cat cat WHERE';
      $query .= ' cat.adm_id = "'.$_SESSION['adm_id'].'" AND cat.is_active AND'; 
      $query .= ' cat.ins_id = "'.$tmp_ins_id.'"';
      $query .= ' ORDER BY update_date DESC LIMIT 1';
      $result = db_retrieve(db_query($query));
      $lastCatUpdate = (isset($result['update_date'])?$result['update_date']:0);
    
      // Find the date the studyguide was synchronized with the catalog
      $query = 'SELECT sgd_sync_date FROM '.$SPL_CFG['prefix'].'sgd sgd WHERE';
      $query .= ' sgd.usr_id = "'.$tmp_usr_id.'" AND'; 
      $query .= ' sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active'; 
      $query .= ' ORDER BY sgd_sync_date DESC LIMIT 1';
      $result = db_retrieve(db_query($query));
      $lastSgdUpdate = (isset($result['sgd_sync_date'])?$result['sgd_sync_date']:0);

      // If the guide was synchronized after changes in the catalog, there is no need to continue
      if ($lastSgdUpdate && ((int)$lastSgdUpdate>=(int)$lastCatUpdate)){
        return (bool) true;
      }
    }
    else {
      // If there is a forced rebuild, start by cleaning the obsolete records
      // (the records associated with subjects no longer applicable)
      $query  = 'DELETE FROM '.$SPL_CFG['prefix'].'sgd WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
      $query .= ' usr_id = "'.$tmp_usr_id.'" AND';
      $query .= ' sts_id = "0" AND';
      $query .= ' NOT is_done AND';
      if (strlen($subjects)){
        $query .= ' sub_id NOT IN ('.$subjects.') AND';
      }
      $query .= ' sgd_courseyear_start >= "'.$courseyear.'"';
      $result = db_query($query);
    }
    
    // Copy the catalog items to the studyguide
    $query  = 'SELECT * FROM '.$SPL_CFG['prefix'].'cat cat WHERE';
    $query .= ' cat.adm_id = "'.$_SESSION['adm_id'].'" AND';
    $query .= ' cat.is_active AND';
    $query .= ' cat.show_in_guide AND'; 
    $query .= ' cat.ins_id = "'.$tmp_ins_id.'" AND';
    $query .= ' (cat.ina_id = "'.$tmp_ina_id.'" OR cat.ina_id = "0") AND';
    $query .= ' (cat.tra_id = "'.$tmp_tra_id.'" OR cat.tra_id = "0") AND';
    $query .= ' (cat.ali_id = "'.$tmp_ali_id.'" OR cat.ali_id = "0") AND';
    $query .= ' cat.cat_courseyear_start <= "'.$courseyear.'" AND';
    $query .= ' cat.cat_courseyear_start >= "'.$entryyear.'" AND';
    if (strlen($subjects)){
      // Only the items related to the students chosen subjects
      $query .= ' (cat.sub_id IN ('.$subjects.') OR cat.sub_id = "0") AND';
    }
    else {
      // When no subjects are chosen, copy all non-subject related items
      $query .= ' cat.sub_id = "0" AND';
    }
    if (!$mustRebuild) {
      $query .= ' cat.update_date >= "'.$lastSgdUpdate.'" AND';
    }
    $query .= ' cat.cat_valid_from <= "'.time().'" AND'; 
    $query .= ' (cat.cat_valid_until = "0" OR cat.cat_valid_until >= "'.time().'")';
    $query .= ' ORDER BY cat.cat_month_start, cat.cat_week_start, cat.cat_day_start, cat.cat_sequence';
    $result = db_query($query);

    while ($row = db_fetch_array($result)) {

      // Check if the catalog item is already in the guide
      $query = 'SELECT sgd_id,is_active FROM '.$SPL_CFG['prefix'].'sgd sgd WHERE';
      $query .= ' sgd.usr_id = "'.$tmp_usr_id.'" AND';
      $query .= ' sgd.cat_id = "'.$row['cat_id'].'" LIMIT 1';
      $existing = db_retrieve(db_query($query));

      if ($existing){
        if ($existing['is_active']){
          $data = $this->_select($this->_primary,$existing['sgd_id'],'1','true');
          $this->_populate($data);
        }
      }
      else {
        $this->_data['sgd_id'] = (int) 0;
        $this->_data['cal_id'] = (int) 0;
        $this->_data['sts_id'] = (int) 0;
        $this->_data['is_active'] = (bool) true;
        $this->_data['create_date'] = time();
        $this->_data['create_usr_id'] = $_SESSION['usr_id'];
        $this->_data['create_src_id'] = $_SESSION['native_id'];
        $this->_status['is_new'] = (bool)true;
      }
      $this->_data['adm_id'] = $row['adm_id'];
      $this->_data['usr_id'] = $tmp_usr_id;
      $this->_data['cat_id'] = $row['cat_id'];
      $this->_data['sub_id'] = $row['sub_id'];
      $this->_data['sug_id'] = $row['sug_id'];
      $this->_data['sgd_courseyear_start'] = $row['cat_courseyear_start']<$courseyear?$courseyear:$row['cat_courseyear_start'];
      $this->_data['sgd_courseyear_finish'] = empty($row['cat_courseyear_end'])?$row['cat_courseyear_start']:$row['cat_courseyear_end'];
      $this->_data['prd_id_start'] = $row['prd_id_start'];
      $this->_data['prd_id_finish'] = empty($row['prd_id_end'])?$row['prd_id_start']:$row['prd_id_end'];
      $this->_data['is_mandatory'] = $row['is_mandatory'];

      // If no cat_month_start, cat_week_start or cat_day_start is known, default to the first week
      if (empty($row['cat_month_start'])&&empty($row['cat_week_start'])&&empty($row['cat_day_start'])){ 
        $row['cat_week_start'] = (int) 1; 
      }

      // If no cat_month_finish, cat_week_finish or cat_day_finish is known, default to the last week
      if (empty($row['cat_month_finish'])&&empty($row['cat_week_finish'])&&empty($row['cat_day_finish'])){ 
        $row['cat_week_finish'] = (int) 52; 
      }

      // Set relative coursedate
      $this->_data['sgd_coursemonth_start'] = $row['cat_month_start']; // 1=first month within period
      $this->_data['sgd_courseweek_start']  = $row['cat_week_start'];  // 1=first week within period
      $this->_data['sgd_courseday_start']   = $row['cat_day_start'];   // 1=first day within period
      if (empty($this->_data['sgd_courseweek_start'])){
        // Translate the relative day to a relative week
        if (!empty($this->_data['sgd_courseday_start'])){
          $this->_data['sgd_courseweek_start'] = ceil((int)$this->_data['sgd_courseday_start']/7);
        }
      }
      if (empty($this->_data['sgd_coursemonth_start'])){
        // Translate a relative week to a relative month
        if (!empty($this->_data['sgd_courseweek_start'])){
          $this->_data['sgd_coursemonth_start'] = ceil((int)$this->_data['sgd_courseweek_start']/4);
        }
      }
      $this->_data['sgd_coursemonth_finish'] = empty($row['cat_month_end'])?$row['cat_month_start']:$row['cat_month_end'];
      $this->_data['sgd_courseweek_finish']  = empty($row['cat_week_end'])?$row['cat_week_start']:$row['cat_week_end'];
      $this->_data['sgd_courseday_finish']   = empty($row['cat_day_end'])?$row['cat_day_start']:$row['cat_day_end'];
      if (empty($this->_data['sgd_courseweek_finish'])){
        // Translate the relative day to a relative week
        if (!empty($this->_data['sgd_courseday_finish'])){
          $this->_data['sgd_courseweek_finish'] = ceil((int)$this->_data['sgd_courseday_finish']/7);
        }
      }
      if (empty($this->_data['sgd_coursemonth_finish'])){
        // Translate a relative week to a relative month
        if (!empty($this->_data['sgd_courseweek_finish'])){
          $this->_data['sgd_coursemonth_finish'] = ceil((int)$this->_data['sgd_courseweek_finish']/4);
        }
      }

      // Find calendar dates related to the given periods
      // The 'firstperiod' states when the student will have to start the activity 
      $firstperiod_year_start  = $schoolyear_start;
      $firstperiod_month_start = $schoolmonth_start;
      $firstperiod_day_start   = $schoolday_start;
      $firstperiod_year_end    = $schoolyear_end;
      $firstperiod_month_end   = $schoolmonth_end;
      $firstperiod_day_end     = $schoolday_end;
      if (!empty($row['prd_id_start'])) {
        $query  = 'SELECT * FROM '.$SPL_CFG['prefix'].'prd prd WHERE';
        $query .= ' prd.prd_id = "'.$row['prd_id_start'].'" AND'; 
        $query .= ' prd.adm_id = "'.$_SESSION['adm_id'].'" LIMIT 1'; 
        $period = db_retrieve(db_query($query));
        if (isset($period['prd_month_start'])){
          $firstperiod_year_start  = (int)$period['prd_month_start']<$schoolmonth_start?$schoolyear_start+1:$schoolyear_start;
          $firstperiod_month_start = (int)$period['prd_month_start'];
          $firstperiod_day_start   = (int)($period['prd_day_start']);
          $firstperiod_year_end    = (int)$period['prd_month_end']<$schoolmonth_start?$schoolyear_start+1:$schoolyear_start;
          $firstperiod_month_end   = (int)$period['prd_month_end'];
          $firstperiod_day_end     = (int)($period['prd_day_end']);
        }
      }
      $firstperiod_date_start = mktime(0,0,0,$firstperiod_month_start,$firstperiod_day_start,$firstperiod_year_start);
      $firstperiod_date_end   = mktime(23,59,59,$firstperiod_month_end,$firstperiod_day_end,$firstperiod_year_end);

      // Find the first day of the first week in the period
      $first_weekday = strtotime("-".(strftime("%w", $firstperiod_date_start)-1)." days", $firstperiod_date_start);
      $this->_data['sgd_caldate_start'] = $first_weekday;
      if (!empty($this->_data['sgd_courseday_start'])){
        for ($i = 1; $i < $this->_data['sgd_courseday_start']; $i++) {
          $this->_data['sgd_caldate_start'] = strtotime("+1 day", $this->_data['sgd_caldate_start']);
        }
        // Set the last date's default
        $this->_data['sgd_caldate_finish'] = mktime (23, 59, 59, date("m",$this->_data['sgd_caldate_start']), date("d",$this->_data['sgd_caldate_start']), date("Y",$this->_data['sgd_caldate_start']));
      }
      else if (!empty($this->_data['sgd_courseweek_start'])){
        for ($i = 1; $i < $this->_data['sgd_courseweek_start']; $i++) {
          $this->_data['sgd_caldate_start'] = strtotime("+1 week", $this->_data['sgd_caldate_start']);
        }
        // Set the last date's default to the last day of the week
        $this->_data['sgd_caldate_finish'] = strtotime("+6 days", $this->_data['sgd_caldate_start']);
        $this->_data['sgd_caldate_finish'] = mktime (23, 59, 59, date("m",$this->_data['sgd_caldate_finish']), date("d",$this->_data['sgd_caldate_finish']), date("Y",$this->_data['sgd_caldate_finish']));
      } 
      else if (!empty($this->_data['sgd_coursemonth_start'])){
        for ($i = 1; $i < $this->_data['sgd_coursemonth_start']; $i++) {
          $this->_data['sgd_caldate_start'] = strtotime("+1 month", $this->_data['sgd_caldate_start']);
        }
        // Set the last date's default to the last day of the month
        $this->_data['sgd_caldate_finish'] = strtotime("-1 day", strtotime("+1 month", $this->_data['sgd_caldate_start']));
        $this->_data['sgd_caldate_finish'] = mktime (23, 59, 59, date("m",$this->_data['sgd_caldate_finish']), date("d",$this->_data['sgd_caldate_finish']), date("Y",$this->_data['sgd_caldate_finish']));
      } 

      $this->_data['sgd_calyear_start']  = date("Y",$this->_data['sgd_caldate_start']);
      $this->_data['sgd_calmonth_start'] = date("n",$this->_data['sgd_caldate_start']);
      $this->_data['sgd_calweek_start']  = date("W",$this->_data['sgd_caldate_start']);
      $this->_data['sgd_calday_start']   = date("j",$this->_data['sgd_caldate_start']);

      // Recalculate the course data from the calendar data
      // Remember these are the numberes relative to the start of the period, so
      // courseweek=6 means the 6th week of the period
      $cal = new cal();
      $this->_data['sgd_coursemonth_start'] = $cal->getPeriodMonthNumber($this->_data['sgd_caldate_start'],$firstperiod_date_start);
      $this->_data['sgd_courseweek_start'] = $cal->getPeriodWeekNumber($this->_data['sgd_caldate_start'],$firstperiod_date_start);
      $this->_data['sgd_courseday_start'] = $cal->getPeriodDayNumber($this->_data['sgd_caldate_start'],$firstperiod_date_start);


      // If the second period is not in the current courseyear, adjust the finish date
      if ($row['cat_courseyear_end']>$courseyear){
        $tmp = '+'.$row['cat_courseyear_end']-$courseyear.' year';
        $this->_data['sgd_caldate_finish'] = strtotime($tmp, $this->_data['sgd_caldate_finish']);
      }
      $this->_data['sgd_calyear_finish']  = date("Y",$this->_data['sgd_caldate_finish']);
      $this->_data['sgd_calmonth_finish'] = date("n",$this->_data['sgd_caldate_finish']);
      $this->_data['sgd_calweek_finish']  = date("W",$this->_data['sgd_caldate_finish']);
      $this->_data['sgd_calday_finish']   = date("j",$this->_data['sgd_caldate_finish']);

      // The 'secondperiod', if any, states when the student has to finish the activity
      if (!empty($row['prd_id_end'])&&$row['prd_id_end']!=$row['prd_id_start']){
        $query  = 'SELECT prd_month_start,prd_day_start,prd_month_end,prd_day_end FROM '.$SPL_CFG['prefix'].'prd prd WHERE';
        $query .= ' prd.prd_id = "'.$row['prd_id_end'].'" AND'; 
        $query .= ' prd.adm_id = "'.$_SESSION['adm_id'].'" LIMIT 1'; 
        $period = db_retrieve(db_query($query));
        if (isset($period['prd_month_start'])){
          $secondperiod_year_start  = (int)$period['prd_month_start']<$schoolmonth_start?$schoolyear_start+1:$schoolyear_start;
          if ($row['cat_courseyear_end']>$courseyear){
            $secondperiod_year_start = $secondperiod_year_start + ($row['cat_courseyear_end']-$courseyear);
          }
          $secondperiod_month_start = (int)$period['prd_month_start'];
          $secondperiod_day_start   = (int)($period['prd_day_start']);
          $secondperiod_year_end    = (int)$period['prd_month_end']<$schoolmonth_start?$schoolyear_start+1:$schoolyear_start;
          if ($row['cat_courseyear_end']>$courseyear){
            $secondperiod_year_end = $secondperiod_year_end + ($row['cat_courseyear_end']-$courseyear);
          }
          $secondperiod_month_end   = (int)$period['prd_month_end'];
          $secondperiod_day_end     = (int)($period['prd_day_end']);
          $secondperiod_date_start  = mktime(0,0,0,$secondperiod_month_start,$secondperiod_day_start,$secondperiod_year_start);
          $secondperiod_date_end    = mktime(23,59,59,$secondperiod_month_end,$secondperiod_day_end,$secondperiod_year_end);
        }

        // Find the last day in the period
        if (isset($secondperiod_date_end)) $this->_data['sgd_caldate_finish'] = $secondperiod_date_end;
        // Sunday
        if (date("w",$this->_data['sgd_caldate_finish'])==0){
          $this->_data['sgd_caldate_finish'] = strtotime("-2 days", $this->_data['sgd_caldate_finish']);
        }
        // Saturday
        if (date("w",$this->_data['sgd_caldate_finish'])==6){
          $this->_data['sgd_caldate_finish'] = strtotime("-1 day", $this->_data['sgd_caldate_finish']);
        }
        $this->_data['sgd_calyear_finish']  = date("Y",$this->_data['sgd_caldate_finish']);
        $this->_data['sgd_calmonth_finish'] = date("n",$this->_data['sgd_caldate_finish']);
        $this->_data['sgd_calweek_finish']  = date("W",$this->_data['sgd_caldate_finish']);
        $this->_data['sgd_calday_finish']   = date("j",$this->_data['sgd_caldate_finish']);
        
        // Recalculate the course data from the calendar data
        // Remember these are the numberes relative to the start of the period, so
        // courseweek=6 means the 6th week of the period
        $cal = new cal();
        $this->_data['sgd_coursemonth_finish'] = $cal->getPeriodMonthNumber($this->_data['sgd_caldate_finish'],$secondperiod_date_start);
        $this->_data['sgd_courseweek_finish'] = $cal->getPeriodWeekNumber($this->_data['sgd_caldate_finish'],$secondperiod_date_start);
        $this->_data['sgd_courseday_finish'] = $cal->getPeriodDayNumber($this->_data['sgd_caldate_finish'],$secondperiod_date_start);
      }


      $this->_data['update_date']   = time();
      $this->_data['update_usr_id'] = $_SESSION['usr_id'];
      $this->_data['update_src_id'] = $_SESSION['native_id'];
      $this->_data['sgd_sync_date'] = time();
      $this->_status['is_dirty']    = (bool)true;

      // Save this data
      //return $this->save();
      $this->save();
    }
    
    if ($mustRebuild) {
      // Reset the force_rebuild flag
      $stu->setProperty('stu_rebuild_sgd',false);
    }
  }

  /**
   * Clear any teacher notifications for this activity
   */
  function clearNotification() {
    
    global $SPL_CFG;
    
    $query = 'DELETE FROM '.$SPL_CFG['prefix'].'tes WHERE sgd_id = "'.$this->_data['sgd_id'].'"';
    db_query($query);
  }
  
  /**
   * Retrieves an array with information about the current item
   *
   * @param integer $tmp_usr_id The user
   * @param integer $sug_id The suggestion
   * @return array The information
   */
  function getInfo($tmp_usr_id, $sug_id) {
    
    global $SPL_CFG;
    
    $info = array();
    $info['num_activities'] = 0;
    $info['num_activities_done'] = 0;
    $info['perc_activities_done'] = 0;

    // Determine progress
    if ($sug_id) {
      // Find all activities
      $query = 'SELECT sug_id FROM '.$SPL_CFG['prefix'].'sug sug WHERE';
      $query .= ' sug.sug_id_parent = "'.$sug_id.'" AND'; 
      $query .= ' sug.adm_id = "'.$_SESSION['adm_id'].'" AND sug.is_active';
      $result = db_query($query);
      while ($row = db_fetch_array($result)) {
        // See if the student finished this activity
        $query = 'SELECT is_done FROM '.$SPL_CFG['prefix'].'sgd sgd WHERE';
        $query .= ' sgd.usr_id = "'.$tmp_usr_id.'" AND';
        $query .= ' sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active AND';
        $query .= ' sug_id = "'.$row['sug_id'].'"';
        $activity = db_retrieve(db_query($query));
        ++$info['num_activities'];
        if (isset($activity['is_done'])&&$activity['is_done']){
          ++$info['num_activities_done'];
        }
      }
      if ($info['num_activities_done']>0) {
        $info['perc_activities_done'] = $info['num_activities_done']/$info['num_activities']*100; 
      }
    }
    
    return $info;
    
  }

  /**
   * Retrieves an array with statistics
   * 
   * @param    integer $tmp_usr_id    The user id 
   * @return   array The statistics
   * @access   public
   */
  function getStatistics($tmp_usr_id) {
  
    global $SPL_CFG;
    
    $stats = array();
    
    // Get the total number of items
    $query = 'SELECT count(*) FROM '.$SPL_CFG['prefix'].'sgd sgd WHERE';
    $query .= ' sgd.usr_id = "'.$tmp_usr_id.'" AND'; 
    $query .= ' sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active'; 
    $result = db_retrieve(db_query($query));
    $stats['total_items'] = (isset($result['count(*)'])?$result['count(*)']:0);

    // Find the date the studyguide was synchronized with the catalog
    $query = 'SELECT sgd_sync_date FROM '.$SPL_CFG['prefix'].'sgd sgd WHERE';
    $query .= ' sgd.usr_id = "'.$tmp_usr_id.'" AND'; 
    $query .= ' sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active'; 
    $query .= ' ORDER BY sgd_sync_date DESC LIMIT 1';
    $result = db_retrieve(db_query($query));
    $stats['last_sync_date'] = (isset($result['sgd_sync_date'])?$result['sgd_sync_date']:0);
    
    return $stats;
  }

  /** 
   * Saves the instance data to the database
   *
   * @return boolean Succes or Failure
   * @access public
   * @uses   array   $SPL_TXT Human language translations
   */
  function save() {
    
    parent::save();
    
    if ($this->_status['notify_teacher']) {
      $this->_buildNotification();
      $this->_status['notify_teacher'] = (bool)false;
      $this->_status['notify_as_new']  = (bool)false;
    }
  }

  /** 
   * Sets the instance property with the given value
   *
   * @param  string  $property The name of the property to be set
   * @param  string  $value    The data to bet set
   * @return boolean
   * @access public
   */
  function setProperty($property, $value) {
    
    global $SPL_CFG;

    // Set load when suggestion is done
    if ($property == 'is_done'){
      if ($value){
        $query  = 'SELECT sug_load,sug_reward FROM '.$SPL_CFG['prefix'].'sug sug WHERE';
        $query .= ' sug.adm_id = "'.$_SESSION['adm_id'].'" AND';
        $query .= ' sug.sug_id = "'.$this->_data['sug_id'].'" LIMIT 1';
        $result = db_retrieve(db_query($query));
        if (isset($result['sug_load'])){
          $this->_data['sgd_load_done']   = $result['sug_load'];
          $this->_data['sgd_reward_done'] = $result['sug_reward'];
        }
        $this->_data['sgd_actualdate_finish'] = time();
        $this->_data['is_changed'] = 0;
      }
      else { 
        $this->_data['sgd_load_done']   = 0;
        $this->_data['sgd_reward_done'] = 0;
        $this->_data['sgd_actualdate_finish'] = 0;
      }
    }
    
    // Set the actual start date
    if ($property == 'sts_id'){
      
      // Set start date
      if ($this->_data['sgd_actualdate_start']==0){
        $this->_data['sgd_actualdate_start'] = time();
      }
      
      // Check if teacher statistics need updating
      if (isset($SPL_CFG['teacher_statistics']) && $SPL_CFG['teacher_statistics']=='true'){
        $this->_status['notify_teacher'] = (bool) true;
        $this->_status['notify_as_new']  = ($this->_data['sts_id']==0); 
      }
      
    }
    
    return parent::setProperty($property, $value);
  }

  //=========================================================================
  // Private methods
  //=========================================================================
  
  /**
   * Notifies the teacher statistics.
   * These statistics are used to inform a teacher of new or changed activities.
   * 
   * @private
   */
  function _buildNotification() {
    
    global $SPL_CFG;

    include_once($SPL_CFG['dirroot'].'/class/sts.class.php');
    $sts = new sts();
    if ($sts->init($this->_data['sts_id'])) {
      if ($sts->getProperty('sts_teacher_action')) {
        
        // Get user data
        include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
        $usr = new usr();
        $usr->init($this->_data['usr_id']);
        $tmp_stu_id = $usr->getProperty('stu_id');
        
        // Get student data
        include_once($SPL_CFG['dirroot'].'/class/stu.class.php');
        $stu = new stu();
        $stu->init($tmp_stu_id);
        
        // Get students teachers
        $teachers = $stu->getTeachers($this->_data['sub_id']);
           
        foreach($teachers as $teacher) {
          foreach ($teacher as $tmp_sub_id => $tmp_tea_id){
            $query  = 'INSERT INTO '.$SPL_CFG['prefix'].'tes (tes_id,adm_id,tea_id,sgd_id,is_new,update_date)';
            $query .= ' VALUES(NULL,';
            $query .= '"'.$this->_data['adm_id'].'",';
            $query .= '"'.$tmp_tea_id.'",';
            $query .= '"'.$this->_data['sgd_id'].'",';
            $query .= '"'.$this->_status['notify_as_new'].'",';
            $query .= '"'.time().'"';
            $query .= ')';
            db_query($query);
          }
        }
      }
    }
  }
}

/** 
 * $Log: sgd.class.php,v $
 * Revision 1.22  2006/08/24 17:31:59  jebbol
 * Added support for alerts
 *
 * Revision 1.21  2006/08/08 06:02:16  jebbol
 * *** empty log message ***
 *
 * Revision 1.19  2006/03/08 16:58:04  jebbol
 * *** empty log message ***
 */

?>