<?php
/**
 * stu.class.php
 *
 * $Author: jebbol $
 * $Revision: 1.23 $
 * $Date: 2006/08/24 17:32:00 $
 * 
 * @copyright Copyright (c) 2005,2006 Studiewijzer.nl, all rights reserved 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 * 
 * $Log: stu.class.php,v $
 * Revision 1.23  2006/08/24 17:32:00  jebbol
 * Added support for alerts
 *
 * Revision 1.22  2006/08/21 18:33:19  jebbol
 * Changed default skin when creating a new record
 *
 * Revision 1.21  2006/08/08 06:05:52  jebbol
 * *** empty log message ***
 *
 * Revision 1.20  2006/04/14 10:41:39  jebbol
 * Added support for mentors and parents
 *
 * Revision 1.19  2006/03/14 20:49:33  jebbol
 * Bug #1448747: setProperty did not call parent
 *
 * Revision 1.18  2006/02/06 12:36:02  jebbol
 * (Bug #1425130) When de-activating a user, also de-activate student/teacher data and their related data
 *
 * Revision 1.17  2006/01/30 19:43:01  jebbol
 * Request #1418440: Changed default setting when creating a new student
 *
 * Revision 1.16  2006/01/23 16:54:21  jebbol
 * Added removal of related spt records when deleting a student
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

//=========================================================================
// Class definition
//=========================================================================

/**
 * Student data
 * This class is an addition to the user data (usr)
 */
class stu extends abstractData {

  var $associatedUser; // When adding a record, it must be attached to a valid user.
  var $subjectQueue;   // Queue subjects for a new student row
  
  function stu() {
    
    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['stu_id']            = (int) 0;
    $this->_data['adm_id']            = $_SESSION['adm_id'];
    $this->_data['ins_id']            = (isset($_SESSION['usp_ins_id'])?$_SESSION['usp_ins_id']:(int) 0);
    $this->_data['ina_id']            = (isset($_SESSION['usp_ina_id'])?$_SESSION['usp_ina_id']:(int) 0);
    $this->_data['tra_id']            = (isset($_SESSION['usp_tra_id'])?$_SESSION['usp_tra_id']:(int) 0);
    $this->_data['ali_id']            = (isset($_SESSION['usp_ali_id'])?$_SESSION['usp_ali_id']:(int) 0);
    $this->_data['lst_id']            = (isset($_SESSION['usp_lst_id'])?$_SESSION['usp_lst_id']:(int) 0);
    $this->_data['stu_courseyear']    = (int) 1;
    $this->_data['stu_entryyear']     = (int) 1;
    $this->_data['stu_dispname']      = '';
    $this->_data['stu_sortname']      = '';
    $this->_data['stu_sisnr']         = '';
    $this->_data['stu_grp_id_schedule'] = (int) 0;
    $this->_data['stu_rebuild_sgd']   = (bool) true;
    $this->_data['is_active']         = (bool) true;
    $this->_data['create_date']       = time();
    $this->_data['create_usr_id']     = $_SESSION['usr_id'];
    $this->_data['create_src_id']     = $_SESSION['native_id'];
    $this->_data['update_date']       = time();
    $this->_data['update_usr_id']     = $_SESSION['usr_id'];
    $this->_data['update_src_id']     = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary                   = 'stu_id';
    $this->_table                     = 'stu';
    $this->_required[]                = 'ins_id';
    $this->_required[]                = 'ina_id';
    $this->_required[]                = 'stu_sortname';
    
    // Initialize the subject queue
    $this->subjectQueue = (bool) false;
    
      // Set user permissions for this type of data
    if (!isset($_SESSION[$this->_table.'_permission'])) {
      $_SESSION[$this->_table.'_permission'] = $this->_getPermissions();
    }
  
  }

  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Returns the students alignment
   *
   * @return integer The alignment id
   * @access public
   */
  function getAlignment() {
    return $this->_data['ali_id'];
  }

  /** 
   * Returns the name of the student
   *
   * @return (string) The (formatted) name
   * @public
   */
  function getName() {

    global $SPL_CFG,$SPL_TXT;
    
    // Make sure session information exists
    if (!isset($_SESSION['usp_nameformat'])) {
      $_SESSION['usp_nameformat'] = $SPL_CFG['nameformat'];
    }
    
    // When a custom student name is knwon, return it
    if (strlen($this->_data['stu_dispname'])>0) {
      return $this->_data['stu_dispname'];
    }

    // Otherwise find the user name
    if ($this->_status['is_loaded']) {

      // The user class owns the name of the student
      include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
      $usr = new usr();
      $result = $usr->initStudent($this->_data[$this->_primary]);
      if ($result) {
        return $usr->getName();
      }
      else {
        return (bool)false;
      }
    }
    else {
      $_dbg['query']  = 'getting name for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_loaded'];
      $_dbg['method'] = 'getName';
      $_dbg['error']  = $SPL_TXT['db_instance_not_loaded'];
      array_push($this->_debug, $_dbg);
      return (bool)false;
    }
    
  }

  /** 
   * Returns the childrens parents
   *
   * @param  (string) $valueType Indicates the type of value to return (long,short,code)
   * @return (array)             An array of parents (key => value)
   * @public
   */
  function getParents($valueType='long') {

    global $SPL_CFG;
    
    $values = array();
    $field = 'par_dispname';

    $query  = 'SELECT par.par_id as "key",par.'.$field.' as "value"';
    $query .= ' FROM '.$SPL_CFG['prefix'].'apl apl LEFT JOIN '.$SPL_CFG['prefix'].'par par ON par.par_id = apl.parent_id';
    $query .= ' WHERE apl.adm_id = "'.$_SESSION['adm_id'].'" AND apl.is_active';
    $query .= ' AND apl.apl_parent = "par"';
    $query .= ' AND apl.apl_child = "stu"';
    $query .= ' AND apl.child_id = "'.$this->_data['stu_id'].'"';
    $query .= ' AND apl.apl_valid_from <= "'.time().'"';
    $query .= ' AND (apl.apl_valid_until = "0" OR apl.apl_valid_until >= "'.time().'")';
    $query .= ' ORDER BY apl.apl_sequence ASC, par.par_sortname ASC';

    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      // Try to extract first name
      if (($valueType != 'long') && strstr($row['value'],' ')) {
        $short = explode(' ',$row['value']);
        $values[] = array($row['key']=>$short[0]);
      }
      // Get full name
      else {
        $values[] = array($row['key']=>$row['value']);
      }
    }
    return $values;
  }

  /**
   * Retrieve a list of students
   * 
   * @param  (string) $displayField The field to return (optional)
   * @param  (string) $sortField    The field to used for sorting (optional)
   * @param  (string) $exclude      Any student id's to exclude from the result (optional)
   * @param  (int)    $insId        The institution id (optional)
   * @param  (int)    $inaId        The institution address id (optional)
   * @param  (int)    $traId        The training id (optional)
   * @param  (int)    $aliId        The alignment id (optional)
   * @return array An array of students (key => value)
   * @public
   */
  function getStudents($displayField='stu_dispname',$sortField='stu_sortname',$exclude=null,$insId=null,$inaId=null,$traId=null,$aliId=null) {
    
    global $SPL_CFG;
    
    $values = array();
    
    $query  = 'SELECT stu_id as "key",'.$displayField.' as "value" FROM '.$SPL_CFG['prefix'].'stu stu';
    $query .= ' WHERE stu.adm_id = "'.$_SESSION['adm_id'].'" AND stu.is_active';
    if (isset($insId)) {
      $query .= ' AND ins_id = "'.$insId.'"';
    }
    if (isset($inaId)) {
      $query .= ' AND ina_id = "'.$inaId.'"';
    }
    if (isset($traId)) {
      $query .= ' AND tra_id = "'.$traId.'"';
    }
    if (isset($aliId)) {
      $query .= ' AND ali_id = "'.$aliId.'"';
    }
    if (isset($exclude)) {
      $query .= ' AND stu_id NOT IN ('.$exclude.')';;
    }
    $query .= ' ORDER BY stu.'.$sortField;

    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      $values[] = array($row['key']=>$row['value']);
    }
    return $values;
  }

  /** 
   * Returns the students subjects
   *
   * @param  (string) $valueType Indicates the type of value to return (long,short,code)
   * @return (array)             An array of subjects (key => value)
   * @public
   */
  function getSubjects($valueType='long') {

    global $SPL_CFG;
    
    $values = array();
    $field = 'sub_name';
    if ($valueType=='short'){
      $field = 'sub_short';
    }
    if ($valueType=='code'){
      $field = 'sub_code';
    }

    $query  = 'SELECT sub.sub_id as "key",sub.'.$field.' as "value"';
    $query .= ' FROM '.$SPL_CFG['prefix'].'apl apl LEFT JOIN '.$SPL_CFG['prefix'].'sub sub ON sub.sub_id = apl.child_id';
    $query .= ' WHERE apl.adm_id = "'.$_SESSION['adm_id'].'" AND apl.is_active';
    $query .= ' AND apl.apl_parent = "stu"';
    $query .= ' AND apl.parent_id = "'.$this->_data['stu_id'].'"';
    $query .= ' AND apl.apl_child = "sub"';
    $query .= ' AND apl.apl_valid_from <= "'.time().'"';
    $query .= ' AND (apl.apl_valid_until = "0" OR apl.apl_valid_until >= "'.time().'")';
    $query .= ' ORDER BY apl.apl_sequence ASC, sub.'.$field.' ASC';

    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      $values[] = array($row['key']=>$row['value']);
    }
    return $values;
  }

  /**
   * Returns the students teachers
   * 
   * @param  (integer) $subject The subject to find the teachers for (optional)
   * @return (array)            An array of sub_id, tea_id
   * @public
   */
  function getTeachers($subject=null) {
    
    global $SPL_CFG;
    
    $myTeachers = array();
    
    if (is_numeric($subject)){
      $mySubjects[] = array($subject=>''); 
    }
    else {
      $mySubjects = $this->getSubjects();
    }
    
    // Loop through the subjects and find teachers for it
    foreach ($mySubjects as $row){
      foreach ($row as $sub_id => $description) {
        $query  = 'SELECT parent_id';
        $query .= ' FROM '.$SPL_CFG['prefix'].'apl apl';
        $query .= ' WHERE apl.adm_id = "'.$_SESSION['adm_id'].'" AND apl.is_active';
        $query .= ' AND apl.apl_parent = "tea"';
        $query .= ' AND apl.apl_child = "sub"';
        $query .= ' AND apl.child_id = "'.$sub_id.'"';
        $query .= ' AND apl.apl_valid_from <= "'.time().'"';
        $query .= ' AND (apl.apl_valid_until = "0" OR apl.apl_valid_until >= "'.time().'")';
        $result = db_query($query);

        while ($apl = db_fetch_array($result)) {
          $myTeachers[] = array($sub_id => (int)$apl['parent_id']);
        }
        
      }
    }
 
    return $myTeachers;
  }

  /** 
   * Returns the students training
   *
   * @return (integer) The training id
   * @public
   */
  function getTraining() {
    return $this->_data['tra_id'];
  }

  /** 
   * Saves the instance data to the database
   *
   * @return (boolean)
   * @public
   */
  function save() {
    
    global $SPL_CFG;
    
    // When creating a new record, attach it to the user
    if ($this->_status['is_new']) {
      if (empty($this->associatedUser)){
        $_dbg['query']  = 'examining the value of associatedUser id '.$this->_data[$this->_primary];
        $_dbg['result'] = 'empty';
        $_dbg['method'] = 'save';
        $_dbg['error']  = 'No user set. Call setUser prior to saving.';
        array_push($this->_debug, $_dbg);
        return (bool)false;
      }
      include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
      $usr = new usr();
      if (!$usr->init($this->associatedUser)){
        $_dbg['query']  = 'initializing user id '.$this->associatedUser;
        $_dbg['result'] = 'failed';
        $_dbg['method'] = 'save';
        $_dbg['error']  = 'Could not initialize user object.';
        array_push($this->_debug, $_dbg);
        return (bool)false;
      }
      parent::save();
      $usr->setProperty('stu_id',$this->_data['stu_id']);
      $usr->save();
      $usr->setPreference('usp_startpane','menu_sgd.php');
      $usr->save();
    }
    else {
      parent::save();
    }

    // Check for subjects in the queue
    if ($this->subjectQueue) {
      $this->setSubjects($this->subjectQueue);
      $this->subjectQueue = (bool) false;
    }

  }
  
  /** 
   * Sets the instance property with the given value
   *
   * @param  (string)  $property The name of the property to be set
   * @param  (string)  $value    The data to bet set
   * @return (boolean) Succes or Failure
   * @public
   */
  function setProperty($property, $value) {
    
    global $SPL_CFG;
    
    parent::setProperty($property, $value);

    // When de-activating the student, also de-activate the related records
    if ($property == 'is_active'){
      $query  = 'UPDATE '.$SPL_CFG['prefix'].'spt';
      $query .= ' SET is_active = "'.$value.'" WHERE';
      $query .= ' stu_id = "'.$this->_data[$this->_primary].'"';
      $result = db_query($query);
    }
    
  }
  
  /** 
   * Sets the students subjects
   *
   * @param  (array)   $subjects An array of subjects to set for the student
   * @return (boolean)           Success or Failure
   * @public
   */
  function setSubjects($subjects=NULL) {

    global $SPL_CFG;
    
    if (isset($subjects)){
      
      // First delete old subjects first
      $this->unsetSubjects();
      
      // Than insert the new ones
      $seq = (int)0;
      foreach ($subjects as $id){
        // Find subject code
        $query  = 'SELECT sub_code FROM '.$SPL_CFG['prefix'].'sub sub ';
        $query .= 'WHERE sub.adm_id = "'.$_SESSION['adm_id'].'" ';
        $query .= 'AND sub.sub_id = "'.$id.'" LIMIT 1';
        $result = db_retrieve(db_query($query));
        $code = (isset($result['sub_code'])?$result['sub_code']:'');
        
        $seq = $seq + 100;
        $query  = 'INSERT INTO '.$SPL_CFG['prefix'].'apl ';
        $query .= '(apl_id,adm_id,apl_parent,parent_id,apl_child,child_id,apl_sequence,apl_code,apl_valid_from,apl_valid_until,is_active) ';
        $query .= 'VALUES (NULL,';
        $query .= '"'.$_SESSION['adm_id'].'",';
        $query .= '"stu",';
        $query .= '"'.$this->_data['stu_id'].'",';
        $query .= '"sub",';
        $query .= '"'.$id.'",';
        $query .= '"'.$seq.'",';
        $query .= '"'.$code.'",';
        $query .= '"0",';
        $query .= '"0",';
        $query .= '"1")';
        $result = db_query($query);
      }
    }
  }

  /**
   * Queues the selection of subjects until a new student row has been created
   * 
   * @param (string) $subjects A list of subjects
   * @public
   */
  function queueSubjects($subjects) {
    $this->subjectQueue = $subjects;
  }

  /** 
   * Sets the userid of the associated user
   *
   * @param (integer) $userid A valid user id
   * @public
   */
  function setUser($userid) {
    $this->associatedUser = $userid;
  }
  
  /** 
   * Unsets the students subjects
   *
   * @return (boolean) Success or Failure
   * @public
   */
  function unsetSubjects() {

    global $SPL_CFG;

    if (!$this->_status['is_new']) {
      $query  = 'DELETE FROM '.$SPL_CFG['prefix'].'apl';
      $query .= ' WHERE adm_id = "'.$_SESSION['adm_id'].'"';
      $query .= ' AND apl_parent = "stu"';
      $query .= ' AND parent_id = "'.$this->_data['stu_id'].'"';
      $query .= ' AND apl_child = "sub"';
      $result = db_query($query);
    }
    return (bool) true;

  }
  
  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Deletes a row from the database.
   * 
   * @return (boolean) Success or Failure
   * @protected
   */
  function _delete() {
    
    global $SPL_CFG;
    
      // Remove student id from user data
      include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
      $usr = new usr();
      $result = $usr->initStudent($this->_data[$this->_primary]);
      if ($result) {
        $usr->setProperty('stu_id',0);
        $usr->save();
      }
      
      // Remove student from 'students per teacher'
      $query = 'DELETE FROM '.$SPL_CFG['prefix'].'spt WHERE stu_id = "'.$this->_data[$this->_primary].'"';
      db_query($query);

      return (parent::_delete());
  }
  
  /** 
   * Inserts a new row in the database.
   * 
   * This function will be called by {@link save()} when adding a new instance to the database.
   * 
   * @return (boolean) Success or Failure
   * @protected
   */
  function _insert() {
    
    global $SPL_CFG;
    
    if (parent::_insert()) {
      // When insert is succesfull, set default authorization
      include_once($SPL_CFG['dirroot'].'/class/usa.class.php');
      
      // Calendar
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','cal');      
      $usa->setProperty('usa_permission','11');   
      $usa->save();   

      // Catalog
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','cat');      
      $usa->setProperty('usa_permission','1');      
      $usa->save();   

      // Studyguide
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','sgd');      
      $usa->setProperty('usa_permission','11');   
      $usa->save();   

      // Suggestions
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','sug');      
      $usa->setProperty('usa_permission','1');   
      $usa->save();   

      // Period
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','prd');      
      $usa->setProperty('usa_permission','1');   
      $usa->save();   

      // Statusses
      $usa = new usa();
      $usa->setProperty('usr_id',$this->associatedUser);      
      $usa->setProperty('usa_key','sts');      
      $usa->setProperty('usa_permission','1');   
      $usa->save();
      
      // Set preferences
      include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
      $usr = new usr();
      if ($usr->init($this->associatedUser)) {
        $usr->setPreference('usp_calmode','calendardate');
        $usr->setPreference('usp_startpane','menu_sgd.php');
        $usr->setPreference('usp_menumode','normal');
        $usr->setPreference('usp_numrows','16');
      }
    }

  }  

}

?>