<?php
/**
 * Class containing synchronization logic.
 *
 * @author    Jebbol
 * @version   1.0
 * @revision  2005-03-09
 * @package   spl
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */
 
//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

// Human language translations
if (!isset($_SESSION['usp_lang'])) {$_SESSION['usp_lang'] = 'en';}
require_once ($SPL_CFG['dirroot'].'/lang/'.$_SESSION['usp_lang'].'/syn.lang.php');

//=========================================================================
// Class definition
//=========================================================================

/** 
 * Synchronize external data
 */
class syn extends abstractData{
  
  function syn() {
    
    // Create an empty instance
    $this->_data['syn_id'] = 0;
    $this->_data['syn_table'] = '';
    $this->_data['syn_table_id'] = 0;
    $this->_data['syn_src_id'] = $_SESSION['src_id'];
    $this->_data['syn_src_key'] = '';
    $this->_data['syn_date'] = '';
    $this->_data['syn_dirty'] = (bool) false;
	  
  }
  
  //=========================================================================
  // Public methods
  //=========================================================================
  
  /** 
   * Updates data in the target application with the data from the source
   * application.
   * 
   * In order for data to be set correctly, the array containing the data must
   * be self-describing. That is, the array contains metadata as well as normal
   * data. 
   * 
   * All metadata information starts with 'xmeta_'. 
   * Required metadata is:
   * 
   * xmeta_source: 
   * the name of the external source as registered by the target
   * 
   * xmeta_sourcetype: 
   * indicating the type of data as known to the external source

   *    * @param  (array)   $data     An array of data to be set
   * @param  (integer) $sourceID (optional) The id of the source.
   * @param  (integer) $targetID (optional) The id of the target.
   * @return (boolean)           <code>true</code> when succesfull,
   *                             otherwise <code>false</code>
   * @public
   */
  function setData($data,$sourceID=NULL,$targetID=NULL) {
    
    global $SPL_CFG,$SPL_TXT;
    
    // Debug information
    $_dbg = array();
    
    // Check if the data submitted is an array
    if (!is_array($data)){
      $_dbg['query'] = 'is_array';
      $_dbg['result'] = 'false';
      $_dbg['method'] = 'setData';
      $_dbg['error'] = $SPL_TXT['syn_no_array'];
      array_push($this->_debug, $_dbg);         
      return (bool) false;
    }

    // Make sure we know the ID of the native application
    if (!isset($SPL_CFG['nativeID'])){
      $query = "SELECT src_id FROM ".$SPL_CFG['prefix']."src src WHERE src.src_is_native = '1' LIMIT 1";
      $result = db_retrieve(db_query($query));
      if (!$result) {
        $_dbg['query'] = $query;
        $_dbg['result'] = $result;
        $_dbg['method'] = 'setData';
        $_dbg['error'] = $SPL_TXT['syn_no_native_source'];
        array_push($this->_debug, $_dbg);         
        return (bool) false;
      }
      else {
        $SPL_CFG['nativeID'] = $result['src_id'];
      }
    }

    // Check if the sourceID exists
    if ($sourceID != NULL){
      $query = "SELECT * FROM ".$SPL_CFG['prefix']."src src WHERE src.src_id = '".$sourceID."' LIMIT 1";
      $result = db_retrieve(db_query($query));
      if (!$result) {
        $_dbg['query'] = $query;
        $_dbg['result'] = $result;
        $_dbg['method'] = 'setData';
        $_dbg['error'] = $SPL_TXT['syn_invalid_sourceID'];
        array_push($this->_debug, $_dbg);         
        return (bool) false;
      }
    }
    else {
      $sourceID = $SPL_CFG['nativeID'];
    }
    
    // Check if the targetID exists
    if ($targetID != NULL){
      $query = "SELECT * FROM ".$SPL_CFG['prefix']."src src WHERE src.src_id = '".$targetID."' LIMIT 1";
      $result = db_retrieve(db_query($query));

      if (!$result) {
        $_dbg['query'] = $query;
        $_dbg['result'] = $result;
        $_dbg['method'] = 'setData';
        $_dbg['error'] = $SPL_TXT['syn_invalid_targetID'];
        array_push($this->_debug, $_dbg);         
        return (bool) false;
      }
    }
    else {
      $targetID = $SPL_CFG['nativeID'];
    }
    
    
    // Check if the array contains a reference to a table
    if (!isset($data['type'])){
      $_dbg['query'] = 'type';
      $_dbg['result'] = 'false';
      $_dbg['method'] = 'setData';
      $_dbg['error'] = $SPL_TXT['syn_no_type'];
      array_push($this->_debug, $_dbg);         
      return (bool) false;
    }
    else {
    	$type = $data['type'];
    	unset($data['type']);
    }

    // Find out if its a valid type
    $result = db_is_table($type);
    if (!$result) {
      $_dbg['query'] = $query;
      $_dbg['result'] = $result;
      $_dbg['method'] = 'setData';
      $_dbg['error'] = $SPL_TXT['syn_unknown_type']." (".$type.")";
      array_push($this->_debug, $_dbg);         
      return (bool) false;
    }
    
    // An external source sets data in the native system
    if ($targetID == $SPL_CFG['nativeID']) {
    	
      // Check if the array contains a reference to a table
      if (!isset($data['mapping'])){
        $_dbg['query'] = 'mapping';
        $_dbg['result'] = 'false'; 
        $_dbg['method'] = 'setData';
        $_dbg['error'] = $SPL_TXT['syn_no_mapping'];
        array_push($this->_debug, $_dbg);         
        return (bool) false;
      }
      else {
        $mapping = $data['mapping'];
        unset($data['mapping']);
      }
    	
      $targetkey=(isset($data['targetkey'])?$data['targetkey']:$type."_id");
      $sourcekey=(isset($data['sourcekey'])?$data['sourcekey']:$targetkey);
      
      include_once($SPL_CFG['dirroot'].'/class/'.$type.'.class.php');
      foreach ($data as $record) {
        $query  = "SELECT syn_table, syn_table_id FROM ".$SPL_CFG['prefix']."syn ";
	    $query .= "WHERE syn_src_id = ".$sourceID." AND syn_src_key = '".$record[$sourcekey]."' LIMIT 1";
	    $result = db_retrieve(db_query($query));
	    // external record not available in target. create
        $targetobject = new $type();
	    if (!$result) {
	    	
	    	foreach ($mapping as $target => $source) {
               $targetobject->setProperty($target,$record[$source]);
	    	}
            $targetobject->save();
            
	        $query  = "INSERT INTO ".$SPL_CFG['prefix']."syn ";
		    $query .= "(syn_id, syn_table, syn_table_id, syn_src_id, syn_src_key, syn_date, syn_dirty)";
		    $query .= " VALUES (NULL,'".$type."',".$targetobject->getProperty($targetkey).",".$sourceID.",'".$record[$sourcekey]."',".time().",1)";
		    $result = db_query($query);
		    
	    }
	    // external record already available. update
	    else {
	    	$targetobject->init($result['syn_table_id']);
	    	foreach ($mapping as $target => $source) {
	    	   $targetobject->setProperty($target,$record[$source]);
	    	}
            $targetobject->save();
	    }
      }  
      
    }
    
    return (bool) true;

  }

  //=========================================================================
  // Private methods
  //=========================================================================

}

?>
