<?php
/**
 * Abstract teacher.
 *
 * $Author: jebbol $
 * $Revision: 1.16 $
 * $Date: 2006/08/21 18:33:19 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Defaults
include_once($SPL_CFG['dirroot'].'/inc/defaults.inc.php');

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

// Methodology
include_once($SPL_CFG['dirroot'].'/class/methodology.class.php');

//=========================================================================
// Class definition
//=========================================================================

/**
 * Abstract Teacher
 */
class abstractTea extends abstractData {

  var $associatedUser; // When adding a record, it must be attached to a valid user.
  
  function abstractTea() {
    
    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Core data
    $this->_data['tea_id'] = (int) 0;
    $this->_data['adm_id'] = $_SESSION['adm_id'];
    $this->_data['tea_dispname'] = '';
    $this->_data['tea_sortname'] = '';
    $this->_data['tea_schedulename'] = '';
    $this->_data['tea_rebuild_spt'] = (bool) true;
    $this->_data['is_active'] = (bool) true;
    $this->_data['create_date'] = time();
    $this->_data['create_usr_id'] = $_SESSION['usr_id'];
    $this->_data['create_src_id'] = $_SESSION['native_id'];
    $this->_data['update_date'] = time();
    $this->_data['update_usr_id'] = $_SESSION['usr_id'];
    $this->_data['update_src_id'] = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary = 'tea_id';
    $this->_table   = 'tea';
    
    // Set user permissions for this type of data
    if (!isset($_SESSION[$this->_table.'_permission'])) {
      $_SESSION[$this->_table.'_permission'] = $this->_getPermissions();
    }
    
  }

  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Returns the teachers institutions.
   * If no teacher institutions can be found, all institutions will be returned
   *
   * @param  (string) $valueType Indicates the type of value to return (long,short,code)
   * @return (array)             A list of institutions (key,value)
   * @public
   */
  function getInstitutions($valueType='long') {

    global $SPL_CFG;
    
    $values = array();
    $field = 'ins_name';
    if ($valueType=='short'){
      $field = 'ins_short';
    }
    if ($valueType=='code'){
      $field = 'ins_code';
    }

    // Find institutions
    $query  = 'SELECT ins.ins_id as "key",ins.'.$field.' as "value"';
    $query .= ' FROM '.$SPL_CFG['prefix'].'apl apl LEFT JOIN '.$SPL_CFG['prefix'].'ins ins ON ins.ins_id = apl.child_id';
    $query .= ' WHERE apl.adm_id = "'.$_SESSION['adm_id'].'" AND apl.is_active';
    $query .= ' AND apl.apl_parent = "tea"';
    $query .= ' AND apl.parent_id = "'.$this->_data['tea_id'].'"';
    $query .= ' AND apl.apl_child = "ins"';
    $query .= ' AND apl.apl_valid_from <= "'.time().'"';
    $query .= ' AND (apl.apl_valid_until = "0" OR apl.apl_valid_until >= "'.time().'")';
    $query .= ' ORDER BY apl.apl_sequence ASC';
    $result = db_query($query);
    
    // If no institutions have been found, return all institutions
    if (db_num_rows($result)==0){
      include_once($SPL_CFG['dirroot'].'/class/ins.class.php');
      $ins = new ins();
      $ins->_name = $field;
      return $ins->getKeyValues();
    }
    
    // Construct the valuelist
    while ($row = db_fetch_array($result)) {
      $values[] = array($row['key']=>$row['value']);
    }
    return $values;
  }

  /** 
   * Returns the teachers institutions addresses.
   * If no teachers addresses can be found, all addresses will be returned
   *
   * @param  (string)  $valueType Indicates the type of value to return (long,short,code)
   * @param  (integer) $insID     Limits the addresses to those associated with this institution
   * @return (array)              A list of addresses (key,value)
   * @public
   */
  function getInstitutionAddresses($valueType='long',$insID=0) {

    global $SPL_CFG;
    
    $values = array();
    $field = 'ina_name';
    if ($valueType=='short'){$field = 'ina_short';}
    if ($valueType=='code'){$field = 'ina_code';}

    // Find preferred institution addresses
    $query  = 'SELECT ina.ina_id as "key",ina.'.$field.' as "value"';
    $query .= ' FROM '.$SPL_CFG['prefix'].'apl apl LEFT JOIN '.$SPL_CFG['prefix'].'ina ina ON ina.ina_id = apl.child_id';
    $query .= ' WHERE apl.adm_id = "'.$_SESSION['adm_id'].'" AND apl.is_active';
    $query .= ' AND apl.apl_parent = "tea"';
    $query .= ' AND apl.parent_id = "'.$this->_data['tea_id'].'"';
    $query .= ' AND apl.apl_child = "ina"';
    $query .= ' AND apl.apl_valid_from <= "'.time().'"';
    $query .= ' AND (apl.apl_valid_until = "0" OR apl.apl_valid_until >= "'.time().'")';
    $query .= ' ORDER BY apl.apl_sequence ASC';
    $result = db_query($query);
    
    // If no preferred institution addresses have been found, return all addresses
    if (db_num_rows($result)==0){
      include_once($SPL_CFG['dirroot'].'/class/ina.class.php');
      $ina = new ina();
      $ina->_name = $field;
      return $ina->getKeyValues('ins',($insID>0?$insID:false),false,'ASC');
    }
    
    // Construct the valuelist
    while ($row = db_fetch_array($result)) {
      $values[] = array($row['key']=>$row['value']);
    }
    return $values;
  }

  /** 
   * Returns the name of the student
   *
   * @return (string) The (formatted) name
   * @public
   */
  function getName() {

    global $SPL_CFG,$SPL_TXT;
    
    // Make sure session information exists
    if (!isset($_SESSION['usp_nameformat'])) {
      $_SESSION['usp_nameformat'] = $SPL_CFG['nameformat'];
    }

    // This will only work on a correctly instantiated object
    if ($this->_status['is_loaded']) {

      // The user class owns the name of the student
      include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
      $usr = new usr();
      $result = $usr->initTeacher($this->_data[$this->_primary]);
      if ($result) {
        return $usr->getName();
      }
      else {
        return (bool)false;
      }
    }
    else {
      $_dbg['query'] = 'getting name for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_loaded'];
      $_dbg['method'] = 'getName';
      $_dbg['error'] = $SPL_TXT['db_instance_not_loaded'];
      array_push($this->_debug, $_dbg);
      return (bool)false;
    }
    
  }

  /** 
   * Saves the instance data to the database
   *
   * @return (boolean) Succes or Failure
   * @public
   */
  function save() {
    
    global $SPL_CFG;
    
    if ($this->_status['is_new']) {
      // When creating a new record, attach it to the user
      if (empty($this->associatedUser)){
        $_dbg['query']  = 'examining the value of associatedUser id '.$this->_data[$this->_primary];
        $_dbg['result'] = 'empty';
        $_dbg['method'] = 'save';
        $_dbg['error']  = 'No user set. Call setUser prior to saving.';
        array_push($this->_debug, $_dbg);
        return (bool)false;
      }
      include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
      $usr = new usr();
      if (!$usr->init($this->associatedUser)){
        $_dbg['query']  = 'initializing user id '.$this->associatedUser;
        $_dbg['result'] = 'failed';
        $_dbg['method'] = 'save';
        $_dbg['error']  = 'Could not initialize user object.';
        array_push($this->_debug, $_dbg);
        return (bool)false;
      }
      parent::save();
      $usr->setProperty('tea_id',$this->_data['tea_id']);
      $usr->save();
      $usr->setPreference('usp_startpane','menu_dash.php');
      $usr->setPreference('usp_numrows','15');
      $usr->setPreference('usp_menumode','normal');
      $usr->save();
    }
    else {
      parent::save();
    }
  }
  
  /** 
   * Sets the userid of the associated user
   *
   * @param  (integer) $userid A valid user id
   * @public
   */
  function setUser($userid) {
    $this->associatedUser = $userid;
  }
   
  //=========================================================================
  // Private methods
  //=========================================================================

}

/**
 * $Log: tea.abstract.php,v $
 * Revision 1.16  2006/08/21 18:33:19  jebbol
 * Changed default skin when creating a new record
 *
 * Revision 1.15  2006/08/08 06:02:16  jebbol
 * *** empty log message ***
 *
 * Revision 1.13  2006/04/16 18:58:54  jebbol
 * Now also setting preferences 'numrows' and 'menumode' (Feature #1400359)
 *
 * Revision 1.12  2006/04/16 18:34:46  jebbol
 * Now setting the startpane of a new teacher to 'dashboard' (Feature #1454313)
 *
 */

?>