<?php
/**
 * User authorization class.
 *
 * @author    Jebbol
 * @version   1.0
 * @revision  2005-03-09
 * @package   spl
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

//=========================================================================
// Class definition
//=========================================================================

/**
 * User authorization
 */
class usa extends abstractData {
  
  function usa() {

    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['usa_id'] = (int) 0;
    $this->_data['adm_id'] = $_SESSION['adm_id'];
    $this->_data['usr_id'] = (int) 0;
    $this->_data['usa_key'] = '';
    $this->_data['usa_permission'] = (int) 0;
    
    // Register the required class properties
    $this->_primary = 'usa_id';
    $this->_table   = 'usa';
    $this->_name    = 'usa_key';
    
    // Register properties that must have a value
    $this->_required[] = 'usr_id';
    $this->_required[] = 'usa_key';
  }

  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Deletes the instance. 
   * By  default a soft delete will be performed, which means the data is not
   * actually  deleted from the database, but merely flagged as 'inactive'.
   *
   * @param  (boolean) $soft    Soft delete (true) or record delete (default) 
   * @return (boolean)
   * @public
   */
  function delete($soft=false) {
    
    // Delete the record. There is no soft delete with this kind of data
    if (parent::delete(false)) {
      // When deleting is succesfull, delete al related information as well
      return (bool) true;
    }
    else {
      return (bool) false;
    }
  }

  /** 
   * Returns the property of an instance
   *
   * @param  (string) $property The name of the propert to be returned
   * @return (string)           The value of the property
   * @public
   */
  function getProperty($property) {
    
    // Only known properties can be returned
    if (array_key_exists($property, $this->_data)) {
      if ($property=='usa_permission'){
        return ($this->_explodePermissions($this->_data['usa_permission']));
      }
      else {
        return $this->_data[$property];
      }
    } 
    // If it's not a known property, return an empty string
    else {
      return '';
    }
  }

  /**
   * Sets a property to a given value
   * 
   * @param  (string) $property The property to be set
   * @param  (string) $value    The value to be set
   * @return (boolean)          Succes or Failure 
   * @public
   */
  function setProperty($property, $value) {
    
    // Only known properties can be set
    if (array_key_exists($property, $this->_data)) {

      // Set the property
      if (!get_magic_quotes_gpc()) {
        $this->_data[$property] = addslashes($value);
      } 
      else {
        $this->_data[$property] = $value;
      }
      $this->_status['is_dirty'] = (bool) true;
      return (bool) true;
    } 
    else {
      $tmp_permissions = $this->_explodePermissions($this->_data['usa_permission']);
      $tmp_permissions[$property] = $value;
      // Set read permission if any other permission is true
      if ($tmp_permissions['write']||$tmp_permissions['change']||$tmp_permissions['delete']||$tmp_permissions['admin']){
        $tmp_permissions['read']=true;
      }
      $this->_data['usa_permission'] = $this->_implodePermissions($tmp_permissions);
    }
  }

  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Inserts a new row in the database.
   * 
   * @return (boolean) Succes or Failure
   * @private
   */
  function _insert() {
    
    global $SPL_CFG,$SPL_TXT;
    
    // Instance must be new. If not, update should be used
    if (!$this->_status['is_new']) {
      $_dbg['query'] = 'examining the value of is_new for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_new'];
      $_dbg['method'] = '_insert';
      $_dbg['error'] = $SPL_TXT['db_instance_not_new'];
      array_push($this->_debug, $_dbg);     
      return (bool)false;    
    }
    
    // Instance must dirty, otherwise it is an empty instance
    if (!$this->_status['is_dirty']) {
      $_dbg['query'] = 'examining the value of is_dirty for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_status['is_dirty'];
      $_dbg['method'] = '_insert';
      $_dbg['error'] = $SPL_TXT['db_instance_not_dirty'];
      array_push($this->_debug, $_dbg);         
      return (bool)false;    
    }

    // usr_id must be known
    if (empty($this->_data['usr_id'])) {
      $_dbg['query'] = 'examining the value of usr_id for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_data['usr_id'];
      $_dbg['method'] = '_insert';
      $_dbg['error'] = $SPL_TXT['db_column_empty'];
      array_push($this->_debug, $_dbg);         
      return (bool)false;    
    }

    // usa_key must be known
    if (empty($this->_data['usa_key'])) {
      $_dbg['query'] = 'examining the value of usa_key for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $this->_data['usa_key'];
      $_dbg['method'] = '_insert';
      $_dbg['error'] = $SPL_TXT['db_column_empty'];
      array_push($this->_debug, $_dbg);         
      return (bool)false;    
    }

    // Cannot create duplicate authorization 
    $query  = "SELECT ".$this->_primary." FROM ".$SPL_CFG['prefix'].$this->_table." WHERE ";
    $query .= "usr_id  = '".$this->_data['usr_id']."' AND ";
    $query .= "adm_id  = '".$this->_data['adm_id']."' AND ";
    $query .= "usa_key = '".$this->_data['usa_key']."' LIMIT 1";
    $result = db_retrieve(db_query($query));
    if (isset($result[$this->_primary])){
      $_dbg['query'] = 'examining duplicate entry for id '.$this->_data[$this->_primary];
      $_dbg['result'] = $result[$this->_primary];
      $_dbg['method'] = '_insert';
      $_dbg['error'] = $SPL_TXT['db_column_not_unique'];
      array_push($this->_debug, $_dbg);         
      return (bool)false;    
    }

    // Write the record
    $query  = "INSERT INTO ".$SPL_CFG['prefix'].$this->_table." (";
    $query .= "usa_id,usr_id,adm_id,usa_key,usa_permission";
    $query .= ") VALUES (NULL,'";
    $query .= $this->_data['usr_id']."','";
    $query .= $this->_data['adm_id']."','";
    $query .= $this->_data['usa_key']."','";
    $query .= $this->_data['usa_permission']."')";

    $result = db_query($query);
    if ($result) {
      $this->_data[$this->_primary] = db_get_generated_key();
      $this->_status['is_new'] = (bool) false;
      $this->_status['is_dirty'] = (bool) false;
      $this->_status['is_loaded'] = (bool) true;
      return (bool) true;
    }
    else {
      $_dbg['query'] = $query;
      $_dbg['result'] = $result;
      $_dbg['method'] = '_insert';
      $_dbg['error'] = $SPL_TXT['db_cannot_insert'];
      array_push($this->_debug, $_dbg);         
      return (bool) false;
    }

  }

  /** 
   * Selects rows from the database.
   * 
   * @param  (string)  $column  The column to select on
   * @param  (string)  $value   The value to match
   * @param  (integer) $limit   (optional) The maximum number of records
   * @param  (string)  $active  Only active ('true'), only inactive ('false') or both (anything else)
   * @private
   */
  function _select($column,$value,$limit=0,$active='true') {
    
    global $SPL_CFG,$SPL_TXT;

    $query = "SELECT * FROM ".$SPL_CFG['prefix'].$this->_table." WHERE ".$column." = '".$value."'".($limit>0?" LIMIT ".$limit:"");
    $result = db_retrieve(db_query($query));
    if (!$result) {
      $_dbg['query'] = $query;
      $_dbg['result'] = $result;
      $_dbg['method'] = '_select in usa.class.php';
      $_dbg['error'] = $SPL_TXT['db_cannot_select'];
      array_push($this->_debug, $_dbg);         
      return (bool) false;
    }
    else {
      return $result;
    }      
  }

  /** 
   * Transforms an array of permissions into a bitmask
   *
   * @param  (array)   $p  The permission array
   * @return (integer)     The bitmask
   * @private
   */
  function _implodePermissions($p) {
    $perms = new permissions();
    $perms->permissions['read']=isset($p['read'])?$p['read']:(bool)false;
    $perms->permissions['write']=isset($p['write'])?$p['write']:(bool)false;
    $perms->permissions['change']=isset($p['change'])?$p['change']:(bool)false;
    $perms->permissions['delete']=isset($p['delete'])?$p['delete']:(bool)false;
    $perms->permissions['admin']=isset($p['admin'])?$p['admin']:(bool)false;
    return ($perms->toBitmask());
  } 

  /** 
   * Updates an existing row in the database.
   *
   * @return (boolean) Succes or Failure
   * @private 
   */
  function _update() {

    global $SPL_CFG,$SPL_TXT;

    // To keep track of the current position in the array
    $currentPos = 0;
    // The number of items in the array
    $maxPos = count($this->_data);
    
    // Assemble the update statement
    $query = "UPDATE ".$SPL_CFG['prefix'].$this->_table." SET ";
    foreach ($this->_data as $property => $value) {
      $currentPos++;
      if ($property != $this->_primary){
        $query .= $property." = '".$value."'".($currentPos<$maxPos?",":"");
      }
    }
    $query .= " WHERE ".$this->_primary." = '".$this->_data[$this->_primary]."'"; 
    $result = db_query($query);
    if ($result) {
      $this->_status['is_new'] = (bool) false;
      $this->_status['is_dirty'] = (bool) false;
      $this->_status['is_loaded'] = (bool) true;
      return (bool) true;
    }
    else {
      $_dbg['query'] = $query;
      $_dbg['result'] = $result;
      $_dbg['method'] = '_update';
      $_dbg['error'] = $SPL_TXT['db_cannot_update'];
      array_push($this->_debug, $_dbg);         
      return (bool) false;
    }
  }

}

?>