<?php
/**
 * usr.class.php
 *
 * $Author: jebbol $
 * $Revision: 1.32 $
 * $Date: 2006/08/21 18:33:19 $
 * 
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Configuration
include_once($SPL_CFG['dirroot'].'/inc/config.php');

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

// Human language translations
if (!isset($_SESSION['usp_lang'])) {$_SESSION['usp_lang'] = 'en';}
include_once($SPL_CFG['dirroot'].'/lang/'.$_SESSION['usp_lang'].'/usr.lang.php');

// Encryption
include_once($SPL_CFG['dirroot'].'/class/rc4crypt.class.php');

//=========================================================================
// Class definition
//=========================================================================

/**
 * User data
 */
class usr extends abstractData {
  
  function usr() {

    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['usr_id'] = (int) 0;
    $this->_data['usr_login'] = '';
    $this->_data['usr_psswd'] = '';
    $this->_data['usr_psswd_exp'] = (int) 0;   // never
    $this->_data['usr_psswd_retry'] = (int) 0; // unlimited
    $this->_data['usr_email'] = '';
    $this->_data['usr_msn'] = '';
    $this->_data['usr_skype'] = '';
    $this->_data['usr_phone'] = '';
    $this->_data['usr_name'] = '';
    $this->_data['per_id'] = (int) 0;
    $this->_data['stu_id'] = (int) 0;
    $this->_data['tea_id'] = (int) 0;
    $this->_data['men_id'] = (int) 0;
    $this->_data['par_id'] = (int) 0;
    $this->_data['usr_login_count'] = (int) 0;
    $this->_data['usr_login_date_first'] = (int) 0;
    $this->_data['usr_login_date_prev'] = (int) 0;
    $this->_data['usr_login_date_last'] = (int) 0;
    $this->_data['usr_allow_date_from'] = (int) 0;
    $this->_data['usr_allow_date_until'] = (int) 0;
    $this->_data['usr_allow_time_from'] = '00:00:00';
    $this->_data['usr_allow_time_until'] = '23:59:59';
    $this->_data['usr_log'] = (bool) false;
    $this->_data['is_sysadm'] = (bool) false;
    $this->_data['is_active'] = (bool) true;
    $this->_data['create_date'] = time();
    $this->_data['create_usr_id'] = $_SESSION['usr_id'];
    $this->_data['create_src_id'] = $_SESSION['native_id'];
    $this->_data['update_date'] = time();
    $this->_data['update_usr_id'] = $_SESSION['usr_id'];
    $this->_data['update_src_id'] = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary    = 'usr_id';
    $this->_table      = 'usr';
    $this->_required[] = 'usr_login';
    $this->_unique[]   = 'usr_login';
    
    // Set field types
    $this->_type['usr_allow_date_from']  = array('type'=>'timestamp');
    $this->_type['usr_allow_date_until'] = array('type'=>'timestamp');
    $this->_type['usr_psswd_exp']        = array('type'=>'timestamp');
  }

  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Authenticate user credentials.
   * Please  note that this does NOT login the user. It just authenticates him.
   * 
   * @param  (string)  $user     The login code
   * @param  (string)  $password The password
   * @param  (string)  $source   (optional) The id of the source owning this login
   * @return (integer)				   The user id, or <code>false</code>
   * @public
   */
  function authenticate($user,$password,$source='native') {
    
    global $SPL_CFG,$SPL_TXT;
    $org = $password;
    // Password is stored encrypted
    $password = base64_encode(rc4crypt::encrypt($SPL_CFG['rc4key'], $password));
    
    $id = 0;
    
    // If the source is not the native application,
    // check the user credentials with the source and retrieve 
    // the native user id
    if (is_numeric($source) && $source != $_SESSION['native_id']) {
      $query  =  "SELECT usr_id FROM ".$SPL_CFG['prefix']."uss";
      $query .= " WHERE src_id = '".$source."' AND uss_login = '".$user."' AND uss_psswd = '".$password."' LIMIT 1";
      $result = db_retrieve(db_query($query));
      if (!$result) {
        $_dbg['query'] = $query;
        $_dbg['result'] = $result;
        $_dbg['method'] = 'authenticate';
        $_dbg['error'] = $SPL_TXT['usr_unknown_in_source'];
        array_push($this->_debug, $_dbg);         
        return (bool) false;
      }
      else {
        // User is known. Retrieve the native user login and password
        if (isset($result['usr_id'])) {
          $query  =  "SELECT usr_login,usr_psswd FROM ".$SPL_CFG['prefix'].'uss';
          $query .= " WHERE usr_id = '".$result['usr_id']."' LIMIT 1";
          $result = db_retrieve(db_query($query));
          $user = (isset($result['usr_login'])?$result['usr_login']:$user);
          $password = (isset($result['usr_psswd'])?$result['usr_psswd']:$password);
        }
      }
    }
    
    // Check the credentials against the native data
    $query  =  "SELECT usr_id,usr_allow_date_from,usr_allow_date_until,usr_allow_time_from,usr_allow_time_until FROM ".$SPL_CFG['prefix'].$this->_table;
    $query .= " WHERE usr_login = '".$user."' AND usr_psswd = '".$password."' AND is_active LIMIT 1";
    $result = db_retrieve(db_query($query));
    if (!$result) {
      $_dbg['query'] = $query;
      $_dbg['result'] = $result;
      $_dbg['method'] = 'authenticate';
      $_dbg['error'] = $SPL_TXT['usr_unknown'];
      array_push($this->_debug, $_dbg);         
      return (bool) false;
    }
    
    // User is known. Now check if he is allowed to login at this moment
    if (isset($result['usr_allow_date_from'])) {
      if (intval($result['usr_allow_date_from']) > 0) {
        if (intval($result['usr_allow_date_from']) > time()) {
          $_dbg['query'] = 'evaluating usr_allow_date_from';
          $_dbg['result'] = 'false';
          $_dbg['method'] = 'authenticate';
          $_dbg['error'] = $SPL_TXT['usr_not_allowed'];
          array_push($this->_debug, $_dbg);         
          return (bool) false;
        }
      }
    }

    if (isset($result['usr_allow_date_until'])) {
      if (intval($result['usr_allow_date_until']) > 0) {
        if (intval($result['usr_allow_date_until']) < time()) {
          $_dbg['query'] = 'evaluating usr_allow_date_until';
          $_dbg['result'] = 'false';
          $_dbg['method'] = 'authenticate';
          $_dbg['error'] = $SPL_TXT['usr_not_allowed'];
          array_push($this->_debug, $_dbg);         
          return (bool) false;
        }
      }
    }

    if (isset($result['usr_allow_time_from'])) {
      if (time() < strtotime('today '.$result['usr_allow_time_from'])) {
        $_dbg['query'] = 'evaluating usr_allow_time_from';
        $_dbg['result'] = 'false';
        $_dbg['method'] = 'authenticate';
        $_dbg['error'] = $SPL_TXT['usr_not_allowed'];
        array_push($this->_debug, $_dbg);         
        return (bool) false;
      }
    }

    if (isset($result['usr_allow_time_until'])) {
      if (time() > strtotime('today '.$result['usr_allow_time_until'])) {
        $_dbg['query'] = 'evaluating usr_allow_time_until';
        $_dbg['result'] = 'false';
        $_dbg['method'] = 'authenticate';
        $_dbg['error'] = $SPL_TXT['usr_not_allowed'];
        array_push($this->_debug, $_dbg);         
        return (bool) false;
      }
    }

    return $result['usr_id'];
  }

  /** 
   * Returns all user properties
   * 
   * @return (array) The objects properties
   * @public
   */
  function getData() {
    
    $rawData = parent::getData();

    // Filter out any security sensitive properties
    if (isset($rawData['usr_login'])){
      if (!isset($_SESSION['usr_permission'])||!$_SESSION['usr_permission']['admin']) {
        unset($rawData['usr_login']);
      }
    }
    
    if (isset($rawData['usr_psswd'])){
      if (!isset($_SESSION['usr_permission'])||!$_SESSION['usr_permission']['admin']) {
        unset($rawData['usr_psswd']);
      }
    }
    
    return $rawData;
  }

  /** 
   * Returns the default value for the named property
   * 
   * @param  (string) $property The property to get the default for
   * @param  (string) $role     The user role (user, student, teacher)
   * @return (string) The value
   * @public
   */
  function getDefault($property,$role='user') {
    
    global $SPL_CFG;
    
    $value = (bool) false;
    switch ($role) {
      case 'mentor':
        if (!$this->_data['men_id']){
          return (bool) false;
        }
        include_once($SPL_CFG['dirroot'].'/class/men.abstract.php');
        $men = new abstractMen();
        $men->init($this->_data['men_id']);
        if (isset($men->_data[$property])){
          $value = $men->_data[$property];
        }
        break;
      case 'parent':
        if (!$this->_data['par_id']){
          return (bool) false;
        }
        include_once($SPL_CFG['dirroot'].'/class/par.class.php');
        $par = new par();
        $par->init($this->_data['par_id']);
        if (isset($par->_data[$property])){
          $value = $par->_data[$property];
        }
        break;
      case 'student':
        if (!$this->_data['stu_id']){
          return (bool) false;
        }
        include_once($SPL_CFG['dirroot'].'/class/stu.class.php');
        $stu = new stu();
        $stu->init($this->_data['stu_id']);
        if (isset($stu->_data[$property])){
          $value = $stu->_data[$property];
        }
        break;
      case 'teacher':
        if (!$this->_data['tea_id']){
          return (bool) false;
        }
        include_once($SPL_CFG['dirroot'].'/class/tea.abstract.php');
        $tea = new abstractTea();
        $tea->init($this->_data['tea_id']);
        if (isset($tea->_data[$property])){
          $value = $tea->_data[$property];
        }
        break;
      default:
        if (isset($this->_data[$property])){
          $value = $this->_data[$property];
        }
        break;
    }
    return $value;    
  }

  /** 
   * Returns the user id
   *
   * @param  (integer) $tmp_stu_id The student id 
   * @param  (integer) $tmp_tea_id The teacher id  
   * @param  (integer) $tmp_men_id The mentor id  
   * @param  (integer) $tmp_par_id The parent id  
   * @return (integer) The user id
   * @public
   */
  function getId($tmp_stu_id=null,$tmp_tea_id=null,$tmp_men_id=null,$tmp_par_id=null) {
    
    // Get data from the database
    if (isset($tmp_stu_id)) {
      $data = $this->_select('stu_id',$tmp_stu_id,'1','true');
      return (isset($data['usr_id'])?$data['usr_id']:(bool)false);
    }
    else if (isset($tmp_tea_id)) {
      $data = $this->_select('tea_id',$tmp_tea_id,'1','true');
      return (isset($data['usr_id'])?$data['usr_id']:(bool)false);
    }
    else if (isset($tmp_men_id)) {
      $data = $this->_select('men_id',$tmp_men_id,'1','true');
      return (isset($data['usr_id'])?$data['usr_id']:(bool)false);
    }
    else if (isset($tmp_par_id)) {
      $data = $this->_select('par_id',$tmp_par_id,'1','true');
      return (isset($data['usr_id'])?$data['usr_id']:(bool)false);
    }
    else {
      return (bool)false;
    }
  }
  
  /** 
   * Returns the name of the user
   *
   * @return (string) The (formatted) name
   * @public
   */
  function getName() {
    
    global $SPL_CFG;
    
    if (isset($SPL_CFG['person_details']) && $SPL_CFG['person_details']!='false') {
      // Find person data
      // If the user tries to access his own data, bypass any restrictions
      if (isset($_SESSION['usr_id']) && $_SESSION['usr_id']==$this->_data['usr_id']){
        $id = $this->_data['per_id'];
      }
      else {
        $id = $this->getProperty('per_id');
      }

      if ($id) {
        include_once($SPL_CFG['dirroot'].'/class/per.class.php');
        $per = new per();
        if ($per->init($id)){
          return $per->getName();
        }
      }
    }
    
    return (!empty($this->_data['usr_name'])?$this->_data['usr_name']:$this->_data['usr_login']);
  }

  /** 
   * Returns the name of the user for sorting purposes
   *
   * @return (string) The (formatted) name
   * @public
   */
  function getNameSorting() {
    
    global $SPL_CFG;
    
    $sortname = $this->_data['usr_login'];
    
    if ($SPL_CFG['person_details'] != 'false') {
      // Find person data
      // If the user tries to access his own data, bypass any restrictions
      if (isset($_SESSION['usr_id']) && $_SESSION['usr_id']==$this->_data['usr_id']){
        $id = $this->_data['per_id'];
      }
      else {
        $id = $this->getProperty('per_id');
      }

      if ($id) {
        include_once($SPL_CFG['dirroot'].'/class/per.class.php');
        $per = new per();
        if ($per->init($id)){
          return $per->getNameSorting();
        }
      }
    }
    
    if (!empty($this->_data['usr_name'])) {
      $pos = strrpos($this->_data['usr_name'], ' ');
      if ($pos) {
        $sortname = substr($this->_data['usr_name'],$pos+1).', '.substr($this->_data['usr_name'],0,$pos-1);
      }
      else {
        $sortname = $this->_data['usr_name'];
      }
    }
    
    return $sortname;
  }
  
  /** 
   * Returns a single property of an instance
   *
   * @param  (string) $property The name of the propert to be returned
   * @return (string)           The value of the property
   * @public
   */
  function getProperty($property) {

    global $SPL_CFG;

    $value = parent::getProperty($property);

    // In order to view login and password, you need administrator rights
    if ($property == 'usr_psswd'){
      if (isset($_SESSION['usr_permission'])&&$_SESSION['usr_permission']['admin']) {
        $value = rc4crypt::decrypt($SPL_CFG['rc4key'], base64_decode($value));
      }
      else {
        return (bool) false;
      }
    }

    // If the user tries to access his onw data, bypass any restrictions
    if (isset($_SESSION['usr_id']) && $_SESSION['usr_id']==$this->_data['usr_id'] && empty($value)){
      if ($property=='usr_login_count' || $property=='usr_login_date_first' || $property=='usr_login_date_prev'){
        return $this->_data[$property];
      } 
    }
    
    return $value;
  }
  
  /** 
   * Returns a formatted property of an instance
   *
   * @param  (string) $property The name of the property to be returned
   * @return (string)           The formatted data
   * @public
   */
  function getPropertyFormatted($property) {

    // There are no formatted properties in this class
    return (bool) false;
  }
  
  /**
   * Returns data for printing a report
   * 
   * @return (array) The data to print
   * @public
   */
  function getReportData() {
    
    global $SPL_CFG;
    
    $query  = 'SELECT usr_login,usr_name,usr_email,usr_phone,IF(usr_login_date_last>0,FROM_UNIXTIME(usr_login_date_last),"") AS login_date_last,usr_login_count';
    $query .= ' FROM '.$SPL_CFG['prefix'].'usr usr';
    $query .= ' WHERE usr_id > 0 AND is_active';
    $query .= ' ORDER BY usr_login';

    $data = array();
    $result = db_query($query);
    while($data[] = db_retrieve($result)) {}
    return $data;

  }

  /** 
   * Instantiates the object given a mentor id
   *
   * @param  (integer) $id The id of the mentor  
   * @return (boolean) Success or Failure
   * @public
   */
  function initMentor($id) {
    
    global $SPL_TXT;
        
    // Get data from the database
    $data = $this->_select('men_id',$id,'1','true');
    if (!$data){
      $_dbg['query'] = 'calling _select with men_id = '.$id.' and limit = 1';
      $_dbg['result'] = $data;
      $_dbg['method'] = 'initMentor';
      $_dbg['error'] = $SPL_TXT['cannot_instantiate'];
      array_push($this->_debug, $_dbg);
      return (bool) false;
    }
    
    // Populate the objects properties
    $this->_populate($data);
  
    return (bool) true;
  }

  /** 
   * Instantiates the object given a parent id
   *
   * @param  (integer) $id The id of the parent  
   * @return (boolean) Success or Failure
   * @public
   */
  function initParent($id) {
    
    global $SPL_TXT;
        
    // Get data from the database
    $data = $this->_select('par_id',$id,'1','true');
    if (!$data){
      $_dbg['query'] = 'calling _select with par_id = '.$id.' and limit = 1';
      $_dbg['result'] = $data;
      $_dbg['method'] = 'initParent';
      $_dbg['error'] = $SPL_TXT['cannot_instantiate'];
      array_push($this->_debug, $_dbg);
      return (bool) false;
    }
    
    // Populate the objects properties
    $this->_populate($data);
  
    return (bool) true;
  }


  /** 
   * Instantiates the object given a student id
   *
   * @param  (integer) $id The id of the student  
   * @return (boolean) Success or Failure
   * @public
   */
  function initStudent($id) {
    
    global $SPL_TXT;
        
    // Get data from the database
    $data = $this->_select('stu_id',$id,'1','true');
    if (!$data){
      $_dbg['query'] = 'calling _select with stu_id = '.$id.' and limit = 1';
      $_dbg['result'] = $data;
      $_dbg['method'] = 'initStudent';
      $_dbg['error'] = $SPL_TXT['cannot_instantiate'];
      array_push($this->_debug, $_dbg);
      return (bool) false;
    }
    
    // Populate the objects properties
    $this->_populate($data);
  
    return (bool) true;
  }

  /** 
   * Instantiates the object given a teacher id
   *
   * @param  (integer) $id The id of the teacher  
   * @return (boolean)
   * @public
   */
  function initTeacher($id) {
    
    global $SPL_TXT;
        
    // Get data from the database
    $data = $this->_select('tea_id',$id,'1','true');
    if (!$data){
      $_dbg['query'] = 'calling _select with tea_id = '.$id.' and limit = 1';
      $_dbg['result'] = $data;
      $_dbg['method'] = 'initTeacher';
      $_dbg['error'] = $SPL_TXT['cannot_instantiate'];
      array_push($this->_debug, $_dbg);
      return (bool) false;
    }
    
    // Populate the objects properties
    $this->_populate($data);
  
    return (bool) true;
  }

  /** 
   * Evaluates if a user is a mentor
   *
   * @return (boolean) Mentor or Not
   * @public
   */
  function isMentor() {
    if (!$this->_status['is_loaded']){
      return(bool)false;
    }
    return (!empty($this->_data['men_id']));
  }

  /** 
   * Evaluates if a user is a parent
   *
   * @return (boolean) Parent or Not
   * @public
   */
  function isParent() {
    if (!$this->_status['is_loaded']){
      return(bool)false;
    }
    return (!empty($this->_data['par_id']));
  }

  /** 
   * Evaluates if a user is a student
   *
   * @return (boolean) Student or Not
   * @public
   */
  function isStudent() {
    if (!$this->_status['is_loaded']){
      return(bool)false;
    }
    return (!empty($this->_data['stu_id']));
  }

  /** 
   * Evaluates if a user is a system administrator
   *
   * @return (boolean) System Administrator or Not
   * @public
   */
  function isSysAdm() {
    if (!$this->_status['is_loaded']){
      return (bool)false;
    }
    return ($this->_data['is_sysadm']=='1');
  }

  /** 
   * Evaluates if a user is a teacher
   *
   * @return (boolean) Teacher or Not
   * @public
   */
  function isTeacher() {
    if (!$this->_status['is_loaded']){
      return (bool)false;
    }
    return (!empty($this->_data['tea_id']));
  }

  /** 
   * Provides access to known users.
   * 
   * @param  (string)  $user     The login code
   * @param  (string)  $password The password
   * @param  (string)  $source   (optional) The id of the source owning this login
   * @return (boolean)
   * @public
   */
  function login($user,$password,$source='native') {
    
    global $SPL_CFG;
    
    // First authenticate the user
    $id = $this->authenticate($user,$password,$source);
    if (!$id) {
      return (bool) false;
    }
    
    // Load instance data
    if (!$this->init($id)) {
      return (bool) false;
    }

    // Set session information
    $src = ($source=='native'?1:$source);
    $_SESSION['usr_id']     = $id;
    $_SESSION['src_id']     = $src;
    $_SESSION['is_admin']   = $this->isSysAdm();
    $_SESSION['is_teacher'] = $this->isTeacher();
    $_SESSION['is_student'] = $this->isStudent();
    $_SESSION['is_mentor']  = $this->isMentor();
    $_SESSION['is_parent']  = $this->isParent();
    $_SESSION['my_per_id']  = $this->_data['per_id'];
    $_SESSION['my_stu_id']  = $this->_data['stu_id'];
    $_SESSION['my_tea_id']  = $this->_data['tea_id'];
    $_SESSION['my_men_id']  = $this->_data['men_id'];
    $_SESSION['my_par_id']  = $this->_data['par_id'];
    $_SESSION['my_name']    = $this->getName();
    
    // Update the activity log
    if ($this->_data['usr_log']=='1'){
      $this->_setLog($id,$src);
    }
    
    // Update the users login status
    $loginCount = intval($this->_data['usr_login_count']);
    if ($loginCount==0) {
      $this->setProperty('usr_login_date_first',time());
    }
    $this->setProperty('usr_login_count',++$loginCount);
    $this->setProperty('usr_login_date_prev',$this->_data['usr_login_date_last']);
    $this->setProperty('usr_login_date_last',time());
    $this->save();
    
    // Load preferences
    $_SESSION['usp_ins_id'] = (!isset($_SESSION['usp_ins_id'])?0:$_SESSION['usp_ins_id']);
    $_SESSION['usp_ina_id'] = (!isset($_SESSION['usp_ina_id'])?0:$_SESSION['usp_ina_id']);
    $_SESSION['usp_tra_id'] = (!isset($_SESSION['usp_tra_id'])?0:$_SESSION['usp_tra_id']);
    $_SESSION['usp_ali_id'] = (!isset($_SESSION['usp_ali_id'])?0:$_SESSION['usp_ali_id']);
    $_SESSION['usp_lst_id'] = (!isset($_SESSION['usp_lst_id'])?0:$_SESSION['usp_lst_id']);
    $this->_getPreferences($id);
    
    // Set permissions
    $_SESSION[$this->_table.'_permission'] = $this->_getPermissions();
    
    $query  = 'SELECT usa_key,usa_permission FROM '.$SPL_CFG['prefix'].'usa WHERE';
    $query .= ' usr_id = "'.$id.'" AND adm_id = "'.$_SESSION['adm_id'].'"';
    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      if (isset($row['usa_permission'])) {
        $_SESSION[$row['usa_key'].'_permission'] = ($this->_explodePermissions($row['usa_permission']));
      }
      else {
        $_SESSION[$row['usa_key'].'_permission'] =  array("read"=>false,"write"=>false,"delete"=>false,"change"=>false,"admin"=>false);
      }
    }

    // Base classes
    foreach ($SPL_CFG['classes'] as $class) {
      if (!isset($_SESSION[$class.'_permission'])){
        $_SESSION[$class.'_permission'] =  array("read"=>false,"write"=>false,"delete"=>false,"change"=>false,"admin"=>false);
      }
    }
    
    return (bool) true;
  }

  /** 
   * Sets the default value for the named property
   * 
   * @param  (string)  $property The property to set the default for
   * @param  (string)  $value    The value to set
   * @param  (string)  $role     The user role (user, student, teacher)
   * @return (boolean) Succes or Failure
   * @public
   */
  function setDefault($property,$value,$role='user') {
    
    global $SPL_CFG;
    
    switch ($role) {
      case 'mentor':
        if (!$this->_data['men_id']){
          return (bool) false;
        }
        include_once($SPL_CFG['dirroot'].'/class/men.abstract.php');
        $men = new abstractMen();
        $men->init($this->_data['men_id']);
        $men->setProperty($property,$value);
        return $men->save();
      case 'parent':
        if (!$this->_data['par_id']){
          return (bool) false;
        }
        include_once($SPL_CFG['dirroot'].'/class/par.class.php');
        $par = new par();
        $par->init($this->_data['par_id']);
        $par->setProperty($property,$value);
        return $par->save();
      case 'student':
        if (!$this->_data['stu_id']){
          return (bool) false;
        }
        include_once($SPL_CFG['dirroot'].'/class/stu.class.php');
        $stu = new stu();
        $stu->init($this->_data['stu_id']);
        $stu->setProperty($property,$value);
        return $stu->save();
      case 'teacher':
        if (!$this->_data['tea_id']){
          return (bool) false;
        }
        include_once($SPL_CFG['dirroot'].'/class/tea.abstract.php');
        $tea = new abstractTea();
        $tea->init($this->_data['tea_id']);
        $tea->setProperty($property,$value);
        return $tea->save();
      default:
        break;
    }
    return $value;    
  }

  /** 
   * Sets a new password
   * 
   * @param  (string) $old   The old password
   * @param  (string) $new   The new password
   * @param  (string) $again The new password again
   * @return (boolean) Succes or a failure text
   * @public
   */
  function setPassword($old,$new,$again) {
    
    global $SPL_TXT;
    
    // Check if new password and agin are the same
    if ($new!=$again){
      if (!isset($SPL_TXT['invalid_password_again'])) {
        $SPL_TXT['invalid_password_again'] = 'The new password does not match the password typed again';
      }
      return $SPL_TXT['invalid_password_again'];
    }
    
    // Check if the old password is correct
    if (!$this->authenticate($this->_data['usr_login'],$old)){
      if (!isset($SPL_TXT['invalid_password_old'])) {
        $SPL_TXT['invalid_password_old'] = 'The current password is invalid';
      }
      return $SPL_TXT['invalid_password_old'];
    }
    
    return $this->setProperty('usr_psswd',$new);
  }
  
  /** 
   * Sets a preference
   *
   * @param  (string)  $property  The name of the preference
   * @param  (string)  $value     The value to set
   * @return (boolean) Succes or Failure
   * @public
   */
  function setPreference($property,$value) {

    global $SPL_CFG,$SPL_TXT;

    // Find existing preferences
    $query = "SELECT usp_id FROM ".$SPL_CFG['prefix']."usp WHERE usr_id = '".$this->_data['usr_id']."' LIMIT 1";
    $result = db_retrieve(db_query($query));
    
    // If none-existent, create them
    if (!$result){
      $query =  "INSERT INTO ".$SPL_CFG['prefix']."usp (usp_id,usr_id,adm_id,usp_lang,usp_nameformat,usp_locale,usp_numrows,usp_startpage,usp_startpane,usp_calmode,usp_menumode,usp_fontsize,usp_skin,usp_ins_id,usp_ina_id,usp_tra_id,usp_ali_id) VALUES (NULL,";
      $query .= "'".$this->_data['usr_id']."',";
      $query .= "'".($property=='adm_id'?$value:(isset($_SESSION['adm_id'])?$_SESSION['adm_id']:'1'))."',";
      $query .= "'".($property=='usp_lang'?$value:$SPL_CFG['language'])."',";
      $query .= "'".($property=='usp_nameformat'?$value:$SPL_CFG['nameformat'])."',";
      $query .= "'".($property=='usp_locale'?$value:(strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')?'nld_nld':'nl_NL')."',";
      $query .= "'".($property=='usp_numrows'?$value:'16')."',";
      $query .= "'".($property=='usp_startpage'?$value:'ui/menu.php')."',";
      $query .= "'".($property=='usp_startpane'?$value:'menu_sgd.php')."',";
      $query .= "'".($property=='usp_calmode'?$value:'calendardate')."',";
      $query .= "'".($property=='usp_menumode'?$value:'normal')."',";
      $query .= "'".($property=='usp_fontsize'?$value:'normal')."',";
      $query .= "'".($property=='usp_skin'?$value:'default')."',";
      $query .= "'".($property=='usp_ins_id'?$value:'')."',";
      $query .= "'".($property=='usp_ina_id'?$value:'')."',";
      $query .= "'".($property=='usp_tra_id'?$value:'')."',";
      $query .= "'".($property=='usp_ali_id'?$value:'')."')";
      db_query($query);
    }
    else {
      $query = "UPDATE ".$SPL_CFG['prefix']."usp SET ";
      $query .= $property." = '".$value."'";
      $query .= " WHERE usp_id = '".$result['usp_id']."'"; 
      db_query($query);
      
      if ($_SESSION['usr_id']==$this->_data['usr_id']) {
        // Set session
        $_SESSION[$property] = $value;
      }
    }
   
    return (bool) true;
  }

  /** 
   * Sets the instance property with the given value
   *
   * @param  (string)  $property The name of the property to be set
   * @param  (string)  $value    The data to bet set
   * @return (boolean) Succes or Failure
   * @public
   */
  function setProperty($property, $value) {
    
    global $SPL_CFG;

    // Encrypt user login, password and email
    if ($property == 'usr_psswd'){
      $value = base64_encode(rc4crypt::encrypt($SPL_CFG['rc4key'], $value));
      // Set expire date
      $this->_setPasswordExpire();
    }
    
    // Translate dates into unix timestamps
    if ($property == 'usr_psswd_exp' || $property == 'usr_allow_date_from' || $property == 'usr_allow_date_until') {
      $value = ($value?strtotime($value):$value);
    }
    
    // When de-activating the user, also de-activate the student and/or teacher records
    if ($property == 'is_active'){
      // Mentor
      if ((int)$this->_data['men_id']>0) {
        include_once($SPL_CFG['dirroot'].'/class/men.abstract.php');
        $men = new abstractMen();
        if ($men->init($this->_data['men_id'])) {
          $men->setProperty('is_active',$value);
        }
      }
      // Parent
      if ((int)$this->_data['par_id']>0) {
        include_once($SPL_CFG['dirroot'].'/class/par.class.php');
        $par = new par();
        if ($par->init($this->_data['par_id'])) {
          $par->setProperty('is_active',$value);
        }
      }
      // Student
      if ((int)$this->_data['stu_id']>0) {
        include_once($SPL_CFG['dirroot'].'/class/stu.class.php');
        $stu = new stu();
        if ($stu->init($this->_data['stu_id'])) {
          $stu->setProperty('is_active',$value);
        }
      }
      // Teacher
      if ((int)$this->_data['tea_id']>0) {
        include_once($SPL_CFG['dirroot'].'/class/tea.abstract.php');
        $tea = new abstractTea();
        if ($tea->init($this->_data['tea_id'])) {
          $tea->setProperty('is_active',$value);
        }
      }
    }
    
    
    return parent::setProperty($property, $value);
  }

  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Adds an entry to the user activity log
   *
   * @param  (integer)  $id  The user id
   * @return (array) An array of preferences
   * @private
   */
  function _getPreferences($id) {

    global $SPL_CFG,$SPL_TXT;

    $query = "SELECT * FROM ".$SPL_CFG['prefix']."usp WHERE usr_id = '".$id."' LIMIT 1";
    $result = db_retrieve(db_query($query));

    // Set administration
    if (isset($result['adm_id'])) {
      $_SESSION['adm_id'] = $result['adm_id'];
    }
    else {
      // If no administration is found, set the first one available
      $query = "SELECT adm_id FROM ".$SPL_CFG['prefix']."adm WHERE adm_is_default = '1' LIMIT 1";
      $admResult = db_retrieve(db_query($query));
      $this->setPreference('adm_id',(isset($admResult['adm_id'])?$admResult['adm_id']:'1'));
    } 
    
    // Set skin
    if (isset($result['usp_skin'])) {
      $_SESSION['usp_skin'] = $result['usp_skin'];
    }
    else {
      $this->setPreference('usp_skin',$SPL_CFG['skin']);
    }

    // Set language
    if (isset($result['usp_lang'])) {
      $_SESSION['usp_lang'] = $result['usp_lang'];
    }
    else {
      $this->setPreference('usp_lang',$SPL_CFG['language']);
    }
    
    // Set name format
    if (isset($result['usp_nameformat'])) {
      $_SESSION['usp_nameformat'] = $result['usp_nameformat'];
    }
    else {
      $this->setPreference('usp_nameformat',$SPL_CFG['nameformat']);
    }
    
    // Set locale
    if (isset($result['usp_locale'])) {
      setlocale(LC_ALL,$result['usp_locale']);
    }
    else {
      $this->setPreference('usp_locale',(strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')?'nld_nld':'nl_NL');
      setlocale(LC_ALL,(strtoupper(substr(PHP_OS, 0, 3)) === 'WIN')?'nld_nld':'nl_NL');
    }
    
    // Set startpage
    if (isset($result['usp_startpage'])) {
      $_SESSION['usp_startpage'] = $result['usp_startpage'];
    }
    else {
      $this->setPreference('usp_startpage','ui/menu.php');
    }
    
    // Set startpane
    if (isset($result['usp_startpane'])) {
      $_SESSION['usp_startpane'] = $result['usp_startpane'];
    }
    else {
      $this->setPreference('usp_startpane','menu_sgd.php');
    }
    
    // Set calendar mode
    // Valid modes: calendardate, schooldate, none
    if (isset($result['usp_calmode'])) {
      $_SESSION['usp_calmode'] = $result['usp_calmode'];
    }
    else {
      $this->setPreference('usp_calmode','calendardate');
    }

    // Set menu mode
    // Valid modes: normal, small, none
    if (isset($result['usp_menumode'])) {
      $_SESSION['usp_menumode'] = $result['usp_menumode'];
    }
    else {
      $this->setPreference('usp_menumode','normal');
    }

    // Set default number of rows to display in a table
    if (isset($result['usp_numrows'])) {
      $_SESSION['usp_numrows'] = $result['usp_numrows'];
    }

    // Set default font siuze for text areas
    if (isset($result['usp_fontsize'])) {
      $_SESSION['usp_fontsize'] = $result['usp_fontsize'];
    }

    // Set default institution
    if (isset($result['usp_ins_id'])) {
      $_SESSION['usp_ins_id'] = $result['usp_ins_id'];
    }
    
    // Set default institution address
    if (isset($result['usp_ina_id'])) {
      $_SESSION['usp_ina_id'] = $result['usp_ina_id'];
    }

    // Set default training
    if (isset($result['usp_tra_id'])) {
      $_SESSION['usp_tra_id'] = $result['usp_tra_id'];
    }
    
    // Set default alignment
    if (isset($result['usp_ali_id'])) {
      $_SESSION['usp_ali_id'] = $result['usp_ali_id'];
    }
    
    // Set default learning style
    if (isset($result['usp_lst_id'])) {
      $_SESSION['usp_lst_id'] = $result['usp_lst_id'];
    }

    // If this user is a student, then use the preferences from the stu table
    if (isset($_SESSION['my_stu_id']) && $_SESSION['my_stu_id']>0) {
      $query = "SELECT * FROM ".$SPL_CFG['prefix']."stu WHERE stu_id = '".$_SESSION['my_stu_id']."' LIMIT 1";
      $result = db_retrieve(db_query($query));

      // Set default institution
      if (isset($result['ins_id'])&&$result['ins_id']>0) {
        $_SESSION['usp_ins_id'] = $result['ins_id'];
      }
    
      // Set default institution address
      if (isset($result['ina_id'])&&$result['ina_id']>0) {
        $_SESSION['usp_ina_id'] = $result['ina_id'];
      }

      // Set default training
      if (isset($result['tra_id'])&&$result['tra_id']>0) {
        $_SESSION['usp_tra_id'] = $result['tra_id'];
      }
    
      // Set default alignment
      if (isset($result['ali_id'])&&$result['ali_id']>0) {
        $_SESSION['usp_ali_id'] = $result['ali_id'];
      }
    
      // Set default learning style
      if (isset($result['lst_id'])&&$result['lst_id']>0) {
        $_SESSION['usp_lst_id'] = $result['lst_id'];
      }
    }

    // If this user is a teacher, then use the preferences from the tea table
    if (isset($_SESSION['my_tea_id']) && $_SESSION['my_tea_id']>0) {
      $query = "SELECT * FROM ".$SPL_CFG['prefix']."tea WHERE tea_id = '".$_SESSION['my_tea_id']."' LIMIT 1";
      $result = db_retrieve(db_query($query));

      // Set default institution
      if (isset($result['ins_id'])&&$result['ins_id']>0) {
        $_SESSION['usp_ins_id'] = $result['ins_id'];
      }
    
      // Set default institution address
      if (isset($result['ina_id'])&&$result['ina_id']>0) {
        $_SESSION['usp_ina_id'] = $result['ina_id'];
      }
    }

    // Set (most recently used) role
    $query  =  "SELECT rol_id FROM ".$SPL_CFG['prefix']."uso";
    $query .= " WHERE usr_id = '".$id."' AND adm_id = '".$_SESSION['adm_id']."'";
    $query .= " ORDER BY uso_date_used DESC LIMIT 1";
    $result = db_retrieve(db_query($query));
    if (isset($result['rol_id'])) {
      $_SESSION['rol_id'] = $result['rol_id'];
    }
    else {
      // If no role has yet been found, set a 'guest' role
      $query  =  "SELECT rol_id FROM ".$SPL_CFG['prefix']."rol";
      $query .= " WHERE rol_type = 'guest' AND adm_id = '".$_SESSION['adm_id']."' LIMIT 1";
      $result = db_retrieve(db_query($query));
      if (isset($result['rol_id'])) {
        $_SESSION['rol_id'] = $result['rol_id'];
      }
      else {
        // No guest role found: create one
        $query  =  "INSERT INTO ".$SPL_CFG['prefix']."rol (rol_id,adm_id,rol_desc,rol_short,rol_type)";
        $query .= " VALUES (NULL,'".$_SESSION['adm_id']."','Guest','Guest','guest')";
        $result = db_query($query);
        $_SESSION['rol_id'] = db_get_generated_key();
      }
    }  
  }

  /** 
   * Inserts a new user 
   *
   * @return (boolean) Succes or Failure
   * @protected
   */
  function _insert() {
    
    global $SPL_CFG;
    
   
    if (parent::_insert()) {
      // When insert is succesfull, set default authorization
      include_once($SPL_CFG['dirroot'].'/class/usa.class.php');
      
      // User (for setting preferences)
      $usa = new usa();
      $usa->setProperty('usr_id',$this->_data['usr_id']);      
      $usa->setProperty('usa_key','usr');      
      $usa->setProperty('usa_permission',($this->_data['is_sysadm'])?'31':'9');      
      $usa->save();
      
      // Applicability
      $usa = new usa();
      $usa->setProperty('usr_id',$this->_data['usr_id']);      
      $usa->setProperty('usa_key','apl');      
      $usa->setProperty('usa_permission',($this->_data['is_sysadm'])?'31':'1');      
      $usa->save();

      // Set preferred number of rows
      $this->setPreference('usp_numrows',$SPL_CFG['numrows']);   

      // Set preferred skin
      $this->setPreference('usp_skin',$SPL_CFG['skin']);   

      return (bool) true;
    }
    else {
      return (bool) false;
    }
  }



  /** 
   * Adds an entry to the user activity log
   *
   * @param  (integer)  $id  The user id
   * @param  (integer)  $src The source id
   * @return (boolean)
   * @private
   */
  function _setLog($id, $src) {
    
    global $SPL_CFG,$SPL_TXT;
    
    $query =  "INSERT INTO ".$SPL_CFG['prefix']."usl (";
    $query .= "usl_id,usr_id,src_id,usl_date,usl_host,usl_ip,usl_agent,usl_referer) VALUES (";
    $query .= "NULL,";
    $query .= "'".$id."',";
    $query .= "'".$src."',";
    $query .= "'".time()."',";
    $query .= "'".(isset($_SERVER['REMOTE_HOST'])?$_SERVER['REMOTE_HOST']:'')."',";
    $query .= "'".(isset($_SERVER['REMOTE_ADDR'])?$_SERVER['REMOTE_ADDR']:'')."',";
    $query .= "'".(isset($_SERVER['HTTP_USER_AGENT'])?$_SERVER['HTTP_USER_AGENT']:'')."',";
    $query .= "'".(isset($_SERVER['HTTP_REFERER'])?$_SERVER['HTTP_REFERER']:'')."')";
    $result = db_query($query);
    if (!$result) {
      $_dbg['query'] = $query;
      $_dbg['result'] = $result;
      $_dbg['method'] = 'login';
      $_dbg['error'] = $SPL_TXT['db_cannot_insert'];
      array_push($this->_debug, $_dbg);
      return (bool) false;         
    }
    return (bool) true;
  }

  /**
   * Sets the expire date for the password
   * 
   * @return (boolean) Succes or Failure
   * @private
   */
  function _setPasswordExpire() {
    
    global $SPL_CFG;
    
    if (!isset($SPL_CFG['psswd_expires'])) {
      return (bool) true;
    }
    
    // Password never expires
    if ($SPL_CFG['psswd_expires']=='never') {
      $this->setProperty('usr_psswd_exp',0);
      return (bool) true;
    }
    
    // Password expires next day
    if ($SPL_CFG['psswd_expires']=='day') {
      $this->setProperty('usr_psswd_exp',date('Y-m-d H:i:s',strtotime('+1 day')));
      return (bool) true;
    }
    
    // Password expires next week
    if ($SPL_CFG['psswd_expires']=='week') {
      $this->setProperty('usr_psswd_exp',date('Y-m-d H:i:s',strtotime('+1 week')));
      return (bool) true;
    }

    // Password expires next month
    if ($SPL_CFG['psswd_expires']=='month') {
      $this->setProperty('usr_psswd_exp',date('Y-m-d H:i:s',strtotime('+1 month')));
      return (bool) true;
    }
    
    // Password expires next quarter
    if ($SPL_CFG['psswd_expires']=='quarter') {
      $this->setProperty('usr_psswd_exp',date('Y-m-d H:i:s',strtotime('+3 month')));
      return (bool) true;
    }
    
    // Password expires next halfyear
    if ($SPL_CFG['psswd_expires']=='halfyear') {
      $this->setProperty('usr_psswd_exp',date('Y-m-d H:i:s',strtotime('+6 month')));
      return (bool) true;
    }

    // Password expires next year
    if ($SPL_CFG['psswd_expires']=='year') {
      $this->setProperty('usr_psswd_exp',date('Y-m-d H:i:s',strtotime('+1 year')));
      return (bool) true;
    }

  }

}

/**
 * $Log: usr.class.php,v $
 * Revision 1.32  2006/08/21 18:33:19  jebbol
 * Changed default skin when creating a new record
 *
 * Revision 1.31  2006/04/16 18:29:37  jebbol
 * setPreference wrongly assumed always setting preferences for the current user (Feature #1454313)
 *
 * Revision 1.30  2006/04/14 10:41:39  jebbol
 * Added support for mentors and parents
 *
 * Revision 1.29  2006/02/06 12:36:02  jebbol
 * (Bug #1425130) When de-activating a user, also de-activate student/teacher data and their related data
 *
 * Revision 1.28  2006/01/30 19:43:01  jebbol
 * Request #1418440: Changed default setting when creating a new student
 */

?>
