<?php
/**
 * User class.
 *
 * @author    Torchic
 * @version   1.0
 * @revision  2005-04-05
 * @package   spl
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../inc/test.inc.php');
}

// Configuration
include_once($SPL_CFG['dirroot'].'/inc/config.php');

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

// Human language translations
if (!isset($_SESSION['usp_lang'])) {$_SESSION['usp_lang'] = 'en';}
include_once($SPL_CFG['dirroot'].'/lang/'.$_SESSION['usp_lang'].'/usr.lang.php');

// Encryption
include_once($SPL_CFG['dirroot'].'/class/rc4crypt.class.php');

//=========================================================================
// Class definition
//=========================================================================

class uss extends abstractData {
  
  function uss() {

    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['uss_id'] = (int) 0;
    $this->_data['usr_id'] = (int) 0;
    $this->_data['src_id'] = (int) 0;
    $this->_data['uss_login'] = '';
    $this->_data['uss_psswd'] = '';
    
    // Register the required class properties
    $this->_primary    = 'uss_id';
    $this->_table      = 'uss';
    $this->_required[] = 'uss_login';
    
    $this->_active      = 'both';
  }
  
  

  //=========================================================================
  // Public methods
  //=========================================================================

  
  /** 
   * Deletes the instance. 
   * By  default a soft delete will be performed, which means the data is not
   * actually  deleted from the database, but merely flagged as 'inactive'.
   *
   * @param    boolean $soft    Soft delete (default) or record delete (false) 
   * @return   boolean
   * @access   public
   */
  function delete($soft=false) {
    
    // Delete the record
    if (parent::delete($soft)) {
      // When deleting is succesfull, delete al related information as well
      return (bool) true;
    }
    else {
      return (bool) false;
    }
  }

  /** 
   * Returns a single property of an instance
   *
   * @param  string $property The name of the propert to be returned
   * @return string           The value of the property
   * @access public
   */
  function getProperty($property) {

    global $SPL_CFG;

    $value = parent::getProperty($property);

    // In order to view login and password, you need administrator rights
    if ($property == 'uss_psswd'){
        $value = rc4crypt::decrypt($SPL_CFG['rc4key'], $value);
    }
    return $value;
  }
  
  /** 
   * Sets the instance property with the given value
   *
   * @param  string  $property The name of the property to be set
   * @param  string  $value    The data to bet set
   * @return boolean
   * @access public
   */
  function setProperty($property, $value) {
    
    global $SPL_CFG;

    // Encrypt user login, password and email
    if ($property == 'uss_psswd'){
      $value = rc4crypt::encrypt($SPL_CFG['rc4key'], $value);
    }
    
    return parent::setProperty($property, $value);
  }
  
  /** 
   * Get credentials of the current user for the source application
   *
   * @param  integer $source  The ID of the source application
   * @return boolean
   * @access public
   */
  function getCredentials ($source) {
  	
     global $SPL_CFG,$SPL_TXT;
    
	 if (!isset($_SESSION['usr_id']) || (int)$_SESSION['usr_id']==0) {
	 	return;
	 }
     $query  =  "SELECT * FROM ".$SPL_CFG['prefix']."uss";
     $query .= " WHERE src_id = '".$source."' AND usr_id = '".$_SESSION['usr_id']."' LIMIT 1";
     $result = db_retrieve(db_query($query));
     if (!$result) {
       $_dbg['query'] = $query;
       $_dbg['result'] = $result;
       $_dbg['method'] = 'getCredentials';
       $_dbg['error'] = $SPL_TXT['usr_unknown_in_source'];
       array_push($this->_debug, $_dbg);         
       return (bool) false;
     }
     else {
       $this->_populate($result);
       return (bool) true;
     }
  	 
  }
  

  //=========================================================================
  // Private methods
  //=========================================================================

}

?>
