<?php 
/**
 * inc/text.formatting.php -  Functions for stripping and applying HTML tags to
 * plain text. 
 * 
 * Code for extracting and applying formatting created by Chung Leong and
 * posted on comp.lang.php on 2004-10-28.
 * 
 * Code for serializing the formatting data were taken from postings to the php
 * manual.
 *
 * @author    Chung Leong, static, james at globalmetacorp dot org
 * @version   1.0
 * @revision  2005-01-13
 * @package   spl
 * @copyright Copyright (c) 2005 Chung Leong, Jebbol 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 *
 * Example:
 * $example  = '<p>A very <em>small</em> paragraph.</p>';
 * list($text,$formatting) = ExtractFormatting($example);
 * echo  "<pre style='border:1px solid black'>$text</pre>";
 * $html = ApplyFormatting($text, $formatting);
 * echo "<div style='border:1px solid black'>$html</div>";
 */

//=========================================================================
// Functions
//=========================================================================

/** 
 * Extracts HTML tags from a given text and returns text and formatting 
 * information  in separate arrays
 *
 * @param  string $html       The formatted text (including html tags)
 * @return string $text       The plain unformatted text
 * @return array  $formatting The formatting information
 */
function ExtractFormatting($html) {
  preg_match_all('/<.*?>/', stripslashes($html), $matches, PREG_OFFSET_CAPTURE | PREG_PATTERN_ORDER);
  $tags = $matches[0];
  $formatting = array ();
  foreach ($tags as $tag) {
    list ($tag_html, $tag_offset) = $tag;
    $formatting[$tag_offset] = $tag_html;
  }
  $text = preg_replace('/<.*?>/', '', $html);
  return array ($text, $formatting);
}

/** 
 * Applies formatting to an unformatted text
 *
 * @param  string $text       The plain unformatted text
 * @param  array  $formatting The formatting information
 * @return string             The formatted text
 */
function ApplyFormatting($text, $formatting) {
  $html = $text;
  foreach ($formatting as $tag_offset => $tag_html) {
    $html = substr($html, 0, $tag_offset).$tag_html.substr($html, $tag_offset);
  }
  return $html;
}

/**
 * Serialize an array(including the keys) to a string
 * 
 * @param $glue  string Character to use as separator
 * @param $array array  The array to serialize
 * @return string
 * 
 * This code is taken from a posting to the php manual by 'static'
 */
function implode_with_keys($glue, $array) {
  $output = array();
  foreach( $array as $key => $item ) {
    $output[] = $key."~".$item;
  }
  return implode($glue, $output);
}

/**
 * Unserialize a string into an array with keys
 * 
 * @param  $glue   string Character to use as separator
 * @param  $string string The string to unserialize
 * @return array
 * 
 * This code is taken from a posting to the php manual by 'james at
 * globalmetacorp dot org'
 */
function explode_with_keys($glue, $string)
{
  $output=array();
  $array=explode($glue, $string);
  foreach ($array as $value) {
    $row=explode("~",$value);
    @$output[$row[0]]=$row[1];
  }
  return $output;
}
?>
