<?php
/**
 * Training alignments class.
 *
 * @author    Jebbol
 * @version   1.0
 * @revision  $Id: ali.class.php,v 1.9 2006/08/21 18:36:25 jebbol Exp $
 * @package   spl
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {$SPL_CFG['dirroot'] = '../../..';}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

//=========================================================================
// Class definition
//=========================================================================

class ali extends abstractData {
  
  function ali() {

    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['ali_id'] = (int) 0;
    $this->_data['adm_id'] = $_SESSION['adm_id'];
    $this->_data['ali_name'] = '';
    $this->_data['ali_short'] = '';
    $this->_data['ali_code'] = '';
    $this->_data['is_active'] = (bool) true;
    $this->_data['create_date'] = time();
    $this->_data['create_usr_id'] = $_SESSION['usr_id'];
    $this->_data['create_src_id'] = $_SESSION['native_id'];
    $this->_data['update_date'] = time();
    $this->_data['update_usr_id'] = $_SESSION['usr_id'];
    $this->_data['update_src_id'] = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary = 'ali_id';
    $this->_table   = 'ali';
    $this->_name    = 'ali_name';
    
    // Register properties that must have a value
    $this->_required[] = $this->_name;

  }
  //=========================================================================
  // Public methods
  //=========================================================================

  /** 
   * Returns all subjects associated with the current alignments
   *
   * @param  string $valueType Indicates the type of value to return (long,short,code)
   * @return array An array of subjects (key => value)
   * @access public
   */
  function getSubjects($valueType='long') {
    global $SPL_CFG;
    
    $values = array();
    $field = 'sub_name';
    if ($valueType=='short'){
      $field = 'sub_short';
    }
    if ($valueType=='code'){
      $field = 'sub_code';
    }

    $query  = 'SELECT sub.sub_id as "key",sub.'.$field.' as "value"';
    $query .= ' FROM '.$SPL_CFG['prefix'].'apl apl LEFT JOIN '.$SPL_CFG['prefix'].'sub sub ON sub.sub_id = apl.child_id';
    $query .= ' WHERE apl.adm_id = "'.$_SESSION['adm_id'].'" AND apl.is_active';
    $query .= ' AND apl.apl_parent = "ali"';
    $query .= ' AND apl.parent_id = "'.$this->_data['ali_id'].'"';
    $query .= ' AND apl.apl_child = "sub"';
    $query .= ' AND apl.apl_valid_from <= "'.time().'"';
    $query .= ' AND (apl.apl_valid_until = "0" OR apl.apl_valid_until >= "'.time().'")';
    $query .= ' ORDER BY sub.'.$field.' ASC, apl.apl_sequence ASC';

    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      $values[] = array($row['key']=>$row['value']);
    }
    return $values;
  }
  
  /**
   * Checks if the given column/value combination is referenced from any objects
   * of this class
   * 
   * @param  string  $column The name of the column
   * @param  string  $value  The value to check
   * @return boolean Used or Unused
   * @access public
   */
  function isUsed($column,$value){

    global $SPL_CFG;
    
    $inUse = parent::isUsed($column,$value);
    if (!$inUse) {
      // Check students
      include_once($SPL_CFG['dirroot'].'/class/stu.class.php');
      $stu = new stu();
      $inUse = $stu->isUsed('ali_id',$value);
    }
    return $inUse;
  }

  /** 
   * Sets the instance property with the given value.
   * This overrides the function in data.abstract.php, because in case of
   * an 'is_active' property all 'referees' will have their state switched as well
   *
   * @param  string  $property The name of the property to be set
   * @param  string  $value    The data to bet set
   * @return boolean Succes or Failure
   * @access public
   */
  function setProperty($property, $value) {
    
    global $SPL_CFG;
    
    $succes = parent::setProperty($property, $value); 
    
    if ($succes){

      if ($property=='is_active') {

        // Switch state in the catalog
        $query  = 'UPDATE '.$SPL_CFG['prefix'].'cat';
        $query .= ' SET is_active = "'.$value.'" WHERE';
        $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
        $query .= ' ali_id = "'.$this->_data['ali_id'].'"';
        $result = db_query($query);

        // Remove default alignment from the suggestion
        if (!$value){
          $query  = 'UPDATE '.$SPL_CFG['prefix'].'sug';
          $query .= ' SET ali_id = 0 WHERE';
          $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
          $query .= ' ali_id = "'.$this->_data['ali_id'].'"';
          $result = db_query($query);
        }
      }
    }
  }

  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Deletes a row from the database.
   * Override the default behaviour by adding the removal of child records
   * 
   * @return boolean Success or Failure
   * @access protected
   * @uses   array   $SPL_CFG Global configuration data
   */
  function _delete() {
    
    global $SPL_CFG;
    
    $succes = parent::_delete();
    
    if ($succes) {

      $query  = 'DELETE FROM '.$SPL_CFG['prefix'].'cat WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
      $query .= ' ali_id = "'.$this->_data['ali_id'].'"';
      $result = db_query($query);

      $query  = 'UPDATE '.$SPL_CFG['prefix'].'sug';
      $query .= ' SET ali_id = 0 WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
      $query .= ' ali_id = "'.$this->_data['ali_id'].'"';
      $result = db_query($query);

    }
    
    return $succes;
  }

}

?>