<?php
/**
 * Suggestions used in a traditional methodology
 *
 * $Author: jebbol $
 * $Revision: 1.10 $
 * $Date: 2006/08/08 14:19:36 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {$SPL_CFG['dirroot'] = '../../..';}

// Defaults
include_once($SPL_CFG['dirroot'].'/inc/defaults.inc.php');

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/sug.abstract.php');

//=========================================================================
// Class definition
//=========================================================================

/**
 * Suggestions
 * 
 * This class describes the suggestions being used in a traditional methodology.
 * These suggestions are used by a student to plan his study. 
 * The following properties are applicable only in a traditional environment:
 * - lst_id: The learning style associated with this suggestion
 * - sug_load: The average time it takes to complete this suggestion 
 * - sug_reward: The reward given for completing a suggestion 
 */
class sug extends abstractSug {
  
  function sug() {
    
    // Call the parent's constructor in order to set the defaults
    parent::abstractSug();
    
    // Create methodology related data
    $this->_data['lst_id']         = (int) 0;
    $this->_data['sug_load']       = (int) 0;
    $this->_data['sug_reward']     = (int) 0;
    $this->_data['tra_id']         = (int) 0;
    $this->_data['ali_id']         = (int) 0;
    $this->_data['sub_id']         = (int) 0;
    $this->_data['sug_courseyear'] = (int) 0;
    
  }

  //=========================================================================
  // Public methods
  //=========================================================================

  /**
   * Returns data for printing a report
   * 
   * @return (array) The data to print
   * @public
   */
  function getReportData() {
    
    global $SPL_CFG;
    
    // Methodology objects
    include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/ali.class.php');
    include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/tra.class.php');

    // Training
    $singleTra = false;
    $tra = new tra();
    $singleTra = $tra->isSingle();

    // Alignments
    $singleAli = false;
    $ali = new ali();
    $singleAli = $ali->isSingle();


    $query  = 'SELECT '.((!$singleTra)?'tra_code,':'').((!$singleAli)?'ali_code,':'').'sub_code,sug_courseyear,LEFT(sug_title,50) AS title,sug_short,sug_code';
    $query .= ' FROM '.$SPL_CFG['prefix'].'sug sug'.((!$singleTra)?' LEFT JOIN '.$SPL_CFG['prefix'].'tra tra ON tra.tra_id = sug.tra_id':'').((!$singleAli)?' LEFT JOIN '.$SPL_CFG['prefix'].'ali ali ON ali.ali_id = sug.ali_id':'').' LEFT JOIN '.$SPL_CFG['prefix'].'sub sub ON sub.sub_id = sug.sub_id';
    $query .= ' WHERE sug.adm_id = "'.$_SESSION['adm_id'].'"  AND sug.is_active';
    $query .= ' ORDER BY '.((!$singleTra)?'tra_code,':'').((!$singleAli)?'ali_code,':'').'sub_code,sug_courseyear,sug_title';

    $data = array();
    $result = db_query($query);
    while($data[] = db_retrieve($result)) {}
    return $data;

  }

  /**
   * Retrieves key values not already in a catalog
   * 
   * @param  integer $tmp_ins_id   Institution
   * @param  integer $tmp_ina_id   Address
   * @param  integer $tmp_tra_id   Training
   * @param  integer $tmp_ali_id   Alignment
   * @param  integer $tmp_sub_id   Subject
   * @param  string  $restrict     (optional) SQL restriction (ex:'ina_id = "1"')
   * @param  string  $orderBy      (optional) Alternative ordering (ex:ina_sequence)
   * @param  string  $orderDir     (optional) 'ASC' or 'DESC'
   * @param  integer $my_sug_id    (optional) The current sug_id (to be added to the list)
   * @return array 
   * @access public
   */
  function getUniqueKeyValues($tmp_ins_id,$tmp_ina_id,$tmp_tra_id,$tmp_ali_id,$tmp_sub_id,$restrict=false,$orderBy=false,$orderDir='ASC',$my_sug_id=0) {

    global $SPL_CFG;
    
    // The column name to return
    if (!isset($this->_name)){
      return (bool)false;
    }
    
    $values = array();

    // Table(s)
    $query  = 'SELECT '.$this->_table.'.'.$this->_primary.' as "key",'.$this->_table.'.'.($orderBy?$orderBy:$this->_name).' as "value"';
    $query .= ' FROM '.$SPL_CFG['prefix'].$this->_table.' '.$this->_table;

    // Columns
    $query .= ' WHERE '.$this->_table.'.adm_id = "'.$_SESSION['adm_id'].'" AND '.$this->_table.'.is_active';
    if ($tmp_tra_id>0) {
      $query .= ' AND ('.$this->_table.'.tra_id = "'.$tmp_tra_id.'" OR '.$this->_table.'.tra_id = 0)';
    }
    if ($tmp_ali_id>0) {
      $query .= ' AND ('.$this->_table.'.ali_id = "'.$tmp_ali_id.'" OR '.$this->_table.'.ali_id = 0)';
    }
    if ($tmp_sub_id>0) {
      $query .= ' AND ('.$this->_table.'.sub_id = "'.$tmp_sub_id.'" OR '.$this->_table.'.sub_id = 0)';
    }
    if ($restrict) {
      $query .= ' AND '.$restrict;
    }

    // Ordering
    $query .= ' ORDER BY '.($orderBy?$orderBy:$this->_name).' '.$orderDir;
    $result = db_query($query);
    while ($row = db_fetch_array($result)) {
      
      // Add the current id to the list
      if ($my_sug_id>0 && $row['key']==$my_sug_id) {
        $values[] = array($row['key']=>$row['value']);
        continue;
      }
      
      // Check the catalog to see if the suggestion is not already there
      $query  = 'SELECT cat_id FROM '.$SPL_CFG['prefix'].'cat cat WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND is_active AND';
      $query .= ' ins_id = "'.$tmp_ins_id.'" AND';
      $query .= ' ina_id = "'.$tmp_ina_id.'" AND';
      $query .= ' tra_id = "'.$tmp_tra_id.'" AND';
      $query .= ' ali_id = "'.$tmp_ali_id.'" AND';
      $query .= ' sub_id = "'.$tmp_sub_id.'" AND';
      $query .= ' sug_id = "'.$row['key'].'"';
      $cat_result = db_retrieve(db_query($query));
      
      if (!$cat_result){
        $values[] = array($row['key']=>$row['value']);
      }
    }
    return $values;
  }

  /** 
   * Sets the instance property with the given value.
   * This overrides the function in data.abstract.php, because in case of
   * an 'is_active' property all 'referees' will have their state switched as well
   *
   * @param  string  $property The name of the property to be set
   * @param  string  $value    The data to bet set
   * @return boolean Succes or Failure
   * @access public
   */
  function setProperty($property, $value) {
    
    global $SPL_CFG;
    
    $succes = parent::setProperty($property, $value); 
    
    if ($succes && $property=='is_active'){
      // Switch state in the catalog
      $query  = 'UPDATE '.$SPL_CFG['prefix'].'cat';
      $query .= ' SET is_active = "'.$value.'" WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
      $query .= ' sug_id = "'.$this->_data['sug_id'].'"';
      $result = db_query($query);
    }
  }



  /**
   * Determines if this suggestion has to be shown in studyguides
   * 
   * @return boolean Show or Hide
   */
  function show_in_guide() {
    return ($this->_data['sug_type'] == 'A');
  }

  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Deletes a row from the database.
   * Override the default behaviour by adding the removal of child records
   * 
   * @return boolean Success or Failure
   * @access protected
   * @uses   array   $SPL_CFG Global configuration data
   */
  function _delete() {
    
    global $SPL_CFG;
    
    $succes = parent::_delete();
    
    if ($succes) {
      $query  = 'DELETE FROM '.$SPL_CFG['prefix'].'cat WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
      $query .= ' sug_id = "'.$this->_data['sug_id'].'"';
      $result = db_query($query);
    }
    
    return $succes;
  }

}

/**
 * $Log: sug.class.php,v $
 * Revision 1.10  2006/08/08 14:19:36  jebbol
 * Added reporting capabilities
 *
 * Revision 1.9  2006/07/04 11:32:27  jebbol
 * Changed setProperty so that it now only switches catalog state when is_active is set
 *
 */
 
?>
