<?php
/**
 * Training class.
 *
 * $Author: jebbol $
 * $Revision: 1.9 $
 * $Date: 2006/08/08 07:08:58 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

if (!isset($SPL_CFG['dirroot'])) {$SPL_CFG['dirroot'] = '../../..';}

// Abstract data class
include_once($SPL_CFG['dirroot'].'/class/data.abstract.php');

//=========================================================================
// Class definition
//=========================================================================

class tra extends abstractData {
  
  function tra() {

    // Call the parent's constructor in order to set the defaults
    parent::abstractData();
    
    // Create an empty instance
    $this->_data['tra_id'] = (int) 0;
    $this->_data['adm_id'] = $_SESSION['adm_id'];
    $this->_data['tra_name'] = '';
    $this->_data['tra_short'] = '';
    $this->_data['tra_code'] = '';
    $this->_data['tra_courseyears'] = (int) 0;
    $this->_data['tra_load'] = (int) 0;
    $this->_data['tra_load_per_year'] = '';
    $this->_data['tra_reward'] = (int) 0;
    $this->_data['tra_reward_per_year'] = '';
    $this->_data['is_active'] = (bool) true;
    $this->_data['create_date'] = time();
    $this->_data['create_usr_id'] = $_SESSION['usr_id'];
    $this->_data['create_src_id'] = $_SESSION['native_id'];
    $this->_data['update_date'] = time();
    $this->_data['update_usr_id'] = $_SESSION['usr_id'];
    $this->_data['update_src_id'] = $_SESSION['native_id'];
    
    // Register the required class properties
    $this->_primary = 'tra_id';
    $this->_table   = 'tra';
    $this->_name    = 'tra_name';
    
    // Register properties that must have a value
    $this->_required[] = $this->_name;

  }
  //=========================================================================
  // Public methods
  //=========================================================================

  /**
   * Checks if there is only one active row in the table and returns the id if that is the case
   * 
   * @param  integer $tmp_ina_id When>0, this will check if there is only one training for this address
   * @return integer The row id, or false if there is either zer or more than one row
   * @access public
   */
  function isSingle($tmp_ina_id=0) {
    
    global $SPL_CFG;
    
    // When no address is specified, check the number of trainings within the administration
    if ($tmp_ina_id==0) {
      return parent::isSingle();
    }
    
    // Else check the number of tranings for the given address
    $query  = 'SELECT child_id FROM '.$SPL_CFG['prefix'].'apl apl';
    $query .= ' LEFT JOIN '.$SPL_CFG['prefix'].'tra tra ON tra.tra_id = apl.child_id';
    $query .= ' WHERE apl.adm_id = "'.$_SESSION['adm_id'].'" AND apl.is_active AND tra.is_active';
    $query .= ' AND apl_parent = "ina" AND parent_id = "'.$tmp_ina_id.'" AND apl_child = "tra"';
    $result = db_query($query);
    if (db_num_rows($result)==1){
      $result = db_retrieve($result);
      return $result['child_id'];
    }
    return (bool)false;
  }

  /**
   * Checks if the given column/value combination is referenced from any objects
   * of this class
   * 
   * @param  string  $column The name of the column
   * @param  string  $value  The value to check
   * @return boolean Used or Unused
   * @access public
   */
  function isUsed($column,$value){

    global $SPL_CFG;

    $inUse = parent::isUsed($column,$value);
    if (!$inUse) {
      // Check students
      include_once($SPL_CFG['dirroot'].'/class/stu.class.php');
      $stu = new stu();
      $inUse = $stu->isUsed('tra_id',$value);
    }
    return $inUse;
  }

  /** 
   * Sets the instance property with the given value.
   * This overrides the function in data.abstract.php, because in case of
   * an 'is_active' property all 'referees' will have their state switched as well
   *
   * @param  string  $property The name of the property to be set
   * @param  string  $value    The data to bet set
   * @return boolean Succes or Failure
   * @access public
   */
  function setProperty($property, $value) {
    
    global $SPL_CFG;
    
    $succes = parent::setProperty($property, $value); 
    
    if ($succes){

      if ($property=='is_active') {

        // Switch state in the catalog
        $query  = 'UPDATE '.$SPL_CFG['prefix'].'cat';
        $query .= ' SET is_active = "'.$value.'" WHERE';
        $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
        $query .= ' tra_id = "'.$this->_data['tra_id'].'"';
        $result = db_query($query);

        // When set to inactive
        if (!$value){
          // Remove default training from the suggestion
          $query  = 'UPDATE '.$SPL_CFG['prefix'].'sug';
          $query .= ' SET tra_id = 0 WHERE';
          $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
          $query .= ' tra_id = "'.$this->_data['tra_id'].'"';
          $result = db_query($query);
          
          // Set 'valid until' date on related data
          $query  = 'UPDATE '.$SPL_CFG['prefix'].'apl';
          $query .= ' SET apl_valid_until = "'.time().'" WHERE';
          $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
          $query .= ' apl_child = "tra" AND';
          $query .= ' child_id = "'.$this->_data['tra_id'].'"';
          $result = db_query($query);
        }

      }
      
    }
  }

  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Deletes a row from the database.
   * Override the default behaviour by adding the removal of child records
   * 
   * @return boolean Success or Failure
   * @access protected
   * @uses   array   $SPL_CFG Global configuration data
   */
  function _delete() {
    
    global $SPL_CFG;
    
    $succes = parent::_delete();
    
    if ($succes) {
      $query  = 'DELETE FROM '.$SPL_CFG['prefix'].'cat WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
      $query .= ' tra_id = "'.$this->_data['tra_id'].'"';
      $result = db_query($query);

      // Remove default training from the suggestion
      $query  = 'UPDATE '.$SPL_CFG['prefix'].'sug';
      $query .= ' SET tra_id = 0 WHERE';
      $query .= ' adm_id = "'.$_SESSION['adm_id'].'" AND';
      $query .= ' tra_id = "'.$this->_data['tra_id'].'"';
      $result = db_query($query);

    }
    
    return $succes;
  }

}

/**
 * $Log: tra.class.php,v $
 * Revision 1.9  2006/08/08 07:08:58  jebbol
 * Changed repository (cvs.sourceforge.net -> splanner.cvs.sourceforge.net)
 *
 * Revision 1.8  2006/03/28 20:26:19  jebbol
 * #1459165: Now setting the 'valid_until' date of related rows upon de-activating a training
 */
?>