<?php
/**
 * User interface for manipulating data in a catalog object
 *
 * $Author: jebbol $
 * $Revision: 1.14 $
 * $Date: 2006/08/08 06:59:28 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.nl
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

// Configuration
include_once('../../../inc/config.php');

// Page generation
include_once($SPL_CFG['dirroot'].'/ui/class/page.class.php');

// Utility functions
include_once($SPL_CFG['dirroot'].'/ui/inc/util.inc.php');

// Core data objects
include_once($SPL_CFG['dirroot'].'/class/ina.class.php');
include_once($SPL_CFG['dirroot'].'/class/ins.class.php');
include_once($SPL_CFG['dirroot'].'/class/prd.class.php');
include_once($SPL_CFG['dirroot'].'/class/usd.class.php');

// Methodology classes
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/ali.class.php');
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/cat.class.php');
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/sub.class.php');
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/sug.class.php');
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/tra.class.php');

//=========================================================================
// Sanitize request variables
//=========================================================================

$sanitize = array(
  'cat_id'=>array('type'=>'int'),
  'ins_id'=>array('type'=>'int'),
  'ina_id'=>array('type'=>'int'),
  'tra_id'=>array('type'=>'int'),
  'ali_id'=>array('type'=>'int'),
  'sub_id'=>array('type'=>'int'),
  'sug_id'=>array('type'=>'int'),
  'prd_id_start'=>array('type'=>'int'),
  'prd_id_end'=>array('type'=>'int'),
  'cat_courseyear_start'=>array('type'=>'int','min'=>0,'max'=>99),
  'cat_courseyear_end'=>array('type'=>'int','min'=>0,'max'=>99),
  'cat_month_start'=>array('type'=>'int','min'=>0,'max'=>99),
  'cat_month_end'=>array('type'=>'int','min'=>0,'max'=>99),
  'cat_week_start'=>array('type'=>'int','min'=>0,'max'=>999),
  'cat_week_end'=>array('type'=>'int','min'=>0,'max'=>999),
  'cat_day_start'=>array('type'=>'int','min'=>0,'max'=>9999),
  'cat_day_end'=>array('type'=>'int','min'=>0,'max'=>9999),
  'cat_valid_from'=>array('type'=>'date'),
  'cat_valid_until'=>array('type'=>'date'),
);
if (isset($_SESSION['is_admin'])&&$_SESSION['is_admin']==true) {
  $sanitize['is_active'] = array('type'=>'bool');
}
require_once($SPL_CFG['dirroot'].'/ui/inc/sanitize.inc.php');

//=========================================================================
// Prerequisites
//=========================================================================

// Must be logged in
include_once($SPL_CFG['dirroot'].'/ui/inc/logincheck.php');

//=========================================================================
// Local variables
//=========================================================================

$object = new cat();
$editMode = (bool)false;

// Defaults
$usd = new usd();
$usd_script = 'cat';

//=========================================================================
// Methodology
//=========================================================================

// Human language translations
$m_trans = $SPL_CFG['dirroot'].'/methodology/traditional/ui/lang/'.$_SESSION['usp_lang'].'/cat.lang.php';
// Metadata
$m_meta = $SPL_CFG['dirroot'].'/methodology/traditional/ui/meta/cat.metadata.php';

//=========================================================================
// Handle events
//=========================================================================

// Are we returning from searching a reference field?
$from_id = (bool)false;
if (isset($clean['from'])){
  $from_id = isset($clean['id'])?$clean['id']:false;
}
    
// Populate the object
if (isset($clean['id'])) {
  if ($object->init($clean['id'])){
    $_SESSION['cat_id'] = $clean['id'];
  }
}

// Get the 'current' object
else {
  if (isset($_SESSION['cat_id'])) {
    $object->init($_SESSION['cat_id']);
  }
  // if there is no current object, start a search
  else  if (!isset($clean['button_action'])) {
    redirect($SPL_CFG['wwwroot'].'/methodology/traditional/ui/cat_search.php');
  }
}

// Capture the search on reference fields
if (isset($clean['search_button'])) {
  // Save the form data
  save();
  // Redirect browser to the search page
  if ($clean['search_button']=='ins_id'){
    redirect($SPL_CFG['wwwroot'].'/ui/ins_search.php?from=ins_id');
  }
  if ($clean['search_button']=='ina_id'){
    redirect($SPL_CFG['wwwroot'].'/ui/ina_search.php?from=ina_id');
  }
  if ($clean['search_button']=='tra_id'){
    redirect($SPL_CFG['wwwroot'].'/methodology/traditional/ui/tra_search.php?from=tra_id');
  }
  if ($clean['search_button']=='ali_id'){
    redirect($SPL_CFG['wwwroot'].'/methodology/traditional/ui/ali_search.php?from=ali_id');
  }
  if ($clean['search_button']=='sug_id'){
    redirect($SPL_CFG['wwwroot'].'/methodology/traditional/ui/sug_search.php?from=sug_id');
  }
  if ($clean['search_button']=='cat_id'){
    redirect($SPL_CFG['wwwroot'].'/methodology/traditional/ui/cat_search.php?from=cat_id_parent');
  }
  if ($clean['search_button']=='prd_id'){
    redirect($SPL_CFG['wwwroot'].'/ui/prd_search.php?from=prd_id');
  }
}

// Events from the Toolbar buttons
if (isset($clean['button_action']) && $clean['button_action']){  
  switch ($clean['button_action']) {
    case 'cancel':
      break;
    case 'del':
      //$object->delete($object->getProperty('is_active')=='1');
      $object->delete($object->getProperty(false));
      unset($_SESSION['cat_id']);
      redirect($SPL_CFG['wwwroot'].'/methodology/traditional/ui/cat_search.php');
      break;
    case 'edit':
      $editMode = (bool)true;
      break;
    case 'find':
      save();
      redirect($SPL_CFG['wwwroot'].'/methodology/traditional/ui/cat_search.php');
      break;
    case 'new':
      $object = new cat();
      $editMode = (bool)true;
      unset($_SESSION['cat_id']);
      break;
    case 'print':
      if ($editMode){
        $object->save();
      }
      redirect($SPL_CFG['wwwroot'].'/ui/print.php?class=cat');
      break;
    case 'save':
      $editMode = (bool)true;
      if (!save()) {
        // Retrieve message
        $tmp = $object->getMessages();
        if (is_array($tmp)&&count($tmp)>0) {
          $_SESSION['msg']                 = array_pop($tmp);
          $_SESSION['referred_from']       = $_SERVER['PHP_SELF'];
          $_SESSION['messagebox_icontype'] = 'disappointed';
          redirect($SPL_CFG['wwwroot'].'/ui/message.php');
        }
      }
      $editMode = (bool)false;
      break;
    case 'list':
      set();
      $editMode = (bool)true;
      break;
    default:
      break;
  }
}

// Set the chosen reference field
if ($from_id){
  if ((!isset($_SESSION['cat_id'])) || ($from_id != $_SESSION['cat_id'])){
    $object->setProperty($clean['from'],$from_id);
  }
}

//=========================================================================
// Definition of the page
//=========================================================================

$page = new page();
$page->init('cat','default','template.htm',true,false,$m_trans,$m_meta);
$page->startForm();
  $page->addToolbar(!$editMode,!$editMode&&$object->isLoaded(),$editMode,!$editMode,$editMode,!$editMode);
  $page->startAreaData();
  $page->startSection();
    $page->startSubsection();
    
      // Is this a new object?
      $new = $object->isNew();

      // Institution
      require ($SPL_CFG['dirroot'].'/ui/inc/field.ins_id.inc.php');

      // Institution address: when there is only one instance, hide it
      require ($SPL_CFG['dirroot'].'/ui/inc/field.ina_id.inc.php');

      // Training
      require ($SPL_CFG['dirroot'].'/ui/inc/field.tra_id.inc.php');
      
      // Alignment
      require ($SPL_CFG['dirroot'].'/ui/inc/field.ali_id.inc.php');
      
      // Subject
      require ($SPL_CFG['dirroot'].'/ui/inc/field.sub_id.inc.php');
      
      // Suggestion
      if ($editMode){
        // When this catalog item has a parent, show child-suggestions only
        $parent_sug = 0;
        if ((int)$object->getProperty('cat_id_parent')>0){
          $parent_sug = $object->getParentSuggestion();
        }
        $sug = new sug();
        if ($parent_sug>0) {
          $page->addField('sug_id',$object->getProperty('sug_id'),!$editMode,false,false,$sug->getUniqueKeyValues($object->getProperty('ins_id'),$object->getProperty('ina_id'),$object->getProperty('tra_id'),$object->getProperty('ali_id'),$object->getProperty('sub_id'),'sug_id_parent = "'.$parent_sug.'"','sug_title','ASC',$object->getProperty('sug_id')));
        }
        else {
          $page->addField('sug_id',$object->getProperty('sug_id'),!$editMode,false,false,$sug->getUniqueKeyValues($object->getProperty('ins_id'),$object->getProperty('ina_id'),$object->getProperty('tra_id'),$object->getProperty('ali_id'),$object->getProperty('sub_id'),'sug_type = ""','sug_title','ASC',$object->getProperty('sug_id')));
        }
      }
      else {
        $page->addField('sug_id',$object->getProperty('sug_id'),!$editMode);
      }

      $page->addField('cat_valid_from',$object->getProperty('cat_valid_from'),!$editMode);
      $page->addField('cat_valid_until',$object->getProperty('cat_valid_until'),!$editMode);

      // Active
      require ($SPL_CFG['dirroot'].'/ui/inc/field.is_active.inc.php');

      $page->showFormButtons();
    $page->endSubsection();
    $page->startSubsection('subsection2');
      $page->startRow();
      $page->startColumn();
      $page->startPane($SPL_TXT['pane_start'],'noclose','page');
        $page->startTable();
        
        //  Courseyear start
        if ($editMode&&$new) {
          $tmp = $usd->getDefault($usd_script,'cat_courseyear_start');
          if ($tmp) {
            $object->setProperty('cat_courseyear_start',$tmp);
          }
        }
        $page->addField('cat_courseyear_start',$object->getProperty('cat_courseyear_start'),!$editMode);

        // Period start
        if ($editMode&&$new){
          $tmp = $usd->getDefault($usd_script,'prd_id_start');
          if ($tmp) {
            $object->setProperty('prd_id_start',$tmp);
          }
          $prd = new prd();
          $page->addField('prd_id_start',$object->getProperty('prd_id_start'),!$editMode,false,false,$prd->getKeyValues('ina',$object->getProperty('ina_id')));
        }
        else {
          $page->addField('prd_id_start',$object->getProperty('prd_id_start'),!$editMode);
        }

        // Month start
        if ($editMode&&$new) {
          $tmp = $usd->getDefault($usd_script,'cat_month_start');
          if ($tmp) {
            $object->setProperty('cat_month_start',$tmp);
          }
        }
        $page->addField('cat_month_start',$object->getProperty('cat_month_start'),!$editMode);
        
        // Week start
        if ($editMode&&$new) {
          $tmp = $usd->getDefault($usd_script,'cat_week_start');
          if ($tmp) {
            $object->setProperty('cat_week_start',$tmp);
          }
        }
        $page->addField('cat_week_start',$object->getProperty('cat_week_start'),!$editMode);

        // Day start
        if ($editMode&&$new) {
          $tmp = $usd->getDefault($usd_script,'cat_day_start');
          if ($tmp) {
            $object->setProperty('cat_day_start',$tmp);
          }
        }
        $page->addField('cat_day_start',$object->getProperty('cat_day_start'),!$editMode);
        $page->endTable();
      $page->endPane('page');
      $page->endColumn();
      $page->endRow();
      $page->startRow();
      $page->startColumn();
      $page->startPane($SPL_TXT['pane_end'],'noclose','page');
        $page->startTable();

        // Courseyear end
        if ($editMode&&$new) {
          $tmp = $usd->getDefault($usd_script,'cat_courseyear_end');
          if ($tmp) {
            $object->setProperty('cat_courseyear_end',$tmp);
          }
        }
        $page->addField('cat_courseyear_end',$object->getProperty('cat_courseyear_end'),!$editMode);

        // Period end
        if ($editMode&&$new){
          $tmp = $usd->getDefault($usd_script,'prd_id_end');
          if ($tmp) {
            $object->setProperty('prd_id_end',$tmp);
          }
          $prd = new prd();
          $page->addField('prd_id_end',$object->getProperty('prd_id_end'),!$editMode,false,false,$prd->getKeyValues('ina',$object->getProperty('ina_id')));
        }
        else {
          $page->addField('prd_id_end',$object->getProperty('prd_id_end'),!$editMode);
        }

        // Month end
        if ($editMode&&$new) {
          $tmp = $usd->getDefault($usd_script,'cat_month_end');
          if ($tmp) {
            $object->setProperty('cat_month_end',$tmp);
          }
        }
        $page->addField('cat_month_end',$object->getProperty('cat_month_end'),!$editMode);

        // Week end
        if ($editMode&&$new) {
          $tmp = $usd->getDefault($usd_script,'cat_week_end');
          if ($tmp) {
            $object->setProperty('cat_week_end',$tmp);
          }
        }
        $page->addField('cat_week_end',$object->getProperty('cat_week_end'),!$editMode);

        // Day end
        if ($editMode&&$new) {
          $tmp = $usd->getDefault($usd_script,'cat_day_end');
          if ($tmp) {
            $object->setProperty('cat_day_end',$tmp);
          }
        }
        $page->addField('cat_day_end',$object->getProperty('cat_day_end'),!$editMode);
        $page->endTable();
      $page->endPane('page');
      $page->endColumn();
      $page->endRow();
    $page->endSubsection();
  $page->endSection();
  $page->endAreaData();
  $page->addFootbar();
$page->endForm();

//=========================================================================
// Show the page
//=========================================================================

$page->setContent();
$page->createPage();
if ($object->isLoaded()) {
  $page->setRowStatistics($object->getPrimary(),$object->getProperty('create_date'),$object->getProperty('create_usr_id'),$object->getProperty('create_src_id'),$object->getProperty('update_date'),$object->getProperty('update_usr_id'),$object->getProperty('update_src_id'));
}
else {
  $page->setRowStatistics();
}
$page->showPage();

//=========================================================================
// Functions
//=========================================================================

function set(){
  
  global $object,$clean;

  if (isset ($clean['ins_id']))               {$object->setProperty('ins_id', $clean['ins_id']);}
  if (isset ($clean['ina_id']))               {$object->setProperty('ina_id', $clean['ina_id']);}
  if (isset ($clean['tra_id']))               {$object->setProperty('tra_id', $clean['tra_id']);}
  if (isset ($clean['ali_id']))               {$object->setProperty('ali_id', $clean['ali_id']);}
  if (isset ($clean['sub_id']))               {$object->setProperty('sub_id', $clean['sub_id']);}
  if (isset ($clean['cat_courseyear_start'])) {$object->setProperty('cat_courseyear_start', $clean['cat_courseyear_start']);}
  if (isset ($clean['cat_courseyear_end']))   {$object->setProperty('cat_courseyear_end', $clean['cat_courseyear_end']);}
  if (isset ($clean['prd_id_start']))         {$object->setProperty('prd_id_start', $clean['prd_id_start']);}
  if (isset ($clean['cat_month_start']))      {$object->setProperty('cat_month_start', $clean['cat_month_start']);}
  if (isset ($clean['cat_week_start']))       {$object->setProperty('cat_week_start', $clean['cat_week_start']);}
  if (isset ($clean['cat_day_start']))        {$object->setProperty('cat_day_start', $clean['cat_day_start']);}
  if (isset ($clean['prd_id_end']))           {$object->setProperty('prd_id_end', $clean['prd_id_end']);}
  if (isset ($clean['cat_month_end']))        {$object->setProperty('cat_month_end', $clean['cat_month_end']);}
  if (isset ($clean['cat_week_end']))         {$object->setProperty('cat_week_end', $clean['cat_week_end']);}
  if (isset ($clean['cat_day_end']))          {$object->setProperty('cat_day_end', $clean['cat_day_end']);}
  if (isset ($clean['cat_sequence']))         {$object->setProperty('cat_sequence', $clean['cat_sequence']);}
  if (isset ($clean['sug_id']))               {$object->setProperty('sug_id', $clean['sug_id']);}
  if (isset ($clean['cat_id_parent']))        {$object->setProperty('cat_id_parent', $clean['cat_id_parent']);}
  if (isset ($clean['is_mandatory']))         {$object->setProperty('is_mandatory', $clean['is_mandatory']);}
  if (isset ($clean['is_active']))            {$object->setProperty('is_active', $clean['is_active']);}
}

function save(){
  
  global $object,$usd,$usd_script,$editMode,$clean;
  
  if (!$editMode){
    return;
  }
  
  if (isset($clean['button_action'])){
    if ($clean['button_action']=='cancel'){
      return;
    }
  }
  set();
  $object->save();
  $_SESSION['cat_id'] = $object->getProperty('cat_id');
  
  // Save user defaults
  $usd->setDefault($usd_script,'ins_id',$object->getProperty('ins_id'));
  $usd->setDefault($usd_script,'ina_id',$object->getProperty('ina_id'));
  $usd->setDefault($usd_script,'tra_id',$object->getProperty('tra_id'));
  $usd->setDefault($usd_script,'ali_id',$object->getProperty('ali_id'));
  $usd->setDefault($usd_script,'sub_id',$object->getProperty('sub_id'));
  $usd->setDefault($usd_script,'cat_courseyear_start',$object->getProperty('cat_courseyear_start'));
  $usd->setDefault($usd_script,'cat_courseyear_end',$object->getProperty('cat_courseyear_end'));
  $usd->setDefault($usd_script,'prd_id_start',$object->getProperty('prd_id_start'));
  $usd->setDefault($usd_script,'prd_id_end',$object->getProperty('prd_id_end'));
  $usd->setDefault($usd_script,'cat_month_start',$object->getProperty('cat_month_start'));
  $usd->setDefault($usd_script,'cat_month_end',$object->getProperty('cat_month_end'));
  $usd->setDefault($usd_script,'cat_week_start',$object->getProperty('cat_week_start'));
  $usd->setDefault($usd_script,'cat_week_end',$object->getProperty('cat_week_end'));
  $usd->setDefault($usd_script,'cat_day_start',$object->getProperty('cat_day_start'));
  $usd->setDefault($usd_script,'cat_day_end',$object->getProperty('cat_day_end'));
}

/**
 * $Log: cat.php,v $
 * Revision 1.14  2006/08/08 06:59:28  jebbol
 * When deleting an item, the item is now removed from the database (instead of being set to inactive)
 *
 * Revision 1.13  2006/02/11 16:38:35  jebbol
 * Renamed ui/inc/util.php -> ui/inc/util.inc.php
 * Added UI areas to support multiple skins
 */

?>
