<?php
/**
 * Include file containing the pdf print logic for a traditional studyguide 
 *
 * $Author: jebbol $
 * $Revision: 1.3 $
 * $Date: 2006/08/08 12:55:46 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Data objects
//=========================================================================
include_once($SPL_CFG['dirroot'].'/class/sgd.class.php');
include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
include_once($SPL_CFG['dirroot'].'/class/stu.class.php');
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/sug.class.php');

// PDF class
include_once($SPL_CFG['dirroot'].'/class/fpdf/fpdf.class.php');

//=========================================================================
// Parameters
//=========================================================================

// Find the preferred student
$prefUsr = $_SESSION['usr_id'];
if (isset($_GET['sgd_search_user'])){
  $prefUsr = $_GET['sgd_search_user'];
}

// Find the preferred subject
$prefSub = 0;
if (isset($_GET['sgd_search_subjects'])){
  $prefSub = $_GET['sgd_search_subjects'];
  $subCode = _getDesc('sub',$prefSub,'sub_short');
}

// Find the preferred period
$prefPer = 0;
if (isset($_GET['sgd_search_periods'])){
  $prefPer = $_GET['sgd_search_periods'];
  $prdCode = _getDesc('prd',$prefPer,'prd_short');
}

// Find the preferred week
$prefWk = 0;
if (isset($_GET['sgd_search_week'])){
  $prefWk = $_GET['sgd_search_week'];
  $wkCode = $SPL_TXT['week'].' '.$prefWk;
}

// Find the preferred status
$prefSts = 0;
if (isset($_GET['sgd_search_status'])){
  $prefSts = $_GET['sgd_search_status'];
  $stsCode = _getDesc('sts',$prefPer,'sts_short');
}

//=========================================================================
// Local variables
//=========================================================================
$descBuffer = array();

if( !function_exists( 'html_entity_decode' ) ) {
  function html_entity_decode( $given_html, $quote_style = ENT_QUOTES ) {
    $trans_table = array_flip(get_html_translation_table( HTML_SPECIALCHARS, $quote_style ));
    $trans_table['&#39;'] = "'";
    return ( strtr( $given_html, $trans_table ) );
  }
}

// User data
$usr = new usr();
$usr->init($prefUsr);
$name = $usr->getName();

// Student data
$stu = new stu();
$stu->init($usr->getProperty('stu_id'));
$courseyear = $SPL_TXT['courseyear'].' '.$stu->getProperty('stu_courseyear');

// Page title
$title = str_replace('%s',$name,$SPL_TXT['report_title']);

// Studyguide
$sgd = new sgd();
$sgd->actualize($prefUsr);
$stats = $sgd->getStatistics($prefUsr);

//=========================================================================
// Initialize report
//=========================================================================

$pdf = new PDF();
$pdf->AliasNbPages();

//=========================================================================
// Assemble the report
//=========================================================================

// Title page
$pdf->addPage();
$pdf->SetFont('Times','B',22);
$pdf->Ln(60);
$pdf->Cell(0,0,$title,0,1,'C');
$pdf->SetFont('Times','B',18);
$pdf->Ln(10);
$pdf->Cell(0,0,$courseyear,0,1,'C');

if ($prefSub >0) {
  $pdf->Ln(10);
  $pdf->Cell(0,0,_getDesc('sub',$prefSub,'sub_name'),0,1,'C');
}
if ($prefPer >0) {
  $pdf->Ln(10);
  $pdf->Cell(0,0,_getDesc('prd',$prefSub,'prd_name'),0,1,'C');
}
if ($prefWk >0) {
  $pdf->Ln(10);
  $pdf->Cell(0,0,$SPL_TXT['week'].' '.$prefWk,0,1,'C');
}
if ($prefSts >0) {
  $pdf->Ln(10);
  $pdf->Cell(0,0,_getDesc('sts',$prefSts,'sts_name'),0,1,'C');
}


// Find suggestions
$sugQuery['columns']  = 'DISTINCT sug_id_parent'; 
$sugQuery['tblName']  = 'sgd sgd LEFT JOIN '.$SPL_CFG['prefix'].'sug sug ON sug.sug_id = sgd.sug_id';
$sugQuery['where']    = 'WHERE sgd.usr_id = "'.$prefUsr.'" AND sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active AND NOT sgd.is_done AND sug_type = "A"';
if ((int)$prefSub>0){
  $sugQuery['where'] .= ' AND sgd.sub_id = "'.$prefSub.'"';
}
if ((int)$prefPer>0){
  $sugQuery['where'] .= ' AND sgd.prd_id_start = "'.$prefPer.'"';
}
if ((int)$prefWk>0){
  $sugQuery['where'] .= ' AND sgd.sgd_courseweek_start = "'.$prefWk.'"';
}
if ((int)$prefSts>0){
  $sugQuery['where'] .= ' AND sgd.sts_id = "'.$prefSts.'"';
}
$sugQuery['orderBy']  = 'sgd_caldate_start,sug_id_parent,sug_sequence';
$sugQuery['orderDir'] = 'ASC';
$query = 'SELECT '.$sugQuery['columns'].' FROM '.$SPL_CFG['prefix'].$sugQuery['tblName'].' '.$sugQuery['where'].' ORDER BY '.$sugQuery['orderBy'].' '.$sugQuery['orderDir'];
$allSuggestions = db_query($query);
$done = array();
while($parent = db_retrieve($allSuggestions)) {
  
  if (in_array($parent['sug_id_parent'],$done)){
    continue;
  }
  $done[] = $parent['sug_id_parent'];
  
  // Get suggestion
  $query  = 'SELECT sug_id,sug_title,sug_description,sug_knowledge,sug_directions,sug_prerequisites,sug_materials';
  $query .= ' FROM '.$SPL_CFG['prefix'].'sug sug';
  $query .= ' WHERE sug_id = "'.$parent['sug_id_parent'].'" LIMIT 1';

  $aSuggestion = db_retrieve(db_query($query));
  $sug = new sug();
  $sug->init($aSuggestion['sug_id']);

  $pdf->addPage();
  $pdf->SetFont('Times','B',18);
  $pdf->Cell(0,0,$aSuggestion['sug_title'],0,1,'C');
  
  // Get formatted description
  $desc = $sug->getPropertyFormatted('sug_description');
  if (!empty($desc)){
    $desc = html_entity_decode($desc);
    $pdf->Ln(10);
    $pdf->SetFont('Arial','B',12);
    $pdf->Cell(0,0,$SPL_TXT['sug_description']);
    $pdf->Ln(5);
    $pdf->SetFont('Arial','',10);
    $pdf->WriteHTML($desc);
  }
  
  // Get formatted knowledge
  $desc = $sug->getPropertyFormatted('sug_knowledge');
  if (!empty($desc)){
    $desc = html_entity_decode($desc);
    $pdf->Ln(16);
    $pdf->SetFont('Arial','B',12);
    $pdf->Cell(0,0,$SPL_TXT['sug_knowledge']);
    $pdf->Ln(5);
    $pdf->SetFont('Arial','',10);
    $pdf->WriteHTML($desc);
  }

  // Get formatted directions
  $desc = $sug->getPropertyFormatted('sug_directions');
  if (!empty($desc)){
    $desc = html_entity_decode($desc);
    $pdf->Ln(16);
    $pdf->SetFont('Arial','B',12);
    $pdf->Cell(0,0,$SPL_TXT['sug_directions']);
    $pdf->Ln(5);
    $pdf->SetFont('Arial','',10);
    $pdf->WriteHTML($desc);
  }
  
  // Get formatted prerequisites
  $desc = $sug->getPropertyFormatted('sug_prerequisites');
  if (!empty($desc)){
    $desc = html_entity_decode($desc);
    $pdf->Ln(16);
    $pdf->SetFont('Arial','B',12);
    $pdf->Cell(0,0,$SPL_TXT['sug_prerequisites']);
    $pdf->Ln(5);
    $pdf->SetFont('Arial','',10);
    $pdf->WriteHTML($desc);
  }
  
  // Get formatted materials
  $desc = $sug->getPropertyFormatted('sug_materials');
  if (!empty($desc)){
    $desc = html_entity_decode($desc);
    $pdf->Ln(16);
    $pdf->SetFont('Arial','B',12);
    $pdf->Cell(0,0,$SPL_TXT['sug_materials']);
    $pdf->Ln(5);
    $pdf->SetFont('Arial','',10);
    $pdf->WriteHTML($desc);
  }

  // Get activities
  $actQuery['columns']  = ((isset($SPL_CFG['weeknumber_mode'])&&$SPL_CFG['weeknumber_mode']=='calendar')?'sgd_calweek_start':'sgd_courseweek_start').',IF(sug_description="",sug_title,sug_description) AS description,fmt_short,sts_short';
  $actQuery['tblName']  = 'sgd sgd LEFT JOIN '.$SPL_CFG['prefix'].'sug sug ON sug.sug_id = sgd.sug_id LEFT JOIN '.$SPL_CFG['prefix'].'sts sts ON sts.sts_id = sgd.sts_id LEFT JOIN '.$SPL_CFG['prefix'].'fmt fmt ON fmt.fmt_id = sug.fmt_id';
  //$actQuery['where']    = 'WHERE sgd.usr_id = "'.$prefUsr.'" AND sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active AND NOT sgd.is_done AND sug_type = "A" AND sug_id_parent = "'.$aSuggestion['sug_id'].'"';
  $actQuery['where']    = 'WHERE sgd.usr_id = "'.$prefUsr.'" AND sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active AND sug_type = "A" AND sug_id_parent = "'.$aSuggestion['sug_id'].'"';
  $actQuery['orderBy']  = 'sgd_courseweek_start';
  $actQuery['orderDir'] = 'ASC';
  $query = 'SELECT '.$actQuery['columns'].' FROM '.$SPL_CFG['prefix'].$actQuery['tblName'].' '.$actQuery['where'].' ORDER BY '.$actQuery['orderBy'].' '.$actQuery['orderDir'];

  $pdf->addPage();
  $pdf->SetFont('Arial','',8);


  // Add columns
  $pdf->AddCol(((isset($SPL_CFG['weeknumber_mode'])&&$SPL_CFG['weeknumber_mode']=='calendar')?'sgd_calweek_start':'sgd_courseweek_start'),10,'Wk','C');
  $pdf->AddCol('description',130,$SPL_TXT['sug_title'],'L');
  $pdf->AddCol('fmt_short',20,$SPL_TXT['fmt_short'],'L');
  $pdf->AddCol('sts_short',20,$SPL_TXT['sts_short'],'L');
  $pdf->SetWidths(array(10,130,20,20));
  $pdf->Table($query);


}
// End suggestions


// Show report
$pdf->Output();


/**
 * Retrieves a description from a table
 * 
 * @param  string  $table Table name
 * @param  integer $id    Row id
 * @param  string  $field name of the field containing the description
 * @return string         Description
 */
function _getDesc($table,$id,$field) {
  
  global $SPL_CFG,$descBuffer;
  
  if (is_array($descBuffer)) {
    if (!array_key_exists($table,$descBuffer)) {
      $descBuffer[$table] = array();
    }
  
    if (!array_key_exists($id,$descBuffer[$table])) {
      $query = 'SELECT '.$field.' FROM '.$SPL_CFG['prefix'].$table.' WHERE '.$table.'_id = "'.$id.'" LIMIT 1';
      $result = db_retrieve(db_query($query));
      if (isset($result[$field])) {
        $descBuffer[$table][$id] = $result[$field];
      }
    }
  
    return $descBuffer[$table][$id];
  }
  
  else {
    return '';
  }

}

/**
 * $Log: sgd_fpdf.inc.php,v $
 * Revision 1.3  2006/08/08 12:55:46  jebbol
 * Added calender/courseweek setting
 *
 * Revision 1.2  2006/08/08 12:48:28  jebbol
 * Added a check for non-existing array in _getDesc
 *
 * Revision 1.1  2006/08/08 07:03:33  jebbol
 * Added in version 0.15
 *
 * Revision 1.5  2006/04/17 10:54:39  jebbol
 * Added decoding html_entity_decode
 *
 */
 
?>
