<?php
/**
 * Include file containing the pdf print logic for a traditional studyguide 
 *
 * $Author: jebbol $
 * $Revision: 1.5 $
 * $Date: 2006/04/17 10:54:39 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Data objects
//=========================================================================
include_once($SPL_CFG['dirroot'].'/class/sgd.class.php');
include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/sug.class.php');

// PDF class
include_once($SPL_CFG['dirroot'].'/class/pdf/pdf.class.php');

//=========================================================================
// Parameters
//=========================================================================

// Find the preferred student
$prefUsr = $_SESSION['usr_id'];
if (isset($_GET['sgd_search_user'])){
  $prefUsr = $_GET['sgd_search_user'];
}

// Find the preferred subject
$prefSub = 0;
if (isset($_GET['sgd_search_subjects'])){
  $prefSub = $_GET['sgd_search_subjects'];
}

// Find the preferred period
$prefPer = 0;
if (isset($_GET['sgd_search_periods'])){
  $prefPer = $_GET['sgd_search_periods'];
}

// Find the preferred week
$prefWk = 0;
if (isset($_GET['sgd_search_week'])){
  $prefWk = $_GET['sgd_search_week'];
}

// Find the preferred status
$prefSts = 0;
if (isset($_GET['sgd_search_status'])){
  $prefSts = $_GET['sgd_search_status'];
}

//=========================================================================
// Local variables
//=========================================================================
$descBuffer = array();

if( !function_exists( 'html_entity_decode' ) )
{
   function html_entity_decode( $given_html, $quote_style = ENT_QUOTES ) {
       $trans_table = array_flip(get_html_translation_table( HTML_SPECIALCHARS, $quote_style ));
       $trans_table['&#39;'] = "'";
       return ( strtr( $given_html, $trans_table ) );
       }
}

//=========================================================================
// Initialize report
//=========================================================================

$pdf =& new pdf();
$pdf->selectFont($SPL_CFG['dirroot'].'/class/pdf/fonts/Helvetica');

// Studyguide
$sgd = new sgd();
$sgd->actualize($prefUsr);
$stats = $sgd->getStatistics($prefUsr);

//=========================================================================
// Assemble the report
//=========================================================================
$pdf -> ezSetMargins(50,70,50,50);

// Page header
$all = $pdf->openObject();
$pdf->saveState();
$pdf->setStrokeColor(0,0,0,1);
$pdf->addText(20,20,6,'Studiewijzer.nl');
$pdf->line(20,40,578,40);
$pdf->restoreState();
$pdf->closeObject();
// note that object can be told to appear on just odd or even pages by changing 'all' to 'odd'
// or 'even'.
$pdf->addObject($all,'all');


// Title page
$usr = new usr();
$usr->init($prefUsr);
$name = $usr->getName();
$title = str_replace('%s',$name,$SPL_TXT['report_title']);

$pdf->addJpegFromFile($SPL_CFG['dirroot'].'/ui/skin/studiewijzer_logo.jpg',95,$pdf->y-100);
$pdf->ezSetDy(-140);
$pdf->ezText($title."\n",30,array('justification'=>'centre'));
if ($prefSub >0) {
  $pdf->ezText(_getDesc('sub',$prefSub,'sub_name'),18,array('justification'=>'centre'));
}
if ($prefPer >0) {
  $pdf->ezText(_getDesc('prd',$prefPer,'prd_name'),18,array('justification'=>'centre'));
}
if ($prefWk >0) {
  $pdf->ezText($SPL_TXT['week'].' '.$prefWk,18,array('justification'=>'centre'));
}
if ($prefSts >0) {
  $pdf->ezText(_getDesc('sts',$prefSts,'sts_name'),18,array('justification'=>'centre'));
}
$pdf->ezStartPageNumbers(300,50,20,'','',1);

// Find suggestions
$sugQuery['columns']  = 'DISTINCT sug_id_parent'; 
$sugQuery['tblName']  = 'sgd sgd LEFT JOIN '.$SPL_CFG['prefix'].'sug sug ON sug.sug_id = sgd.sug_id';
$sugQuery['where']    = 'WHERE sgd.usr_id = "'.$prefUsr.'" AND sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active AND NOT sgd.is_done AND sug_type = "A"';
if ((int)$prefSub>0){
  $sugQuery['where'] .= ' AND sgd.sub_id = "'.$prefSub.'"';
}
if ((int)$prefPer>0){
  $sugQuery['where'] .= ' AND sgd.prd_id_start = "'.$prefPer.'"';
}
if ((int)$prefWk>0){
  $sugQuery['where'] .= ' AND sgd.sgd_courseweek_start = "'.$prefWk.'"';
}
if ((int)$prefSts>0){
  $sugQuery['where'] .= ' AND sgd.sts_id = "'.$prefSts.'"';
}
$sugQuery['orderBy']  = 'sgd_caldate_start,sug_id_parent,sug_sequence';
$sugQuery['orderDir'] = 'ASC';
$query = 'SELECT '.$sugQuery['columns'].' FROM '.$SPL_CFG['prefix'].$sugQuery['tblName'].' '.$sugQuery['where'].' ORDER BY '.$sugQuery['orderBy'].' '.$sugQuery['orderDir'];
$allSuggestions = db_query($query);
$done = array();
while($parent = db_retrieve($allSuggestions)) {
  
  if (in_array($parent['sug_id_parent'],$done)){
    continue;
  }
  $done[] = $parent['sug_id_parent'];
  
  // Get suggestion
  $query  = 'SELECT sug_id,sug_title,sug_description,sug_knowledge,sug_directions,sug_prerequisites,sug_materials';
  $query .= ' FROM '.$SPL_CFG['prefix'].'sug sug';
  $query .= ' WHERE sug_id = "'.$parent['sug_id_parent'].'" LIMIT 1';
  $aSuggestion = db_retrieve(db_query($query));

  $pdf->ezNewPage();
  $sug = new sug();
  $sug->init($aSuggestion['sug_id']);
  
  $pdf->addJpegFromFile($SPL_CFG['dirroot'].'/ui/skin/studiewijzer_book.jpg',50,$pdf->y-40);
  $pdf->ezText('     '.$aSuggestion['sug_title'],30);
  if ($prefSub!=0) {
    $pdf->ezText("\n"._getDesc('sub',$prefSub,'sub_name'),12);
  }
  if ($prefPer!=0) {
    $pdf->ezText(($prefSub>0?"\n":'')._getDesc('prd',$prefPer,'prd_name'),12);
  }

  // Get formatted description
  $desc = $sug->getPropertyFormatted('sug_description');
  if (!empty($desc)){
    $desc = html_entity_decode($desc);
    $pdf->ezText("\n".$SPL_TXT['sug_description'],18);
    $pdf->ezText($desc,12);
  }
  
  // Get formatted knowledge
  $desc = $sug->getPropertyFormatted('sug_knowledge');
  if (!empty($desc)){
    $desc = html_entity_decode($desc);
    $pdf->ezText("\n".$SPL_TXT['sug_knowledge'],18);
    $pdf->ezText($desc,12);
  }

  // Get formatted directions
  $desc = $sug->getPropertyFormatted('sug_directions');
  if (!empty($desc)){
    $desc = html_entity_decode($desc);
    $pdf->ezText("\n".$SPL_TXT['sug_directions'],18);
    $pdf->ezText($desc,12);
  }
  
  // Get formatted prerequisites
  $desc = $sug->getPropertyFormatted('sug_prerequisites');
  if (!empty($desc)){
    $desc = html_entity_decode($desc);
    $pdf->ezText("\n".$SPL_TXT['sug_prerequisites'],18);
    $pdf->ezText($desc,12);
  }
  
  // Get formatted materials
  $desc = $sug->getPropertyFormatted('sug_materials');
  if (!empty($desc)){
    $desc = html_entity_decode($desc);
    $pdf->ezText("\n".$SPL_TXT['sug_materials'],18);
    $pdf->ezText($desc,12);
  }
  
  // Get activities
  $actQuery['columns']  = 'sgd_id,sub.sub_id,sub_short,sug.sug_id,sug_title,prd.prd_id,prd_short,sgd_courseweek_start,sgd_caldate_finish'.($SPL_CFG['learning_load']=='true'?',sug_load':'').($SPL_CFG['learning_reward']=='true'?',sug_reward':'').',sts_short';
  $actQuery['tblName']  = 'sgd sgd LEFT JOIN '.$SPL_CFG['prefix'].'sub sub ON sub.sub_id = sgd.sub_id LEFT JOIN '.$SPL_CFG['prefix'].'sug sug ON sug.sug_id = sgd.sug_id LEFT JOIN '.$SPL_CFG['prefix'].'sts sts ON sts.sts_id = sgd.sts_id LEFT JOIN '.$SPL_CFG['prefix'].'prd prd ON prd.prd_id = sgd.prd_id_start';
  $actQuery['where']    = 'WHERE sgd.usr_id = "'.$prefUsr.'" AND sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active AND NOT sgd.is_done AND sug_type = "A" AND sug_id_parent = "'.$aSuggestion['sug_id'].'"';
  $actQuery['orderBy']  = 'prd_short, sgd_courseweek_start, sub_short,sug_id_parent,sug_sequence';
  $actQuery['orderDir'] = 'ASC';
  $query = 'SELECT '.$actQuery['columns'].' FROM '.$SPL_CFG['prefix'].$actQuery['tblName'].' '.$actQuery['where'].' ORDER BY '.$actQuery['orderBy'].' '.$actQuery['orderDir'];
  $allActivities = db_query($query);
  
  if (db_num_rows($allActivities)>0) {
    
    $pdf->ezNewPage();
    $pdf->ezText($SPL_TXT['activities'],18);
  
    while($anActivity = db_retrieve($allActivities)) {
      
      $pdf->ezText("\n".$anActivity['sug_title'],14);
      $dayname   = $SPL_TXT[date("l",$anActivity['sgd_caldate_finish'])];
      $monthname = $SPL_TXT[date("F",$anActivity['sgd_caldate_finish'])];
      $pdf->ezText($SPL_TXT['sgd_caldate_finish'].' '.$dayname.' '.date("j",$anActivity['sgd_caldate_finish']).' '.$monthname.' '.date("Y",$anActivity['sgd_caldate_finish']),12);

      $load = '';
      if ($SPL_CFG['learning_load']=='true') {
        if ($anActivity['sug_load']>0) {
          $load .= $SPL_TXT['sug_load_long'].': '.$anActivity['sug_load'].' '.$SPL_TXT['hour'];
        }
      }
      if ($SPL_CFG['learning_reward']=='true') {
        if ($anActivity['sug_reward']>0) {
          $load .= ($anActivity['sug_load']>0?', ':'').$SPL_TXT['sug_reward_long'].': '.$anActivity['sug_reward'];
        }
      }
      if (!empty($load)){
        $pdf->ezText($load,12);
      }
    
      $data = array();
      $cols = array();
      $act = new sug();
      $act->init($anActivity['sug_id']);

      // Description
      $tmp = $act->getProperty('sug_description');
      if (!empty($tmp)) {
        $tmp = html_entity_decode($tmp);
        $pdf->ezText("\n".$SPL_TXT['sug_description'],10);
        $pdf->ezText($tmp,10);
      }
  
      // Knowledge
      $tmp = $act->getPropertyFormatted('sug_knowledge');
      if (!empty($tmp)) {
        $tmp = html_entity_decode($tmp);
        $pdf->ezText("\n".$SPL_TXT['sug_knowledge'],10);
        $pdf->ezText($tmp,10);
      }
    
      // Directions
      $tmp = $act->getPropertyFormatted('sug_directions');
      if (!empty($tmp)) {
        $tmp = html_entity_decode($tmp);
        $pdf->ezText("\n".$SPL_TXT['sug_directions'],10);
        $pdf->ezText($tmp,10);
      }
    
      // Prerequisites
      $tmp = $act->getPropertyFormatted('sug_prerequisites');
      if (!empty($tmp)) {
        $tmp = html_entity_decode($tmp);
        $pdf->ezText("\n".$SPL_TXT['sug_prerequisites'],10);
        $pdf->ezText($tmp,10);
      }
    
      // Materials
      $tmp = $act->getPropertyFormatted('sug_materials');
      if (!empty($tmp)) {
        $tmp = html_entity_decode($tmp);
        $pdf->ezText("\n".$SPL_TXT['sug_materials'],10);
        $pdf->ezText($tmp,10);
      }
    }
  }
  // End activities

}
// End suggestions


// Show report
$pdf->stream();


/**
 * Retrieves a description from a table
 * 
 * @param  string  $table Table name
 * @param  integer $id    Row id
 * @param  string  $field name of the field containing the description
 * @return string         Description
 */
function _getDesc($table,$id,$field) {
  
  global $SPL_CFG,$descBuffer;
  
  if (!array_key_exists($table,$descBuffer)) {
    $descBuffer[$table] = array();
  }
  
  if (!array_key_exists($id,$descBuffer[$table])) {
    $query = 'SELECT '.$field.' FROM '.$SPL_CFG['prefix'].$table.' WHERE '.$table.'_id = "'.$id.'" LIMIT 1';
    $result = db_retrieve(db_query($query));
    if (isset($result[$field])) {
      $descBuffer[$table][$id] = $result[$field];
    }
  }
  
  return $descBuffer[$table][$id];
}

/**
 * $Log: sgd_pdf.inc.php,v $
 * Revision 1.5  2006/04/17 10:54:39  jebbol
 * Added decoding html_entity_decode
 *
 */
 
?>
