<?php
/**
 * Search suggestions
 *
 * $Author: jebbol $
 * $Revision: 1.11 $
 * $Date: 2006/08/08 07:08:58 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.nl
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

// Configuration
include_once('../../../inc/config.php');

// Page generation
include_once($SPL_CFG['dirroot'].'/ui/class/page.class.php');

// HTML Table support
include_once($SPL_CFG['dirroot'].'/ui/class/table.class.php');

// Utility functions
include_once($SPL_CFG['dirroot'].'/ui/inc/util.inc.php');

// Defaults
include_once($SPL_CFG['dirroot'].'/class/usd.class.php');

// Methodology objects
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/ali.class.php');
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/sub.class.php');
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/tea.class.php');
include_once($SPL_CFG['dirroot'].'/methodology/traditional/class/tra.class.php');


//=========================================================================
// Sanitize request variables
//=========================================================================
$sanitize = array(
  'sug_search_tra'=>array('type'=>'int'),
  'sug_search_ali'=>array('type'=>'int'),
  'sug_search_sub'=>array('type'=>'int'),
  'sug_search_year'=>array('type'=>'int'),
  'search_show_inactive'=>array('type'=>'bool'),
  'search_show_mine'=>array('type'=>'bool'),
);
require_once($SPL_CFG['dirroot'].'/ui/inc/sanitize.inc.php');

//=========================================================================
// Prerequisites
//=========================================================================

// Must be logged in
include_once($SPL_CFG['dirroot'].'/ui/inc/logincheck.php');

//=========================================================================
// Methodology
//=========================================================================

// Human language translations
$m_trans = $SPL_CFG['dirroot'].'/methodology/traditional/ui/lang/'.$_SESSION['usp_lang'].'/sug.lang.php';
include_once($m_trans);

// Metadata
$m_meta = $SPL_CFG['dirroot'].'/methodology/traditional/ui/meta/sug.metadata.php';
include_once($m_meta);

//=========================================================================
// Defaults
//=========================================================================

$usd = new usd();
$usd_script = 'sug_search';

$singleTra = false;
$singleAli = false;
$singleSub = false;

$isTeacher = $_SESSION['is_teacher'];
$isAdmin   = $_SESSION['is_admin'];

$tea = new tea();
if ($isTeacher) {
  $tea->init($_SESSION['my_tea_id']);
}


// Preferred Training
$prefTra = $usd->getDefault($usd_script,'sug_search_tra');
if (isset($clean['sug_search_tra'])){
  $prefTra = $clean['sug_search_tra'];
  $usd->setDefault($usd_script,'sug_search_tra',$prefTra);
}

// Preferred Alignment
$prefAli = $usd->getDefault($usd_script,'sug_search_ali');
if (isset($clean['sug_search_ali'])){
  $prefAli = $clean['sug_search_ali'];
  $usd->setDefault($usd_script,'sug_search_ali',$prefAli);
}

// Preferred Subject
$prefSub = $usd->getDefault($usd_script,'sug_search_sub');
if (isset($clean['sug_search_sub'])){
  $prefSub = $clean['sug_search_sub'];
  $usd->setDefault($usd_script,'sug_search_sub',$prefSub);
}

// Preferred Courseyear
$prefYr = $usd->getDefault($usd_script,'sug_search_year');
if (isset($clean['sug_search_year'])){
  $prefYr = $clean['sug_search_year'];
  $usd->setDefault($usd_script,'sug_search_year',$prefYr);
}

// Show only items created by myself?
$prefMine = $clean['search_show_mine']; 

// Only administrator can see inactive items
$prefInactive = false;
// Show inactive items
if ($isAdmin) {
  $prefInactive = $clean['search_show_inactive'];
}

//=========================================================================
// Selections
//=========================================================================

$area = '<table style="margin-bottom:3px"><tr><td>'."\r\n";
  
  // Training
  $tra = new tra();
  $singleTra = $tra->isSingle();
  if ($singleTra){
    // Bug #1467590: $prefTra = $singleTra;
    $prefTra = 0;
  }
  else {
    if ($isAdmin) {
      $list = $tra->getKeyValues('ina',false,'tra_short','ASC',false,'tra_short');
    }
    else {
      $list = $tea->getTrainings('short',$_SESSION['usp_ina_id']);
    }
    $area .= ' '.$SPL_TXT['tra_short'].' <select class="listbox" id="sug_search_tra" name="sug_search_tra" style="width:7em" onChange="document.sugsearchform.submit();"><option value="0">'.$SPL_TXT['list_all'].'</option>';
    foreach ($list as $record){
      foreach ($record as $key => $item) {
        $area .= '<option value="'.$key.'"';
        if ($prefTra>0){
          if ((int)$prefTra==(int)$key){
            $area .= ' selected="selected"';
          }
        }
        $area .= '>'.$item.'</option>';
      }
    }
    $area .='</select>'."\r\n";
  }

  // Alignments
  $ali = new ali();
  $singleAli = $ali->isSingle();
  if ($singleAli){
    // Bug #1467590: $prefAli = $singleAli;
    $prefAli = 0;
  }
  else {
    if ($isAdmin) {
      $list = $ali->getKeyValues('tra',$prefTra,'ali_code','ASC',false,'ali_code');
    }
    else {
      $list = $tea->getAlignments('code',$prefTra);
    }
    $area .= ' '.$SPL_TXT['ali_short'].' <select class="listbox" id="sug_search_ali" name="sug_search_ali" style="width:6em" onChange="document.sugsearchform.submit();"><option value="0">'.$SPL_TXT['list_all'].'</option>';
    foreach ($list as $record){
      foreach ($record as $key => $item) {
        $area .= '<option value="'.$key.'"';
        if ($prefAli>0){
          if ((int)$prefAli==(int)$key){
            $area .= ' selected="selected"';
          }
        }
        $area .= '>'.$item.'</option>';
      }
    }
    $area .='</select>'."\r\n";
  }

  // Subjects
  $sub = new sub();
  $singleSub = $sub->isSingle();
  if ($singleSub){
    // Bug #1467590: $prefSub = $singleSub;
    $prefSub = 0;
  }
  else {
    if ($isAdmin) {
      $list = $sub->getKeyValues('ali',$prefAli,'sub_code','ASC',false,'sub_code');
    }
    else {
      $list = $tea->getSubjects('code');
    }
    $area .= $SPL_TXT['sub_short'].' <select class="listbox" id="sug_search_sub" name="sug_search_sub" style="width:6em" onChange="document.sugsearchform.submit();"><option value="0">'.$SPL_TXT['list_all'].'</option>';
    foreach ($list as $record){
      foreach ($record as $key => $item) {
        $area .= '<option value="'.$key.'"';
        if ($prefSub>0){
          if ((int)$prefSub==(int)$key){
            $area .= ' selected="selected"';
          }
        }
        $area .= '>'.$item.'</option>';
      }
    }
    $area .= '</select>'."\r\n";
  }

  // Courseyear
  $area .= ' '.$SPL_TXT['sug_courseyear'].' <input type="text" name="sug_search_year" id="sug_search_year" value="'.((int)$prefYr>0?$prefYr:'').'" class="text" style="width:2em;margin-bottom:0px;padding-bottom:0px" onBlur="document.sugsearchform.submit();" />'."\r\n";
  
  // Show only items created by me
  $area .= ' '.' <input type="checkbox" class="checkbox" style="margin-bottom:0px;padding-bottom:0px;" name="search_show_mine" id="search_show_mine"'.($prefMine?' checked="checked"':'').' onClick="document.sugsearchform.submit();" />'.' '.$SPL_TXT['show_mine']."\r\n";
  
  // Show inactive items
  if ($isAdmin) {
    $area .= ' '.' <input type="checkbox" class="checkbox" style="margin-bottom:0px;padding-bottom:0px;" name="search_show_inactive" id="search_show_inactive"'.($prefInactive?' checked="checked"':'').' onClick="document.sugsearchform.submit();" />'.' '.$SPL_TXT['show_inactive']."\r\n";
  }
    
$area .= '</td></tr></table>'."\r\n";

//=========================================================================
// Query
//=========================================================================

$query['columns']  = 'sug_id AS id,"sug.php'.(isset($clean['from'])?'?from='.$clean['from']:'').'" AS target'.((!$singleTra&&$prefTra==0)?',tra_short':'').((!$singleAli&&$prefAli==0)?',ali_code':'').(((int)$prefSub==0)?',sub_code':'').',sug_courseyear,LEFT(sug_title,50) AS title,sug_short,sug_code'.(isset($clean['search_show_inactive'])?',sug.is_active':'');
$query['tblName']  = 'sug sug'.((!$singleTra)?' LEFT JOIN '.$SPL_CFG['prefix'].'tra tra ON tra.tra_id = sug.tra_id':'').((!$singleAli)?' LEFT JOIN '.$SPL_CFG['prefix'].'ali ali ON ali.ali_id = sug.ali_id':'').(((int)$prefSub==0)?' LEFT JOIN '.$SPL_CFG['prefix'].'sub sub ON sub.sub_id = sug.sub_id':'');
$query['where']    = 'WHERE sug.adm_id = "'.$_SESSION['adm_id'].'" AND sug.sug_type=""';
if (!$prefInactive) {
  $query['where']   .= ' AND sug.is_active';
}
if ($prefMine) {
  $query['where']   .= ' AND sug.create_usr_id = "'.$_SESSION['usr_id'].'"';
}
$query['orderBy']  = 'sug_short';
$query['orderDir'] = 'ASC';

// Limit the view to a single training?
if ((int)$prefTra>0){
  $query['where'] .= ' AND sug.tra_id = "'.$prefTra.'"';
}
else {
  $query['orderBy'] .= (strlen($query['orderBy'])>0?',':'').'sug.tra_id';
}

// Limit the view to a single alignment?
if ((int)$prefAli>0){
  $query['where'] .= ' AND sug.ali_id = "'.$prefAli.'"';
}
else {
  $query['orderBy'] .= (strlen($query['orderBy'])>0?',':'').'sug.ali_id';
}

// Limit the view to a single subject?
if ((int)$prefSub>0){
  $query['where'] .= ' AND sug.sub_id = "'.$prefSub.'"';
}
else {
  $query['orderBy'] .= (strlen($query['orderBy'])>0?',':'').'sug.sub_id';
}

// Limit the view to a single courseyear?
if ((int)$prefYr>0){
  $query['where'] .= ' AND sug.sug_courseyear = "'.$prefYr.'"';
}
else {
  $query['orderBy'] .= (strlen($query['orderBy'])>0?',':'').'sug.sug_courseyear';
}

//=========================================================================
// Handling of POST and GET
//=========================================================================

if (isset($clean['search_for'])) {
  if ($clean['search_by']=='sug_description' || $clean['search_by']=='sug_directions' || $clean['search_by']=='sug_prerequisites' || $clean['search_by']=='sug_materials'){
    $query['where'] .= ' AND MATCH ('.$clean['search_by'].') AGAINST (\''.$clean['search_for'].'\')';
  }
  else {
    $query['where'] .= ' AND '.$clean['search_by'].' LIKE "'.$clean['search_for'].'%"';
  }
}

//The column to orderBy
if (isset ($clean['orderBy'])) $query['orderBy'] = $clean['orderBy'];

//The starting offset for record paging 
if (isset ($clean['viewStart'])) $query['viewStart'] = $clean['viewStart'];

//Sort ascending or descending ('ASC' or 'DESC') 
if (isset ($clean['orderDir']))  $query['orderDir'] = $clean['orderDir']; 

// Add show mine switch
if (isset($clean['search_show_mine'])) $query['showMine'] = $clean['search_show_mine'];

// Add inactive switch
if (isset($clean['search_show_inactive'])) $query['showInactive'] = $clean['search_show_inactive'];

// Encode URL
$query['encode'] = false;

//=========================================================================
// Definition of the page
//=========================================================================

// Create the HTML table
$table = new table($query);
$table->init('sug','',$m_trans,$m_meta);
$table->enableRowSelection();
$table->setCallbackScript($_SERVER['PHP_SELF']);
$table->setMetaOverrule('is_active', array('hidden'=>true));
$table->addNewButton(NULL,$SPL_CFG['wwwroot'].'/methodology/'.$_SESSION['methodology'].'/ui/sug.php?toolbar_button=new&new"');

// Create the page
$page = new page();
$page->init('sug','default','template_search.htm',true,false,$m_trans,$m_meta);
$page->setHeadScripts($table->getScripts());
$table->setImageDir($page->getImageDir());
  $page->startForm('sugsearchform');
  $page->startSection();
  $page->startColumn();
  if (isset($area)){
    $page->startAreaSearch();
    $page->addContent($area);
    $page->endAreaSearch();
  }
  $page->startAreaData();
  $page->addContent($table->getTable());
  $page->endAreaData();
  $page->endColumn();
  $page->endSection();
$page->endForm();

//=========================================================================
// Show the page
//=========================================================================

$page->setContent();
$page->createPage();
$page->showPage();

/**
 * $Log: sug_search.php,v $
 * Revision 1.11  2006/08/08 07:08:58  jebbol
 * Changed repository (cvs.sourceforge.net -> splanner.cvs.sourceforge.net)
 *
 * Revision 1.10  2006/04/14 11:27:28  jebbol
 * Bug #1467590: Added check on single instance
 *
 * Revision 1.9  2006/02/11 16:52:06  jebbol
 * Renamed ui/inc/util.php -> ui/inc/util.inc.php
 * Added UI areas to support multiple skins
 */

?>
