<?php
    /**
     * Copyright 2005 Zervaas Enterprises (www.zervaas.com.au)
     *
     * Licensed under the Apache License, Version 2.0 (the "License");
     * you may not use this file except in compliance with the License.
     * You may obtain a copy of the License at
     *
     *     http://www.apache.org/licenses/LICENSE-2.0
     *
     * Unless required by applicable law or agreed to in writing, software
     * distributed under the License is distributed on an "AS IS" BASIS,
     * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
     * See the License for the specific language governing permissions and
     * limitations under the License.
     */

    include_once($SPL_CFG['dirroot'].'/class/ina.class.php');
    include_once($SPL_CFG['dirroot'].'/class/ins.class.php');
    include_once($SPL_CFG['dirroot'].'/ui/class/ajax/AjaxACApplication.class.php');

    class stuAjax extends AjaxACApplication
    {
        // the file location data is stored in. Each city should be stored on
        // a separate line, in the form: CountryName,RegionName,CityName
        var $data = 'locations.txt';


        function stuAjax($config = null)
        {
            parent::AjaxACApplication($config);
            $this->registerActions('getInstitutions', 'getAddresses');

            // This application contains 3 actions:
            //  1. action_getInstitutions(), for fetching a country list
            //  2. action_getAddresses(), for fetching a region list for a given country
            //  3. action_getcities(), for fetching a city list for a given country/region
            // See below for the implementation of each of these methods

            $this->setup();
        }

        function setup() {
            
            global $SPL_CFG;
            
            $this->addJsLib($SPL_CFG['dirroot'].'/ui/class/ajax/stu.ajax.class.js');
            // there's a bunch of utility functions that never change in this file


            // Next we need to create the HTML select elements. There is one
            // to list countries, one to list regions and one to list cities.
            // We want to perform certain actions on each when they are loaded,
            // so the onload event is added to each of them, with their own
            // callback (event_countryinit(), event_regioninit(), event_cityinit()).
            // Additionally, when a country is changed, we need to update the
            // region list, so we add the onchange element to the country.
            // Likewise we add onchange to regions so the cities dropdown
            // can be updated

            // And finally, each widget is added to the application using addWidget()

            $tmp_ins_id = $this->createWidget('ins_id');
            $tmp_ins_id->addEvent(AJAXAC_EV_ONLOAD, 'ins_idinit');
            $tmp_ins_id->addEvent(AJAXAC_EV_ONCHANGE, 'ins_idchange');
            $this->addWidget($tmp_ins_id);

            $tmp_ina_id = $this->createWidget('ina_id');
            $tmp_ina_id->addEvent(AJAXAC_EV_ONLOAD, 'ina_idinit');
            //$tmp_ina_id->addEvent(AJAXAC_EV_ONCHANGE, 'ina_idchange');
            $this->addWidget($tmp_ina_id);

            /* 
            $city = $this->createWidget('city');
            $city->addEvent(AJAXAC_EV_ONLOAD, 'cityinit');
            $this->addWidget($city);
            */
            // after having all done this, a number of things must now be done:
            //  1. Create a HTML page (see index.php) containing 3 dropdowns,
            //     as well as attaching them to each of these widgets
            //  2. Implement event_ins_idinit()
            //  3. Implement event_ins_idchange()
            //  4. Implement event_ina_idinit()
            //  5. Implement event_ina_idchange()
            //  6. Implement event_cityinit()
        }

        /**
         * Handle the getInstitutions action, by returning a JavaScript array
         * of all the countries, or an empty array if none are found
         */
        function action_getInstitutions()
        {
            $countries = $this->getInstitutions();
            $this->sendResponseData('jsarray', $countries);
        }

        /**
         * Handle the getAddresses action, by returning a JavaScript array
         * of all the ina_ids for the request ins_id, or an empty array
         * if none are found
         */
        function action_getAddresses()
        {
            $tmp_ina_ids = $this->getAddresses($this->getRequestValue('c'));
            $this->sendResponseData('jsarray', $tmp_ina_ids);
        }

        /**
         * Handle the getcities action, by returning a JavaScript array
         * of all the cities for the request ina_id/ins_id, or an empty array
         * if none are found
         */
        /*
        function action_getcities()
        {
            $cities = $this->getCities($this->getRequestValue('c'), $this->getRequestValue('r'));
            $this->sendResponseData('jsarray', $cities);
        }
        */

        /**
         * Initialises the ins_id dropdown selector
         */
        function event_ins_idinit(&$widget, $event)
        {
            // create a new xmlhttp widget so we can fetch the initial list
            // of countries. the action to do this is getInstitutions.
            // once the ins_id list is ready, we process this data with
            // the handlecountries event callback
            $xmlhttp = $this->XMLHttpRequest('r1', AJAXAC_METH_GET);
            $xmlhttp->setFilenameFromString($this->getApplicationUrl('getInstitutions'));
            $xmlhttp->addEvent(AJAXAC_EV_ONXMLHTTPSUCCESS, 'handlecountries');

            // callback description:
            //  1. Initialize various text values for the element such as default text
            //  2. Set the 3 dropdowns to their required state while the ins_id list is loading
            //  3. Submit the HTTP subrequest to fetch the ins_id data

            $callback = "
                            function()
                            {
                                this.emptyDefault = '%s';
                                this.fullDefault = '%s';
                                this.loadingText = '%s';

                                loadingins_id(this, %s, %s);

                                try {
                                    %s
                                }
                                catch (e) { }


                                return false;
                            }
                        ";
            /*
            $callback = sprintf($callback,
                                $this->escapeJs($this->getConfigValue('ins_idEmptyDefault')),
                                $this->escapeJs($this->getConfigValue('ins_idFullDefault')),
                                $this->escapeJs($this->getConfigValue('ins_idLoadingText')),
                                $this->getHookName('ina_id'),
                                $this->getHookName('city'),
                                $xmlhttp->getJsCode());
            */
            $callback = sprintf($callback,
                                $this->escapeJs($this->getConfigValue('ins_idEmptyDefault')),
                                $this->escapeJs($this->getConfigValue('ins_idFullDefault')),
                                $this->escapeJs($this->getConfigValue('ins_idLoadingText')),
                                $this->getHookName('ina_id'),
                                $xmlhttp->getJsCode());

            return $callback;
        }

        /**
         * Initialises the ins_id dropdown selector. Basically just sets up text strings
         * for various states
         */
        function event_ina_idinit(&$widget, $event)
        {
            $callback = "
                            function()
                            {
                                this.emptyDefault = '%s';
                                this.fullDefault = '%s';
                                this.loadingText = '%s';
                            }
                        ";
            $callback = sprintf($callback,
                                $this->escapeJs($this->getConfigValue('ina_idEmptyDefault')),
                                $this->escapeJs($this->getConfigValue('ina_idFullDefault')),
                                $this->escapeJs($this->getConfigValue('ina_idLoadingText')));

            return $callback;
        }

        /**
         * Initialises the ins_id dropdown selector. Basically just sets up text strings
         * for various states. It has an extra one to ina_ids, as there is text to indicate
         * to select a ins_id when no ins_id has been selected, or to select a ina_id when
         * a ins_id has been selected but no ina_id has.
         */
        /*
        function event_cityinit(&$widget, $event)
        {
            $callback = "
                            function()
                            {
                                this.emptyDefault = '%s';
                                this.emptyDefaultina_id = '%s';
                                this.fullDefault = '%s';
                                this.loadingText = '%s';
                            }
                        ";
            $callback = sprintf($callback,
                                $this->escapeJs($this->getConfigValue('cityEmptyNoins_idDefault')),
                                $this->escapeJs($this->getConfigValue('cityEmptyNoina_idDefault')),
                                $this->escapeJs($this->getConfigValue('cityFullDefault')),
                                $this->escapeJs($this->getConfigValue('cityLoadingText')));

            return $callback;
        }
        */

        /**
         * Deals with the returned ins_id data once the XMLHttp request is completed
         */
        function event_handlecountries(&$widget, $event)
        {
            // callback description:
            //  1. Firstly checks the the HTTP subrequest is complete
            //  2. Next checks that it was a valid 200 response header
            //  3. Assigns the returned JavaScript array from action_getInstitutions() to _data
            //  4. Populates the ins_id dropdown with what is in _data
            //  5. Set the 3 dropdowns to their required state for ins_id data just being loaded
            //     (this is basically, tell user to pick a ins_id, and disable ina_id/city until this is done
            $callback = "
                            function()
                            {
                                _data = ajaxac_receivejsarray(%1\$s.responseText);
                                populateSelect(%2\$s, _data);
                                enableins_id(%2\$s, %3\$s, %4\$s);
                            }
                        ";
            /*
            $callback = sprintf($callback,
                                $widget->getHookName(),
                                $this->getHookName('ins_id'),
                                $this->getHookName('ina_id'),
                                $this->getHookName('city'));
            */
            $callback = sprintf($callback,
                                $widget->getHookName(),
                                $this->getHookName('ins_id'),
                                $this->getHookName('ina_id'));
            return $callback;
        }

        /**
         * Handles the value in the ins_id dropdown being changed
         */
        function event_ins_idchange(&$widget, $event)
        {
            // when a ins_id is changed, we need to fetch the associated
            // ina_ids, so we create a HTTP request to do so. The action to
            // do this is getAddresses, and we also need to pass to it the
            // selected ins_id in the 'c' request parameter. Because this
            // ins_id value is stored in a JavaScript variable, we pass the name of
            // the variable. And finally, once the request is complete, we
            // need to insert the ina_id data, which is done in event_handleina_ids
            $xmlhttp = $this->XMLHttpRequest('r2', AJAXAC_METH_GET);
            $xmlhttp->setFilenameFromString($this->getApplicationUrl('getAddresses'));
            $xmlhttp->addParamFromHookValue('c', $this->getHookName('ins_id'));
            $xmlhttp->addEvent(AJAXAC_EV_ONXMLHTTPSUCCESS, 'handleina_ids');

            // callback description:
            //  1. Check if a ins_id has been selected
            //  2. If not, set the state back to needing to select a ins_id as when countries were initially loaded
            //  3. If so, set the state of ina_id/city to indicate that a ina_id is being loaded, and perform the subrequest
            //
            $callback = "
                            function()
                            {
                                try {
                                    if (this.selectedIndex == 0)
                                        enableins_id(this, %2\$s, %3\$s);
                                    else {
                                        loadingina_id(%2\$s, %3\$s);
                                        %4\$s
                                    }

                                }
                                catch (e) { }


                                return false;
                            }
                        ";
            /*
            $callback = sprintf($callback,
                                $this->getHookName('ins_id'),
                                $this->getHookName('ina_id'),
                                $this->getHookName('city'),
                                $xmlhttp->getJsCode());
            */
            $callback = sprintf($callback,
                                $this->getHookName('ins_id'),
                                $this->getHookName('ina_id'),
                                $xmlhttp->getJsCode());

            return $callback;
        }

        /**
         * Deals with the returned ina_id data once the XMLHttp request is completed
         */
        function event_handleina_ids(&$widget, $event)
        {
            // callback description:
            //  1. Firstly checks the the HTTP subrequest is complete
            //  2. Next checks that it was a valid 200 response header
            //  3. Assigns the returned JavaScript array from action_getAddresses() to _data
            //  4. Populates the ina_id dropdown with what is in _data
            //  5. Set the ina_id/city dropdowns to their required state for ina_id data just being loaded
            //     (this is basically, tell user to pick a ina_id, and disable city until this is done
            $callback = "
                            function()
                            {
                                _data = ajaxac_receivejsarray(%1\$s.responseText);
                                populateSelect(%2\$s, _data);
                                enableina_id(%2\$s, %3\$s);
                            }
                        ";
            /*
            $callback = sprintf($callback,
                                $widget->getHookName(),
                                $this->getHookName('ina_id'),
                                $this->getHookName('city'));
            */
            $callback = sprintf($callback,
                                $widget->getHookName(),
                                $this->getHookName('ina_id'));
            return $callback;
        }

        /**
         * Handles the value in the ina_id dropdown being changed
         */
        /*
        function event_ina_idchange(&$widget, $event)
        {
            // when a ina_id is changed, we need to fetch the associated
            // cities, so we create a HTTP request to do so. The action to
            // do this is getcities, and we also need to pass to it the
            // selected ins_id in the 'c' request parameter and the selected
            // ina_id in the 'r' request parameter. Because the ins_id and
            // ina_id values are stored in JavaScript variables, we pass the name of
            // the variables. And finally, once the request is complete, we
            // need to insert the city data, which is done in event_handlecities
            $xmlhttp = $this->XMLHttpRequest('r3', AJAXAC_METH_GET);
            $xmlhttp->setFilenameFromString($this->getApplicationUrl('getcities'));
            $xmlhttp->addParamFromHookValue('c', $this->getHookName('ins_id'));
            $xmlhttp->addParamFromHookValue('r', $this->getHookName('ina_id'));
            $xmlhttp->addEvent(AJAXAC_EV_ONXMLHTTPSUCCESS, 'handlecities');

            // callback description:
            //  1. Check if a ina_id has been selected
            //  2. If not, set the state back to needing to select a ina_id as when ina_ids were initially loaded
            //  3. If so, set the state of city to indicate that a city is being loaded, and perform the subrequest
            //
            $callback = "
                            function()
                            {
                                try {
                                    if (this.selectedIndex == 0)
                                        enableina_id(%2\$s, %3\$s);
                                    else {
                                        loadingCity(%3\$s);
                                        %1\$s
                                    }
                                }
                                catch (e) { }

                                return false;
                            }
                        ";

            $callback = sprintf($callback,
                                $xmlhttp->getJsCode(),
                                $this->getHookName('ina_id'));

            return $callback;
        }
        */

        /**
         * Deals with the returned city data once the XMLHttp request is completed
         */
        /*
        function event_handlecities(&$widget, $event)
        {
            // callback description:
            //  1. Firstly checks the the HTTP subrequest is complete
            //  2. Next checks that it was a valid 200 response header
            //  3. Assigns the returned JavaScript array from action_getcities() to _data
            //  4. Populates the city dropdown with what is in _data
            //  5. Set the city dropdowns to its required state for city data just being loaded
            //     (this is basically, tell user to pick a city
            $callback = "
                            function()
                            {
                                _data = ajaxac_receivejsarray(%1\$s.responseText);
                                populateSelect(%2\$s, _data);
                                enableCity(%2\$s);
                            }
                        ";
            $callback = sprintf($callback,
                                $widget->getHookName(),
                                $this->getHookName('city'));
            return $callback;
        }
        */

        function event_handlefailure(&$widget, $event)
        {
            $callback = "
                            function()
                            {
                                alert('An error occurred loading the data');
                            }
                        ";
            return $callback;
        }

        /**
         * Retrieve the list of countries and return them in an array
         */
        function getInstitutions()
        {
            // uncomment this to notice the disabling/loading status update
            usleep(300000);
            return array_keys($this->getNestedData());
        }

        /**
         * Retrieve the list of ina_ids for the given ins_id, or null if
         * the ins_id was not found
         */
        function getAddresses($tmp_ins_id)
        {
            // uncomment this to notice the disabling/loading status update
            usleep(300000);
            $data = $this->getNestedData();
            if (isset($data[$tmp_ins_id]))
                return array_keys($data[$tmp_ins_id]);
            return null;
        }

        /**
         * Retrieve the list of cities for the given ins_id and ina_id, or
         * null if the ins_id or ina_id were not found
         */
        /*
        function getCities($tmp_ins_id, $tmp_ina_id)
        {
            // uncomment this to notice the disabling/loading status update
            usleep(300000);
            $data = $this->getNestedData();
            if (isset($data[$tmp_ins_id][$tmp_ina_id]))
                return $data[$tmp_ins_id][$tmp_ina_id];
            return null;
        }
        */

        /**
         * A utility function to parse our location data into a usable format
         */
        function getNestedData()
        {
            $ret = array();
            $lines = file($this->data);

            foreach ($lines as $line) {
                $line = trim($line);
                if (strlen($line) == 0)
                    continue;
                $parts = explode(',', $line, 3);
                $tmp_ins_id = $parts[0];
                $tmp_ina_id = $parts[1];
                $city = $parts[2];
                $ret[$tmp_ins_id][$tmp_ina_id][] = $city;
            }
            return $ret;
        }
    }
?>