<?php
/**
 * Generates a user interface.
 *
 * $Author: jebbol $
 * $Revision: 1.45 $
 * $Date: 2006/09/03 18:32:13 $
 * 
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================
if (!isset($SPL_CFG['dirroot'])) {
  // Automatic test scripts and documentation
  $test_basename = basename(__FILE__);
  $test_filename = __FILE__;
  include_once('../../inc/test.inc.php');
}

// Configuration
if (!isset($SPL_CFG['dirroot'])) {include_once('../inc/config.php');}

// Database functions
if (!isset($SPL_CFG['dbtype'])) {$SPL_CFG['dbtype'] = 'mysql';}
include_once($SPL_CFG['dirroot'].'/db/'.$SPL_CFG['dbtype'].'/db.inc.php');

// System setting
include_once($SPL_CFG['dirroot'].'/inc/settings.sys.php');

// Human language translations
if (!isset($_SESSION['usp_lang'])) {$_SESSION['usp_lang'] = 'en';}
include_once($SPL_CFG['dirroot'].'/ui/lang/'.$_SESSION['usp_lang'].'/spl.lang.php');

// User interface elements
if (!isset($_SESSION['usp_skin'])) {$_SESSION['usp_skin'] = $SPL_CFG['skin'];}
if (!isset($SPL_CFG['helproot'])) {$SPL_CFG['helproot'] = 'http://doc.studiewijzer.org';}
include_once($SPL_CFG['dirroot'].'/ui/skin/'.$_SESSION['usp_skin'].'/elements.php');
// Make sure we have a valid mode
if (!isset($SPL_UI['layout_mode'])) {$SPL_UI['layout_mode'] = 'full';}

//=========================================================================
// Class definition
//=========================================================================

class page {

  var $_ajax;                  // When true, ajax support will be included
  var $_ajaxlists;             // When true, ajax list support will be included
  var $_bodyOptions = array(); // Body tag options
  var $_bodyHeader  = array(); // Body header content
  var $_bodyFooter  = array(); // Body header content
  var $_border;                // states if a border has to be drawn around the viewport
  var $_calendar;              // states if the calendar must be enabled
  var $_checklist;             // Checklist support
  var $_content;               // the body content html
  var $_dataClass;             // The name of the class supplying the data
  var $_diagram;               // states if the diagram capabilities must be enabled
  var $_editor;                // When true the wysiwyg editor is added
  var $_focus;                 // Field to receive the focus
  var $_formButtons = array(); // A list of form buttons to display
  var $_formAction;            // the action of the generated form
  var $_formName;              // the name of the generated form
  var $_formOptions = array(); // Form tag options
  var $_help;                  // When true, helptext will be added
  var $_frameKickout;          // When true, the window will always be "top"
  var $_html;                  // The generated html
  var $_icon;                  // The icon to display
  var $_iframe;                // Is an iframe active on this page
  var $_imageDir;              // path to the images
  var $_nativeSearch;          // Adds a hidden field for capturing the search event
  var $_paneStart;             // Appends a pane div to a form tag
  var $_parameters = array (); // the parameters to be substituted
  var $_popup;                 // When true the winlike popups are enabled
  var $_skin;                  // style sheet file name (full path and filename)
  var $_template;              // template name (full path and filename)
  var $_topmenu;               // The menu to display in case of a topmenu
  var $_startTime;             // Start of the page-generation timer
  var $_validate;              // When true, javascript field validation is added
  var $_validationRules;       // Validation to perform on submit

  function page() {
    
    $this->_ajax = (bool)false;         // Add ajax list support
    $this->_ajaxlists = (bool)false;    // Add ajax list support
    $this->_border = (bool)true;        // Draw a border
    $this->_calendar = (bool)false;     // Add calendar support?
    $this->_checklist = (bool)false;    // Add checklist for IE support?
    $this->_diagram = (bool)false;      // Add diagram support?
    $this->_editor = (bool)false;       // Add Editor support?
    $this->_help = (bool)false;         // Add help support?
    $this->_frameKickout = (bool)false; // Make window the top window?
    $this->_icon = 'default.png';       // Use the default icon
    $this->_iframe = (bool)false;       // Is an iframe displayed on this page?
    $this->_nativeSearch = (bool)false; // Add search event capturing?
    $this->_popup = (bool)false;        // Add popup support?
    $this->_topmenu = '';
    $this->_validate = (bool) false;    // Add form validation
    $this->_validationRules = '';       // Empty the rules
    
    $this->_startTime = microtime(true);
    
  }

  //=========================================================================
  // Public methods
  //=========================================================================

  /**
   * Adds a bar to a chart
   * 
   * @param integer $percentage The percentage the bar has to cover
   * @param string  $label The label to display
   * @param string  $type The type of bar ('default', 'alert' or 'ok')
   */
  function addBar($percentage=0,$label=null,$type='default') {
    
    global $SPL_UI;
    
    // Label
    if (isset($label)) {
      $this->addContent(str_replace('{BAR_LABEL}',$this->translate($label),$SPL_UI['simple_chart_label']));
    }
    
    // Percentage
    $this->addContent(str_replace('{BAR_PERCENTAGE}',$percentage.'%',$SPL_UI['simple_chart_percentage']));
    
    // Bar
    switch ($type) {
      case 'alert':
        $this->addContent(str_replace('{BAR_PERCENTAGE}',$percentage.'%',$SPL_UI['simple_chart_bar_alert']));
        break;
      case 'ok':
        $this->addContent(str_replace('{BAR_PERCENTAGE}',$percentage.'%',$SPL_UI['simple_chart_bar_ok']));
        break;
      default:
        $this->addContent(str_replace('{BAR_PERCENTAGE}',$percentage.'%',$SPL_UI['simple_chart_bar_default']));
        break;
    }
  }
  
  /** 
   * Adds an item to the body footer
   * 
   * @param $content string The content to add
   * @access public
   */
  function addBodyFooter($content) {
    $this->_bodyFooter[] = $content;
  }

  /** 
   * Adds an item to the body header
   * 
   * @param $content string The content to add
   * @access public
   */
  function addBodyHeader($content) {
    $this->_bodyHeader[] = $content;
  }


  /** 
   * Adds options to the body tag
   * 
   * @param $event string   The name of the event to catch
   * @param $handler string The event handler (add a semicolon!)
   * 
   * @access public
   */
  function addBodyOption($event,$handler) {
    if (!isset($this->_bodyOptions[$event])){
      $this->_bodyOptions[$event] = $handler;
    }
    else {
      $this->_bodyOptions[$event] .= $handler;
    }
  }

  /** 
   * Adds a button to the page
   *
   * @param $sName       (string)  Name of the button 
   * @param $sValue      (string)  Value to display 
   * @param $sType       (string)  (optional) Button type
   * @param $sOnClick    (string)  (optional) The onClick event 
   * @param $sSize       (string)  (optional) The size of the button 
   * @param $sClass      (string)  (optional) The class of the button
   * @param $sDisabled   (boolean) (optional) Disable the button or not 
   * @param $accesskey   (string)  (optional) The accesskey to assign to the button
   * @param $resetCancel (boolean) (optional) Overwites the cancel button as a 'reset' button
   * @access public
   */
  function addButton($sName='default',$sValue='default',$sType='default',$sOnClick='default',$sSize='default',$sClass='default',$sDisabled='no',$accesskey=null,$resetCancel=false) {
    $sTmp = $sName;
    $sName = ($sName == 'default' ? '' : ' name="'.$sName.'"');
    $sId = ($sTmp == 'default' ? '' : ' id="'.$sTmp.'"');
    $sValue = ' value="'. ($sValue == 'default' ? 'Ok' : $this->translate($sValue)).'"';
    // If the value contains an accesskey, filter it out
    if (strpos($sValue,'/u')){
      $accesskey = substr($sValue,(strpos($sValue,'/u')-2),1);
      $sValue = strip_tags($sValue);
    }
    $sType = ($sType == 'default' ? ' type="button"' : ' type="'.$sType.'"');
    $sOnClick = ($sOnClick == 'default' ? '' : ' onClick="'.$sOnClick.'"');

    // Overwrite cancel button behaviour
    if ($sTmp=='cancel' && $resetCancel){
      $sType = ' type="reset"';
      $sOnClick = ' onClick="document.location=\''.$this->_formAction.'\'"';
    }

    if ($sClass != 'default') {
      if ($sDisabled == 'yes') {
        $sClass = ' class="'.$sClass.'Disabled"';
      } else {
        $sClass = ' class="'.$sClass.'"';
      }
    } else {
      $sClass = ' class="button"';
    }
    $sSize = ($sSize == 'default' ? '' : ' size="'.$sSize.'" style="width:'.$sSize.'em"');
    $sDisabled = ($sDisabled == 'yes' ? ' disabled="disabled"' : '');
    $accesskey = (isset($accesskey)?' accesskey="'.$accesskey.'"':'');

    $this->addContent('<input'.$sName.$sId.$sType.$sDisabled.$sSize.$sClass.$sValue.$accesskey.$sOnClick.' />');
  }

  /** 
   * Adds content to the body of the page
   * 
   * @param $sContent  string the HTML code to be added
   * @access public
   */
  function addContent($content,$linefeed='none') {
    $this->_content .= $content.($linefeed!='none'?"\r\n":"");
  }

  /** 
   * Adds a data field 
   * 
   * @param (string)  $name      The field name 
   * @param (string)  $value     The field value
   * @param (boolean) $readonly  (optional) Is this field readonly
   * @param (boolean) $disabled  (optional) Disable the field
   * @param (boolean) $overrule  (optional) Overrule the default authorization (true=allow everything, string=use autho for this class)
   * @param (array)   $valuelist (optional) Listvalues (key=>value)
   * @param (string)  $focus     (optional) Sets the field receiving start focus
   * @param (boolean) $makerow   (optional) Display each field in its own row
   * @param (string)  $rowoption (optional) Options to add to the startrow tag
   * @param (bool)    $hidden    (optional) When true, adds a hidden field
   * @param (integer) $tabindex  (optional) The tabindex to use
   * @param (integer) $maxsize   (optional) The maximum display size
   * @access public
   */
  function addField($name,$value='',$readonly=false,$disabled=false,$overrule=false,$valuelist=false,$focus=null,$makerow=true,$rowoption=false,$hidden=false,$tabindex=null,$maxsize=0) {
    
    global $SPL_CFG,$SPL_TXT,$SPL_META,$SPL_UI;

    // Do we need to add style to the element?
    $style = (bool)false;
    
    // Must we hide the field?
    if (!$hidden){
      $hidden = isset($SPL_META[$name]['hidden'])&&$SPL_META[$name]['hidden'];
    }
    
    // Set the defaults for checking the maximum length
    $maxlength=0;

    // Is the default metadata setting overruled?
    $autho_class = $this->_dataClass;
    if ($overrule) {
      if ($overrule==true) {
        $disabled=(bool)false;
        $readonly=(bool)false;
      }
      else {
        $autho_class = $overrule;
      }
    }
    else {
      // Do not check the permission for hidden fields
      if ($hidden){
        // do nothing, just skip the test
      }
      // User permissions can only be set by users having administrator rights on the user data
      else if ($autho_class=='usa' && $_SESSION['usr_permission']['admin']==(bool)true){
        // do nothing, just skip the test
      }
      // Check permissions
      else {
        // When the user has no read permissions, don't display anything
        if (!isset($_SESSION[$autho_class.'_permission'])){
          return (bool) false;
        }
        if ($_SESSION[$autho_class.'_permission']['read']==(bool)false) {
          return (bool) false;
        }
        // When the user has only read permission, make fields 'readonly'
        if ($_SESSION[$autho_class.'_permission']['write']==(bool)false && $_SESSION[$autho_class.'_permission']['change']==(bool)false) {
          $readonly=(bool)true;
        }
      }
    }

    // Remove any slashes from the value
    if (strpos($value, "\'")) {
      do {
        $value = str_replace("\'", "'", "$value");
      } while (strpos($value, "\'"));
    } else if (strpos($value, "\"")) {
      do {
        $value = str_replace('\"', '"', "$value");
      } while (strpos($value, '\"'));
    }

    // Find metadata. If none, assume defaults
    if (!isset($SPL_META[$name])) {
      $type = 'text';
      $size = '15';
      $required = (bool) false;
    }
    
    // Metadata is available
    else {
      $type      = (isset($SPL_META[$name]['type'])?$SPL_META[$name]['type']:'text');
      $size      = (isset($SPL_META[$name]['size'])?$SPL_META[$name]['size']:'0');
      $maxlength = (isset($SPL_META[$name]['maxlength'])?$SPL_META[$name]['maxlength']:'0');
      $required  = (isset($SPL_META[$name]['required'])?$SPL_META[$name]['required']:false);
      $readonly  = (isset($SPL_META[$name]['displayonly'])?$SPL_META[$name]['displayonly']:$readonly);
      if ($maxsize>0) {
        $size = $maxsize;
      }
    }

    // In case of a reference field, check if we have to visualize it as a listbox
    if ($type=='reference' && !$readonly && !$disabled && !$hidden && isset($SPL_CFG['max_listitems']) && ($SPL_CFG['max_listitems']>0)){
      // Values were entered as a parameter
      if (is_array($valuelist) && count($valuelist)>0 && count($valuelist)<=$SPL_CFG['max_listitems']){
        $type = 'list';
      }
      // Find values in the database
      else if (isset($SPL_META[$name]['referencetable'])){
        $query = 'SELECT COUNT(*) FROM '.$SPL_CFG['prefix'].$SPL_META[$name]['referencetable'].' WHERE adm_id = "'.$_SESSION['adm_id'].'" AND is_active';
        $result = db_retrieve(db_query($query));
        if (isset($result['COUNT(*)']) && $result['COUNT(*)']>0 && $result['COUNT(*)']<=$SPL_CFG['max_listitems']){
          // When person dat ais requested, make sure we use the preferred formatting
          if ($SPL_META[$name]['referencetable']=='per') {
            include_once($SPL_CFG['dirroot'].'/class/per.class.php');
            $per = new per();
            $valuelist = $per->getKeyValues();
          }  
          else {
            $query = 'SELECT '.$SPL_META[$name]['referencekey'].' AS "key", '.$SPL_META[$name]['referencedisplay'].' AS "value" FROM '.$SPL_CFG['prefix'].$SPL_META[$name]['referencetable'].' WHERE adm_id = "'.$_SESSION['adm_id'].'" AND is_active ORDER BY '.$SPL_META[$name]['referencedisplay'];
            $result = db_query($query);
            while ($row = db_fetch_array($result)) {
              $valuelist[] = array($row['key']=>$row['value']);
            }
          }
          $type = 'list';
        }
      } 
    }

    // Hidden field
    if ($hidden){
      $row = '<div class="hidden"><input type="hidden" name="'.$name.'" id="'.$name.'" value="'.$value.'" /></div>';
      $this->addContent($row,'yes');
      return;
    }

    // Construct the field
    $row = '<td class="label'.($disabled?'_disabled':($readonly?'_readonly':'')).'">{LABEL_VALUE}</td><td class="sign">{FIELD_SIGN}</td><td>{FIELD_PREFIX}<input type="{FIELD_TYPE}"{FIELD_NAME}{FIELD_ID} value="{FIELD_VALUE}"'.($disabled?' disabled="disabled"':'').($readonly?' readonly="readonly"':'').'{FIELD_CHECKED}{FIELD_CLASS}{FIELD_SIZE}{FIELD_MAXLENGTH}{FIELD_STYLE}{FIELD_TITLE}{FIELD_TABINDEX}{FIELD_ACCESSKEY}{FIELD_EVENT}>{FIELD_SUFFIX}</td>'; 
    $property = (isset($SPL_META[$name]['multiple'])?'check':'').$type.($disabled?'_disabled':($readonly?'_readonly':''));
    if ($required && !$disabled && !$readonly){
      $property = (isset($SPL_META[$name]['multiple'])?'check':'').$type.'_required';
    }
    if (isset($SPL_META[$name]['labelpos']) && $SPL_META[$name]['labelpos']=='none') {
      $property .= '_nolabel';
    }
    if (isset($SPL_UI[$property])) {
      $row = $SPL_UI[$property];
    }
    else {
      if (isset($SPL_UI[$type])) { 
        $row = $SPL_UI[$type];
      }
    }
    // Display this field on its own row
    if ($makerow){
      $row = $SPL_UI['fieldrow_start'].$row.$SPL_UI['fieldrow_end'];
      $row = str_replace('{FIELDROW_OPTIONS}',($rowoption?$rowoption:''),$row);
    }
    
    // Set focus
    if (!$readonly&&!$disabled) {
      if (isset($focus)){
        $this->_focus = $name;
      }
      if (!isset($this->_focus)){
        $this->_focus = $name;
      }
    }
    
    // Set label value
    $labelText = (isset($SPL_META[$name]['label'])&&isset($SPL_TXT[$SPL_META[$name]['label']])?$SPL_TXT[$SPL_META[$name]['label']]:(isset($SPL_TXT[$name])?$SPL_TXT[$name]:''));
    if (isset($SPL_TXT[$name.'_help'])){
      $labelText = '<a '.$SPL_UI['help'].' tabindex="-32768" href="?" onclick="showHelpTip(event, \''.$SPL_TXT[$name.'_help'].'\'); return false">'.$labelText.'</a>';
      $this->_help = (bool)true;
    }
    $row = str_replace('{LABEL_VALUE}',$labelText,$row);
    
    // Set field name
    $row = str_replace('{FIELD_NAME}',' name="'.$name.'"',$row);
    $row = str_replace('{FIELD_ID}',($SPL_CFG['doctype']=='html3'?'':' id="'.$name.'"'),$row);
    
    // Set field class
    $row = str_replace('{FIELD_CLASS}',($property==''?'':' class="'.$property.'"'),$row);

    // Set field size
    $checklength = ($maxlength>0?' maxlength="'.$maxlength.'"':'');
    if ($size>0){
      // XHTML documents
      if (substr($SPL_CFG['doctype'],0,1)=='x') {
        $row = str_replace('{FIELD_SIZE}',$checklength,$row);
        $style .= 'width:'.$size.'em;';
      }
      // HTML documents
      else {
        if ($type=='list') {
          // Clear out the size attribute, for it will force a multiple select 
          $row = str_replace('{FIELD_SIZE}','',$row);
          $style .= 'width:'.$size.'em;';
        }
        else {
          $row = str_replace('{FIELD_SIZE}',($size=='0'?'':' size="'.$size.'"'.$checklength),$row);
          $style .= 'width:'.$size.'em;';
        }
      }
    }
    // If there is no size, clear the placeholder
    else {
      $row = str_replace('{FIELD_SIZE}','',$row);
    }
    
    // SPL_TODO: Set maxlength from db metadata
    $row = str_replace('{FIELD_MAXLENGTH}','',$row);

    // Set tabindex
    if (isset($tabindex)){
      $row = str_replace('{FIELD_TABINDEX}',' tabindex="'.$tabindex.'"',$row);
    }
    else {
      $row = str_replace('{FIELD_TABINDEX}','',$row);
    }
    
    // SPL_TODO: Set accesskey
    $row = str_replace('{FIELD_ACCESSKEY}','',$row);
    
    // Set title
    $row = str_replace('{FIELD_TITLE}',(isset($SPL_META[$name]['title'])&&isset($SPL_TXT[$SPL_META[$name]['title']])?' title="'.$SPL_TXT[$SPL_META[$name]['title']].'"':''),$row);
    
    switch ($type) {
      case 'checkbox':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','checkbox',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}',($value?' checked="checked"':''),$row);
        $row = str_replace('{FIELD_VALUE}',$value,$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}',(($readonly||$disabled)?'':' onChange="javascript:setDirtyFlag();"'),$row);
        break;
      case 'date':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        $row = str_replace('{FIELD_VALUE}',($value=='0000-00-00'?'':$value),$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}',(($readonly||$disabled)?'':' onChange="javascript:setDirtyFlag();"'),$row);
        // Add calendar
        if (!$readonly&&!$disabled){
          if ($this->_browser_detection('browser')=='msie') {
            // Hack to prevent 'Page has expired' error in IE
            $this->_calendar = (bool) true;
            $row = str_replace('{FIELD_SIGN}','',$row);
          }
          else {
            $this->_calendar = (bool) true;
            $row = str_replace('{FIELD_SIGN}','<img align="middle" id="img_'.$name.'" src="'.$this->_imageDir.$SPL_UI['sign_calendar'].'" alt="calendar" /><dlcalendar click_element_id="img_'.$name.'" input_element_id="'.$name.'" tool_tip="'.$SPL_TXT['calendar'].'" months="'.$SPL_TXT['calendar_months'].'" days="'.$SPL_TXT['calendar_days'].'" firstday="'.$SPL_TXT['calendar_firstday'].'" nav_images="'.$this->_imageDir.'prevyear.gif,'.$this->_imageDir.'prevmonth.gif,'.$this->_imageDir.'nextmonth.gif,'.$this->_imageDir.'nextyear.gif" navbar_style="'.$SPL_UI['calendar_navbar_style'].'" daybar_style ="'.$SPL_UI['calendar_daybar_style'].'"></dlcalendar>',$row);
          }
        }
        break;
      case 'datetime':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        $row = str_replace('{FIELD_VALUE}',$value,$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}',(($readonly||$disabled)?'':' onChange="javascript:setDirtyFlag();"'),$row);
        break;
      case 'dynamiclist':
        // Set up a result field
        $this->addContent('<div class="hidden"><input type="hidden" name="'.$name.'" id="'.$name.'" value="'.$value.'" /></div>','yes');
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);

        // Find description
        if (isset($SPL_META[$name]['referencetable']) && $value>0){
          $query = 'SELECT '.$SPL_META[$name]['referencedisplay'].' FROM '.$SPL_CFG['prefix'].$SPL_META[$name]['referencetable'].' WHERE '.$SPL_META[$name]['referencekey'].' = "'.$value.'" AND adm_id = "'.$_SESSION['adm_id'].'"';
          $result = db_retrieve(db_query($query));
          if (isset($result[$SPL_META[$name]['referencedisplay']])){
            $row = str_replace('{FIELD_VALUE}',' '.$result[$SPL_META[$name]['referencedisplay']],$row);
          }
          else {
            $row = str_replace('{FIELD_VALUE}',($value=='0'?'':$value),$row);
          }
        }
        $row = str_replace('{FIELD_VALUE}','',$row);

        $ev = ' onkeyup="ajax_setFile(\''.$SPL_CFG['wwwroot'].'/ui/ajax/'.$SPL_META[$name]['referencedisplay'].'.php\');ajax_showOptions(this,\'getByLetters\',event);"';
        // Set events
        if (!$readonly && $required && isset($SPL_CFG['client_validate']) && $SPL_CFG['client_validate']=='default'){
          $this->_validate = true;
          $ev .= ' onChange="javascript:validatePresent(this,\''.$name.'\');"';
          $this->_validationRules .= 'if(!validatePresent(document.forms.'.$this->_formName.'.'.$name.',\''.$name.'\')){errs+=1};';
        }
        $row = str_replace('{FIELD_EVENT}',$ev,$row);
        // Replace field name (value will be placed in hidden field)
        $row = str_replace(' name="'.$name,' name="'.$name.'_display',$row);
        $row = str_replace(' id="'.$name,' id="'.$name.'_display',$row);
        break;
      case 'email':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        $row = str_replace('{FIELD_VALUE}',$value,$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}','',$row);
        if (!empty($value)) {
          $row = str_replace('{FIELD_SIGN}','<a href="mailto:'.$value.'" title="'.$SPL_TXT['email'].'"><img align="middle" id="img_'.$name.'" src="'.$this->_imageDir.$SPL_UI['sign_mail'].'" class="sign" border="0" alt="mail" /></a>',$row);
        }
        else {
          $row = str_replace('{FIELD_SIGN}','',$row);
        }
      case 'msn':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        $row = str_replace('{FIELD_VALUE}',$value,$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}','',$row);
        if (!empty($value)) {
          $row = str_replace('{FIELD_SIGN}','<a href="msnim:chat?contact='.$value.'" title="'.$SPL_TXT['messenger'].'"><img align="middle" id="img_'.$name.'" src="'.$this->_imageDir.$SPL_UI['sign_messenger'].'" class="sign" border="0" alt="mail" /></a>',$row);
        }
        else {
          $row = str_replace('{FIELD_SIGN}','',$row);
        }
      case 'list':
        // Select multiple items
        if (isset($SPL_META[$name]['multiple'])) {
          // Add :hover support for IE
          if ($this->_browser_detection('browser')=='msie') {
            $this->_checklist = true;
          }
        }
        $row = str_replace('{LIST_MULTIPLE}',(isset($SPL_META[$name]['multiple'])&&($SPL_META[$name]['multiple']==true)?' multiple="multiple"':''),$row);
        $option = (bool)false;
        // If no valuelist has been specified, check the meta definitions
        if (!$valuelist && isset($SPL_META[$name]['valuelist'])){
          $valuelist = $SPL_META[$name]['valuelist'];
        }
        if (isset($tabindex)){
          $row = str_replace('{FIELD_TABINDEX}',' tabindex="'.$tabindex.'"',$row);
        }
        // Hide empty values
        if (($readonly || $disabled) && $value=='0'){
          $row = str_replace('{FIELD_VALUE}','',$row);
        }
        else if ($valuelist) {
          // Add an empty option at the top of a single-selection list
          if (!$required && !$disabled && !$readonly && !isset($SPL_META[$name]['multiple'])){
            $option .= $SPL_UI['list_item'];
            $option = str_replace('{LIST_ITEM_KEY}','0',$option);
            $option = str_replace('{LIST_ITEM_VALUE}','&nbsp;',$option);
            $option = str_replace('{LIST_ITEM_SELECTED}','',$option);
          }
          // Change the sort order of the items in the list
          if (isset($SPL_META[$name]['sort'])&&$SPL_META[$name]['sort']==true){
            $valuelist = $this->_sortValuelist($valuelist);
          }
          foreach ($valuelist as $record){
            foreach ($record as $key => $item) {
              // Single selection mode
              if (!isset($SPL_META[$name]['multiple'])) {
                // Only display the selected item
                if ($readonly||$disabled){
                  if ($key==$value){
                    $row = str_replace('{FIELD_VALUE}',$this->translate($item),$row);
                    break;
                  }
                }
                $option .= $SPL_UI['list_item'];
                $option = str_replace('{LIST_ITEM_KEY}',$key,$option);
                $option = str_replace('{LIST_ITEM_VALUE}',$this->translate($item),$option);
                $option = str_replace('{LIST_ITEM_SELECTED}',($key==$value?' selected="selected"':''),$option);
              }
              // Multiple selection mode
              else {
                $option .= $SPL_UI['checklist_item'];
                $option = str_replace('{LIST_ITEM_KEY}',($name.'_'.$key),$option);
                $option = str_replace('{LIST_ITEM_VALUE}',$this->translate($item),$option);
                $option = str_replace('{LIST_ITEM_SELECTED}',($key==$value?' checked="checked"':''),$option);
                if ($readonly || $disabled) {
                  $option = str_replace('{LIST_ITEM_DISABLED}',' disabled="disabled"',$option);
                }
                else {
                  $option = str_replace('{LIST_ITEM_DISABLED}','',$option);
                }
              }
            }
          }
        }
        $row = str_replace('{LIST_ITEMS}',($option?$option:''),$row);
        if (isset($SPL_META[$name]['submitonchange'])&&($SPL_META[$name]['submitonchange']==true)){
          $row = str_replace('{LIST_ONCHANGE}',(($this->_editor&&$SPL_CFG['editor']=='tinyMCE')?'tinyMCE.triggerSave();':'').'document.'.$this->_formName.'.toolbar_button.value=\'list\';document.'.$this->_formName.'.submit();',$row);
        }
        else {
          $row = str_replace('{LIST_ONCHANGE}','',$row);
        } 
      case 'numeric':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        $row = str_replace('{FIELD_VALUE}',(empty($value)&&$readonly?'&nbsp;':$value),$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}','',$row);
        break;
      case 'option':
        break;
      case 'password':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','password',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        // Hide the password
        if ($readonly) {
          $row = str_replace('{FIELD_VALUE}','<span title="'.$value.'" style="cursor:help">*********</span>',$row);
        }
        else {
          $row = str_replace('{FIELD_VALUE}',$value,$row);
        }
        // Set events
        if (!$readonly && $required && isset($SPL_CFG['client_validate']) && $SPL_CFG['client_validate']=='default'){
          $this->_validate = true;
          $ev = ' onChange="javascript:validatePresent(this,\''.$name.'\');"';
          $this->_validationRules .= 'if(!validatePresent(document.forms.'.$this->_formName.'.'.$name.',\''.$name.'\')){errs+=1};';
        }
        else {
          $ev = '';
        }
        $row = str_replace('{FIELD_EVENT}',$ev,$row);
        break;
      case 'reference':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        $row = str_replace('{FIELD_VALUE}',($value=='0'?'':$value),$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}','',$row);
        if (!$readonly && !$disabled){
          $this->_popup = (bool) true;
          // When using the native search, create a placeholder for capturing the search event
          if ($SPL_CFG['popup']=='none'){
            $this->_nativeSearch = (bool)true;
          }
          $row = str_replace('{FIELD_SIGN}','<a href=\''.($SPL_CFG['popup']=='winlike'?$SPL_CFG['wwwroot'].'/ui/'.$SPL_META[$name]['referencescript'].'?from='.$name.'':'javascript:'.(($this->_editor&&$SPL_CFG['editor']=='tinyMCE')?'tinyMCE.triggerSave();':'').'document.'.$this->_formName.'.search_button.value="'.$name.'";document.'.$this->_formName.'.submit();').'\' '.($SPL_CFG['popup']=='winlike'?' target="popup"':'').' title="'.$SPL_TXT['search'].'"><img id="img_'.$name.'" src="'.$this->_imageDir.$SPL_UI['sign_search'].'" class="sign" alt="search"<!-- {SPL_ENDTAG} -->></a>',$row);
        }
        // Find description
        if (isset($SPL_META[$name]['referencetable']) && $value>0){
          $query = 'SELECT '.$SPL_META[$name]['referencedisplay'].' FROM '.$SPL_CFG['prefix'].$SPL_META[$name]['referencetable'].' WHERE '.$SPL_META[$name]['referencekey'].' = "'.$value.'" AND adm_id = "'.$_SESSION['adm_id'].'"'.($readonly?'':' AND is_active');
          $result = db_retrieve(db_query($query));
          if (isset($result[$SPL_META[$name]['referencedisplay']])){
            $row = str_replace('{FIELD_SUFFIX}',' '.$result[$SPL_META[$name]['referencedisplay']],$row);
          }
        }
        break;
      case 'textarea':
        // Set columns
        $row = str_replace('{FIELD_COLS}',(isset($SPL_META[$name]['cols'])?' cols="'.$SPL_META[$name]['cols'].'"':''),$row);
        // Set rows
        $row = str_replace('{FIELD_ROWS}',(isset($SPL_META[$name]['rows'])?' rows="'.$SPL_META[$name]['rows'].'"':''),$row);
        if (isset($SPL_META[$name]['rows'])) {
          if (!$readonly) {
            // Provide screen space for the editor buttons
            $row = str_replace('{FIELD_STYLE}',' style="height:'.($SPL_META[$name]['rows']+2).'em"',$row);
          }
          else {
            $row = str_replace('{FIELD_STYLE}',' style="height:'.($SPL_META[$name]['rows']).'em"',$row);
          }
        }
        // Set field value
        $row = str_replace('{FIELD_VALUE}',$value,$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}',(($readonly||$disabled)?'':' onKeyDown="javascript:setDirtyFlag();event.cancelBubble=true;"'),$row);
        // Add the editor
        if (!$readonly && !$disabled){
          if ($SPL_CFG['editor']!='none'){
            $this->_editor = (bool)true;
            // Patch the width of a TinyMCE editor
            if ($SPL_CFG['editor']=='tinyMCE'){
              $style .= 'width:100%;';
            }
          }
        }
        break;
      case 'time':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        $row = str_replace('{FIELD_VALUE}',$value,$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}','',$row);
        break;
      case 'timestamp':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        $row = str_replace('{FIELD_VALUE}',($value>0?strftime("%Y-%m-%d %H:%M",$value):''),$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}',(($readonly||$disabled)?'':' onChange="javascript:setDirtyFlag();"'),$row);
        // Add calendar
        if ($readonly=='0' && $disabled=='0'){
          if ($this->_browser_detection('browser')=='msie') {
            // Hack to prevent 'Page has expired' error in IE
            $this->_calendar = (bool) true;
            $row = str_replace('{FIELD_SIGN}','',$row);
          }
          else {
            $this->_calendar = (bool) true;
            $row = str_replace('{FIELD_SIGN}','<img id="img_'.$name.'" class="sign" src="'.$this->_imageDir.$SPL_UI['sign_calendar'].'" alt="calendar" /><dlcalendar click_element_id="img_'.$name.'" input_element_id="'.$name.'" tool_tip="'.$SPL_TXT['popup_calendar'].'" months="'.$SPL_TXT['popup_calendar_months'].'" days="'.$SPL_TXT['popup_calendar_days'].'" firstday="'.$SPL_TXT['popup_calendar_firstday'].'" nav_images="'.$this->_imageDir.'prevyear.gif,'.$this->_imageDir.'prevmonth.gif,'.$this->_imageDir.'nextmonth.gif,'.$this->_imageDir.'nextyear.gif" navbar_style="'.$SPL_UI['calendar_navbar_style'].'" daybar_style ="'.$SPL_UI['calendar_daybar_style'].'"></dlcalendar>',$row);
          }
        }
        break;
      case 'year':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        $row = str_replace('{FIELD_VALUE}',strftime("%Y",$value),$row);
        // Set events
        $row = str_replace('{FIELD_EVENT}',(($readonly||$disabled)?'':' onChange="javascript:setDirtyFlag();"'),$row);
        break;
      case 'url':
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        if ($readonly && !empty($value)){
          $row = str_replace('{FIELD_VALUE}','<a href="'.$value.'" target="_blank" title="'.$value.'">'.$SPL_TXT['start'].'</a>',$row);
        }
        else {
          $row = str_replace('{FIELD_VALUE}',$value,$row);
        }
        // Set events
        $row = str_replace('{FIELD_EVENT}','',$row);
        break;
      default:
        // Set field type
        $row = str_replace('{FIELD_TYPE}','text',$row);
        // Set field value
        $row = str_replace('{FIELD_CHECKED}','',$row);
        $row = str_replace('{FIELD_VALUE}',$value,$row);
        // Set events
        if (!$readonly && $required && isset($SPL_CFG['client_validate']) && $SPL_CFG['client_validate']=='default'){
          $this->_validate = true;
          $ev = ' onChange="javascript:validatePresent(this,\''.$name.'\');"';
          $this->_validationRules .= 'if(!validatePresent(document.forms.'.$this->_formName.'.'.$name.',\''.$name.'\')){errs+=1};';
        }
        else {
          $ev = '';
        }
        $row = str_replace('{FIELD_EVENT}',$ev,$row);
        break;
    }
    
    // Build style
    $row = str_replace('{FIELD_STYLE}',($style?' style="'.$style.'"':''),$row);

    // Clear out any stuff left
    $row = str_replace('{FIELD_SIGN}','',$row);
    $row = str_replace('{FIELD_PREFIX}','',$row);
    $row = str_replace('{FIELD_SUFFIX}','',$row);

    // Add the field to the page
    $this->addContent($row,'yes');
    
  }
  
  /**
   * Adds a footer bar
   */
  function addFootbar($title=null) {
    if ($_SESSION['usp_skin'] != 'default') {
      $this->addContent('<div class="areaFootbar">'.(isset($title)?$title:'&copy; studiewijzer.nl').'</div>','yes');
    }
  }

  /** 
   * Adds options to the form tag
   * 
   * @param $event string   The name of the event to catch
   * @param $handler string The event handler (add a semicolon!)
   * 
   * @access Private
   */
  function addFormOption($event,$handler) {
    if (!isset($this->_formOptions[$event])){
      $this->_formOptions[$event] = $handler;
    }
    else {
      $this->_formOptions[$event] .= $handler;
    }
  }

  /**
   * Adds an iframe to the page
   * 
   * @param (string) $sTargetPage The url of the target page
   * @param (string) $sFrameName  The name of the frame (optional)
   * @param (string) $sStyle      The style to use (optional)
   * @param (bool)   $lScrolling  Add scrollbars (optional)
   * @param (int)    $fixedHeight The absolute height (optional)
   */
  function addIFrame($sTargetPage,$sFrameName='subframe',$sStyle='default',$lScrolling='no',$fixedHeight=null) {
    
    global $SPL_CFG,$SPL_UI;
    
    $this->_iframe = (bool)true;
    $browser = $this->_browser_detection('browser');
 
    if ($sFrameName!='resizableframe'){
      if ($sStyle=='default'){
        $sStyle = 'width:100%;height:';
        /*
        if ($browser=='msie') {
          $sStyle = 'width:99%;height:';
        }
        */
        if (isset($fixedHeight)) {
          $sStyle .= $fixedHeight;
        }
        else if (isset($SPL_UI['iframe_height'])) {
          $sStyle .= $SPL_UI['iframe_height'];
        }
        else {
          $height = (isset($_SESSION['usp_numrows'])?(((int)$_SESSION['usp_numrows']*2)+4):'46');
          // Ugly hack to compensate for the selection area in the studyguide
          if ($sFrameName=='fixedframe') {
            $height += 8;
          }
          $sStyle .= $height.'em';
        }
      }
    }
    else if ($sStyle=='default'){
      $sStyle = 'width:100%';
    }
    
    // IE6 Hack: doesn't display empty td's (thinks the iframe is empty)
    //$filler = $this->_browser_detection('browser')=='msie'?'&nbsp;':'';
    $filler = '';
    if ($browser=='msie') {
      $filler = '&nbsp;';
      //$sStyle .= ';position:relative;top:-1.4em;margin-bottom:-1.4em';
    }
    $this->addContent('<tr id="'.$sFrameName.'Row"><td style="background-color:transparent">'.$filler.'<iframe id="'.$sFrameName.'" name="'.$sFrameName.'" style="'.$sStyle.'" frameborder="0" marginwidth="0" marginheight="0" hspace="0" vspace="0" scrolling="'.$lScrolling.'" src="'.$SPL_CFG['wwwroot'].'/ui/'.$sTargetPage.'"></iframe></td></tr>','yes');
  }

  /** 
   * Starts a field with a label on the right site 
   * 
   * @param $label string The label 
   * @param $field string The field
   * @param $value string The field value
   * @param $type  string (optional) The type of input field
   * @param $class string (optional) The class to use
   * @access public
   */
  function addLabelledField($label,$field,$value,$type='text',$class='default') {
    if ($class=='default') {
      $class = $type.'input';
    }
    $label = $this->translate($label);
    $this->addContent('<tr><td class="label">'.$label.'</td><td><input type="'.$type.'" class="'.$class.'" name="'.$field.'" value="'.$value.'"></td></tr>','yes');
  }

  /** 
   * Adds a menu item 
   * 
   * @param $class string The class name 
   * @param $path  string An optional path to the class
   * @access public
   */
  function addMenuItem($class,$path=NULL) {
    
    global $SPL_UI;

    if (strlen($class)>0) {
      if (!isset($SPL_UI['menubar_allowed']) || (isset($SPL_UI['menubar_allowed'])&&(strstr($SPL_UI['menubar_allowed'],$class)))){
        $label = $this->translate($class);
        $content = str_replace('{MENU_TEXT}',$label,$SPL_UI['menubar_content']);
        $content = str_replace('{MENU_SCRIPT}',(isset($path)?$path.'/'.$class:$class),$content);
        $content = str_replace('{MENU_ICON}',$this->_imageDir.$class,$content);
        $this->addContent($content);
      }
    }
  }

  function addOptionButton($sTitle,$sTargetFrame,$sTargetPage,$lSelected='no') {
    $sClass = ($lSelected == 'yes' ? "navButtonIn" : "navButton");
    $tmp = '<td id="'.$sClass.'" onClick="document.getElementById(\''.$sTargetFrame.'\').src=\''.$sTargetPage.'\'">'.$this->translate($sTitle)."</td>\r\n".'<!-- {OPTION_BUTTON} -->';
    $this->_html = str_replace('<!-- {OPTION_BUTTON} -->', $tmp, $this->_html);
  }

  function addOptionContent($sContent) {
    $sContent = $sContent.'<!-- {OPTION_BUTTON} -->';
    $this->_html = str_replace('<!-- {OPTION_BUTTON} -->', $sContent, $this->_html);
  }

  function addOptionSeparator() {
    $tmp = '<td id="navSep">'."</td>\r\n".'<!-- {OPTION_BUTTON} -->';
    $this->_html = str_replace('<!-- {OPTION_BUTTON} -->', $tmp, $this->_html);
  }

  /** 
   * Adds a searchbar
   * 
   * @param  array  $searchBy      Array with column names to search by
   * @param  string $searchFor     Value of the 'search for:' field (or 'empty')
   * @param  string $searchOptions Any optiobal data
   * @param  bool   $showInactive  The default for the 'show_inactive' checkbox
   * @access public
   */
  function addSearchbar($searchBy=NULL,$searchSelected='',$searchFor=null,$searchOptions=null,$showInactive=false) {

    global $SPL_CFG,$SPL_UI,$SPL_TXT;
    
    if (isset($SPL_UI['searchbar'])) {
      $by='';
      $for='';
      $opt='';
      $option='';
      
      // Create the 'search by' entry
      if (isset($searchBy)) {
        // If it is an array, create a listbox for it
        if (is_array($searchBy)){
          $len = 0;
          foreach ($searchBy as $key){
              $option .= $SPL_UI['list_item'];
              $option = str_replace('{LIST_ITEM_KEY}',$key,$option);
              $option = str_replace('{LIST_ITEM_VALUE}',$SPL_TXT[$key],$option);
              $len = (strlen($SPL_TXT[$key])>$len?strlen($SPL_TXT[$key]):$len);
              $option = str_replace('{LIST_ITEM_SELECTED}',($searchSelected==$key?' selected="selected"':''),$option);
          }
          $option = str_replace('{LIST_ITEMS}',$option,$SPL_UI['list_light']);
          $option = str_replace('{FIELD_NAME}',' name="search_by"',$option);
          $option = str_replace('{FIELD_ID}',($SPL_CFG['doctype']=='html3'?'':' id="search_by"'),$option);
          $by = str_replace('{SEARCH_BY}',$SPL_TXT['search_by'].$option,$SPL_UI['searchbar_by']);
        }
        // Otherwise, just display its description
        else {
          if (isset($SPL_TXT[$searchBy])){
            $by = str_replace('{SEARCH_BY}',$SPL_TXT[$searchBy].$option,$SPL_UI['searchbar_by']);
            $len = strlen($SPL_TXT['search_by']);
          }
        }
        $len = (int)$len + 1;
        $by = str_replace('{LIST_STYLE}',($len>0?' style="width:'.$len.'em"':''),$by);
      }
      
      // Create the 'search for' entry
      if (isset($searchFor)){
        $for = str_replace('{FIELD_NAME}',' name="search_for"',$SPL_UI['text_light']);
        $for = str_replace('{FIELD_ID}',($SPL_CFG['doctype']=='html3'?'':' id="search_for"'),$for);
        $for = str_replace('{FIELD_VALUE}',$searchFor,$for);
        $for = str_replace('{SEARCH_FOR}',(isset($SPL_TXT['search_for'])?$SPL_TXT['search_for']:'').$for,$SPL_UI['searchbar_for']);
        $for = str_replace('{SEARCH_FOR_STYLE}',';width:'.(strlen($SPL_TXT['search_for'])+15).'em;padding-bottom:0px;margin-bottom:0px;',$for);
      }
      
      // Create the 'options' entry
      $opt = str_replace('{SEARCH_OPTIONS}',(isset($searchOptions)?$searchOptions:''),$SPL_UI['searchbar_options']);
      $opt = str_replace('{SEARCH_OPTION_STYLE}','',$opt);
      
      // Create the submit button
      $but = str_replace('{SEARCH_BUTTON}',strip_tags($SPL_TXT['search_go']),$SPL_UI['searchbar_button']);

      // If the button text contains an accesskey, filter it out
      if (strpos($SPL_TXT['search_go'],'/u')){
        $but = str_replace('{SEARCH_ACCESSKEY}',' accesskey="'.substr($SPL_TXT['search_go'],(strpos($SPL_TXT['search_go'],'/u')-2),1).'"',$but);
      }
      else {
        $but = str_replace('{SEARCH_ACCESSKEY}','',$but);
      }
      
      $but = str_replace('<!-- {SPL_ENDTAG} -->',(substr($SPL_CFG['doctype'],0,1)=='x'?' /':''),$but);

      // For administrators, add a 'show inactive' switch
      $inactive='';
      if (isset($_SESSION['is_admin'])&&$_SESSION['is_admin']==true) {
        $inactive = ' <input type="checkbox" class="checkbox" style="padding-bottom:0px;margin-bottom:0px;" name="search_show_inactive" id="search_show_inactive"'.($showInactive?' checked="checked"':'').' onClick="document.'.$this->_formName.'.submit();" />'.' '.$SPL_TXT['show_inactive'];
      }

      // Assemble the searchbar
      $row = str_replace('{SEARCH_ITEMS}',$by.$for.$opt.$inactive.$but,$SPL_UI['searchbar']);
      $this->startAreaSearch();
      $this->addContent($row,'yes');
      $this->endAreaSearch();
    }
  }

  /** 
   * Adds a section title
   *
   * @param $title string The title of the section 
   * @param $class string (optional) The name of the class to use
   * @access public
   */
  function addSectionTitle($title,$class='sectiontitle') {
    
    global $SPL_UI;
    
    $title = $this->translate($title);
    if (!isset($SPL_UI['section_title'])) {
      $this->addContent('<div class="'.$class.'">'.$title.'</div>','yes');
    }
    else {
      $row = str_replace('{SECTION_TITLE_CLASS}',$class,$SPL_UI['section_title']);
      $row = str_replace('{SECTION_TITLE}',$title,$row);
      $this->addContent($row,'yes');
    }
  }

  /** 
   * Adds a bar with tabs
   *  
   * @param $current string  (optional) The name (or id) of the current tab
   * @param $target  string  (optional) The name of the target frame
   * @param $tabs    array   (optional) Tabidentifier in the metadata
   * @param $ruler   boolean (optional) Display a ruler (default=no)
   * @param $args    string  (optional) Arguments to append to the action
   * @param $type    string  (optional) The type of tabs. null=default, 'sub'=subtabs
   * @access public
   * 
   * The $tabs array may contain description of all tabs. When empty, the function 
   * will try to find the default tab setting for this class (from the metadata).
   * The follwoing items can be added to the array:
   * - name: the name of the tab
   * - title: the title to display
   * - widget: the name of a widget in te form to which the tab is associated. This widget will be turned on/off using the style 'display:none'
   * - tabaction: the action to perform when clicking the tab. Note that when a 'widget' is specified, the default action is to display the associated widget.
   * - empty: a boolean indicating if the tab content is emtpy (default=false)
   */
  function addTabbar($current=null,$target=null,$tabs=null,$ruler=false,$args=null,$type=null) {
    
    global $SPL_CFG,$SPL_UI,$SPL_TXT,$SPL_META;

    if (!$tabs) {
      $tabs = $SPL_META[$this->_dataClass.'_tabs'];
    }
    if (!isset($tabs)){
      return (bool)false;
    }
    
    $tabnames = '';
    $widgetnames = '';
    
    // Add a hidden field to capture the chosen tab
    $row = '<tr style="display:none"><td><input type="hidden" name="selected_tab" id="selected_tab" value="'.($current?$current:'').'"<!-- {SPL_ENDTAG} -->></td></tr>'."\r\n";
    
    $row .= $SPL_UI[($type?$type:'').'tab_start'];
    foreach($tabs as $tab){
      // SPL_TODO: permission
      if (!isset($tab['name'])&&isset($tab['title'])){
        $tab['name'] = $tab['title'];
      }
      if (!isset($tab['title'])){
        $tab['title'] = $tab['name'];
      }
      $tabnames .= ($tabnames?",'".$tab['name']."'":"'".$tab['name']."'");
      if (isset($tab['widget'])){
        $widgetnames .= ($widgetnames?",'".$tab['widget']."'":"'".$tab['widget']."'");
      }
      // Action
      if (isset($tab['tabaction'])){
        $tabaction = $tab['tabaction'];
        if ($args){
          $tabaction .= '&amp;'.$args;
        }
      }
      else {
        $tabaction = '#';
      }
      $tabaction .= '" onClick="'.$this->_dataClass.'_settab(\''.$tab['name'].'\');';
      // Empty tab indicator
      $empty = (isset($tab['empty']) && $tab['empty']==true); 
      //Target
      if (isset($tab['target'])) {
        $target = $tab['target'];
      }
      $row .= ($tab['name']==$current?$SPL_UI['tab_current']:$SPL_UI['tab']);
      $row = str_replace('{TAB_NAME}','id="'.$tab['name'].'"',$row);
      $row = str_replace('{TAB_ACTION}',$tabaction,$row);
      $row = str_replace('{TAB_TITLE}',($empty?$SPL_UI['tab_empty_start']:'').$this->translate($tab['title']).($empty?$SPL_UI['tab_empty_end']:''),$row);
      $row = str_replace('{TAB_TARGET}',($target?' target="'.$target.'"':''),$row);
    }
    $row .= $SPL_UI['tab_end'];
    $this->addContent($row);
    if ($ruler) {
      $this->addTabRuler($type);
    }

    // Create javascript to drive the tabs
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">function '.$this->_dataClass.'_settab(tab){';
    $sContent .= "var tabs=new Array(".$tabnames.");";
    $sContent .= "var widgets=new Array(".$widgetnames.");";
    $sContent .= "document.".$this->_formName.".selected_tab.value=tab;";
    // Do the tabs activate form widgets?
    $sContent .= "for(i=0;i<(tabs.length);i++){if(tabs[i]==tab){document.getElementById(tabs[i]).className='current';";
    if (!empty($widgetnames)){
      $sContent .= "document.getElementById(widgets[i]).style.display='';";
    }
    $sContent .= "}else{document.getElementById(tabs[i]).className='';";
    if (!empty($widgetnames)){
      $sContent .= "document.getElementById(widgets[i]).style.display='none';";
    }
    $sContent .= '}}}</script>';
    $this->setHeadScripts($sContent);
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- Tab support -->";
    $this->setHeadScripts($sContent);
  }
  
  /** 
   * Adds a ruler in tabbar style to the content
   *
   * @param string $type (optional) The type of tabs. null=default, 'sub'=subtabs
   * @access public
   */
  function addTabRuler($type=null) {

    global $SPL_UI;
    
    $tmp = $SPL_UI[($type?$type:'').'tab_ruler'];
    if (isset($tmp)&& !empty($tmp)){
      $this->addContent($tmp);
    }
  }

  /** 
   * Adds a toolbar 
   * @param bool   $new    (optional) Display the new button
   * @param bool   $edit   (optional) Display the edit button
   * @param bool   $save   (optional) Display the save button
   * @param bool   $del    (optional) Display the delete button
   * @param bool   $cancel (optional) Display the cancel button
   * @param bool   $find   (optional) Display the find button
   * @param string $icon   (optional) The icon to display
   * @access public
   */
  function addToolbar($new=false,$edit=false,$save=false,$del=false,$cancel=false,$find=false,$icon='default.png') {
    
    global $SPL_CFG,$SPL_UI,$SPL_TXT;

    // Add image support
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/images.js"></script>';
    $this->setHeadScripts($sContent);
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- Image support -->";
    $this->setHeadScripts($sContent);
        
    if (isset($SPL_UI['toolbar'])) {
      
      // New
      if ($new && isset($SPL_UI['toolbar_new']) && (isset($SPL_UI['toolbar_new_position'])&&$SPL_UI['toolbar_new_position']=='toolbar')) {
        // Must have 'write' permission
        if (isset($_SESSION[$this->_dataClass.'_permission']['write']) && $_SESSION[$this->_dataClass.'_permission']['write'] == true) {
          $tmp = str_replace('{TOOLBAR_NEW_TEXT}',$SPL_TXT['toolbar_new'],$SPL_UI['toolbar_new']);
          if (strpos($SPL_TXT['toolbar_new'],'/u')){
            $SPL_TXT['toolbar_new_title'].= '" accesskey="'.substr($SPL_TXT['toolbar_new'],(strpos($SPL_TXT['toolbar_new'],'/u')-2),1);
          }
          $tmp = str_replace('{TOOLBAR_NEW_TITLE}',$SPL_TXT['toolbar_new_title'],$tmp);
          $tmp = str_replace('{FORM_NAME}',$this->_formName,$tmp);
          $row = str_replace('{TOOLBAR_NEW}',$tmp,$SPL_UI['toolbar']);
        }
        else {
          $row = str_replace('{TOOLBAR_NEW}','',$SPL_UI['toolbar']);
        }
      }
      else {
        $row = str_replace('{TOOLBAR_NEW}','',$SPL_UI['toolbar']);
      }
      
      // Edit
      if ($edit && isset($SPL_UI['toolbar_edit']) && (isset($SPL_UI['toolbar_edit_position'])&&$SPL_UI['toolbar_edit_position']=='toolbar')) {
        // Must have 'change' permission
        if (isset($_SESSION[$this->_dataClass.'_permission']['change']) && $_SESSION[$this->_dataClass.'_permission']['change'] == true) {
          $tmp = str_replace('{TOOLBAR_EDIT_TEXT}',$SPL_TXT['toolbar_edit'],$SPL_UI['toolbar_edit']);
          if (strpos($SPL_TXT['toolbar_edit'],'/u')){
            $SPL_TXT['toolbar_edit_title'].= '" accesskey="'.substr($SPL_TXT['toolbar_edit'],(strpos($SPL_TXT['toolbar_edit'],'/u')-2),1);
          }
          $tmp = str_replace('{TOOLBAR_EDIT_TITLE}',$SPL_TXT['toolbar_edit_title'],$tmp);
          $tmp = str_replace('{FORM_NAME}',$this->_formName,$tmp);
          $row = str_replace('{TOOLBAR_EDIT}',$tmp,$row);
        }
        else {
          $row = str_replace('{TOOLBAR_EDIT}','',$row);
        }
      }
      else {
        $row = str_replace('{TOOLBAR_EDIT}','',$row);
      }
      
      // Cancel
      if ($cancel && isset($SPL_UI['toolbar_cancel']) && (isset($SPL_UI['toolbar_cancel_position'])&&$SPL_UI['toolbar_cancel_position']=='toolbar')) {
        // Must have change or write permission
        if ((isset($_SESSION[$this->_dataClass.'_permission']['change']) && $_SESSION[$this->_dataClass.'_permission']['change'] == true) ||
            (isset($_SESSION[$this->_dataClass.'_permission']['write']) && $_SESSION[$this->_dataClass.'_permission']['write'] == true)) {
          $tmp = str_replace('{TOOLBAR_CANCEL_TEXT}',$SPL_TXT['toolbar_cancel'],$SPL_UI['toolbar_cancel']);
          if (strpos($SPL_TXT['toolbar_cancel'],'/u')){
            $SPL_TXT['toolbar_cancel_title'].= '" accesskey="'.substr($SPL_TXT['toolbar_cancel'],(strpos($SPL_TXT['toolbar_cancel'],'/u')-2),1);
          }
          $tmp = str_replace('{TOOLBAR_CANCEL_TITLE}',$SPL_TXT['toolbar_cancel_title'],$tmp);
          $tmp = str_replace('{FORM_NAME}',$this->_formName,$tmp);
          $row = str_replace('{TOOLBAR_CANCEL}',$tmp,$row);
        }
        else {
          $row = str_replace('{TOOLBAR_CANCEL}','',$row);
        }
      }
      else {
        $row = str_replace('{TOOLBAR_CANCEL}','',$row);
      }
      
      // Save
      if ($save && isset($SPL_UI['toolbar_save'])) {
        // Must have change or write permission
        if ((isset($_SESSION[$this->_dataClass.'_permission']['change']) && $_SESSION[$this->_dataClass.'_permission']['change'] == true) ||
            (isset($_SESSION[$this->_dataClass.'_permission']['write']) && $_SESSION[$this->_dataClass.'_permission']['write'] == true)) {
          
          if (isset($SPL_UI['toolbar_save_position'])&&$SPL_UI['toolbar_save_position']=='form'){
            $this->_formButtons[] = 'save';
            $row = str_replace('{TOOLBAR_SAVE}','',$row);
          }
          else {
            $tmp = str_replace('{TOOLBAR_SAVE_TEXT}',$SPL_TXT['toolbar_save'],$SPL_UI['toolbar_save']);
            if (strpos($SPL_TXT['toolbar_save'],'/u')){
              $SPL_TXT['toolbar_save_title'].= '" accesskey="'.substr($SPL_TXT['toolbar_save'],(strpos($SPL_TXT['toolbar_save'],'/u')-2),1);
            }
            $tmp = str_replace('{TOOLBAR_SAVE_TITLE}',$SPL_TXT['toolbar_save_title'],$tmp);
            $tmp = str_replace('{FORM_NAME}',$this->_formName,$tmp);
            $row = str_replace('{TOOLBAR_SAVE}',$tmp,$row);
          }
        }
        else {
          $row = str_replace('{TOOLBAR_SAVE}','',$row);
        }
      }
      else {
        $row = str_replace('{TOOLBAR_SAVE}','',$row);
      }

      // Delete
      if ($del && isset($SPL_UI['toolbar_del'])) {
        // Must have 'delete' permission
        if (isset($_SESSION[$this->_dataClass.'_permission']['delete']) && $_SESSION[$this->_dataClass.'_permission']['delete'] == true) {
          if (isset($SPL_UI['toolbar_del_position'])&&$SPL_UI['toolbar_del_position']=='form'){
            $this->_formButtons[] = 'del';
            $row = str_replace('{TOOLBAR_DEL}','',$row);
          }
          else {
            $tmp = str_replace('{TOOLBAR_DEL_TEXT}',$SPL_TXT['toolbar_del'],$SPL_UI['toolbar_del']);
            if (strpos($SPL_TXT['toolbar_del'],'/u')){
              $SPL_TXT['toolbar_del_title'].= '" accesskey="'.substr($SPL_TXT['toolbar_del'],(strpos($SPL_TXT['toolbar_del'],'/u')-2),1);
            }
            $tmp = str_replace('{TOOLBAR_DEL_TITLE}',$SPL_TXT['toolbar_del_title'],$tmp);
            $tmp = str_replace('{TOOLBAR_DEL_CONFIRM}',$SPL_TXT['toolbar_del_confirm'],$tmp);
            $tmp = str_replace('{FORM_NAME}',$this->_formName,$tmp);
            $row = str_replace('{TOOLBAR_DEL}',$tmp,$row);
          }
        }
        else {
          $row = str_replace('{TOOLBAR_DEL}','',$row);
        }
      }
      else {
        $row = str_replace('{TOOLBAR_DEL}','',$row);
      }

      // Find
      if ($find && isset($SPL_UI['toolbar_find']) && (isset($SPL_UI['toolbar_find_position'])&&$SPL_UI['toolbar_find_position']=='toolbar')) {
        // Must have read permission
        if (isset($_SESSION[$this->_dataClass.'_permission']['read']) && $_SESSION[$this->_dataClass.'_permission']['read'] == true) {
          $tmp = str_replace('{TOOLBAR_FIND_TEXT}',$SPL_TXT['toolbar_find'],$SPL_UI['toolbar_find']);
          if (strpos($SPL_TXT['toolbar_find'],'/u')){
            $SPL_TXT['toolbar_find_title'].= '" accesskey="'.substr($SPL_TXT['toolbar_find'],(strpos($SPL_TXT['toolbar_find'],'/u')-2),1);
          }
          $tmp = str_replace('{TOOLBAR_FIND_TITLE}',$SPL_TXT['toolbar_find_title'],$tmp);
          $tmp = str_replace('{FORM_NAME}',$this->_formName,$tmp);
          $row = str_replace('{TOOLBAR_FIND}',$tmp,$row);
        }
        else {
          $row = str_replace('{TOOLBAR_FIND}','',$row);
        }
      }
      else {
        $row = str_replace('{TOOLBAR_FIND}','',$row);
      }
      
      // Help
      if (isset($SPL_UI['toolbar_help']) && (isset($SPL_UI['toolbar_help_position'])&&$SPL_UI['toolbar_help_position']=='toolbar')) {
        $tmp = str_replace('{TOOLBAR_HELP_TITLE}',$SPL_TXT['toolbar_help_title'],$SPL_UI['toolbar_help']);
        $tmp = str_replace('{FORM_NAME}',$this->_formName,$tmp);
        $tmp = str_replace('{DATA_CLASS}',$this->_dataClass,$tmp);
        $row = str_replace('{TOOLBAR_HELP}',$tmp,$row);
      }
      else {
        $row = str_replace('{TOOLBAR_HELP}','',$row);
      }

      // Print
      if (isset($SPL_UI['toolbar_print']) && (isset($SPL_UI['toolbar_print_position'])&&$SPL_UI['toolbar_print_position']=='toolbar')) {
        $tmp = str_replace('{TOOLBAR_PRINT_TITLE}',$SPL_TXT['toolbar_print_title'],$SPL_UI['toolbar_print']);
        $tmp = str_replace('{FORM_NAME}',$this->_formName,$tmp);
        $row = str_replace('{TOOLBAR_PRINT}',$tmp,$row);
      }
      else {
        $row = str_replace('{TOOLBAR_PRINT}','',$row);
      }

      // Icon
      $this->_icon = $icon;
      
      $this->startAreaToolbar();
      // Add a hidden field
      $this->addContent('<div class="hidden"><input type="hidden" name="toolbar_button" id="toolbar_button" value=""<!-- {SPL_ENDTAG} -->></div>','yes');
      $this->addContent($row);
      $this->endAreaToolbar();
    }
  }

  /**
   * Prepares the page for displaying it on the screen.
   * This doen not do the actial display, just the preparation of the page.
   * To display the page, use <code>showPage()</code>
   *
   * @access public
   */
  function createPage() {

    global $SPL_CFG;
    
    // Set the body header
    $this->_setBodyHeader();


    // Add ajax support
    if ($this->_ajax) {
      $this->_addAjax();
    }

    // Add a checklist :hover support
    if ($this->_checklist) {
      $this->_addChecklist();
    }

    // Add a calendar
    if ($this->_calendar) {
      $this->_addCalendar();
    }
   
    // Add a diagram
    if ($this->_diagram) {
      $this->_addDiagram();
    }

    // Add iframe resize capabilities
    if ($this->_iframe) {
      $this->_addIframeResize();
    }

    // Add an editor
    if ($this->_editor) {
      $this->_addEditor();
    }
    
    // Add an popups
    if ($this->_popup) {
      $this->_addPopup();
    }
    
    // Add helptexts
    if ($this->_help){
      $this->_addHelp();
    }
    
    // Kick out of frames
    if ($this->_frameKickout){
      $this->_addFrameKickout();
    }

    // Add client-side validation
    if ($this->_validate){
      $this->_addValidation();
      $this->_html = str_replace('/*fields_to_validate*/',$this->_validationRules,$this->_html);
    }

    // Add resizer
    /*
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/scale.js"></script>';
    $this->setHeadScripts($sContent);
    */

    // Set the body footer
    $this->_setBodyFooter();

    // Set the focus
    if (!empty($this->_focus)){
      $this->addBodyOption('onLoad','document.'.$this->_formName.'.'.$this->_focus.'.focus();');
    }
    
    // Set the toolbar button presubmits
    $presubmit = '';
    if ($this->_editor){
      if ($SPL_CFG['editor']=='tinyMCE') {
        $presubmit .= 'tinyMCE.triggerSave();';
      }
    }
    if ($this->_validate){
      $presubmit = 'validatePresubmitFields();if(submitResult)';
    }
    $this->_html = str_replace('{TOOLBAR_PRESUBMIT}',$presubmit,$this->_html);
    
    // Set body options
    $this->_setBodyOptions();
    
    // Set form options
    if (!empty($this->_formOptions)){
      $this->_setFormOptions();
    }

    // Loop through all parameters and set the variables to values
    foreach ($this->_parameters as $key => $value) {
      $template_name = '<!-- {'.$key.'} -->';
      $this->_html = str_replace($template_name, $value, $this->_html);
    }
    
    // Add the onload handler
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/onload.js"></script>';
    $this->setHeadScripts($sContent);

    $sContent = "<!-- {HEAD_SCRIPTS} -->"."<!-- onLoad handler -->";
    $this->setHeadScripts($sContent);
    
    // Set any search inputs
    $this->_html = str_replace('{SEARCH_INPUT}',$this->_getSearch(),$this->_html);
    
    // Set the icon
    $this->_html = str_replace('{ICON_NAME}',$this->_getIcon(),$this->_html);
    
    // Set top menu
    $this->_html = str_replace('{MENU_TOP}',$this->_topmenu,$this->_html);
    
    // Set user name
    $this->_html = str_replace('{USR_NAME}',(isset($_SESSION['my_name'])?': '.$_SESSION['my_name']:''),$this->_html);

    // Remove template placeholders
    $this->_html = str_replace('<!-- {BODY_CONTENT} -->', '<!-- Content -->'."\r\n".'<div id="bodyContent">', $this->_html);
    $this->_html = str_replace('<!-- {END_BODY_CONTENT} -->', '</div>', $this->_html);
    $this->_html = str_replace('<!-- {HEAD_SCRIPTS} -->', '', $this->_html);
    $this->_html = str_replace('<!-- {END_HEAD_SCRIPTS} -->', '', $this->_html);
  }

  /**
   * Ends a button area
   */
  function endAreaButton() {
    $this->addContent('</div>');
  }

  /**
   * Ends a data area
   */
  function endAreaData() {
    $this->addContent('</div>');
  }

  /**
   * Ends a grid area
   */
  function endAreaGrid() {
    $this->addContent('</div>');
  }

  /**
   * Ends a search area
   */
  function endAreaSearch() {
    $this->addContent('</div>');
  }

  /**
   * Ends a tab area
   */
  function endAreaTab() {
    $this->addContent('</div>');
  }

  /**
   * Ends a tabsheet area
   */
  function endAreaTabSheet() {
    $this->addContent('</div>');
  }

  /**
   * Ends a text area
   */
  function endAreaText() {
    $this->addContent('</div>');
  }

  /**
   * Ends a toolbar area
   */
  function endAreaToolbar() {
    $this->addContent('</div>');
  }

  /**
   * Adds an 'end chart' tag to the content
   * 
   * @param string $type The chart type (default='simple')
   */
  function endChart($type='simple') {
    global $SPL_UI;
    if ($type=='simple') {
      $this->addContent(isset($SPL_UI['simple_chart_end'])?$SPL_UI['simple_chart_end']:'</tr></table>');
    }
  }

  /**
   * Adds an 'end column' tag to the content
   * 
   * @access public
   */
  function endColumn() {
    $this->addContent('</td>');
  }


  /**
   * Adds an 'end fieldset' tag to the content
   * 
   * @access public
   */
  function endFieldset() {
    $this->addContent('</table></fieldset></td></tr>');
  }

  /**
   * Adds an 'end form' tag to the content
   * 
   * @param  (bool)   $wide     When true, no margins left and right will be added
   * @param  (bool)   $plain    When true, no table will be added
   * @access public
   */
  function endForm($wide=false,$plain=false) {
    
    global $SPL_UI;
    
    if (!isset($SPL_UI['form_end'])){
      $SPL_UI['form_end'] = '</form>';
    }
    
    // Create a hidden field for capturing search events
    if ($this->_nativeSearch){
      $this->addContent('<div class="hidden"><input type="hidden" name="search_button" id="search_button" value=""<!-- {SPL_ENDTAG} -->></div>','yes');
    }
    
    // See if we should end a pane
    if (!empty($this->_paneStart)){
      $this->endPane();
      $this->addContent(' <!-- end pane -->','yes');
    }
    
    $tmp = $SPL_UI['form_end'];
    
    if ($wide && isset($SPL_UI['form_end_wide'])) {
      $tmp = $SPL_UI['form_end_wide'];
    }
    if ($plain && isset($SPL_UI['form_end_plain'])) {
      $tmp = $SPL_UI['form_end_plain'];
    }
    
    $this->addContent($tmp,'yes');
  }


  /**
   * Adds an 'end menu' tag to the content
   * 
   * @access public
   */
  function endMenu() {
    global $SPL_UI;
    $this->addContent($SPL_UI['menubar_end'],'yes');
  }


  /**
   * Adds an 'end pane' tag to the content
   * 
   * @access public
   */
  function endPane($type='') {
    
    global $SPL_UI;
    
    $this->addContent(isset($SPL_UI[$type.'pane_right'])?$SPL_UI[$type.'pane_right']:$SPL_UI['pane_right']);
    $this->addContent(isset($SPL_UI[$type.'pane_bottom'])?$SPL_UI[$type.'pane_bottom']:$SPL_UI['pane_bottom']);
  }

  /**
   * Ends a scrollable area
   * 
   * @access public
   */
  function endScrollableArea() {
    global $SPL_UI;
    
    if ($this->_browser_detection('browser')!='mozilla') {
      if (!isset($SPL_UI['scroller_end'])) {
        $SPL_UI['scroller_end'] = '</div>';
      }
      $this->addContent($SPL_UI['scroller_end'],'yes');
    }
  }

  /**
   * Adds an 'end row' tag to the content
   * 
   * @access public
   */
  function endRow() {
    $this->addContent('</tr>','yes');
  }

  /**
   * Adds an 'end section' tag to the content
   * 
   * @access public
   */
  function endSection($name=NULL) {
    global $SPL_UI;
    if ($this->_border) {
      $this->addContent($SPL_UI['border'],'no');
    }
    $this->addContent($SPL_UI[($name?$name:'section_end')],'yes');
  }

  /**
   * Adds an 'end subsection' tag to the content
   * 
   * @access public
   */
  function endSubsection() {
    global $SPL_UI;
    $this->addContent($SPL_UI['subsection_end'],'yes');
  }

  /**
   * Adds an 'end table' tag to the content
   * 
   * @access public
   */
  function endTable() {
    
    global $SPL_UI;
    
    if (isset($SPL_UI['table_end'])) {
      $this->addContent($SPL_UI['table_end'],'yes');
    }
    else {
      $this->addContent('</table>','yes');
    }
  }

  /** 
   * Returns the generated HTML
   *
   * @return string The HTML
   * @access public
   */
  function getHTML() {
    return $this->_html;
  }

  /** 
   * Returns the path to the images directory
   *
   * @return string The path to the images
   * @access public
   */
  function getImageDir() {
    return $this->_imageDir;
  }

  /** 
   * Returns full path to the skin file
   *
   * @return string containing the path
   * @access public
   */
  function getSkin() {
    return $this->_skin;
  }


/** 
 * Initializes the metadata, the translations and the default template
 * 
 * @param string $class        The name of the class supplying the data
 * @param string $translation  (optional) The name of the translation file
 * @param string $template     (optional) The template name
 * @param string $title        (optional) The screen title (false = no title)
 * @param bool   $disabled     (optional) Disable title bar buttons
 * @param string $qTranslation (optional) Fully qualified path to the translation file
 * @param string $qMetadata    (optional) Fully qualified path to the metadata file
 */
  function init($class,$translation='default',$template='template.htm',$title=true,$disabled=false,$qTranslation=NULL,$qMetadata=NULL) {
    
    global $SPL_UI;
    
    // Set the name of the dataclass
    $this->_dataClass = $class;
    
    // Load translations
    if (!empty($qTranslation)){
      $this->_setTranslation($qTranslation,true);
    }
    else {
      $translation = ($translation=='default'?$class:$translation);
      $this->_setTranslation($translation);
    }
    
    // Meta data
    if (!empty($qMetadata)){
      $this->_setMetadata($qMetadata,true);
    }
    else {
      $this->_setMetadata($class);
    }
    
    // Template
    $this->_setTemplate($template);
    
    // Title
    if ($title){
      if ($SPL_UI['layout_mode']=='columns' && $class != 'spl') {
        if (is_bool($title)){
          $this->_paneStart = $this->_getPaneStart($this->translate($class.'_title_screen'),($disabled?'noclose':null));
        }
        else {
          $this->_paneStart = $this->_getPaneStart($this->translate($title),($disabled?'noclose':null));
        }
      }
      else {
        if (is_bool($title)){
          $this->_setTitle($class.'_title_screen',$disabled);
        }
        else {
          $this->_setTitle($title,$disabled);
        }
      }
    }
    else {
      // disable border
      $this->_border = (bool)false;
    }
    
  }
  
  /** 
   * Enables ajax dropdownlists
   *
   * @access public
   */
  function setAjaxLists() {
    $this->_ajax = true;
    $this->_ajaxlists = true;
  }


  function setContent($sContent = 'none') {
    /* Sets the body content
     *
     * param $sContent  string the body text
     * @access public
     */
    if ($sContent != 'none') {
      $this->addContent($sContent);
    }
    $this->_content = "<!-- {BODY_CONTENT} -->\r\n".$this->_content;
    $this->_html = str_replace('<!-- {BODY_CONTENT} -->', $this->_content, $this->_html);
  }

  /** 
   * Sets the scripts in the header section
   *
   * @param string $sContent The body text
   * @access public
   */
  function setHeadScripts($sContent) {
    $this->_html = str_replace('<!-- {HEAD_SCRIPTS} -->', $sContent, $this->_html);
  }

  /** 
   * Enables helptips
   *
   * @access public
   */
  function setHelpTips() {
    $this->_help = true;
  }

  /** 
   * Ensures the window is not inclosed in another frame
   *
   * @access public
   */
  function setKickOutOfFrames() {
    $this->_frameKickout = true;
  }

  /** 
   * Sets the scripts in the mappings section
   *
   * @param string $sContent The body text
   * @access public
   */
  function setMappings($sContent) {
    $sContent = "<!-- {MAPPINGS} -->\r\n".$sContent;
    $this->_html = str_replace('<!-- {MAPPINGS} -->', $sContent, $this->_html);
  }


  /** 
   * Sets a parameter to a value
   *
   * @param string $sName  The name of the parameter
   * @param string $sValue The value of the parameter
   * @access public
   */
  function setParameter($sName, $sValue) {
    $this->_parameters[$sName] = $this->translate($sValue);
  }

  /** 
   * Sets the record id information
   *
   * @param string $sID The record id 
   * @access public
   */
  function setRecordID($sID) {
    if ($sID != '') {
      $this->_html = str_replace('<!-- {RECORD_ID} -->', ' ('.$sID.') ', $this->_html);
    }
  }

  /** 
   * Sets the row statistics information 
   * 
   * @param $createdAt  timestamp (optional) The date the row was created 
   * @param $createdBy  integer   (optional) The userid creating the row
   * @param $createdSrc integer   (optional) The id of the creating source
   * @param $updatedAt  timestamp (optional) The date the row was updated 
   * @param $updatedBy  integer   (optional) The userid updating the row
   * @param $updatedSrc integer   (optional) The id of the updating source 
   * @access public
   * 
   * Be sure to make this call pretty much the last thing before you call
   * 'createPage()'.
   */
  function setRowStatistics($objectID='',$createdAt=false,$createdBy=false,$createdSrc=false,$updatedAt=false,$updatedBy=false,$updatedSrc=false) {
    
    global $SPL_CFG,$SPL_UI,$SPL_TXT;

    // Show object if
    if (strpos($this->_html,'{OBJECT_ID}')>0){
      if (isset($SPL_UI['object_id'])){
        $content = $SPL_UI['object_id'];
        $content = str_replace('{ID}',$objectID,$content);
      }
      else {
        $content = $objectID;
      }
      $this->_html = str_replace('{OBJECT_ID}',$content,$this->_html);
      $this->_html = str_replace('{FORM_NAME}',$this->_formName,$this->_html);
      $this->_html = str_replace('<!-- {SPL_ENDTAG} -->',(substr($SPL_CFG['doctype'],0,1)=='x'?' /':''),$this->_html);
    }

    // Show statistics
    if (strpos($this->_html,'{ROW_STATISTICS}')>0){
      
      // Get a readable date
      $createdAt = ($createdAt?strftime("%Y-%m-%d %H:%M",$createdAt):$createdAt);
      $updatedAt = ($updatedAt?strftime("%Y-%m-%d %H:%M",$updatedAt):$updatedAt);
  
      // Find the user. Don't print the name if the current user is the 
      // one that created the row
      if ($createdBy && ($createdBy!=$_SESSION['usr_id'])){
        include_once ($SPL_CFG['dirroot'].'/class/usr.class.php');
        $usr = new usr();
        $usr->init($createdBy);
        $createdUser = $usr->getName();
      }
      else {
        $createdUser=(bool)false;
      }
      
      if ($updatedBy){
        if ($updatedBy==$createdBy){
          $updatedUser = $createdUser;
        }
        else {
          // SPL_TODO: Find user
          $updatedUser=(bool)false;
        }
      }
      else {
        $updatedUser=(bool)false;
      } 
      
      // SPL_TODO: Find the creating source
      if ($createdSrc){
        $createdSource=(bool)false;
        $updatedSource=(bool)false;
      }
      else {
        $createdSource=(bool)false;
        $updatedSource=(bool)false;
      }
  
      $row = $SPL_UI['row_statistics'];

      // Assemble create statistics
      if ($createdAt) {
        $create = $SPL_TXT['create_date'].' '.$createdAt;
        if ($createdUser){
          $create.= ' '.$SPL_TXT['create_usr_id'].' '.$createdUser;
        }
        if ($createdSource){
          $create.= ' ('.$createdSource.')';
        }
        $row = str_replace('{ROW_CREATED}',$create, $row);
      }
      else {
        $row = str_replace('{ROW_CREATED}','', $row);
      }

      // Assemble update statistics
      if ($updatedAt) {
        $update = $SPL_TXT['update_date'].' '.$updatedAt;
        if ($updatedUser){
          $update.= ' '.$SPL_TXT['update_usr_id'].' '.$updatedUser;
        }
        if ($updatedSource){
          $update.= ' ('.$updatedSource.')';
        }
        $row = str_replace('{ROW_UPDATED}',$update, $row);
      }
      else {
        $row = str_replace('{ROW_UPDATED}','', $row);
      }
    
      // Build the statistics
      $this->_html = str_replace('{ROW_STATISTICS}',$row,$this->_html);
    }
  }

  /** 
   * Sets the top menubar
   */
  function setTopMenu($menu) {
    $this->_topmenu = $menu;
  }

  /** 
   * Displays any form buttons
   *
   * @access public
   */
  function showFormButtons() {
    
    global $SPL_CFG,$SPL_TXT;
    
    if (count($this->_formButtons)){
      $this->startRow();
      $this->startColumn();
      $this->endColumn();
      $this->startColumn();
      $this->endColumn();
      foreach ($this->_formButtons as $name){
        $this->startColumn();
        $this->addButton($name,'toolbar_'.$name,'submit','default','8','button');
        $this->endColumn();
      }
      $this->endRow();
    }
  }

  /** 
   * Sends the page to the browser
   *
   * @access public
   */
  function showPage() {
    
    global $SPL_CFG,$SPL_TXT;

    // Clear any tags left
    $this->_html = str_replace('{ROW_STATISTICS}', '', $this->_html);
    $this->_html = str_replace('<!-- {RECORD_ID} -->', '', $this->_html);
    $this->_html = str_replace('<!-- {UPDATE_INFO} -->', '', $this->_html);

    // Set the default header
    include ($SPL_CFG['dirroot'].'/ui/inc/header.php');

    // Add timer information
    if ($SPL_CFG['show_generation_time']=="true") {  
      $endTime = microtime(true);
      $diff = number_format(((substr($endTime,0,9)) + (substr($endTime,-10)) - (substr($this->_startTime,0,9)) - (substr($this->_startTime,-10))),4);   
      $this->_html = str_replace('{TIMER_INFO}', $SPL_TXT['timer_prefix'].$diff.$SPL_TXT['timer_suffix'], $this->_html);
    }
    else {
      $this->_html = str_replace('{TIMER_INFO}', '', $this->_html);
    }
    
    // SPL_HACK: For browsers not supporting PNG's correctly, use the GIF version
    if (!$this->_browser_detection('png')) {
      $this->_html = str_replace('.png','.gif',$this->_html);
    }
    // END_OF_HARDCODED_HACK

    // send html
    echo $this->_html;
  }

  /**
   * Starts a button area
   */
  function startAreaButton() {
    $this->addContent('<div class="areaButton">');
  }

  /**
   * Starts a data area
   */
  function startAreaData() {
    $this->addContent('<div class="areaData">');
  }

  /**
   * Starts a grid area
   */
  function startAreaGrid() {
    $this->addContent('<div class="areaGrid">');
  }

  /**
   * Starts a search area
   */
  function startAreaSearch() {
    $this->addContent('<div class="areaSearch">');
  }

  /**
   * Starts a tab area
   */
  function startAreaTab() {
    $this->addContent('<div class="areaTab">');
  }

  /**
   * Starts a tabsheet area
   */
  function startAreaTabSheet() {
    $this->addContent('<div class="areaTabSheet">');
  }

  /**
   * Starts a text area
   */
  function startAreaText() {
    $this->addContent('<div class="areaText">');
  }

  /**
   * Starts a toolbar area
   */
  function startAreaToolbar() {
    $this->addContent('<div class="areaToolbar">');
  }

  /**
   * Start a chart
   * 
   * @param string $type The chart type (default='simple')
   */
  function startChart($type='simple') {
    
    global $SPL_UI;
    
    if ($type=='simple') {
      $this->addContent(isset($SPL_UI['simple_chart_start'])?$SPL_UI['simple_chart_start']:'<table><tr>');
    }
  }

  /** Starts a table column
   *
   * @param string $class the name of the class (optional)
   * @param string $options the options (optional)
   * @access public
   */
  function startColumn($class='none',$options='none') {
    $columnClass = ($class == 'none' ? "" : ' class="'.$class.'"');
    $columnOptions = ($options == 'none' ? "" : " ".$options);
    $this->addContent("<td".$columnClass.$columnOptions.">");
  }


  /** 
   * Starts a fieldset
   *
   * @param $class   string  (optional) The name of the class to use
   * @param $title   string  (optional) The title of the field set
   * @param $line_up boolean When true, the set is lined up with input fields
   * @access public
   */
  function startFieldset($title=NULL,$class='fieldset',$line_up=false) {
    
    $this->addContent('<tr>'.($line_up?'<td></td><td></td>':'').'<td><fieldset class="'.$class.'" id="'.uniqid('spl_').'"><legend>'.$title.'</legend><table>');
  }

  /** 
   * Starts a form 
   * 
   * @param  (string) $sName   the name of the form 
   * @param  (string) $sAction the action to be performed
   * @param  (bool)   $setToken When true, a token wille be generated in $_SESSION['token']
   * @param  (bool)   $wide     When true, no margins left and right will be added
   * @param  (bool)   $editMode When true, an editable form is assumed
   * @param  (bool)   $plain    When true, no table will be added
   * @public
   */
  function startForm($name=null,$action='default',$setToken=false,$wide=false,$editMode=true,$plain=false) {
    
    global $SPL_CFG, $SPL_UI;

    // Create a default formname
    if (!isset($name)){
      if (isset($this->_dataClass)){
        $name = $this->_dataClass.'form';
      }
      else {
        $name = 'splform';
      }
    }
    $this->_formName = $name;
    if ($action=='default') {
      $action = $_SERVER['PHP_SELF'];
    }
    $this->_formAction = $action;
    $method = 'post';

    if (!isset($SPL_UI['form_start'])) {
      $SPL_UI['form_start'] = '<form name="{FORM_NAME}" id="{FORM_NAME}" action="{FORM_ACTION}" method="{FORM_METHOD}">';
    }
    
    $tmp = $SPL_UI['form_start'];
    
    if ($wide && isset($SPL_UI['form_start_wide'])) {
      $tmp = $SPL_UI['form_start_wide'];
    }
    
    if ($plain && isset($SPL_UI['form_start_plain'])) {
      $tmp = $SPL_UI['form_start_plain'];
    }

    $tmp = str_replace('{FORM_NAME}',$name,$tmp);
    $tmp = str_replace('{FORM_ID}',$name,$tmp);
    $tmp = str_replace('{FORM_ACTION}',$action,$tmp);
    $tmp = str_replace('{FORM_METHOD}',$method,$tmp);
    $this->addContent($tmp,'yes');
    if ($editMode) {
      $this->addFormOption('onreset','resetDirtyFlag();');
    }
    
    // See if we should start a pane
    if (!empty($this->_paneStart)){
      $this->addContent($this->_paneStart);
    }
    
    if ($setToken) {
      // Add a token to prevent Cross-site Request Forgories attacks
      $_SESSION['token']      = md5(uniqid(rand(), TRUE));
      $_SESSION['token_time'] = time();
      $this->addContent('<input type="hidden" name="token" value="'.$_SESSION['token'].'" />');
    }
  }

  function startFormUpload($sName, $sAction) {
    /* Sets the form tag
     *
     * param $sName   string the name of the form
     * param $sAction string the action to be performed
     * @access public
     */

    $this->_formName = $sName;
    $this->addContent("<form enctype=\"multipart/form-data\" name='".$sName."' id='".$sName."' action='".$sAction."' method='post'>");
  }

  /** 
   * Starts the menu
   */
  function startMenu() {
    
    global $SPL_UI;
    
    $this->addContent($SPL_UI['menubar_start']);
  }


  /** 
   * Starts a pane
   * 
   * @param  $title string The title to show on top of the pane
   * @param  $close string Action to perform when clicking the clode icon ('noclose'=don't display the icon)
   * @access public
   */
  function startPane($title=NULL,$close=NULL,$type='') {
    
    $this->addContent($this->_getPaneStart($title,$close,$type));
  }


  /** 
   * Starts a table row
   *
   * @param $sClass   string the name of the class (optional) 
   * @param $sOptions string the options (optional)
   * @access public
   */
  function startRow($sClass = 'none', $sOptions = 'none') {
    $rowClass = ($sClass == 'none' ? "" : ' class="'.$sClass.'"');
    $rowOptions = ($sOptions == 'none' ? "" : " ".$sOptions);
    $this->addContent("<tr".$rowClass.$rowOptions.">");
  }

  /** 
   * Starts a scrollable area
   * 
   * @access public
   */
  function startScrollableArea() {
    
    global $SPL_UI;

    if ($this->_browser_detection('browser')=='mozilla') {
      $this->addContent('<div class="scrollable">','yes');
    }
    else {
      if (!isset($SPL_UI['scroller_start'])) {
        $SPL_UI['scroller_start'] = '<div class="scrollable" style="height:100%;background-color:transparent;border:none">';
      }
      $this->addContent($SPL_UI['scroller_start'],'yes');
    }

  }
  
  /** 
   * Starts a section
   *
   * @param $class string (optional) The name of the class to use
   * @access public
   */
  function startSection($class='section',$name=NULL) {
    
    global $SPL_UI;
    
    $row = str_replace('{SECTION_CLASS}',$class,$SPL_UI[($name?$name:'section_start')].($this->_border?$SPL_UI['border']:''));
    $this->addContent($row,'yes');
  }

  /** 
   * Starts a subsection
   *
   * @param (string) $class      (optional) The name of the class to use
   * @param (string) $tdStyle    (optional) The td style property
   * @param (string) $tableStyle (optional) The table style property
   * @access public
   */
  function startSubsection($class='subsection',$tdStyle=null,$tableStyle=null) {
    
    global $SPL_UI;
    
    $row = str_replace('{SUBSECTION_CLASS}',$class,$SPL_UI['subsection_start']);
    if (isset($tdStyle)) {
      $row = str_replace('{TD_STYLE}',' style="'.$tdStyle.'"',$row);
    }
    else {
      $row = str_replace('{TD_STYLE}','',$row);
    }
    if (isset($tableStyle)) {
      $row = str_replace('{TABLE_STYLE}',' style="'.$tableStyle.'"',$row);
    }
    else {
      $row = str_replace('{TABLE_STYLE}','',$row);
    }
    $this->addContent($row,'yes');
  }

  /** 
   * Starts a table
   * 
   * @param  string $sClass   The name of the class (optional) 
   * @param  string $sOptions The options (optional)
   * @access public
   */
  function startTable($sClass='none',$sOptions='none') {
    
    global $SPL_UI;
    
    $tableClass = ($sClass == 'none' ? "" : ' class="'.$sClass.'"');
    $tableOptions = ($sOptions == 'none' ? "" : " ".$sOptions);
    
    // Find out if a skin has overridden the default table start
    if (isset($SPL_UI['table_start'])) {
      $tmp = $SPL_UI['table_start'];
      $tmp = str_replace('{TABLE_CLASS}',$tableClass,$tmp);
      $tmp = str_replace('{TABLE_OPTIONS}',$tableOptions,$tmp);
      $this->addContent($tmp,'yes');
    }
    else {
      $this->addContent('<table'.$tableClass.$tableOptions.' cellspacing="0" cellpadding="0">','yes');
    }
  }

  /** 
   * Translates the given string and returns the translation
   *
   * @param  string $sProperty the name of the data to be returned
   * @return string the translated property
   * @access public
   */  
  function translate($property) {

    global $SPL_TXT;

    if (array_key_exists(strtolower($property), $SPL_TXT)) {
      return $SPL_TXT[strtolower($property)];
    } else {
      return $property;
    }
  }

  //=========================================================================
  // Private functions
  //=========================================================================

  /** 
   * Adds ajax capabilities to the page
   * 
   * @access Private
   */
  function _addAjax() {
    
    global $SPL_CFG;
    
    // Add the javascript references
    if ($this->_ajaxlists){
      //$sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">ajax_setFile(\''.$SPL_CFG['wwwroot'].'/ui/stu_ajax.php\');</script>';
      //$this->setHeadScripts($sContent);
      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/ajax-dynamic-list.js"></script>';
      $this->setHeadScripts($sContent);
    }

    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/ajax.js"></script>';
    $this->setHeadScripts($sContent);

    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- Ajax -->";
    $this->setHeadScripts($sContent);
    
  }
  
  /**
   * Adds calendar capabilities to the page
   * @private
   */
  function _addCalendar() {
    global $SPL_CFG;
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/dlcalendar.js"></script>';
    $this->setHeadScripts($sContent);

    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- Calendar -->";
    $this->setHeadScripts($sContent);
  }

  /** 
   * Adds :hover functionality on labels for IE
   * @Private
   */
  function _addChecklist() {
    
    global $SPL_CFG;
    
    // Add the javascript references
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">addLoadEvent(function(){initChecklist();});function initChecklist(){if (document.all && document.getElementById){var lists = document.getElementsByTagName("ul");for (i = 0; i < lists.length; i++){var theList = lists[i];if (theList.className.indexOf("checklist") > -1){var labels = theList.getElementsByTagName("label");for (var j = 0; j < labels.length; j++){var theLabel = labels[j];theLabel.onmouseover = function(){this.className += " hover"; };theLabel.onmouseout = function(){this.className = this.className.replace(" hover", "");};}}}}}</script>';
    $this->setHeadScripts($sContent);

    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- checkList :hover workaround for IE -->";
    $this->setHeadScripts($sContent);
  }

  /** 
   * Adds diagram capabilities to the page
   * 
   * @access Private
   */
  function _addDiagram() {
    
    global $SPL_CFG;
    
    // add the javascript references
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/diagram/diagram.js"></script>';
    $this->setHeadScripts($sContent);

    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- Diagram Builder -->";
    $this->setHeadScripts($sContent);
  }

  /** 
   * Adds editor capabilities to the page
   * 
   * @access Private
   */
  function _addEditor() {
    
    global $SPL_CFG;
    
    // TinyMCE
    if ($SPL_CFG['editor']=='tinyMCE'){
      // add the javascript references
      //$sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">tinyMCE.init({mode:"textareas",language:"'.$_SESSION['usp_lang'].'",convert_newlines_to_brs:true,force_p_newlines:false});</script>';
      //$sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">tinyMCE.init({mode:"textareas",language:"'.$_SESSION['usp_lang'].'",theme:"advanced",extended_valid_elements:"font[size|color|face]",theme_advanced_buttons1:"bold,italic,underline,strikethrough,separator,justifyleft,justifycenter,justifyright",theme_advanced_buttons1_add_before:"fontsizeselect,forecolor",theme_advanced_buttons1_add:"separator,bullist,numlist,separator,undo,redo,separator,link,unlink,image,separator,charmap",theme_advanced_buttons2:"",theme_advanced_buttons3:"",theme_advanced_disable:"styleselect,formatselect",convert_newlines_to_brs:false,force_p_newlines:false,remove_linebreaks:true,auto_cleanup_word:true});</script>';
      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">tinyMCE.init({mode:"textareas",language:"'.$_SESSION['usp_lang'].'",theme:"advanced",extended_valid_elements:"font[size|color|face]",theme_advanced_buttons1:"bold,italic,underline,strikethrough,separator,justifyleft,justifycenter,justifyright,separator,bullist,numlist",theme_advanced_buttons2:"fontsizeselect,forecolor,separator,link,unlink,image,separator,charmap",theme_advanced_buttons3:"",theme_advanced_disable:"styleselect,formatselect",convert_newlines_to_brs:false,force_p_newlines:false,remove_linebreaks:true,auto_cleanup_word:true,baseURL:"'.$SPL_CFG['wwwroot'].'/ui/js/tiny_mce",document_base_url:"'.$SPL_CFG['wwwroot'].'/ui/js/tiny_mce"});</script>';
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/tiny_mce/tiny_mce.js"></script>';
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- TinyMCE editor -->";
      $this->setHeadScripts($sContent);
    }
  }

  /** 
   * Ensures the window is a top window
   * 
   * @access Private
   */
  function _addFrameKickout() {
    
    global $SPL_CFG;
    
    // add the javascript references
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">if (window != top) top.location.href = location.href;</script>';
    $this->setHeadScripts($sContent);

    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- Kick out of frames -->";
    $this->setHeadScripts($sContent);
  }

  /** 
   * Adds helptext capabilities to the page
   * 
   * @access Private
   */
  function _addHelp() {
    
    global $SPL_CFG;

    if ($SPL_CFG['help']=='helptip'){
      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/tooltip/helptip.js"></script>';
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- WebFX helptips -->";
      $this->setHeadScripts($sContent);
    }

  }

  /** 
   * Adds iframe resize capabilities to the page
   * 
   * @access Private
   */
  function _addIframeResize() {
    
    global $SPL_CFG;
    
    // add the javascript references
    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/ssi.js"></script>';
    $this->setHeadScripts($sContent);

    $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- IFrame SSI -->";
    $this->setHeadScripts($sContent);
  }

  /** 
   * Adds multipse select support to the page
   * 
   * @access Private
   */
  function _addMultipleSelect() {
    
    global $SPL_CFG;
    
    // add the javascript references
      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/mselect.js"></script>';
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- Multiple select boxes -->";
      $this->setHeadScripts($sContent);
    }

  /** 
   * Adds popup capabilities to the page
   * 
   * @access Private
   */
  function _addPopup() {
    
    global $SPL_CFG;
    
    // Make sure we've got a value
    if (!isset($SPL_CFG['popup'])) {
      $SPL_CFG['popup'] = 'none';
    }
    
    if ($SPL_CFG['popup']=='winlike') {
      
      // add the javascript references
      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">WinLIKE.definewindows=mydefs;function mydefs(){var j=new WinLIKE.window(\'\',200,200,490,500);j.Min=false;j.Nam=\'popup\';j.Vis=false;WinLIKE.addwindow(j);}</script>';
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="js/winlike/winlike/winman/winman.js"></script>';
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="js/winlike/winlike/winman/wininit.js"></script>';
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">WinLIKEskinpath="js/winlike/skins/";WinLIKEfilepath="js/winlike/winlike/";</script>';
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">WinLIKEerrorpage="js/winlike/winlike/winman/hlp-error.html";</script>';
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- WinLIKE popups -->";
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {BODY_HEADER} -->\r\n".'<div class="hidden"><img id="ih_" src="js/winlike/skins/trans.gif" style="z-Index:4000;position:absolute;left:0;top:0;width:100%;height:100%"></div>';
      $this->setBodyHeader($sContent);
    
      $this->addBodyOption('onResize','WinLIKE.resizewindows();');
      $this->addBodyOption('onLoad','WinLIKE.init();');
    }
  }


  /** 
   * Adds validation capabilities to the page
   * 
   * @access Private
   */
  function _addValidation() {
    
    global $SPL_CFG,$SPL_TXT;

    if (isset($SPL_CFG['client_validate']) && $SPL_CFG['client_validate']=='default'){
      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">var submitResult=true;function validatePresubmitFields(){var errs=0;/*fields_to_validate*/if (errs>=1){alert(\''.$SPL_TXT['invalid_form'].'\');}submitResult=(errs==0);}function validateFields(){var errs=0;/*fields_to_validate*/if (errs>=1){alert(\''.$SPL_TXT['invalid_form'].'\');}return (errs==0);}</script>';
      $this->setHeadScripts($sContent);
      
      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript" src="'.$SPL_CFG['wwwroot'].'/ui/js/validation/formval.js"></script>';
      $this->setHeadScripts($sContent);
      
      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n".'<script type="text/javascript">var eRequired="'.$SPL_TXT['required'].'";var eInvalidEmail="'.$SPL_TXT['invalid_email'].'";var wUnusualEmail="'.$SPL_TXT['unusual_email'].'";var eInvalidPhone="'.$SPL_TXT['invalid_phone'].'";var wUnusualPhone="'.$SPL_TXT['unusual_phone'].'";var eInvalidAge="'.$SPL_TXT['invalid_age'].'";var wUnusualAge="'.$SPL_TXT['unusual_age'].'";</script>';
      $this->setHeadScripts($sContent);

      $sContent = "<!-- {HEAD_SCRIPTS} -->\r\n"."<!-- Form validation -->";
      $this->setHeadScripts($sContent);
      
      // Set the form submit handler
      $this->addFormOption('onsubmit','return validateFields();');
    }

  }

  /** 
   * Determines the type of user agent
   */
  function _browser_detection($which_test) {

    // initialize the variables
    $browser       = '';
    $dom_browser   = (bool)false;
    $xhtml_browser = (bool)false;
    $png_browser   = (bool)false;

    if (!isset($_SERVER['HTTP_USER_AGENT'])) {
      // Weird bug reported by Jon Bolton. Assume the worst
      $navigator_user_agent = 'msie';
    }
    else {
      // set to lower case to avoid errors
      $navigator_user_agent = strtolower($_SERVER['HTTP_USER_AGENT']);
    }

    // run through the main browser possibilities, assign them to the main $browser variable
    if (stristr($navigator_user_agent, "opera")) {
      $browser = 'opera';
      $dom_browser = true;
      $xhtml_browser = true;
      $png_browser = true;
    }

    elseif (stristr($navigator_user_agent, "msie 4")) {
      $browser = 'msie';
    }

    elseif (stristr($navigator_user_agent, "msie 5")) {
      $browser = 'msie';
    }

    elseif (stristr($navigator_user_agent, "msie 6")) {
      $browser = 'msie';
    }

    elseif (stristr($navigator_user_agent, "msie 7")) {
      $browser = 'msie';
      $dom_broswer = true;
      $xhtml_browser = true;
      $png_browser = true;
    }

    elseif (stristr($navigator_user_agent, "msie")) {
      $browser = 'msie';
      $dom_browser = true;
    }

    elseif (stristr($navigator_user_agent, "konqueror")) {
      $browser = 'konqueror';
      $dom_browser = true;
      $xhtml_browser = true;
      $png_browser = true;
    }

    else if (stristr($navigator_user_agent, "safari")) {
      $browser = 'safari';
      $dom_browser = true;
      $xhtml_browser = true;
      $png_browser = true;
    }
      
    elseif (stristr($navigator_user_agent, "gecko")) {
      $browser = 'mozilla';
      $dom_browser = true;
      $xhtml_browser = true;
      $png_browser = true;
    }

    elseif (stristr($navigator_user_agent, "mozilla/4")) {
      $browser = 'ns4';
      $dom_browser = false;
    } else {
      $dom_browser = false;
      $browser = false;
    }

    // return the test result you want
    if ($which_test == 'browser') {
      return $browser;
    }
    elseif ($which_test == 'dom') {
      return $dom_browser;
    }
    elseif ($which_test == 'xhtml') {
      return $xhtml_browser;
    }
    elseif ($which_test == 'png') {
      return $png_browser;
    }
  }

  function _getReference($sFields, $sTable, $sWhere, $sValue) {

    $query = "SELECT ".$sFields." FROM ".$sTable." WHERE ".$sWhere." = '".$sValue."' LIMIT 1";
    $_reference = SPL_retrieve(SPL_query($query));

    if (!$_reference) {
      if ($sTable == 'usr') {
        $sValue = $this->translate('System administrator');
      }
    } else {
      // SPL_TODO: Make this dynamic
      if ($sTable == 'usr') {
        $sValue = $_reference['usr_name']." ". (isset ($_reference['usr_middlename']) ? $_reference['usr_middlename'].' ' : '').$_reference['usr_lastname'];
      } else
        if ($sTable == 'ctg') {
          $sValue = $_reference['ctg_name'];
        }
    }
    return $sValue;
  }

  /**
   * Adds a small search input 
   */
  function _getSearch() {
    
    global $SPL_CFG,$SPL_TXT;
    
    return '<span id="search"><input name="q" id="q" type="text" size="10" value="" /></span>';
  }


  /** 
   * Creates the body footer 
   * 
   * @access private
   */
  function _setBodyFooter() {
    $content = '';
    $i = (int)0;
    if (count($this->_bodyFooter)>0) {
      foreach ($this->_bodyFooter as $row) {
        $i++;
        $content .= '  '.$row;
        if ($i<count($this->_bodyFooter)) {
          $content .= "\r\n";
        }
      }
      $this->_html = str_replace('<!-- {BODY_FOOTER} -->', '<!-- Footer -->'."\r\n".'<div id="bodyFooter">'."\r\n".$content, $this->_html);
      $this->_html = str_replace('<!-- {END_BODY_FOOTER} -->', '</div>', $this->_html);
    }
    else {
      $this->_html = str_replace('<!-- {BODY_FOOTER} -->', '<!-- Footer -->', $this->_html);
      $this->_html = str_replace('<!-- {END_BODY_FOOTER} -->', '', $this->_html);
    }
  }

  /** 
   * Creates the body header 
   * 
   * @access private
   */
  function _setBodyHeader() {
    $content = '';
    $i = (int)0;
    if (count($this->_bodyHeader)>0) {
      foreach ($this->_bodyHeader as $row) {
        $i++;
        $content .= '  '.$row;
        if ($i<count($this->_bodyHeader)) {
          $content .= "\r\n";
        }
      }
      $this->_html = str_replace('<!-- {BODY_HEADER} -->', '<!-- Header -->'."\r\n".'<div id="bodyHeader">'."\r\n".$content, $this->_html);
      $this->_html = str_replace('<!-- {END_BODY_HEADER} -->', '</div>', $this->_html);
    }
    else {
      $this->_html = str_replace('<!-- {BODY_HEADER} -->', '<!-- Header -->', $this->_html);
      $this->_html = str_replace('<!-- {END_BODY_HEADER} -->', '', $this->_html);
    }
  }

  /** 
   * Creates the body tag options 
   * 
   * @access private
   */
  function _setBodyOptions() {
    $options = '';
    if (count($this->_bodyOptions)>0) {
      foreach ($this->_bodyOptions as $event => $handler) {
        $options .= ' '.$event.'="'.$handler.'"';
      }
      $this->_html = str_replace('<body', '<body '.$options, $this->_html);
    }
  }

  /** 
   * Creates the form tag options 
   * 
   * @access private
   */
  function _setFormOptions() {
    $options = '';
    if (count($this->_formOptions)>0) {
      foreach ($this->_formOptions as $event => $handler) {
        $options .= ' '.$event.'="'.$handler.'"';
      }
      $this->_html = str_replace('<form', '<form '.$options, $this->_html);
    }
  }

  /**
   * Retrieves the form icon
   * 
   * @return (string) The icon name
   * @private
   */
  function _getIcon() {
    
    global $SPL_UI;
    
    if (!isset($SPL_UI['icon_position'])) {$SPL_UI['icon_position'] = 'toolbar';}
    if (!isset($SPL_UI['icon_size']))     {$SPL_UI['icon_size'] = 'normal';}
    if ($this->_icon=='default.png') {
      if ($SPL_UI['icon_size']=='small') {
        $icon = $this->_dataClass.'_small.png';
      }
      else if ($SPL_UI['icon_size']=='large') {
        $icon = $this->_dataClass.'_large.png';
      }
      else {
        $icon = $this->_dataClass.'.png';
      }
    }
    else {
      $icon = $this->_icon;
    }
    return $icon;
  }

  /** 
   * Loads the metadata
   * 
   * @param  $filename  string The name of the metadata file
   * @param  $qualified bool   The filename give is a fully qualified path
   * @access private
   */
  function _setMetadata($filename,$qualified=false) {
    
    global $SPL_CFG,$SPL_META;

    if ($qualified) {
      @include_once($SPL_CFG['dirroot'].'/ui/meta/spl.metadata.php');
      @include_once($filename);
    }
    else {
      @include_once($SPL_CFG['dirroot'].'/ui/meta/spl.metadata.php');
      @include_once($SPL_CFG['dirroot'].'/ui/meta/'.$filename.'.metadata.php');
    }
  }


  /** 
   * Sets the template to be used
   * 
   * @param  $template  string the name of the template
   * @access private
   */
  function _setTemplate($template='template.htm') {

    global $SPL_CFG,$SPL_TXT,$SPL_UI;

    $template_file = $SPL_CFG['dirroot']."/ui/skin/".$_SESSION['usp_skin']."/".$template;
    $template_date = strftime("%Y-%m-%d %H:%M");
    
    // Style sheet
    $user_browser = $this->_browser_detection('browser');
    if (isset($SPL_CFG['wwwroot'])) {
      $skin_file = $SPL_CFG['wwwroot']."/ui/skin/".$_SESSION['usp_skin']."/".$user_browser."_skin.css";
    }
    else {
      $skin_file = "skin/".$_SESSION['usp_skin']."/".$user_browser."_skin.css";
    }
    
    // check if template file exists
    if (!file_exists($template_file)) {
      // if not, find the default template file for this skin (application independent)
      $template_file = $SPL_CFG['dirroot']."/ui/skin/".$_SESSION['usp_skin']."/default.htm";
      // if even the default cannot be found, raise an error
      if (!file_exists($template_file)) {
        die("Template file cannot be found: ".$SPL_CFG['dirroot']."/ui/skin/".$_SESSION['usp_skin']."/".$template);
      }
    }
    // Set the instance variable
    $this->_template = $template_file;

    // Read template into an array and create a string
    $this->_html = implode("", (file($this->_template)));

    // SPL_HACK: Winlike does not support the xhtml doctype. Force html 4 when using winlike
    if($SPL_CFG['popup']=='winlike'){
      $SPL_CFG['doctype'] = 'html4';
      $SPL_CFG['doctype_dtd'] = 'transitional';
    }

    // SPL_HACK: Change the doctype for browsers not supporting xhtml well
    if (!$this->_browser_detection('browser')) {
      $SPL_CFG['doctype'] = 'html4';
      $SPL_CFG['doctype_dtd'] = 'transitional';
    }
    

    // Determine the document type
    if (!stristr($this->_html,'<!DOCTYPE')) {
      $doctype = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">';
      
      // If the doctype has not been specified in config.php, determine the best for this browser
      if (empty($SPL_CFG['doctype'])){
        if ($this->_browser_detection('dom')) {
          $SPL_CFG['doctype'] = 'xhtml1';
          if (empty($SPL_CFG['doctype_dtd'])) {
            $SPL_CFG['doctype_dtd'] = 'transitional';
          }
        }
        else {
          $SPL_CFG['doctype'] = 'html4';
          if (empty($SPL_CFG['doctype_dtd'])) {
            $SPL_CFG['doctype_dtd'] = 'transitional';
          }
        }
      }
      
      // HTML 3.2
      if ($SPL_CFG['doctype']=='html3') {
        $this->_parameters['SPL_ENDTAG'] = '';
        $doctype = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">';
      }
      
      // HTML 4.01
      if ($SPL_CFG['doctype']=='html4') {
        $this->_parameters['SPL_ENDTAG'] = '';
        if ($SPL_CFG['doctype_dtd']=='strict') {
          $doctype = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/1999/REC-html401-19991224/strict.dtd">';
        }
        else if ($SPL_CFG['doctype_dtd']=='frameset') {
          $doctype = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Frameset//EN" "http://www.w3.org/TR/html4/frameset.dtd">';
        }
        else {
          $doctype = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">';
        } 
      }
      
      // XHTML 1.0
      if ($SPL_CFG['doctype']=='xhtml1') {
        $this->_parameters['SPL_ENDTAG'] = ' /';
        if ($SPL_CFG['doctype_dtd']=='strict') {
          $doctype = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">';
        }
        else if ($SPL_CFG['doctype_dtd']=='frameset') {
          $doctype = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Frameset//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd">';
        }
        else {
          $doctype = '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">';
        }
        $this->_html = str_replace('<html>','<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="'.$_SESSION['usp_lang'].'" lang="'.$_SESSION['usp_lang'].'">',$this->_html); 
      }
      
      // Add the doctype to the page
      $this->_html = $doctype."\r\n".$this->_html;
    }

    // Set the instance variable
    $this->_skin = $skin_file;
    $this->_imageDir = $SPL_CFG['wwwroot']."/ui/skin/".$_SESSION['usp_skin']."/images/";

    // Set some default parameters
    $this->_parameters['SPL_NAME']      = $SPL_CFG['appname'];
    $this->_parameters['SPL_VERSION']   = $_SESSION['appversion'];
    $this->_parameters['SPL_DATE']      = $template_date;
    $this->_parameters['SPL_AUTHOR']    = 'Jebbol';
    $this->_parameters['SPL_CREATOR']   = 'www.studiewijzer.nl';
    $this->_parameters['SPL_LANG']      = $_SESSION['usp_lang'];
    $this->_parameters['SPL_CHARSET']   = (isset($SPL_CFG['charset'])?$SPL_CFG['charset']:'UTF-8');
    $this->_parameters['SPL_SKIN']      = $this->_skin;
    $this->_parameters['SPL_SKIN_NAME'] = $_SESSION['usp_skin'];
    $this->_parameters['SPL_IMAGE']     = $this->_imageDir;
    $this->_parameters['TITLE']         = $SPL_CFG['appname'].'{USR_NAME}';
    $this->_parameters['DESCRIPTION']   = $SPL_CFG['appname']." ".$_SESSION['appversion'];
    $this->_parameters['ROBOTS']        = (isset($SPL_CFG['robots'])?$SPL_CFG['robots']:'noindex');
    $this->_parameters['SPL_WWWROOT']   = $SPL_CFG['wwwroot'];
    $this->_parameters['P3P_POLICY']    = (isset($SPL_CFG['p3p_policy'])?$SPL_CFG['p3p_policy']:'NOI NID STP');
    $this->_parameters['SPL_KEYWORDS']  = (isset($SPL_CFG['keywords'])?$SPL_CFG['keywords']:'');

  }


  /**
   * Retrieves the pan ttitle and pane start divs
   */
  function _getPaneStart($title=NULL,$close=NULL,$type='') {
    
    global $SPL_UI;
    
    $paneDef = '';
    
    if ($close=='smaller') {
      $paneDef = (isset($SPL_UI[$type.'pane_top_smaller'])?$SPL_UI[$type.'pane_top_smaller']:$SPL_UI['pane_top_smaller']);
    }
    
    else if ($close=='show') {
      $paneDef = (isset($SPL_UI[$type.'pane_top_larger'])?$SPL_UI[$type.'pane_top_larger']:$SPL_UI['pane_top_larger']);
    }
    
    else {
      if ($close=='noclose') {
        $paneDef = (isset($SPL_UI[$type.'pane_top_noclose'])?$SPL_UI[$type.'pane_top_noclose']:$SPL_UI['pane_top_noclose']);
      }
      else {
        $paneDef = (isset($SPL_UI[$type.'pane_top'])?$SPL_UI[$type.'pane_top']:$SPL_UI['pane_top']);
      }
    }

    $row = str_replace('{PANE_TITLE}',($title?$title:''),$paneDef);

    if ($close) {
      if ($close!='noclose'&&$close!='hide'&&$close!='show'){
        $row = str_replace('menu.php',$close,$row);
        $row = str_replace('#',$close,$row);
      }
    }

    $row .= (isset($SPL_UI[$type.'pane_left'])?$SPL_UI[$type.'pane_left']:$SPL_UI['pane_left']);
    
    return $row;
    
  }

  /** 
   * Sets the form title
   *
   * @param $title string The title of the form 
   * @access public
   */
  function _setTitle($title,$disabled=false) {
    
    global $SPL_UI,$SPL_TXT;
    
    $title = $this->translate($title);
    if (!isset($SPL_UI['title'])) {
      return;
    }
    else {
      $row = str_replace('{TITLE}',$title,($disabled?$SPL_UI['title_disabled']:$SPL_UI['title']));
      $row = str_replace('{LOGOFF_TITLE}',$SPL_TXT['logoff_title'],$row);
      $row = str_replace('{PREFERENCES_TITLE}',$SPL_TXT['preferences_title'],$row);
      if (isset($SPL_TXT['preferences_tooltip'])) {
        $row = str_replace('{PREFERENCES_TOOLTIP}',$SPL_TXT['preferences_tooltip'],$row);
      }
      else {
        $row = str_replace('{PREFERENCES_TOOLTIP}','',$row);
      }
      if (isset($SPL_TXT['help_title'])) {
        $row = str_replace('{HELP_TITLE}',$SPL_TXT['help_title'],$row);
      }
      else {
        $row = str_replace('{HELP_TOOLTIP}','Help',$row);
      }
      if (isset($SPL_TXT['help_tooltip'])) {
        $row = str_replace('{HELP_TOOLTIP}',$SPL_TXT['help_tooltip'],$row);
      }
      else {
        $row = str_replace('{HELP_TOOLTIP}','Help',$row);
      }
      $row = str_replace('{PASSWORD_TITLE}',$SPL_TXT['menu_password'],$row);
      $row = str_replace('{MAINMENU_TITLE}',$SPL_TXT['mainmenu_title'],$row);
      $row = str_replace('{MAINMENU_TOOLTIP}',$SPL_TXT['mainmenu_tooltip'],$row);
      $row = str_replace('{BACK_TITLE}',$SPL_TXT['back_title'],$row);
      $backscript = (isset($_SERVER['HTTP_REFERER'])?$_SERVER['HTTP_REFERER']:'menu.php');
      // When reversing to a part of the menu, send it back to the menu
      if (strstr($backscript,'menu_')){
        $backscript = 'menu.php';
      }
      // When reversing to a search frame, send it back to the menu
      if (strstr($backscript,'_search')){
        $backscript = 'menu.php';
      }
      $row = str_replace('{BACK_SCRIPT}',$backscript,$row);
      $this->addBodyHeader($row);
    }
    if (isset($SPL_UI['footer'])) {
      $this->addBodyFooter($SPL_UI['footer']);
    }
  }

  /** 
   * Sets the translation file to be used
   * 
   * @param  $filename  string The name of the translation file
   * @param  $qualified bool   The filename give is a fully qualified path
   * @access private
   */
  function _setTranslation($filename,$qualified=false) {
    
    global $SPL_CFG,$SPL_TXT;
    
    // Is a fully qualified path given?
    if ($qualified){
      @include_once($filename);
    }
    // Otherwise use the default path
    else {    
      if (isset($SPL_CFG['dirroot'])) {
        @include_once($SPL_CFG['dirroot'].'/ui/lang/'.$_SESSION['usp_lang'].'/'.$filename.'.lang.php');
      }
      else {
        @include_once('lang/'.$_SESSION['usp_lang'].'/'.$filename.'.lang.php');
      }
    }
  }
  
  /** 
   * Takes an unsorted valuelist, translates the items and returns a sorted valuelist
   * 
   * @param  $valuelist array The list o be sorted
   * @return            array The sorted list
   * @access private
   */
  function _sortValuelist($valuelist){
    
    // Translate the items first
    $newlist=array();
    foreach ($valuelist as $record){
      foreach ($record as $key => $item) {
        $item = $this->translate($item);
        $newlist[]= $item."|".$key;
      }
    }

    // Sort the translated list
    asort($newlist);
    reset($newlist);
    
    // Create a translated, sorted valuelist
    while (list ($arr_key,$arr_value) = each ($newlist)) {
      $item = substr($arr_value, 0, strpos($arr_value,"|"));
      $key = substr($arr_value,strpos($arr_value,"|")+1);
      $sortedlist[] = array($key=>$item); 
    }
    return $sortedlist;
  }  

}

/** 
 * $Log: page.class.php,v $
 * Revision 1.45  2006/09/03 18:32:13  jebbol
 * Changed startScrollableArea for use with mozilla
 *
 * Revision 1.44  2006/08/28 09:20:30  jebbol
 * *** empty log message ***
 *
 * Revision 1.43  2006/08/08 14:36:12  jebbol
 * Suppres errors when the translation file cannot be found in _setTranslation() and metadata cannot be found in _setMetadata()
 *
 * Revision 1.41  2006/08/08 08:07:19  jebbol
 * Changed repository (cvs.sourceforge.net -> splanner.cvs.sourceforge.net)
 *
 * Revision 1.40  2006/06/10 12:00:09  jebbol
 * Added possibility to overrule authorization by supplying a class name
 *
 * Revision 1.39  2006/04/17 09:31:13  jebbol
 * Added resetCancel switch to addButton
 *
 * Revision 1.38  2006/04/17 06:52:27  jebbol
 * Split editor buttons on two rows instead of one
 *
 * Revision 1.37  2006/04/16 19:10:38  jebbol
 * Enhanced the size of the 'search by' item by 1em in addSearchbar (Feauture #1459221)
 *
 * Revision 1.36  2006/04/14 19:51:12  jebbol
 * Added class to several td's, as they don't inherit from div's in IE
 *
 * Revision 1.35  2006/04/14 19:36:35  jebbol
 * Overwite cancelbutton behaviour (now using reset)
 *
 * Revision 1.34  2006/04/14 12:41:21  jebbol
 * *** empty log message ***
 *
 * Revision 1.33  2006/03/13 12:46:10  jebbol
 * Cleaned up HTML validation errors
 *
 * Revision 1.32  2006/03/08 16:32:39  jebbol
 * *** empty log message ***
 */

?>