<?php
/**
 * Sanitize GET and POST data
 *
 * $Author: jebbol $
 * $Revision: 1.6 $
 * $Date: 2006/08/08 08:09:19 $ 
 * 
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 * 
 * $Log: sanitize.inc.php,v $
 * Revision 1.6  2006/08/08 08:09:19  jebbol
 * Changed repository (cvs.sourceforge.net -> splanner.cvs.sourceforge.net)
 *
 * Revision 1.5  2006/03/17 08:12:50  jebbol
 * Added checks for month and year
 *
 */

/** 
 * Sanitizes all request variables
 * 
 * Make sure that $sanitize contains the definition of all incoming data allowed. 
 * All data will be checked against this definition.
 * Data in the request variables but not in $sanitize, will be ignored.
 * Actions invoke by buttons (plain buttons or toolbar buttons) will be in $clean['button_action'].
 * 
 * Format: $sanitize ('variable' => array of metadata)
 * Supported types of data are:
 * - Array:       $sanitize('variable'=>array('type'=>'array'))
 * - Boolean:     $sanitize('variable'=>array('type'=>'bool'))
 * - Button:      $sanitize('variable'=>array('type'=>'button'))
 *                The result will be in $clean['button_action']
 * - Character:   $sanitize('variable'=>array('type'=>'char','minlength'=>1,'maxlength'=>9999))
 *                min and max values are optional.
 * - Date:        $sanitize('variable'=>array('type'=>'date'))
 * - Email:       $sanitize('variable'=>array('type'=>'email'))
 * - Enumerated:  $sanitize('variable'=>array('type'=>'enum','values'=>value1,value2,value3))
 * - Integer:     $sanitize('variable'=>array('type'=>'int','min'=>0,'max'=>9999))
 *                min and max values are optional.
 * - Month:       $sanitize('variable'=>array('type'=>'month'))
 * - Shield:      $sanitize('variable'=>array('type=>'shield'))
 *                This is an encrypted field. The variable 'spl' is a shield variable by default.
 *                The shield assumes format: var1=value1&var2=value2, just like you would append to the url.
 *                Encrypt the field using: base64_encode(rc4crypt::encrypt($SPL_CFG['rc4key'], $value))
 * - Time:        $sanitize('variable'=>array('type'=>'time'))
 * - URL:         $sanitize('variable'=>array('type'=>'url'))
 * - Year:        $sanitize('variable'=>array('type'=>'year'))
*/

// Get encryption support
include_once($SPL_CFG['dirroot'].'/class/rc4crypt.class.php');

if (!isset($sanitize)) {
  $sanitize = array();
}

// Store all clean data in this array (var => value)
if (isset($clean)) unset($clean);
$clean = array();

//=========================================================================
// Setup defaults
//=========================================================================

// Mark 'spl' as the default shield type
if (!isset($sanitize['spl'])) {
  $sanitize['spl'] = array('type'=>'shield');
}

// Add support for tabbed sheets
if (!isset($sanitize['selected_tab'])) {
  $sanitize['selected_tab'] = array('type'=>'char');
}

// Add support for selectors
if (!isset($sanitize['selector_id'])) {
  $sanitize['selector_id'] = array('type'=>'array');
}

// Add support for id 
if (!isset($sanitize['id'])) {
  $sanitize['id'] = array('type'=>'int');
}

// Add support for from 
if (!isset($sanitize['from'])) {
  $sanitize['from'] = array('type'=>'char');
}

// Administration id 
if (!isset($sanitize['adm_id'])) {
  $sanitize['adm_id'] = array('type'=>'int');
}

// Set the default toolbar buttons and the plain buttons
if (!isset($sanitize['toolbar_button'])) {
  $sanitize['toolbar_button'] = array('type'=>'enum','values'=>'cancel,del,edit,find,help,list,new,print,save');
}
$sanitize['cancel'] = isset($sanitize['cancel'])?$sanitize['cancel']:array('type'=>'button'); 
$sanitize['del']    = isset($sanitize['del'])?$sanitize['del']:array('type'=>'button'); 
$sanitize['edit']   = isset($sanitize['edit'])?$sanitize['edit']:array('type'=>'button'); 
$sanitize['find']   = isset($sanitize['find'])?$sanitize['find']:array('type'=>'button'); 
$sanitize['help']   = isset($sanitize['help'])?$sanitize['help']:array('type'=>'button'); 
$sanitize['new']    = isset($sanitize['new'])?$sanitize['new']:array('type'=>'button'); 
$sanitize['print']  = isset($sanitize['print'])?$sanitize['print']:array('type'=>'button'); 
$sanitize['save']   = isset($sanitize['save'])?$sanitize['save']:array('type'=>'button'); 

// Search buttons
if (!isset($sanitize['search_button'])) {
  $sanitize['search_button'] = array('type'=>'char');
}
// Search by
if (!isset($sanitize['search_by'])) {
  $sanitize['search_by'] = array('type'=>'char');
}
// Search for
if (!isset($sanitize['search_for'])) {
  $sanitize['search_for'] = array('type'=>'char');
}
// Search for inactive rows
if (!isset($sanitize['search_show_inactive'])) {
  $sanitize['search_show_inactive'] = array('type'=>'bool');
}
// orderBy
if (!isset($sanitize['orderBy'])) {
  $sanitize['orderBy'] = array('type'=>'char');
}
// orderDir
if (!isset($sanitize['orderDir'])) {
  $sanitize['orderDir'] = array('type'=>'char');
}
// Viewstart
if (!isset($sanitize['viewStart'])) {
  $sanitize['viewStart'] = array('type'=>'char');
}

//=========================================================================
// Start sanitize
//=========================================================================

if (isset($_REQUEST)){
  
  $validRequest = true;
  
  // Check token (can only be a POST)
  // This token has been set to prevent Cross-site Request Forgories attacks
  if (isset($_POST['token'])) {
    $validRequest = false;
    if (isset($_SESSION['token']) && ($_POST['token']==$_SESSION['token'])) {
      if (!isset($SPL_CFG['token_time'])) $SPL_CFG['token_time'] = 300;
      if ((time() - $_SESSION['token_time']) <= $SPL_CFG['token_time']) {
        // Less than five minutes has passed.
        $validRequest = true;
      }
      unset($_SESSION['token']);
      unset($_SESSION['token_time']);
    }
  }

  if ($validRequest) {
  foreach ($sanitize as $sanitize_var => $sanitize_def) {

    // Unchecked checkboxes will not appear in the $_REQUEST
    if ($sanitize_def['type']=='bool' && !isset($_REQUEST[$sanitize_var])) {
      $clean[$sanitize_var] = false;
    }
    
    if (isset($_REQUEST[$sanitize_var])){
      
      // Don't allow anything containing <script>
      if (!is_array($_REQUEST[$sanitize_var])) {
        if (preg_match("/<\s*script/i",$_REQUEST[$sanitize_var])) {
          continue;
        }
      }

      // Determine type
      if (!isset($sanitize_def['type'])) {
        $sanitize_def['type'] = 'char';
      }
      $sanitize_type = $sanitize_def['type'];
    
      switch ($sanitize_type) {
        case 'array':
          $clean[$sanitize_var] = $_REQUEST[$sanitize_var];
          break;
        case 'bool':
          if (strstr('on,1,yes,true',$_REQUEST[$sanitize_var])){
            $clean[$sanitize_var] = true;
          }
          if (strstr('off,0,no,false',$_REQUEST[$sanitize_var])){
            $clean[$sanitize_var] = false;
          }
          break;
        case 'button':
          $clean['button_action'] = $sanitize_var;
          break;
        case 'char':
          if (is_string($_REQUEST[$sanitize_var])) {
            if (isset($sanitize_def['minlength'])&&strlen($_REQUEST[$sanitize_var])<$sanitize_def['minlength']){
              break;
            }
            if (isset($sanitize_def['maxlength'])&&strlen($_REQUEST[$sanitize_var])>$sanitize_def['maxlength']){
              break;
            }
            $clean[$sanitize_var] = $_REQUEST[$sanitize_var];
          }
          break;
        case 'date':
          /*
          if (ereg("^(19|20)[0-9]{2}[-](0[1-9]|1[0-2])[-](0[1-9]|[1-2][0-9]|3[0-1])$", $_REQUEST[$sanitize_var])) {
            $clean[$sanitize_var] = $_REQUEST[$sanitize_var];
          }
          */
          $clean[$sanitize_var] = $_REQUEST[$sanitize_var];
          break;
        case 'email':
          if (preg_match("/^[a-z0-9_\.-]+@([a-z0-9]+([\-]+[a-z0-9]+)*\.)+[a-z]{2,7}$/i", $_REQUEST[$sanitize_var])){
            $clean[$sanitize_var] = $_REQUEST[$sanitize_var];
          }
          break;
        case 'enum':
          if (!empty($_REQUEST[$sanitize_var])) {
            if (strstr($sanitize_def['values'],$_REQUEST[$sanitize_var])){
              if ($sanitize_var=='toolbar_button'){
                $clean['button_action'] = $_REQUEST[$sanitize_var];
              }
              $clean[$sanitize_var] = $_REQUEST[$sanitize_var];
            }
          }
          break;
        case 'int':
          if (is_numeric($_REQUEST[$sanitize_var])) {
            if (!isset($sanitize_def['min'])) {
              $sanitize_def['min'] = 0;
            }
            if (!isset($sanitize_def['max'])) {
              $sanitize_def['max'] = 4294967295; //mySQL max
            }
            if ($_REQUEST[$sanitize_var]<$sanitize_def['min']){
              break;
            }
            if ($_REQUEST[$sanitize_var]>$sanitize_def['max']){
              break;
            }
            $clean[$sanitize_var] = (int) $_REQUEST[$sanitize_var];
          }
          break;
        case 'month':
          if (preg_match("/^[0-9]{1,2}$/", $_REQUEST[$sanitize_var])){
            $clean[$sanitize_var] = $_REQUEST[$sanitize_var];
          }
          break;
        case 'shield':
          $sanitize_tmp = rc4crypt::decrypt($SPL_CFG['rc4key'],base64_decode($_REQUEST[$sanitize_var]));
          $sanitize_tmp = explode('&',ltrim($sanitize_tmp,'?'));
          foreach ($sanitize_tmp as $key_value_pair){
            $key_value_array = explode('=',$key_value_pair);
            if (count($key_value_array)==2) {
              $clean[$key_value_array[0]] = $key_value_array[1];
            }
          }
          break;
        case 'time':
          $clean[$sanitize_var] = $_REQUEST[$sanitize_var];
          break;
        case 'url':
          if (ereg("^(http|https|ftp)\://[a-zA-Z0-9\-\.]+\.[a-zA-Z]{2,3}(:[a-zA-Z0-9]*)?/?([a-zA-Z0-9\-\._\?\,\'/\\\+&%\$#\=~])*$", $_REQUEST[$sanitize_var])) {
            $clean[$sanitize_var] = $_REQUEST[$sanitize_var];
          }
          break;
        case 'year':
          if (preg_match("/^[0-9]{4}$/", $_REQUEST[$sanitize_var])){
            $clean[$sanitize_var] = $_REQUEST[$sanitize_var];
          }
          break;

      } // switch

    } // isset($_REQUEST[$sanitize_var]) 
    
  }} // foreach

}// isset($_REQUEST)

?>
