<?php
/**
 * User interface utility functions.
 *
 * $Author: jebbol $
 * $Revision: 1.1 $
 * $Date: 2006/02/11 16:27:32 $
 * 
 * @copyright Copyright (c) 2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 * 
 * $Log: util.inc.php,v $
 * Revision 1.1  2006/02/11 16:27:32  jebbol
 * Changed the name from util.php into util.inc.php
 * Added function sort_valuelist
 *
 */

/**
 * Finds all available human language translations
 * 
 * @return (array) Translations
 * @public
 */
function getLanguages(){
  $valuelist = array();
  $handle = opendir('../ui/lang');
  while (false!==($file = readdir($handle))) {
    if ($file != "." && $file != "..") {
      $name = @file_get_contents('../ui/lang/'.$file.'/language.desc');
      // Must have a language.desc file
      if (!empty($name)) {
        $record = array();
        $record[$file] = $name;
        $valuelist[] = $record;
      }
    }
  }
  closedir($handle);
  $sorted = sort_valuelist($valuelist);
  return (!empty($sorted)?$sorted:false);
}

/**
 * Finds all available methodologies
 * 
 * @return (array) Methodologies
 * @public
 */
function getMethodologies(){
  
  global $SPL_CFG;
  
  include_once($SPL_CFG['dirroot'].'/class/methodology.class.php');
  
  $valuelist = array();
  $handle = opendir('../methodology');

  while (false!==($dir = readdir($handle))) {
    if ($dir != "." && $dir != "..") {
      
      // Check if the definitions file exists
      $file = $dir.'/methodology.xml';
      if (file_exists($SPL_CFG['dirroot'].'/methodology/'.$file)){
        $m = new methodology();
        $m->init($dir);
        $name = $m->getName();
        if (!empty($name)) {
          $record = array();
          $record[$dir] = $name;
          $valuelist[] = $record;
        }
      }
    }
  }
  closedir($handle);
  $sorted = sort_valuelist($valuelist);
  return (!empty($sorted)?$sorted:false);
}

/**
 * Lists all supported name formats
 * 
 * @return (array) Nameformats
 * @public
 */
function getNameFormats(){
  
  global $SPL_TXT;
  
  $valuelist = array();
  $record = array();
  $record['UIS'] = $SPL_TXT['UIS'];
  $valuelist[] = $record;
  $record = array();
  $record['FIS'] = $SPL_TXT['FIS'];
  $valuelist[] = $record;
  $record = array();
  $record['NIS'] = $SPL_TXT['NIS'];
  $valuelist[] = $record;
  $record = array();
  $record['SNI'] = $SPL_TXT['SNI'];
  $valuelist[] = $record;
  $record = array();
  $record['SUI'] = $SPL_TXT['SUI'];
  $valuelist[] = $record;
  $record = array();
  $record['SFI'] = $SPL_TXT['SFI'];
  $valuelist[] = $record;
  return $valuelist;
}

/**
 * Finds all available skins
 * 
 * @return (array) Skins
 * @public
 */
function getSkins(){
  $valuelist = array();
  $handle = opendir('../ui/skin');
  while (false!==($file = readdir($handle))) {
    if ($file != "." && $file != "..") {
      $name = @file_get_contents('../ui/skin/'.$file.'/skin.desc');
      // Must have a language.desc file
      if (!empty($name)) {
        $record = array();
        $record[$file] = $name;
        $valuelist[] = $record;
      }
    }
  }
  closedir($handle);
  $sorted = sort_valuelist($valuelist);
  return (!empty($sorted)?$sorted:false);
}

/**
 * Utility function redirect the browser location to another url
 * Inspired by from Mambo 4.5.1 (C) 2000 - 2004 Miro International Pty Ltd
 *
 * @param (string) $url The file system path
 * @param (string) $msg A filter for the names
 * @public 
 */
function redirect($url,$msg='') {
  if (trim( $msg )) {
    if (strpos( $url, '?' )) {
      $url .= '&msg=' . urlencode( $msg );
    } else {
      $url .= '?msg=' . urlencode( $msg );
    }
  }

  if (headers_sent()) {
    echo "<script>document.location.href='$url';</script>\n";
  } else {
    header( "Location: $url" );
  }
  exit();
}

/**
 * Sorts a valuelist
 * 
 * @param  (array) $valuelist Unsorted valuelist
 * @return (array)            Sorted valuelist
 * @public
 */ 
function sort_valuelist($valuelist) {
  $unsorted=array();
  foreach ($valuelist as $record){
    foreach ($record as $key => $item) {
      $unsorted[]= $item."|".$key;
    }
  }
  asort($unsorted);
  reset($unsorted);
  while (list ($arr_key,$arr_value) = each ($unsorted)) {
    $item = substr($arr_value, 0, strpos($arr_value,"|"));
    $key = substr($arr_value,strpos($arr_value,"|")+1);
    $sorted[] = array($key=>$item); 
  }
  return $sorted;
}
?>
