<?php
/**
 * User interface for manipulating data in a mentor object
 *
 * $Author: jebbol $
 * $Revision: 1.3 $
 * $Date: 2006/04/17 09:28:40 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.nl
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

// Configuration
include_once('../inc/config.php');

// Defaults
include_once($SPL_CFG['dirroot'].'/inc/defaults.inc.php');

// Page generation
include_once($SPL_CFG['dirroot'].'/ui/class/page.class.php');

// Utility functions
include_once($SPL_CFG['dirroot'].'/ui/inc/util.inc.php');

// Methodology
include_once($SPL_CFG['dirroot'].'/class/methodology.class.php');

// Data objects
include_once($SPL_CFG['dirroot'].'/methodology/'.$_SESSION['methodology'].'/class/men.class.php');
include_once($SPL_CFG['dirroot'].'/class/ina.class.php');
include_once($SPL_CFG['dirroot'].'/class/ins.class.php');
include_once($SPL_CFG['dirroot'].'/class/usr.class.php');

//=========================================================================
// Sanitize request variables
//=========================================================================
$sanitize = array(
  'men_id'=>array('type'=>'int'),
  'men_usr_id'=>array('type'=>'int'),
  'parent_id'=>array('type'=>'int'),
  'ins_id'=>array('type'=>'int'),
  'ina_id'=>array('type'=>'int'),
  'men_sortname'=>array('type'=>'char'),
  'men_dispname'=>array('type'=>'char'),
  'men_rebuild_spm'=>array('type'=>'bool'),
);
if (isset($_SESSION['is_admin'])&&$_SESSION['is_admin']==true) {
  $sanitize['is_active'] = array('type'=>'bool');
}
require_once($SPL_CFG['dirroot'].'/ui/inc/sanitize.inc.php');

//=========================================================================
// Prerequisites
//=========================================================================

// Must be logged in
include_once($SPL_CFG['dirroot'].'/ui/inc/logincheck.php');

// Must have access rights
$perm = 'men';
include_once($SPL_CFG['dirroot'].'/ui/inc/permissioncheck.inc.php');

//=========================================================================
// Local variables
//=========================================================================

global $SPL_META;

$object = new men();
$editMode = (bool)false;

//=========================================================================
// Methodology
//=========================================================================

$m = new methodology();
$m->init($_SESSION['methodology']);

//=========================================================================
// Event handling
//=========================================================================

// Are we returning from searching a reference field?
$from_id = (bool)false;
if (isset($clean['from'])){
  $from_id = $clean['id'];
}
    
$men_user = 0;
if (isset($clean['men_usr_id'])){
  $men_user = $clean['men_usr_id'];
}
else if (isset($clean['parent_id'])){
  $men_user = $clean['parent_id'];
}

// Populate the object
$displayNewNotice = (bool)false;
if ($men_user) {
  $usr = new usr();
  $usr->init($men_user);
  $tmp_men_id = $usr->getProperty('men_id');  
  if ($tmp_men_id) {
    if ($object->init($tmp_men_id)){
      $_SESSION['men_id'] = $tmp_men_id;
    }
  }
  else if (!isset($clean['button_action'])) {
    $displayNewNotice = (bool)true;
  }
}

if (isset($clean['button_action'])){
  // Delete
  if ($clean['button_action']=='del') {
    $object->delete(false);
    $object = new men();
    unset($_SESSION['men_id']);
  }

  // Edit
  if ($clean['button_action']=='edit') {
    $editMode = (bool)true;
  }

  // New
  if ($clean['button_action']=='new') {
    $object = new men();
    if (isset($clean['parent_id'])){
      $object->setUser($clean['parent_id']);
    }
    $editMode = (bool)true;
    unset($_SESSION['men_id']);
  }

  // Save
  if ($clean['button_action']=='save') {
    $editMode = (bool)true;
    save();
    $editMode = (bool)false;
  }

  // Cancel
  if ($clean['button_action']=='cancel') {
    $editMode = (bool)false;
  }
}

// Set the chosen reference field
if ($from_id){
  if ($from_id != $_SESSION['men_id']){
    $object->setProperty($clean['from'],$from_id);
  }
}

//=========================================================================
// Definition of the page
//=========================================================================
$page = new page();
$page->init('men','default','template_iframetabbed.htm',false);
$page->startForm(null,'default',false,true);
  $page->startAreaTabSheet();
  $page->startTable('none','style="width:100%"');
  $page->addTabRuler();
  $page->endTable();
  $page->startTable('detail');
    $page->startRow();
      $page->startColumn('areaTabSheet', 'style="vertical-align:top;width:100%"');
        $page->startTable();
        
        // Add a hidden field to remember the user id
        $page->addField('men_usr_id',$men_user);
        
        // Do we have to display a notice?
        if ($displayNewNotice) {
          $page->addContent($SPL_TXT['new_mentor_notice']);
        }
        
        // No notice
        else {
  
          // Sortname
          $page->addField('men_sortname',$object->getProperty('men_sortname'),!$editMode);
          
          // Display name
          $page->addField('men_dispname',$object->getProperty('men_dispname'),!$editMode);

          // Schedule name
          if (isset($SPL_CFG['scheduler'])&&($SPL_CFG['scheduler']!='none')){
            $page->addField('men_schedulename',$object->getProperty('men_schedulename'),!$editMode);
          }

          // Institution: when there is only one instance, hide it
          /*
          $ins = new ins();
          $single = $ins->isSingle();
          if ($single&&$editMode){
            $object->setProperty('ins_id',$single);
          }
          $page->addField('ins_id',$object->getProperty('ins_id'),!$editMode,false,false,false,NULL,true,false,$single);
          */
          
          // Institution address: when there is only one instance, hide it
          /*
          $ina = new ina();
          $single = $ina->isSingle();
          if ($editMode){
            if ($single){
              $object->setProperty('ina_id',$single);
            }
            $page->addField('ina_id',$object->getProperty('ina_id'),!$editMode,false,false,$ina->getKeyValues('ins',$object->getProperty('ins_id')),NULL,true,false,$single);
          }
          else {
            $page->addField('ina_id',$object->getProperty('ina_id'),!$editMode,false,false,false,NULL,true,false,$single);
          }
          */
   
          // Include methodology related fields
          $m_inc = $m->getMentorEditScript();
          if ($m_inc){
            include_once ($m_inc);
          }
          $page->endRow();
   
        }
        // End of not displaying a notice
        
        // Start buttons
        $page->startRow();
          // Buttons
          $page->startColumn();
          $page->endColumn();
          $page->startColumn();
          $page->endColumn();
          $page->startColumn();
            if ($editMode){
              $page->addButton('save','toolbar_save','submit','default','8','button');
              $page->addButton('cancel','toolbar_cancel','submit','default','8','button','no',null,true);
            }
            else {
              if ($_SESSION['men_permission']['write'] && $displayNewNotice) {
                $page->addButton('new','toolbar_new','submit','default','8','button');
              }
              if ($_SESSION['men_permission']['change'] && !$displayNewNotice) {
                $page->addButton('edit','toolbar_edit','submit','default','8','button');
              }
              if ($_SESSION['men_permission']['delete'] && !$displayNewNotice) {
                $page->addButton('del','toolbar_del','submit','default','8','button');
              }
            }
          $page->endColumn();
        $page->endRow();
        // End buttons
  $page->endTable();
  $page->endAreaTabSheet();
$page->endForm();

//=========================================================================
// Show the page
//=========================================================================

$page->setContent();
$page->createPage();
$page->showPage();

//=========================================================================
// Functions
//=========================================================================

function set(){
  
  global $object,$m,$clean;
  
  if (isset ($clean['men_sortname'])) {$object->setProperty('men_sortname', $clean['men_sortname']);}
  if (isset ($clean['men_dispname'])) {$object->setProperty('men_dispname', $clean['men_dispname']);}
  if (isset ($clean['men_schedulename'])) {$object->setProperty('men_schedulename', $clean['men_schedulename']);}
  if (isset ($clean['ins_id']))       {$object->setProperty('ins_id', $clean['ins_id']);}
  if (isset ($clean['ina_id']))       {$object->setProperty('ina_id', $clean['ina_id']);}
  // Include methodology related fields
  $m_inc = $m->getMentorSetScript();
  if ($m_inc){
    include_once ($m_inc);
  }
}

function save(){
  
  global $object,$editMode,$men_user,$clean;

  if (!$editMode){
    return;
  }
  set();
  $object->setUser($men_user);
  $object->save();
  $_SESSION['men_id'] = $object->getProperty('men_id');
}

/**
 * $Log: men.php,v $
 * Revision 1.3  2006/04/17 09:28:40  jebbol
 * Added resetCancel switch to addButton
 *
 * Revision 1.2  2006/04/14 19:52:01  jebbol
 * Added class to several td's, as they don't inherit from div's in IE
 *
 * Revision 1.1  2006/04/14 12:45:28  jebbol
 * Added support for mentors and parents
 */

?>
