<?php
/**
 * User interface for the main menu of the application
 *
 * $Author: jebbol $
 * $Revision: 1.24 $
 * $Date: 2006/09/03 15:59:46 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.nl
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

// Configuration
include_once('../inc/config.php');

// Page generation
include_once($SPL_CFG['dirroot'].'/ui/class/page.class.php');

// Utility functions
include_once($SPL_CFG['dirroot'].'/ui/inc/util.inc.php');

// Calendar
include_once($SPL_CFG['dirroot'].'/ui/class/calendar.class.php');

//=========================================================================
// Prerequisites
//=========================================================================

// Must be logged in
include_once($SPL_CFG['dirroot'].'/ui/inc/logincheck.php');

//=========================================================================
// Sanitize request parameters
//=========================================================================
$sanitize = array(
  'calendar' =>array('type'=>'enum','values'=>'prevyear,prevmonth,nextmonth,nextyear'),
  'caldate'  =>array('type'=>'int'),
  'msg'      =>array('type'=>'char'),
);
require_once($SPL_CFG['dirroot'].'/ui/inc/sanitize.inc.php');

//=========================================================================
// Local variables
//=========================================================================
 
$cal = new calendar();

// Check for messages to be displayed
$tmp_msg  = (isset($clean['msg']) ? $clean['msg'] : (bool)false);
if (isset($_SESSION['msg'])) {
  $tmp_msg = $_SESSION['msg'];
  unset($_SESSION['msg']);
}

//=========================================================================
// Event handling
//=========================================================================

// Calendar events
if (isset($clean['calendar'])) {
  switch ($clean['calendar']) {
    
    case 'prevyear':
      if (isset($clean['caldate'])){
        if ((int)$clean['caldate']>0){
          $cal->setDate(addslashes($clean['caldate']));
        }
      }
      $cal->setPrevYear();
      break;
    case 'prevmonth':
      if (isset($clean['caldate'])){
        if ((int)$clean['caldate']>0){
          $cal->setDate(addslashes($clean['caldate']));
        }
      }
      $cal->setPrevMonth();
      break;
    case 'nextmonth':
      if (isset($clean['caldate'])){
        if ((int)$clean['caldate']>0){
          $cal->setDate(addslashes($clean['caldate']));
        }
      }
      $cal->setNextMonth();
      break;
    case 'nextyear':
      if (isset($clean['caldate'])){
        if ((int)$clean['caldate']>0){
          $cal->setDate(addslashes($clean['caldate']));
        }
      }
      $cal->setNextYear();
      break;
    default:
      break;
  }
}

//=========================================================================
// Definition of the page
//=========================================================================

$page = new page();
$page->init('spl');
$page->setHelpTips();        // Enable helptips for the calendar
$page->setKickOutOfFrames(); // This window is never enclosed in frame
$page->startForm();
  $page->startSection('menuSection');

    // Left section
    if (isset($_SESSION['usp_calmode']) && $_SESSION['usp_calmode']!='none') {
    $page->startSubsection('menuleftsection');
    
      // Calendar
      $page->startRow();
        $page->startColumn();
          $page->startPane($cal->getCaption(),'noclose','calendar');
            $page->addContent($cal->display());
          $page->endPane('calendar');
        $page->endColumn();
      $page->endRow();

      // Schedule
      if (isset($SPL_CFG['scheduler'])&&($SPL_CFG['scheduler']!='none')){
        include_once($SPL_CFG['dirroot'].'/ui/class/schedule.class.php');
        $sched = new schedule();
        $schedule = $sched->init();
        if ($schedule) {
          $mapping = array('monday','monday','tuesday','wednesday','thursday','friday','monday','monday');
          $weekday = $cal->getWeekDay();
          $dayschedule = $schedule[$mapping[$weekday]];
          $page->startRow();
            $page->startColumn();
              $page->startPane(($SPL_TXT['scheduler'].' '.$page->translate($mapping[$weekday])),'noclose','schedule');
                $i = 1;
                $page->startTable();
                foreach ($dayschedule as $hour) {
                  //if (!empty($hour)) {
                    $page->startRow();
                      $page->startColumn('scheduleHour'.((($i % 2) == 0)?'Even':'Odd'));
                        $page->addContent($i);
                      $page->endColumn();
                      $page->startColumn('schedule'.((($i % 2) == 0)?'Even':'Odd'));
                        $page->addContent(empty($hour)?'&nbsp;':$hour);
                      $page->endColumn();
                    $page->endRow();
                  //}
                  $i++;
                }
                $page->endTable();
              $page->endPane('schedule');
            $page->endColumn();
          $page->endRow();
        }
      }

      // Assessments
      if (isset($SPL_CFG['learning_assessment'])&&($SPL_CFG['learning_assessment']=='true')&&$_SESSION['is_student']){
        include_once($SPL_CFG['dirroot'].'/ui/class/assessment.class.php');
        $assessment = new assessment();
        $assessments = $assessment->init();
        $page->startRow();
          $page->startColumn();
            $page->startPane($SPL_TXT['assessment'],'noclose','assessment');
            if (empty($assessments)) {
              $page->addContent($SPL_TXT['assessment_empty']);
            }
            else {
              $i = 1;
              $page->startTable();
              foreach ($assessments as $a) {
                 $page->startRow('assessment'.((($i % 2) == 0)?'Even':'Odd'));
                   $page->startColumn();
                     $page->addContent($a['sub_short']);
                   $page->endColumn();
                   $page->startColumn();
                     $page->addContent($a['sug_short']);
                   $page->endColumn();
                   $page->startColumn();
                     $page->addContent($a['sgd_assessment']);
                   $page->endColumn();
                   $page->startColumn('none','style="width:16px;height:16px;"');
                     if (isset($a['sgd_compliment']) && $a['sgd_compliment']) {
                       $page->addContent(' <span style="vertical-align:bottom"><img src="'.$page->getImageDir().'compliment.gif" alt=""></span>');
                     } 
                   $page->endColumn();
                 $page->endRow();
                 $i++;
              }
              $page->endTable();
            }
            $page->endPane('assessment');
          $page->endColumn();
        $page->endRow();
      }
      
      // Task list
      if ($_SESSION['is_teacher']) {
      $page->startRow();
        $page->startColumn();
          $page->startPane($SPL_TXT['tasklist_new'],'noclose','task');
            $page->startTable();
              $i=1;
              $today = date("d-m-Y");
              // Find teachers tasks            
              $query  = 'SELECT usr.usr_login,usr.usr_name,stu.stu_dispname,tes.update_date';
              $query .= ' FROM '.$SPL_CFG['prefix'].'tes tes';
              $query .= ' LEFT JOIN '.$SPL_CFG['prefix'].'sgd sgd ON sgd.sgd_id = tes.sgd_id';
              $query .= ' LEFT JOIN '.$SPL_CFG['prefix'].'usr usr ON usr.usr_id = sgd.usr_id';
              $query .= ' LEFT JOIN '.$SPL_CFG['prefix'].'stu stu ON stu.stu_id = usr.stu_id';
              $query .= ' WHERE tes.tea_id = "'.$_SESSION['my_tea_id'].'" AND is_new ORDER BY tes.update_date DESC LIMIT 0,10';
              if ($result = db_query($query)) {
                while ($r = db_fetch_array($result)) {
                  if (isset($r['stu_dispname']) && empty($r['stu_dispname'])) {
                    $user = $r['stu_dispname'];
                  }
                  else if (isset($r['usr_name']) && !empty($r['usr_name'])) {
                    $user = $r['usr_name'];
                  }
                  else if (isset($r['usr_login']) && !empty($r['usr_login'])) {
                    $user = $r['usr_login'];
                  }
                  else {
                    $user = 'Unknown';
                  }
                  $when='';
                  if (date("d-m-Y",$r['update_date'])==$today){
                    $when = date("H:i",$r['update_date']);
                  }
                  else {
                    $when = date("d-m",$r['update_date']);
                  }
                  $page->startRow('task'.((($i % 2) == 0)?'Even':'Odd'));
                    $page->startColumn('taskLabel');
                       $page->addContent($when);
                    $page->endColumn();
                    $page->startColumn('taskField');
                       $page->addContent($user);
                    $page->endColumn();
                  $page->endRow();
                  $i++;
                }
              }
            $page->endTable();
          $page->endPane('task');
        $page->endColumn();
      $page->endRow();
      }
    $page->endSubsection();
    }

    // Middle section
    $page->startSubsection('menumiddlesection');
      if ($tmp_msg) {
        $_SESSION['msg']                 = $tmp_msg;
        $_SESSION['referred_from']       = $SPL_CFG['wwwroot'].'/ui/menu.php';
        $_SESSION['messagebox_icontype'] = 'disappointed';
        $_SESSION['messagebox_style']    = 'menu';
        $page->addIFrame('message.php','fixedframe');
      }
      else {   
        $page->addIFrame((isset($_SESSION['usp_startpane'])?$_SESSION['usp_startpane']:'menu_sgd.php'),'fixedframe');
      }
    $page->endSubsection();

    // Right section
    if (isset($SPL_UI['layout_menu_position'])&&$SPL_UI['layout_menu_position']=='top'){
      //SPL_TODO: This is ugly hacking. Must be redone completely
      $tmp_menu = '';
      if ($_SESSION['is_parent']){
        $tmp_menu .= '<a href="menu_parent.php" target="fixedframe">'.$SPL_TXT['menu_parent'].'</a>';
      }            
      if ($_SESSION['is_teacher'] || $_SESSION['is_mentor']){
        $tmp_menu .= '<a href="menu_dash.php" target="fixedframe">'.$SPL_TXT['menu_dash'].'</a>';
        if ($_SESSION['is_teacher']) {
          $tmp_menu .= ' | <a href="menu_tsk.php" target="fixedframe">'.$SPL_TXT['menu_tsk'].'</a>';
        }
        $tmp_menu .= ' | <a href="menu_sgd.php" target="fixedframe">'.$SPL_TXT['menu_sgd'].'</a>';
        $tmp_menu .= ' | <a href="menu_chr.php" target="fixedframe">'.$SPL_TXT['menu_chr'].'</a>';
        if ($_SESSION['is_teacher']) {
          $tmp_menu .= ' | <a href="menu_data.php" target="fixedframe">'.$SPL_TXT['menu_data'].'</a>';
        }
      }            
      if ($_SESSION['is_student']){
        // Prevent double entries
        if (!$_SESSION['is_teacher']){
          $tmp_menu .= '<a href="menu_sgd.php" target="fixedframe">'.$SPL_TXT['menu_sgd'].'</a>';
          $tmp_menu .= ' | <a href="menu_chr.php" target="fixedframe">'.$SPL_TXT['menu_chr'].'</a>';
        }
      }
      if ($_SESSION['is_admin']){
        // Prevent double entries
        if (!$_SESSION['is_teacher']){
          $tmp_menu .= '<a href="menu_data.php" target="fixedframe">'.$SPL_TXT['menu_data'].'</a>';
        }
        $tmp_menu .= ' | <a href="info.php" target="fixedframe">'.$SPL_TXT['menu_info'].'</a>';
      }
      $page->setTopMenu($tmp_menu);
    }
    else if (isset($_SESSION['usp_menumode']) && $_SESSION['usp_menumode']!='none') {
    $page->startSubsection(($_SESSION['usp_menumode']=='normal'?'menurightsection':'menurightsectionsmall'));
      // Logon history
      $page->startRow();
        $page->startColumn();
          $page->startPane(($_SESSION['usp_menumode']=='normal'?$_SESSION['my_name']:''),'noclose');
            $page->startTable('none','style="margin-top:3px"');

              // Teachers only
              if ($_SESSION['is_teacher'] || $_SESSION['is_mentor']){

                // Dahboard
                $page->startRow();
                  $page->startColumn();
                    $page->addContent('<a href="menu_dash.php" target="fixedframe"><img src="'.$page->getImageDir().'dash_small.png" title="'.$SPL_TXT['menu_dash'].'" alt="'.$SPL_TXT['menu_dash'].'"> '.($_SESSION['usp_menumode']=='normal'?$SPL_TXT['menu_dash']:'').'</a>');
                  $page->endColumn();
                $page->endRow();

                // Tasklist
                if ($_SESSION['is_teacher']) {
                $page->startRow();
                  $page->startColumn();
                    $page->addContent('<a href="menu_tsk.php" target="fixedframe"><img src="'.$page->getImageDir().'tsk_small.png" title="'.$SPL_TXT['menu_tsk'].'" alt="'.$SPL_TXT['menu_tsk'].'"> '.($_SESSION['usp_menumode']=='normal'?$SPL_TXT['menu_tsk']:'').'</a>');
                  $page->endColumn();
                $page->endRow();
                }
              }

              // Display the studyguide to students and teachers only
              if ($_SESSION['is_student'] || $_SESSION['is_teacher'] || $_SESSION['is_mentor']){
                $page->startRow();
                  $page->startColumn();
                    $page->addContent('<a href="menu_sgd.php" target="fixedframe"><img src="'.$page->getImageDir().'sgd_small.png" title="'.$SPL_TXT['menu_sgd'].'" alt="'.$SPL_TXT['menu_sgd'].'"> '.($_SESSION['usp_menumode']=='normal'?$SPL_TXT['menu_sgd']:'').'</a>');
                  $page->endColumn();
                $page->endRow();
                $page->startRow();
                  $page->startColumn();
                    $page->addContent('<a href="menu_chr.php" target="fixedframe"><img src="'.$page->getImageDir().'chr_small.png" title="'.$SPL_TXT['menu_chr'].'" alt="'.$SPL_TXT['menu_chr'].'"> '.($_SESSION['usp_menumode']=='normal'?$SPL_TXT['menu_chr']:'').'</a>');
                  $page->endColumn();
                $page->endRow();
              }

              if ($_SESSION['is_admin'] || $_SESSION['is_teacher']){
                // Data maintenance
                $page->startRow();
                  $page->startColumn();
                    $page->addContent('<a href="menu_data.php" target="fixedframe"><img src="'.$page->getImageDir().'data_small.png" title="'.$SPL_TXT['menu_data'].'" alt="'.$SPL_TXT['menu_data'].'"> '.($_SESSION['usp_menumode']=='normal'?$SPL_TXT['menu_data']:'').'</a>');
                  $page->endColumn();
                $page->endRow();
              }
              
              // Display the parent summary to parents only
              if ($_SESSION['is_parent']){
                $page->startRow();
                  $page->startColumn();
                    $page->addContent('<a href="menu_parent.php" target="fixedframe"><img src="'.$page->getImageDir().'par_small.png" title="'.$SPL_TXT['menu_parent'].'" alt="'.$SPL_TXT['menu_parent'].'"> '.($_SESSION['usp_menumode']=='normal'?$SPL_TXT['menu_parent']:'').'</a>');
                  $page->endColumn();
                $page->endRow();
              }

              // Preferences
              $page->startRow();
                $page->startColumn();
                  $page->addContent('<a href="'.$SPL_CFG['wwwroot'].'/ui/usp.php" target="fixedframe" title="'.$SPL_TXT['preferences_title'].'" alt="'.$SPL_TXT['preferences_title'].'"><img src="'.$page->getImageDir().'personal_small.png"> '.($_SESSION['usp_menumode']=='normal'?$SPL_TXT['preferences_title']:'').'</a>');
                $page->endColumn();
              $page->endRow();

              if ($_SESSION['is_admin']){
                // System info
                $page->startRow();
                  $page->startColumn();
                    $page->addContent('<a href="info.php" target="fixedframe"><img src="'.$page->getImageDir().'sysinfo_small.png" title="'.$SPL_TXT['menu_info'].'" alt="'.$SPL_TXT['menu_info'].'"> '.($_SESSION['usp_menumode']=='normal'?$SPL_TXT['menu_info']:'').'</a>');
                  $page->endColumn();
                $page->endRow();
              }
              
              // Logoff
              $page->startRow();
                $page->startColumn();
                  $page->addContent('<br /><a href="'.$SPL_CFG['wwwroot'].'/ui/logoff.php" title="'.$SPL_TXT['logoff_title'].'" alt="'.$SPL_TXT['logoff_title'].'"><img src="'.$page->getImageDir().'logoff_small.png"> '.($_SESSION['usp_menumode']=='normal'?$SPL_TXT['logoff_title']:'').'</a>');
                $page->endColumn();
              $page->endRow();
              
            $page->endTable();
          $page->endPane();
        $page->endColumn();
      $page->endRow();

    $page->endSubsection();
    }
    
  $page->endSection();
$page->endForm();

//=========================================================================
// Show the page
//=========================================================================

$page->setContent();
$page->createPage();
$page->showPage();

//=========================================================================
// Functions
//=========================================================================

/** 
 * $Log: menu.php,v $
 * Revision 1.24  2006/09/03 15:59:46  jebbol
 * Added the display of compliments to the assessments pane
 *
 * Revision 1.23  2006/08/08 07:11:52  jebbol
 * Added support for parent and mentor
 *
 * Revision 1.22  2006/04/14 12:45:28  jebbol
 * Added support for mentors and parents
 *
 * Revision 1.21  2006/02/11 18:40:47  jebbol
 * Renamed ui/inc/util.php -> ui/inc/util.inc.php
 * Added support for assessments
 */

?>
