<?php
/**
 * User interface for manipulating data in a parent object
 *
 * $Author: jebbol $
 * $Revision: 1.3 $
 * $Date: 2006/04/17 09:28:40 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.nl
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

// Configuration
include_once('../inc/config.php');

// Defaults
include_once($SPL_CFG['dirroot'].'/inc/defaults.inc.php');

// Page generation
include_once($SPL_CFG['dirroot'].'/ui/class/page.class.php');
include_once($SPL_CFG['dirroot'].'/ui/class/selector.class.php');

// Utility functions
include_once($SPL_CFG['dirroot'].'/ui/inc/util.inc.php');

// Methodology
include_once($SPL_CFG['dirroot'].'/class/methodology.class.php');

// Data objects
include_once($SPL_CFG['dirroot'].'/class/par.class.php');
include_once($SPL_CFG['dirroot'].'/class/stu.class.php');
include_once($SPL_CFG['dirroot'].'/class/usr.class.php');

//=========================================================================
// Sanitize request variables
//=========================================================================
$sanitize = array(
  'par_id'=>array('type'=>'int'),
  'par_usr_id'=>array('type'=>'int'),
  'parent_id'=>array('type'=>'int'),
  'par_sortname'=>array('type'=>'char'),
  'par_dispname'=>array('type'=>'char'),
);
if (isset($_SESSION['is_admin'])&&$_SESSION['is_admin']==true) {
  $sanitize['is_active'] = array('type'=>'bool');
}
require_once($SPL_CFG['dirroot'].'/ui/inc/sanitize.inc.php');

//=========================================================================
// Prerequisites
//=========================================================================

// Must be logged in
include_once($SPL_CFG['dirroot'].'/ui/inc/logincheck.php');

// Must have access rights
$perm = 'par';
include_once($SPL_CFG['dirroot'].'/ui/inc/permissioncheck.inc.php');

//=========================================================================
// Local variables
//=========================================================================

global $SPL_META;

$object = new par();
$editMode = (bool)false;

//=========================================================================
// Event handling
//=========================================================================

// Are we returning from searching a reference field?
$from_id = (bool)false;
if (isset($clean['from'])){
  $from_id = $clean['id'];
}
    
$par_user = 0;
if (isset($clean['par_usr_id'])){
  $par_user = $clean['par_usr_id'];
}
else if (isset($clean['parent_id'])){
  $par_user = $clean['parent_id'];
}

// Populate the object
$displayNewNotice = (bool)false;
if ($par_user) {
  $usr = new usr();
  $usr->init($par_user);
  $tmp_par_id = $usr->getProperty('par_id');  
  if ($tmp_par_id) {
    if ($object->init($tmp_par_id)){
      $_SESSION['par_id'] = $tmp_par_id;
    }
  }
  else if (!isset($clean['button_action'])) {
    $displayNewNotice = (bool)true;
  }
}

if (isset($clean['button_action'])){
  // Delete
  if ($clean['button_action']=='del') {
    $object->delete(false);
    $object = new par();
    unset($_SESSION['par_id']);
  }

  // Edit
  if ($clean['button_action']=='edit') {
    $editMode = (bool)true;
  }

  // New
  if ($clean['button_action']=='new') {
    $object = new par();
    if (isset($clean['parent_id'])){
      $object->setUser($clean['parent_id']);
    }
    $editMode = (bool)true;
    unset($_SESSION['par_id']);
  }

  // Save
  if ($clean['button_action']=='save') {
    $editMode = (bool)true;
    save();
    $editMode = (bool)false;
  }

  // Cancel
  if ($clean['button_action']=='cancel') {
    $editMode = (bool)false;
  }
}

// Set the chosen reference field
if ($from_id){
  if ($from_id != $_SESSION['par_id']){
    $object->setProperty($clean['from'],$from_id);
  }
}

//=========================================================================
// Definition of the page
//=========================================================================
$page = new page();
$page->init('par','default','template_iframetabbed.htm',false);
$page->startForm(null,'default',false,true,$editMode);
  $page->startAreaTabSheet();
  $page->startTable('none','width:100%"');
  $page->addTabRuler();
  $page->endTable();
  $page->startTable('detail');
    $page->startRow();
      $page->startColumn('areaTabSheet','style="vertical-align:top;width:100%"');
        $page->startTable();
        
        // Add a hidden field to remember the user id
        $page->addField('par_usr_id',$par_user);
        
        // Do we have to display a notice?
        if ($displayNewNotice) {
          $page->addContent($SPL_TXT['new_parent_notice']);
        }
        
        // No notice
        else {
  
          // Sortname
          $page->addField('par_sortname',$object->getProperty('par_sortname'),!$editMode);
          
          // Display name
          $page->addField('par_dispname',$object->getProperty('par_dispname'),!$editMode);
          
          // List of children
          $page->startRow();
            $page->startColumn('label','style="vertical-align:top;"');
              $page->addContent($page->translate('children'));
            $page->endColumn();
            $page->startColumn('sign');
            $page->endColumn();
            $page->startColumn('field');
              $sel = new selector();
              // Get students
              if (!isset($stu)){
                $stu = new stu();
              }
              $sel->init($stu->getStudents(), $object->getChildren());
              $page->addContent('<div class="scrollable" style="height:28em">');
              $page->addContent($sel->getSelector(!$editMode));
              $page->addContent('</div>');
            $page->endColumn();

          $page->endRow();
   
        }
        // End of not displaying a notice
        
        // Start buttons
        $page->startRow();
          // Buttons
          $page->startColumn();
          $page->endColumn();
          $page->startColumn();
          $page->endColumn();
          $page->startColumn();
            if ($editMode){
              $page->addButton('save','toolbar_save','submit','default','8','button');
              $page->addButton('cancel','toolbar_cancel','submit','default','8','button','no',null,true);
            }
            else {
              if ($_SESSION['par_permission']['write'] && $displayNewNotice) {
                $page->addButton('new','toolbar_new','submit','default','8','button');
              }
              if ($_SESSION['par_permission']['change'] && !$displayNewNotice) {
                $page->addButton('edit','toolbar_edit','submit','default','8','button');
              }
              if ($_SESSION['par_permission']['delete'] && !$displayNewNotice) {
                $page->addButton('del','toolbar_del','submit','default','8','button');
              }
            }
          $page->endColumn();
        $page->endRow();
        // End buttons
  $page->endTable();
  $page->endAreaTabSheet();
$page->endForm();

//=========================================================================
// Show the page
//=========================================================================

$page->setContent();
$page->createPage();
$page->showPage();

//=========================================================================
// Functions
//=========================================================================

function set(){
  
  global $object,$m,$clean;
  
  if (isset ($clean['par_sortname'])) {$object->setProperty('par_sortname', $clean['par_sortname']);}
  if (isset ($clean['par_dispname'])) {$object->setProperty('par_dispname', $clean['par_dispname']);}

  // Set children
  if (isset($clean['selector_id'])){
    $children = array();
    foreach ($clean['selector_id'] as $key => $selected){
      if ($selected=='on'){
        $children[] = $key;
      }
    }
    $object->queueChildren($children);
  }
  else {
    // if $clean['selector_id'] is empty, then delete all children
    $object->unsetChildren();
  }

}

function save(){
  
  global $object,$editMode,$par_user,$clean;

  if (!$editMode){
    return;
  }
  set();
  $object->setUser($par_user);
  $object->save();
  $_SESSION['par_id'] = $object->getProperty('par_id');
}

/** 
 * $Log: par.php,v $
 * Revision 1.3  2006/04/17 09:28:40  jebbol
 * Added resetCancel switch to addButton
 *
 * Revision 1.2  2006/04/14 19:52:02  jebbol
 * Added class to several td's, as they don't inherit from div's in IE
 *
 * Revision 1.1  2006/04/14 12:45:28  jebbol
 * Added support for mentors and parents
 */

?>
