<?php
/**
 * User interface for manipulating data in a student object
 *
 * $Author: jebbol $
 * $Revision: 1.19 $
 * $Date: 2006/08/08 08:07:19 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.nl
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

// Configuration
include_once('../inc/config.php');

// Defaults
include_once($SPL_CFG['dirroot'].'/inc/defaults.inc.php');

// Page generation
include_once($SPL_CFG['dirroot'].'/ui/class/page.class.php');

// Utility functions
include_once($SPL_CFG['dirroot'].'/ui/inc/util.inc.php');

// Methodology
include_once($SPL_CFG['dirroot'].'/class/methodology.class.php');

// Data objects
include_once($SPL_CFG['dirroot'].'/class/ina.class.php');
include_once($SPL_CFG['dirroot'].'/class/ins.class.php');
include_once($SPL_CFG['dirroot'].'/class/stu.class.php');
include_once($SPL_CFG['dirroot'].'/class/usr.class.php');

//=========================================================================
// Sanitize request variables
//=========================================================================
$sanitize = array(
  'stu_id'=>array('type'=>'int'),
  'stu_usr_id'=>array('type'=>'int'),
  'parent_id'=>array('type'=>'int'),
  'ins_id'=>array('type'=>'int'),
  'ina_id'=>array('type'=>'int'),
  'tra_id'=>array('type'=>'int'),
  'ali_id'=>array('type'=>'int'),
  'lst_id'=>array('type'=>'int'),
  'stu_courseyear'=>array('type'=>'int'),
  'stu_entryyear'=>array('type'=>'int'),
  'stu_sortname'=>array('type'=>'char'),
  'stu_dispname'=>array('type'=>'char'),
  'stu_sisnr'=>array('type'=>'char'),
  'stu_grp_id_schedule'=>array('type'=>'int'),
);
if (isset($_SESSION['is_admin'])&&$_SESSION['is_admin']==true) {
  $sanitize['is_active'] = array('type'=>'bool');
}
require_once($SPL_CFG['dirroot'].'/ui/inc/sanitize.inc.php');

//=========================================================================
// Prerequisites
//=========================================================================

// Must be logged in
include_once($SPL_CFG['dirroot'].'/ui/inc/logincheck.php');

// Must have access rights
$perm = 'stu';
include_once($SPL_CFG['dirroot'].'/ui/inc/permissioncheck.inc.php');

//=========================================================================
// Local variables
//=========================================================================

global $SPL_META;

$object = new stu();
$editMode = (bool)false;

//=========================================================================
// Methodology
//=========================================================================

$m = new methodology();
$m->init($_SESSION['methodology']);

//=========================================================================
// Event handling
//=========================================================================

// Are we returning from searching a reference field?
$from_id = (bool)false;
if (isset($clean['from'])){
  $from_id = $clean['id'];
}
    
$stu_user = 0;
if (isset($clean['stu_usr_id'])){
  $stu_user = $clean['stu_usr_id'];
}
else if (isset($clean['parent_id'])){
  $stu_user = $clean['parent_id'];
}

// Populate the object
$displayNewNotice = (bool)false;
if ($stu_user) {
  $usr = new usr();
  $usr->init($stu_user);
  $tmp_stu_id = $usr->getProperty('stu_id');  
  if ($tmp_stu_id) {
    if ($object->init($tmp_stu_id)){
      $_SESSION['stu_id'] = $tmp_stu_id;
    }
  }
  else if (!isset($clean['button_action'])) {
    $displayNewNotice = (bool)true;
  }
}

// Capture the search on reference fields
if (isset($clean['search_button'])) {
  // Save the form data
  set();
  save();
  // Redirect browser to the search page
  if ($clean['search_button']=='ins_id'){
    redirect($SPL_CFG['wwwroot'].'/ui/ins_search.php?from=ins_id');
  }
  if ($clean['search_button']=='ina_id'){
    redirect($SPL_CFG['wwwroot'].'/ui/ina_search.php?from=ina_id');
  }
  if ($clean['search_button']=='tra_id'){
    redirect($SPL_CFG['wwwroot'].'/methodology/traditional/ui/tra_search.php?from=tra_id');
  }
  if ($clean['search_button']=='ali_id'){
    redirect($SPL_CFG['wwwroot'].'/methodology/traditional/ui/ali_search.php?from=ali_id');
  }
}


// Events from the Toolbar buttons
if (isset($clean['button_action']) && $clean['button_action']){  
  switch ($clean['button_action']) {
    case 'cancel':
      break;
    case 'del':
      $object->delete(false);
      $object = new stu();
      unset($_SESSION['stu_id']);
      break;
    case 'edit':
      $editMode = (bool)true;
      break;
    case 'find':
      save();
      redirect($SPL_CFG['wwwroot'].'/methodology/traditional/ui/cat_search.php');
      break;
    case 'new':
      $object = new stu();
      if (isset($clean['parent_id'])){
        $object->setUser($clean['parent_id']);
      }
      $editMode = (bool)true;
      unset($_SESSION['stu_id']);
      break;
    case 'save':
      $editMode = (bool)true;
      save();
      $editMode = (bool)false;
      break;
    case 'list':
      set();
      $editMode = (bool)true;
      $displayNewNotice = (bool)false;
      break;
    default:
      break;
  }
}

// Set the chosen reference field
if ($from_id){
  if ($from_id != $_SESSION['stu_id']){
    $object->setProperty($clean['from'],$from_id);
  }
}

//=========================================================================
// Definition of the page
//=========================================================================
$page = new page();
$page->init('stu','default','template_iframetabbed.htm',false);
$page->startScrollableArea();
$page->startForm(null,'default',false,true);
  $page->startAreaTabSheet();
  $page->startTable('none','style="width:100%"');
  $page->addTabRuler();
  $page->endTable();
  $page->startTable('detail');
    $page->startRow();
      $page->startColumn('areaTabSheet','style="vertical-align:top"');
        $page->startTable();
        
        // Add a hidden field to remember the user id
        $page->addField('stu_usr_id',$stu_user);
        $page->addField('toolbar_button');
        
        // Do we have to display a notice?
        if ($displayNewNotice) {
          $page->addContent($SPL_TXT['new_student_notice']);
        }
        
        // No notice
        else {
          
          // Sortname
          $page->addField('stu_sortname',$object->getProperty('stu_sortname'),!$editMode);
          
          // Display name
          $page->addField('stu_dispname',$object->getProperty('stu_dispname'),!$editMode);
  
          // Student number in Student Information System
          $page->addField('stu_sisnr',$object->getProperty('stu_sisnr'),!$editMode);
          
          // Institution: when there is only one instance, hide it
          $ins = new ins();
          $single = $ins->isSingle();
          if ($single&&$editMode){
            $object->setProperty('ins_id',$single);
          }
          $page->addField('ins_id',$object->getProperty('ins_id'),!$editMode,false,false,false,NULL,true,false,$single);

          // Institution address: when there is only one instance, hide it
          $ina = new ina();
          $single = $ina->isSingle();
          if ($editMode){
            if ($single){
              $object->setProperty('ina_id',$single);
            }
            $page->addField('ina_id',$object->getProperty('ina_id'),!$editMode,false,false,$ina->getKeyValues('ins',$object->getProperty('ins_id')),NULL,true,false,$single);
          }
          else {
            $page->addField('ina_id',$object->getProperty('ina_id'),!$editMode,false,false,false,NULL,true,false,$single);
          }
          
          // Scheduler related data
          if (isset($SPL_CFG['scheduler'])&&($SPL_CFG['scheduler']!='none')){
            if ($editMode){
              include_once($SPL_CFG['dirroot'].'/class/grp.class.php');
              $grp = new grp();
              $page->addField('stu_grp_id_schedule',$object->getProperty('stu_grp_id_schedule'),!$editMode,false,false,$grp->getKeyValues(false,'grp_type = "class"'));
            }
            else {
              $page->addField('stu_grp_id_schedule',$object->getProperty('stu_grp_id_schedule'),!$editMode);
            }
          }          
   
          // Include methodology related fields
          $m_inc = $m->getStudentEditScript();
          if ($m_inc){
            include_once ($m_inc);
          }
          $page->endRow();
   
        }
        // End of not displaying a notice
        
        // Start buttons
        $page->startRow();
          // Buttons
          $page->startColumn('none','colspan="3"');
            if ($editMode){
              $page->addButton('save','toolbar_save','submit','default','8','button');
              $page->addButton('cancel','toolbar_cancel','submit','default','8','button','no',null,true);
            }
            else {
              if ($_SESSION['stu_permission']['write'] && $displayNewNotice) {
                $page->addButton('new','toolbar_new','submit','default','8','button');
              }
              if ($_SESSION['stu_permission']['change'] && !$displayNewNotice) {
                $page->addButton('edit','toolbar_edit','submit','default','8','button');
              }
              if ($_SESSION['stu_permission']['delete'] && !$displayNewNotice) {
                $page->addButton('del','toolbar_del','submit','default','8','button');
              }
            }
          $page->endColumn();
        $page->endRow();
        // End buttons
  $page->endTable();
  $page->endAreaTabSheet();
$page->endForm();
$page->endScrollableArea();

//=========================================================================
// Show the page
//=========================================================================

$page->setContent();
$page->createPage();
$page->showPage();

//=========================================================================
// Functions
//=========================================================================

function set(){
  
  global $object,$m,$clean;

  if (isset ($clean['stu_sortname']))        {$object->setProperty('stu_sortname',        $clean['stu_sortname']);}
  if (isset ($clean['stu_dispname']))        {$object->setProperty('stu_dispname',        $clean['stu_dispname']);}
  if (isset ($clean['stu_sisnr']))           {$object->setProperty('stu_sisnr',           $clean['stu_sisnr']);}
  if (isset ($clean['ins_id']))              {$object->setProperty('ins_id',              $clean['ins_id']);}
  if (isset ($clean['ina_id']))              {$object->setProperty('ina_id',              $clean['ina_id']);}
  if (isset ($clean['stu_grp_id_schedule'])) {$object->setProperty('stu_grp_id_schedule', $clean['stu_grp_id_schedule']);}
  
  // Include methodology related fields
  $m_inc = $m->getStudentSetScript();
  if ($m_inc){
    include_once ($m_inc);
  }
}

function save(){
  
  global $object,$editMode,$stu_user,$clean;

  if (!$editMode){
    return;
  }
  set();
  $object->setUser($stu_user);
  $object->save();
  $_SESSION['stu_id'] = $object->getProperty('stu_id');
}

/**
 * $Log: stu.php,v $
 * Revision 1.19  2006/08/08 08:07:19  jebbol
 * Changed repository (cvs.sourceforge.net -> splanner.cvs.sourceforge.net)
 *
 * Revision 1.18  2006/04/17 09:28:40  jebbol
 * Added resetCancel switch to addButton
 *
 * Revision 1.17  2006/04/14 19:52:02  jebbol
 * Added class to several td's, as they don't inherit from div's in IE
 *
 * Revision 1.16  2006/04/14 12:45:28  jebbol
 * Added support for mentors and parents
 *
 * Revision 1.15  2006/02/11 18:49:36  jebbol
 * Renamed ui/inc/util.php -> ui/inc/util.inc.php
 * Added UI areas to support multiple skins
 */

?>
