<?php
/**
 * User interface for manipulating data in a teacher object
 *
 * $Author: jebbol $
 * $Revision: 1.10 $
 * $Date: 2006/08/08 08:07:19 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.nl
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

// Configuration
include_once('../inc/config.php');

// Defaults
include_once($SPL_CFG['dirroot'].'/inc/defaults.inc.php');

// Page generation
include_once($SPL_CFG['dirroot'].'/ui/class/page.class.php');

// Utility functions
include_once($SPL_CFG['dirroot'].'/ui/inc/util.inc.php');

// Methodology
include_once($SPL_CFG['dirroot'].'/class/methodology.class.php');

// Data objects
include_once($SPL_CFG['dirroot'].'/methodology/'.$_SESSION['methodology'].'/class/tea.class.php');
include_once($SPL_CFG['dirroot'].'/class/ina.class.php');
include_once($SPL_CFG['dirroot'].'/class/ins.class.php');
include_once($SPL_CFG['dirroot'].'/class/usr.class.php');

//=========================================================================
// Sanitize request variables
//=========================================================================
$sanitize = array(
  'tea_id'           => array('type'=>'int'),
  'tea_usr_id'       => array('type'=>'int'),
  'parent_id'        => array('type'=>'int'),
  'ins_id'           => array('type'=>'int'),
  'ina_id'           => array('type'=>'int'),
  'tea_sortname'     => array('type'=>'char'),
  'tea_dispname'     => array('type'=>'char'),
  'tea_schedulename' => array('type'=>'char'),
  'tea_rebuild_spt'  => array('type'=>'bool'),
);
if (isset($_SESSION['is_admin'])&&$_SESSION['is_admin']==true) {
  $sanitize['is_active'] = array('type'=>'bool');
}
require_once($SPL_CFG['dirroot'].'/ui/inc/sanitize.inc.php');

//=========================================================================
// Prerequisites
//=========================================================================

// Must be logged in
include_once($SPL_CFG['dirroot'].'/ui/inc/logincheck.php');

// Must have access rights
$perm = 'tea';
include_once($SPL_CFG['dirroot'].'/ui/inc/permissioncheck.inc.php');

//=========================================================================
// Local variables
//=========================================================================

global $SPL_META;

$object = new tea();
$editMode = (bool)false;

//=========================================================================
// Methodology
//=========================================================================

$m = new methodology();
$m->init($_SESSION['methodology']);

//=========================================================================
// Event handling
//=========================================================================

// Are we returning from searching a reference field?
$from_id = (bool)false;
if (isset($clean['from'])){
  $from_id = $clean['id'];
}
    
$tea_user = 0;
if (isset($clean['tea_usr_id'])){
  $tea_user = $clean['tea_usr_id'];
}
else if (isset($clean['parent_id'])){
  $tea_user = $clean['parent_id'];
}

// Populate the object
$displayNewNotice = (bool)false;
if ($tea_user) {
  $usr = new usr();
  $usr->init($tea_user);
  $tmp_tea_id = $usr->getProperty('tea_id');  
  if ($tmp_tea_id) {
    if ($object->init($tmp_tea_id)){
      $_SESSION['tea_id'] = $tmp_tea_id;
    }
  }
  else if (!isset($clean['button_action'])) {
    $displayNewNotice = (bool)true;
  }
}

if (isset($clean['button_action'])){
  // Delete
  if ($clean['button_action']=='del') {
    $object->delete(false);
    $object = new tea();
    unset($_SESSION['tea_id']);
  }

  // Edit
  if ($clean['button_action']=='edit') {
    $editMode = (bool)true;
  }

  // New
  if ($clean['button_action']=='new') {
    $object = new tea();
    if (isset($clean['parent_id'])){
      $object->setUser($clean['parent_id']);
    }
    $editMode = (bool)true;
    unset($_SESSION['tea_id']);
  }

  // Save
  if ($clean['button_action']=='save') {
    $editMode = (bool)true;
    save();
    $editMode = (bool)false;
  }

  // Cancel
  if ($clean['button_action']=='cancel') {
    $editMode = (bool)false;
  }
}

// Set the chosen reference field
if ($from_id){
  if ($from_id != $_SESSION['tea_id']){
    $object->setProperty($clean['from'],$from_id);
  }
}

//=========================================================================
// Definition of the page
//=========================================================================
$page = new page();
$page->init('tea','default','template_iframetabbed.htm',false);
$page->startForm(null,'default',false,true);
  $page->startAreaTabSheet();
  $page->startTable('none','style="width:100%"');
  $page->addTabRuler();
  $page->endTable();
  $page->startTable('detail');
    $page->startRow();
      $page->startColumn('areaTabSheet','style="vertical-align:top;width:100%"');
        $page->startTable();
        
        // Add a hidden field to remember the user id
        $page->addField('tea_usr_id',$tea_user);
        
        // Do we have to display a notice?
        if ($displayNewNotice) {
          $page->addContent($SPL_TXT['new_teacher_notice']);
        }
        
        // No notice
        else {
  
          // Sortname
          $page->addField('tea_sortname',$object->getProperty('tea_sortname'),!$editMode);
          
          // Display name
          $page->addField('tea_dispname',$object->getProperty('tea_dispname'),!$editMode);

          // Schedule name
          if (isset($SPL_CFG['scheduler'])&&($SPL_CFG['scheduler']!='none')){
            $page->addField('tea_schedulename',$object->getProperty('tea_schedulename'),!$editMode);
          }

          // Include methodology related fields
          $m_inc = $m->getTeacherEditScript();
          if ($m_inc){
            include_once ($m_inc);
          }

          $page->endRow();
   
        }
        // End of not displaying a notice
        
        // Start buttons
        $page->startRow();
          $page->startColumn('none','colspan="3"');
            if ($editMode){
              $page->addButton('save','toolbar_save','submit','default','8','button');
              $page->addButton('cancel','toolbar_cancel','submit','default','8','button','no',null,true);
            }
            else {
              if ($_SESSION['tea_permission']['write'] && $displayNewNotice) {
                $page->addButton('new','toolbar_new','submit','default','8','button');
              }
              if ($_SESSION['tea_permission']['change'] && !$displayNewNotice) {
                $page->addButton('edit','toolbar_edit','submit','default','8','button');
              }
              if ($_SESSION['tea_permission']['delete'] && !$displayNewNotice) {
                $page->addButton('del','toolbar_del','submit','default','8','button');
              }
            }
          $page->endColumn();
        $page->endRow();
        // End buttons

    $page->endTable();
  $page->endAreaTabSheet();

$page->endForm();

//=========================================================================
// Show the page
//=========================================================================

$page->setContent();
$page->createPage();
$page->showPage();

//=========================================================================
// Functions
//=========================================================================

function set(){
  
  global $object,$m,$clean;
  
  if (isset ($clean['tea_sortname'])) {$object->setProperty('tea_sortname', $clean['tea_sortname']);}
  if (isset ($clean['tea_dispname'])) {$object->setProperty('tea_dispname', $clean['tea_dispname']);}
  if (isset ($clean['tea_schedulename'])) {$object->setProperty('tea_schedulename', $clean['tea_schedulename']);}
  if (isset ($clean['ins_id']))       {$object->setProperty('ins_id', $clean['ins_id']);}
  if (isset ($clean['ina_id']))       {$object->setProperty('ina_id', $clean['ina_id']);}
  // Include methodology related fields
  $m_inc = $m->getTeacherSetScript();
  if ($m_inc){
    include_once ($m_inc);
  }
  // Subjects
  $m_inc = $m->getTeacherSubjectsSetScript();
  if ($m_inc){
    include_once ($m_inc);
  }

}

function save(){
  
  global $object,$editMode,$tea_user,$clean;

  if (!$editMode){
    return;
  }
  set();
  $object->setUser($tea_user);
  $object->save();
  $_SESSION['tea_id'] = $object->getProperty('tea_id');
}

/**
 * $Log: tea.php,v $
 * Revision 1.10  2006/08/08 08:07:19  jebbol
 * Changed repository (cvs.sourceforge.net -> splanner.cvs.sourceforge.net)
 *
 * Revision 1.9  2006/04/17 09:29:11  jebbol
 * Added resetCancel switch to addButton and added subjects
 *
 * Revision 1.8  2006/04/14 19:52:02  jebbol
 * Added class to several td's, as they don't inherit from div's in IE
 *
 * Revision 1.7  2006/04/14 12:45:28  jebbol
 * Added support for mentors and parents
 *
 * Revision 1.6  2006/02/11 18:53:21  jebbol
 * Renamed ui/inc/util.php -> ui/inc/util.inc.php
 * Added UI areas to support multiple skins
 */

?>
