<?php
/**
 * User interface for manipulating data in the user object
 *
 * $Author: jebbol $
 * $Revision: 1.19 $
 * $Date: 2006/08/08 08:07:19 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.nl
 * @license   http://www.gnu.org/copyleft/gpl.html
 * 
 * $Log: usr.php,v $
 * Revision 1.19  2006/08/08 08:07:19  jebbol
 * Changed repository (cvs.sourceforge.net -> splanner.cvs.sourceforge.net)
 *
 * Revision 1.18  2006/05/30 12:19:26  jebbol
 * When adding a new user, tabs remain hidden until the newely created record has been saved
 *
 * Revision 1.17  2006/04/14 12:46:33  jebbol
 * Added print support
 *
 * Revision 1.16  2006/03/13 12:50:18  jebbol
 * Cleaned up HTML validation errors
 *
 * Revision 1.15  2006/02/11 19:03:04  jebbol
 * Renamed ui/inc/util.php -> ui/inc/util.inc.php
 * Added UI areas to support multiple skins
 *
 */

//=========================================================================
// Includes
//=========================================================================

// Configuration
include_once('../inc/config.php');

// Page generation
include_once($SPL_CFG['dirroot'].'/ui/class/page.class.php');

// Utility functions
include_once($SPL_CFG['dirroot'].'/ui/inc/util.inc.php');

// Data object
include_once($SPL_CFG['dirroot'].'/class/usr.class.php');

//=========================================================================
// Sanitize request variables
//=========================================================================
$sanitize = array(
  'usr_id'=>array('type'=>'int'),
  'usr_login'=>array('type'=>'char'),
  'usr_psswd'=>array('type'=>'char'),
  'usr_psswd_exp'=>array('type'=>'date'),
  'usr_psswd_retry'=>array('type'=>'int'),
  'usr_email'=>array('type'=>'char'),
  'usr_name'=>array('type'=>'char'),
  'per_id'=>array('type'=>'int'),
  'usr_allow_date_from'=>array('type'=>'date'),
  'usr_allow_date_until'=>array('type'=>'date'),
  'usr_allow_time_from'=>array('type'=>'time'),
  'usr_allow_time_until'=>array('type'=>'time'),
  'usr_log'=>array('type'=>'bool'),
  'is_sysadm'=>array('type'=>'bool'),
);
if (isset($_SESSION['is_admin'])&&$_SESSION['is_admin']==true) {
  $sanitize['is_active'] = array('type'=>'bool');
}
require_once($SPL_CFG['dirroot'].'/ui/inc/sanitize.inc.php');

//=========================================================================
// Prerequisites
//=========================================================================

// Must be logged in
include_once($SPL_CFG['dirroot'].'/ui/inc/logincheck.php');

// Must be an administrator
include_once($SPL_CFG['dirroot'].'/ui/inc/admincheck.inc.php');


//=========================================================================
// Local variables
//=========================================================================

$object = new usr();
$editMode = (bool)false;

//=========================================================================
// Handling of POST and GET
//=========================================================================

// User requested a specific object
if (isset($clean['id'])) {
  $object->init($clean['id']);
  unset($_SESSION['add_user_mode']);
}
else {
  // Make sure we do not instantiate the 'current user'
  // when trying to add a new one
  if (isset($_SESSION['add_user_mode'])){
    unset($_SESSION['add_user_mode']);
  }
  else { 
    $object->init($_SESSION['usr_id']);
  }
}
 
// Check Toolbar buttons
// Events from the Toolbar buttons
if (isset($clean['button_action']) && $clean['button_action']){  
  switch ($clean['button_action']) {
    case 'cancel':
      break;
    case 'del':
      $object->delete($object->getProperty('is_active')=='1');
      $object->save();
      $object = new usr();
      break;
    case 'edit':
      $editMode = (bool)true;
      break;
    case 'find':
      if ($editMode){
        $object->save();
      }
      redirect($SPL_CFG['wwwroot'].'/ui/usr_search.php');
      break;
    case 'new':
      $object = new usr();
      $editMode = (bool)true;
      $_SESSION['add_user_mode']= (bool)true;
      break;
    case 'print':
      if ($editMode){
        $object->save();
      }
      redirect($SPL_CFG['wwwroot'].'/ui/print.php?class=usr');
      break;
    case 'save':
      if (isset ($clean['usr_login']))            {$object->setProperty('usr_login', $clean['usr_login']);}
      if (isset ($clean['usr_psswd']))            {$object->setProperty('usr_psswd', $clean['usr_psswd']);}
      if (isset ($clean['usr_psswd_exp']))        {$object->setProperty('usr_psswd_exp', $clean['usr_psswd_exp']);}
      if (isset ($clean['usr_psswd_retry']))      {$object->setProperty('usr_psswd_retry', $clean['usr_psswd_retry']);}
      if (isset ($clean['usr_email']))            {$object->setProperty('usr_email', $clean['usr_email']);}
      if (isset ($clean['usr_name']))             {$object->setProperty('usr_name', $clean['usr_name']);}
      if (isset ($clean['per_id']))               {$object->setProperty('per_id', $clean['per_id']);}
      if (isset ($clean['usr_allow_date_from']))  {$object->setProperty('usr_allow_date_from', $clean['usr_allow_date_from']);}
      if (isset ($clean['usr_allow_date_until'])) {$object->setProperty('usr_allow_date_until', $clean['usr_allow_date_until']);}
      if (isset ($clean['usr_allow_time_from']))  {$object->setProperty('usr_allow_time_from', $clean['usr_allow_time_from']);}
      if (isset ($clean['usr_allow_time_until'])) {$object->setProperty('usr_allow_time_until', $clean['usr_allow_time_until']);}
      if (isset ($clean['usr_log']))              {$object->setProperty('usr_log', $clean['usr_log']);}
      if (isset ($clean['is_sysadm']))            {$object->setProperty('is_sysadm', $clean['is_sysadm']);}
      if (isset ($clean['is_active']))            {$object->setProperty('is_active', $clean['is_active']);}
      $object->save();
      $editMode = (bool)false;
      break;
    default:
      break;
  }
}
 
//=========================================================================
// Definition of the page
//=========================================================================

$page = new page();
$page->init('usr');
$page->startForm(null,'default',true);
  $page->addToolbar((isset($_SESSION['usr_permission']['admin'])&&$_SESSION['usr_permission']['admin'])&&!$editMode,!$editMode&&$object->isLoaded(),$editMode,(isset($_SESSION['usr_permission']['admin'])&&$_SESSION['usr_permission']['admin'])&&!$editMode,$editMode,(isset($_SESSION['usr_permission']['admin'])&&$_SESSION['usr_permission']['admin'])&&!$editMode);
  $page->startAreaData();
  $page->startSection();
    $page->startSubsection('subsection1');
      if (isset($_SESSION['usr_permission']['admin'])&&$_SESSION['usr_permission']['admin']){
        $page->addField('usr_login',$object->getProperty('usr_login'),!$editMode);
      }
      $page->addField('usr_name',$object->getProperty('usr_name'),!$editMode);
      if (isset($_SESSION['usr_permission']['admin'])&&$_SESSION['usr_permission']['admin']){
        $page->addField('usr_psswd',$object->getProperty('usr_psswd'),!$editMode);
      }
      $page->addField('usr_psswd_exp',$object->getProperty('usr_psswd_exp'),!$editMode);
      $page->addField('usr_psswd_retry',$object->getProperty('usr_psswd_retry'),!$editMode);
      $page->addField('usr_email',$object->getProperty('usr_email'),!$editMode);
      if ($SPL_CFG['person_details'] != 'false') {
        $page->addField('per_id',$object->getProperty('per_id'),!$editMode);
      }
      $page->addField('usr_allow_date_from',$object->getProperty('usr_allow_date_from'),!$editMode);
      $page->addField('usr_allow_date_until',$object->getProperty('usr_allow_date_until'),!$editMode);
      $page->addField('usr_allow_time_from',$object->getProperty('usr_allow_time_from'),!$editMode);
      $page->addField('usr_allow_time_until',$object->getProperty('usr_allow_time_until'),!$editMode);
      $page->addField('usr_login_count',$object->getProperty('usr_login_count'),!$editMode);
      $page->addField('usr_login_date_first',$object->getProperty('usr_login_date_first'),!$editMode);
      $page->addField('usr_login_date_prev',$object->getProperty('usr_login_date_prev'),!$editMode);
      $page->addField('usr_login_date_last',$object->getProperty('usr_login_date_last'),!$editMode);
      if (isset($_SESSION['usr_permission']['admin'])&&$_SESSION['usr_permission']['admin']){
        $page->addField('usr_log',$object->getProperty('usr_log'),!$editMode);
        $page->addField('is_sysadm',$object->getProperty('is_sysadm'),!$editMode);
        $page->addField('is_active',$object->getProperty('is_active'),!$editMode);
      }
      $page->showFormButtons();
    $page->endSubsection();
    $page->startSubsection('subsection2');
      if (!isset($_SESSION['add_user_mode'])||!$_SESSION['add_user_mode']) {
      $tabs = array(array('name'=>'usr_tab1','title'=>'usa','tabaction'=>'usa_search.php?parent=usr'),
                    array('name'=>'usr_tab2','title'=>'stu_id','tabaction'=>'stu.php?parent=usr'.'&amp;parent_id='.$object->getProperty('usr_id'),'empty'=>!$object->isStudent()),
                    array('name'=>'usr_tab3','title'=>'tea_id','tabaction'=>'tea.php?parent=usr'.'&amp;parent_id='.$object->getProperty('usr_id'),'empty'=>!$object->isTeacher()),
                    array('name'=>'usr_tab4','title'=>'men_id','tabaction'=>'men.php?parent=usr'.'&amp;parent_id='.$object->getProperty('usr_id'),'empty'=>!$object->isMentor()),
                    array('name'=>'usr_tab5','title'=>'par_id','tabaction'=>'par.php?parent=usr'.'&amp;parent_id='.$object->getProperty('usr_id'),'empty'=>!$object->isParent()),
                    );
      $selected = (!isset($clean['selected_tab'])||(isset($_SESSION['add_user_mode'])&&$_SESSION['add_user_mode']==true)?'usr_tab1':$clean['selected_tab']);
      $page->addTabbar($selected,'subframe',$tabs,false,'id='.$object->getProperty('usr_id'));
      $page->addIFrame('usa_search.php?id='.$object->getProperty('usr_id'),'subframe','default','no','36em');
      }
    $page->endSubsection();
  $page->endSection();
  $page->endAreaData();
  $page->addFootbar();
$page->endForm();

//=========================================================================
// Show the page
//=========================================================================

$page->setContent();
$page->createPage();
if ($object->isLoaded()) {
  $page->setRowStatistics($object->getPrimary(),$object->getProperty('create_date'),$object->getProperty('create_usr_id'),$object->getProperty('create_src_id'),$object->getProperty('update_date'),$object->getProperty('update_usr_id'),$object->getProperty('update_src_id'));
}
else {
  $page->setRowStatistics();
}
$page->showPage();

?>
