<?php
/**
 * User interface for searching items in the user table
 *
 * $Author: jebbol $
 * $Revision: 1.12 $
 * $Date: 2006/08/21 19:44:52 $
 * 
 * @copyright Copyright (c) 2005-2006 Studiewijzer.nl 
 * @link      http://www.studiewijzer.nl
 * @license   http://www.gnu.org/copyleft/gpl.html
 */

//=========================================================================
// Includes
//=========================================================================

// Configuration
include_once('../inc/config.php');

// Page generation
include_once($SPL_CFG['dirroot'].'/ui/class/page.class.php');

// HTML Table support
include_once($SPL_CFG['dirroot'].'/ui/class/table.class.php');

// Utility functions
include_once($SPL_CFG['dirroot'].'/ui/inc/util.inc.php');

// Defaults
include_once($SPL_CFG['dirroot'].'/class/usd.class.php');

//=========================================================================
// Prerequisites
//=========================================================================

// Must be logged in
include_once($SPL_CFG['dirroot'].'/ui/inc/logincheck.php');

//=========================================================================
// Sanitize request variables
//=========================================================================
$sanitize = array(
  'usr_type'=>array('type'=>'int'),
  'search_show_inactive'=>array('type'=>'bool'),
);
require_once($SPL_CFG['dirroot'].'/ui/inc/sanitize.inc.php');

//=========================================================================
// Local variables
//=========================================================================
$usd = new usd();
$usd_script = 'usr_search';

// Preferred user type
$prefType = $usd->getDefault($usd_script,'usr_type');
if (isset($clean['usr_type'])){
  $prefType = $clean['usr_type'];
  $usd->setDefault($usd_script,'usr_type',$prefType);
}
// User login
$prefLogin = $usd->getDefault($usd_script,'search_for');
if (isset($clean['search_for'])){
  $prefLogin = $clean['search_for'];
  $usd->setDefault($usd_script,'search_for',$prefLogin);
}
// Only administrator can see inactive items
$prefInactive = false;
// Show inactive items
if ($_SESSION['is_admin']) {
  $prefInactive = $clean['search_show_inactive'];
}

$query['columns']  = 'usr_id AS id,"usr.php'.(isset($clean['from'])?'?from='.$clean['from']:'').'" AS target,usr_login,usr_name,usr_phone,usr_email,IF(usr_login_date_last>0,FROM_UNIXTIME(usr_login_date_last,"%Y-%m-%d %H:%i"),"") AS login_date_last,usr_login_count'.(isset($clean['search_show_inactive'])&&$clean['search_show_inactive']==true?',is_active':'');
$query['tblName']  = 'usr';
$query['where']    = 'WHERE usr_id > 0'.((!isset($clean['search_show_inactive'])||(isset($clean['search_show_inactive'])&&$clean['search_show_inactive']==false))?' AND is_active':'');
if (isset($prefType) && ((int)$prefType>0)) {
  switch ((int)$prefType) {
    case 1:
      // Students
      $query['where'] .= ' AND stu_id > 0';
      break;
    case 2:
      // Teachers
      $query['where'] .= ' AND tea_id > 0';
      break;
    case 3:
      // Mentors
      $query['where'] .= ' AND men_id > 0';
      break;
    case 4:
      // Parents
      $query['where'] .= ' AND par_id > 0';
      break;
  }
}
$query['orderBy']  = 'usr_login';
$query['orderDir'] = 'ASC';

//=========================================================================
// Handling of POST and GET
//=========================================================================

if(isset($prefLogin)) {
  $query['where'] .= ' AND usr_login LIKE "'.$prefLogin.'%"';
} 

//The column to orderBy
if (isset ($clean['orderBy'])) $query['orderBy'] = $clean['orderBy'];

//The starting offset for record paging 
if (isset ($clean['viewStart'])) $query['viewStart'] = $clean['viewStart'];

//Sort ascending or descending ('ASC' or 'DESC') 
if (isset ($clean['orderDir']))  $query['orderDir'] = $clean['orderDir']; 

// Add inactive switch
if (isset($clean['search_show_inactive'])) $query['showInactive'] = $clean['search_show_inactive'];

// Encode URL
$query['encode'] = false;

//=========================================================================
// Definition of the page
//=========================================================================

// Create the page
$page = new page();
$page->init('usr','default','template_search.htm');

// Selection area
$area = '<table style="margin-bottom:3px"><tr><td>'."\r\n";

// User type
$area .= ' '.$SPL_TXT['usr_type'].' <select class="listbox" id="usr_type" name="usr_type" style="width:7em" onChange="document.usrsearchform.submit();"><option value="0">'.$SPL_TXT['list_all'].'</option><option value="1"'.((int)$prefType==1?' selected="selected"':'').'>'.$SPL_TXT['stu'].'</option><option value="2"'.((int)$prefType==2?' selected="selected"':'').'>'.$SPL_TXT['tea'].'</option><option value="3"'.((int)$prefType==3?' selected="selected"':'').'>'.$SPL_TXT['men'].'</option><option value="4"'.((int)$prefType==4?' selected="selected"':'').'>'.$SPL_TXT['par'].'</option></select>';
$area .= ' '.$SPL_TXT['usr_login'].' <input type="text" name="search_for" id="search_for" value="'.$prefLogin.'" class="text">';
if ($_SESSION['is_admin']) {
  $area .= ' '.' <input type="checkbox" class="checkbox" style="margin-bottom:0px;padding-bottom:0px;" name="search_show_inactive" id="search_show_inactive"'.($prefInactive?' checked="checked"':'').' onClick="document.sugsearchform.submit();" />'.' '.$SPL_TXT['show_inactive']."\r\n";
}
$area .= '</td></tr></table>'."\r\n";


// Create the HTML table
$table = new table($query);
$table->init('usr');
$table->enableRowSelection();
$table->setCallbackScript($_SERVER['PHP_SELF']);
$table->addNewButton();


$page->setHeadScripts($table->getScripts());
$table->setMetaOverrule('is_active', array('hidden'=>true));
$page->startForm("usrsearchform",'default',false,true);
  $page->startSection();
  $page->startColumn();
  $page->startAreaSearch();
  //$page->addSearchbar(array('usr_login','usr_login_date_last'),(isset($clean['search_by'])?$clean['search_by']:''),isset($clean['search_for'])?$clean['search_for']:'',null,$clean['search_show_inactive']);
  $page->addContent($area);
  $page->endAreaSearch();
  $table->setImageDir($page->getImageDir());
  $page->startAreaData();
  $page->addContent($table->getTable());
  $page->endAreaData();
  $page->endColumn();
  $page->endSection();
$page->endForm();

//=========================================================================
// Show the page
//=========================================================================

$page->setContent();
$page->createPage();
$page->showPage();

/** 
 * $Log: usr_search.php,v $
 * Revision 1.12  2006/08/21 19:44:52  jebbol
 * Changed display format of the last-login date
 *
 * Revision 1.11  2006/08/21 18:40:17  jebbol
 * Added user type to the search options
 *
 * Revision 1.10  2006/04/14 12:45:28  jebbol
 * Added support for mentors and parents
 *
 * Revision 1.9  2006/02/11 19:00:45  jebbol
 * Renamed ui/inc/util.php -> ui/inc/util.inc.php
 * Added UI areas to support multiple skins
 */

?>
