<?php
/**
 * ws/spl.webservice.php
 * Web service provider.
 *
 * @author    Jebbol
 * @version   1.0
 * @revision  2005-04-22
 * @package   spl
 * @copyright Copyright (c) 2005 Studiewijzer.nl 
 * @link      http://www.studiewijzer.net
 * @license   http://www.gnu.org/copyleft/gpl.html
 * 
 * In order to use the methods in this libary, the client must provide a valid
 * ticket. A ticket can be obtained by signing on (calling 'signon') and
 * supplying a valid user login and password.
 */

//=========================================================================
// Includes
//=========================================================================

// Configuration
include_once('../inc/config.php');

// Database functions
if (!isset($SPL_CFG['dbtype'])) {$SPL_CFG['dbtype'] = 'mysql';}
include_once($SPL_CFG['dirroot'].'/db/'.$SPL_CFG['dbtype'].'/db.inc.php');

// nuSOAP version 1.81 or higher
include_once ($SPL_CFG['dirroot']."/ws/nusoap.php");

//=========================================================================
// General purpose requests
//=========================================================================

/**
 * Adds a user
 *
 * @param  $ticket   A valid session ticket
 * @param  $user     The user login name
 * @param  $password The user's password
 * @return boolean
 */
function addUser ($ticket,$user,$password) {
  
  global $SPL_CFG;
  
  if (_checkTicket($ticket)) {
    include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
    $usr = new usr();
    $usr->setProperty('usr_login',$user);
    $usr->setProperty('usr_psswd',$password);
    $usr->setProperty('usr_name',$user);
    return ($usr->save());
  }
  return (bool) false;
}

/**
 * Retrieves the study guide for a student
 *
 * @param  $ticket   A valid session ticket
 * @param  $userid   The user id (default: the user who signed on)
 * @return boolean
 */
function getStudyGuide ($ticket,$userid='myself') {
  
  global $SPL_CFG;

  if (_checkTicket($ticket)) {
    
    // Find userid
    if ($userid=='myself'){
      $userid = $_SESSION['usr_id'];
    }
    
    if (is_numeric($userid)){
      $query  = 'SELECT sgd.sgd_id,sgd.cat_id,sgd.sub_id,sgd.sug_id,sgd.prd_id_start,sgd.sgd_calyear_start,sgd.sgd_calmonth_start,sgd.sgd_calday_start,sub.sub_short,sub.sub_name,sug.sug_short,sug.sug_title,sug.sug_load,sts.sts_short';
      $query .= ' FROM '.$SPL_CFG['prefix'].'sgd sgd LEFT JOIN '.$SPL_CFG['prefix'].'sub sub ON sub.sub_id = sgd.sub_id LEFT JOIN '.$SPL_CFG['prefix'].'sug sug ON sug.sug_id = sgd.sug_id LEFT JOIN '.$SPL_CFG['prefix'].'sts sts ON sts.sts_id = sgd.sts_id';
      $query .= ' WHERE sgd.usr_id = "'.$userid.'" AND sgd.adm_id = "'.$_SESSION['adm_id'].'" AND sgd.is_active AND NOT sgd.is_done';
      $query .= ' ORDER BY sgd.sgd_calyear_start,sgd.sgd_calmonth_start,sgd.sgd_calday_start,sub.sub_short ASC';
      $result = db_query($query);

      $rows = array();
      while ($row = db_fetch_array($result)) {
        $rows[] = $row;
      }
      return $rows;
    }
    
  }
  return (bool) false;
}

/**
 * Validates the user credentials and returns a session ticket
 *
 * @param  $user     The user login name
 * @param  $password The user's password
 * @param  $source   (optional) The id of the source requesting signon
 * @return           A session ticket, or <code>False</code>.
 */
function signon ($user,$password,$source='native') {
  
  global $SPL_CFG;
  
  // Set login required default to true
  $mustLogin = (bool) true;

  // If this is not a native call, check the source
  if (is_numeric($source)) {
    include_once($SPL_CFG['dirroot'].'/class/src.class.php');
    $src = new src();
    if ($src->init($source)) {
      // Is login required for this source
      $mustLogin = $src->_data['src_login'];
    }
    else {
      // If source cannot be found, abort this proces
      return (bool) false;
    }
  }
  
  // Check user credentials
  if ($mustLogin) {
    include_once($SPL_CFG['dirroot'].'/class/usr.class.php');
    $usr = new usr();
    if (!$usr->login($user,$password,$source)){
      return (bool) false;
    }
    $_SESSION['usr_id'] = $usr->_data['usr_id'];
  }

  // Obtain a session ticket
  //return session_id();
  return _newTicket();
}

  //=========================================================================
  // Private methods
  //=========================================================================

  /** 
   * Validates a ticket and restores the session data
   * 
   * @param  string  $ticket The ticket to validate
   * @return array           Session data
   * @access private
   * @uses   array   $SPL_CFG Global configuration data
   */
  function _checkTicket($ticket) {
    
    global $SPL_CFG;
    
    $query = "SELECT tic_data FROM ".$SPL_CFG['prefix']."tic  WHERE tic_id = '".$ticket."' LIMIT 1";
    $result = db_retrieve(db_query($query));
    if ($result) {
      $data = unserialize($result['tic_data']);
      foreach ($data as $property => $value) {
        $_SESSION[$property] = $value;
      }
      return (bool) true;
    }
    else {
      return (bool) false;
    }
  }


  /** 
   * Deletes any expired tickets
   * 
   * @access private
   * @uses   array   $SPL_CFG Global configuration data
   */
  function _delTicket() {
    
    global $SPL_CFG;
    
    $query = "DELETE FROM ".$SPL_CFG['prefix']."tic  WHERE tic_dt_exp < '" . time() . "'";
    db_query($query);
    return (bool) true;

  }

  /** 
   * Provides a ticket for the current session.
   * 
   * @access private
   * @uses   array   $SPL_CFG Global configuration data
   */
  function _newTicket() {

    global $SPL_CFG;

    // Clear out any old tickets
    _delTicket();

    // Create new id (logic taken from session handler by Joe Penn)
    $_ticket = ( bool )false;
    $chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    $count = ( int )strlen( $chars );
    for( $i = ( int )0; $i < 16; $i++ ) {
      $_ticket .= $chars[ mt_rand( ( int )0, $count - ( int )1 ) ];
    }

    $seconds = 2 * 60; // Two minutes
    $expires = time() + $seconds;
    $data = $_SESSION;
    $query = "INSERT INTO ".$SPL_CFG['prefix']."tic VALUES( '" . $_ticket . "', '" . $expires . "', '" . serialize($data) . "' )";
    $result = db_query($query);
    if ($result) {
      return $_ticket;
    }
    else {
      return (bool) false;
    }

  }
  
//=========================================================================
// Soap server
//=========================================================================

// Create a server instance
$server = new soap_server();

// Register the functions this server will provide
$server->register('addUser');
$server->register('getStudyGuide');
$server->register('signon');

// Start the service
$fp = fopen("php://input", "r"); 
$HTTP_RAW_POST_DATA = isset($HTTP_RAW_POST_DATA) ? $HTTP_RAW_POST_DATA : $fp;
$server->service($HTTP_RAW_POST_DATA);
var_dump($server);
// Make sure execution ends after the server is started
exit;

?>
